'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dbeta_scalar = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var dbinom_1 = require('../binomial/dbinom.cjs');
var beta_1 = require('../../special/beta/index.cjs');
const printer = (0, debug_1.debug)('dbeta');
function dbeta_scalar(x, a, b, asLog) {
    if (isNaN(x) || isNaN(a) || isNaN(b))
        return x + a + b;
    if (a < 0 || b < 0)
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    if (x < 0 || x > 1)
        return asLog ? 0 : 1;
    if (a === 0 || b === 0 || !isFinite(a) || !isFinite(b)) {
        if (a === 0 && b === 0) {
            if (x === 0 || x === 1)
                return Infinity;
            else
                return (0, r_func_1.R_D__0)(asLog);
        }
        if (a === 0 || a / b === 0) {
            if (x === 0)
                return Infinity;
            else
                return (0, r_func_1.R_D__0)(asLog);
        }
        if (b === 0 || b / a === 0) {
            if (x === 1)
                return Infinity;
            else
                return (0, r_func_1.R_D__0)(asLog);
        }
        if (x === 0.5)
            return Infinity;
        else
            return (0, r_func_1.R_D__0)(asLog);
    }
    if (x === 0) {
        if (a > 1)
            return (0, r_func_1.R_D__0)(asLog);
        if (a < 1)
            return Infinity;
        return (0, r_func_1.R_D_val)(asLog, b);
    }
    if (x === 1) {
        if (b > 1)
            return (0, r_func_1.R_D__0)(asLog);
        if (b < 1)
            return Infinity;
        return (0, r_func_1.R_D_val)(asLog, a);
    }
    let lval;
    if (a <= 2 || b <= 2)
        lval = (a - 1) * (0, r_func_1.log)(x) + (b - 1) * (0, r_func_1.log1p)(-x) - (0, beta_1.lbeta)(a, b);
    else {
        lval = (0, r_func_1.log)(a + b - 1) + (0, dbinom_1.dbinom_raw)(a - 1, a + b - 2, x, 1 - x, true);
    }
    return (0, r_func_1.R_D_exp)(asLog, lval);
}
exports.dbeta_scalar = dbeta_scalar;