'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pbeta = exports.pbeta_raw = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var toms708_1 = require('../../common/toms708/toms708.cjs');
var NumberW_1 = require('../../common/toms708/NumberW.cjs');
const printer_pbeta_raw = (0, debug_1.debug)('pbeta_raw');
const printer_pbeta = (0, debug_1.debug)('pbeta');
function pbeta_raw(x, a, b, lower_tail, log_p) {
    if (a === 0 && b === 0)
        return log_p ? -Math.LN2 : 0.5;
    if (a === 0 || a / b === 0)
        return (0, r_func_1.R_DT_1)(lower_tail, log_p);
    if (b === 0 || b / a === 0)
        return (0, r_func_1.R_DT_0)(lower_tail, log_p);
    if (!isFinite(a) || !isFinite(b)) {
        if (x < 0.5)
            return (0, r_func_1.R_DT_0)(lower_tail, log_p);
        else
            return (0, r_func_1.R_DT_1)(lower_tail, log_p);
    }
    const x1 = 0.5 - x + 0.5;
    const w = new NumberW_1.NumberW(0);
    const wc = new NumberW_1.NumberW(0);
    const ierr = new NumberW_1.NumberW(0);
    printer_pbeta_raw('before Toms708.bratio, a=%d, b=%d, x=%d, w=%d,wc=%d, ierr=%d', a, b, x, w.val, wc.val, ierr.val);
    toms708_1.Toms708.bratio(a, b, x, x1, w, wc, ierr);
    printer_pbeta_raw('after Toms708.bratio, a=%d, b=%d, x=%d, w=%d,wc=%d, ierr=%d', a, b, x, w.val, wc.val, ierr.val);
    if (ierr.val && ierr.val !== 11 && ierr.val !== 14)
        printer_pbeta_raw('pbeta_raw(%d, a=%d, b=%d, ..) -> bratio() gave error code %d', x, a, b, ierr);
    if (log_p) {
        w.val = (0, r_func_1.log)(w.val);
        wc.val = (0, r_func_1.log)(wc.val);
    }
    return lower_tail ? w.val : wc.val;
}
exports.pbeta_raw = pbeta_raw;
function pbeta(q, a, b, lowerTail = true, logP = false) {
    printer_pbeta('pbeta(q=%d, a=%d, b=%d, l.t=%s, ln=%s)', q, a, b, lowerTail, logP);
    if (isNaN(q) || isNaN(a) || isNaN(b))
        return NaN;
    if (a < 0 || b < 0)
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pbeta, logger_1.lineInfo4);
    if (q <= 0)
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    if (q >= 1)
        return (0, r_func_1.R_DT_1)(lowerTail, logP);
    return pbeta_raw(q, a, b, lowerTail, logP);
}
exports.pbeta = pbeta;