'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pnbeta = exports.pnbeta2 = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var lgammafn_sign_1 = require('../../special/gamma/lgammafn_sign.cjs');
var toms708_1 = require('../../common/toms708/toms708.cjs');
var NumberW_1 = require('../../common/toms708/NumberW.cjs');
const printer = (0, debug_1.debug)('pnbeta_raw');
const printer_pnbeta2 = (0, debug_1.debug)('pnbeta2');
const errmax = 1e-9;
const itrmax = 10000;
function pnbeta_raw(x, o_x, a, b, ncp) {
    let errbd;
    const temp = new NumberW_1.NumberW(0);
    const tmp_c = new NumberW_1.NumberW(0);
    const ierr = new NumberW_1.NumberW(0);
    let ans;
    let ax;
    let gx;
    let q;
    let sumq;
    if (ncp < 0 || a <= 0 || b <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const c = ncp / 2;
    const x0 = (0, r_func_1.floor)((0, r_func_1.max)(c - 7 * (0, r_func_1.sqrt)(c), 0));
    const a0 = a + x0;
    const lbeta = (0, lgammafn_sign_1.lgammafn_sign)(a0) + (0, lgammafn_sign_1.lgammafn_sign)(b) - (0, lgammafn_sign_1.lgammafn_sign)(a0 + b);
    toms708_1.Toms708.bratio(a0, b, x, o_x, temp, tmp_c, ierr);
    gx = (0, r_func_1.exp)(a0 * (0, r_func_1.log)(x) + b * (x < 0.5 ? (0, r_func_1.log1p)(-x) : (0, r_func_1.log)(o_x)) - lbeta - (0, r_func_1.log)(a0));
    if (a0 > a)
        q = (0, r_func_1.exp)(-c + x0 * (0, r_func_1.log)(c) - (0, lgammafn_sign_1.lgammafn_sign)(x0 + 1));
    else
        q = (0, r_func_1.exp)(-c);
    sumq = 1 - q;
    ans = ax = q * temp.val;
    let j = (0, r_func_1.floor)(x0);
    do {
        j++;
        temp.val -= gx;
        gx *= x * (a + b + j - 1) / (a + j);
        q *= c / j;
        sumq -= q;
        ax = temp.val * q;
        ans += ax;
        errbd = (temp.val - gx) * sumq;
    } while (errbd > errmax && j < itrmax + x0);
    if (errbd > errmax)
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'pnbeta', printer);
    if (j >= itrmax + x0)
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_NOCONV, 'pnbeta', printer);
    return ans;
}
function pnbeta2(x, o_x, a, b, ncp, lower_tail, log_p) {
    let ans = pnbeta_raw(x, o_x, a, b, ncp);
    if (isNaN(ans))
        return NaN;
    if (lower_tail) {
        return log_p ? (0, r_func_1.log)(ans) : ans;
    } else {
        if (ans > 1 - 1e-10)
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'pnbeta', printer_pnbeta2);
        if (ans > 1)
            ans = 1;
        return log_p ? (0, r_func_1.log1p)(-ans) : 1 - ans;
    }
}
exports.pnbeta2 = pnbeta2;
function pnbeta(x, a, b, ncp, lower_tail, log_p) {
    if (isNaN(x) || isNaN(a) || isNaN(b) || isNaN(ncp))
        return x + a + b + ncp;
    const rc = (0, r_func_1.R_P_bounds_01)(lower_tail, log_p, x, 0, 1);
    if (rc !== undefined) {
        return rc;
    }
    return pnbeta2(x, 1 - x, a, b, ncp, lower_tail, log_p);
}
exports.pnbeta = pnbeta;