'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qnbeta = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var pnbeta_1 = require('./pnbeta.cjs');
const printer_qnbeta = (0, debug_1.debug)('qnbeta');
const accu = 1e-15;
const eps = 1e-14;
function qnbeta(p, a, b, ncp, lower_tail, log_p) {
    let ux;
    let lx;
    let nx;
    let pp;
    if (isNaN(p) || isNaN(a) || isNaN(b) || isNaN(ncp))
        return p + a + b + ncp;
    if (!isFinite(a))
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qnbeta, logger_1.lineInfo4);
    if (ncp < 0 || a <= 0 || b <= 0)
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qnbeta, logger_1.lineInfo4);
    const rc = (0, logger_1.R_Q_P01_boundaries)(lower_tail, log_p, p, 0, 1);
    if (rc !== undefined) {
        return rc;
    }
    p = (0, expm1_1.R_DT_qIv)(lower_tail, log_p, p);
    if (p > 1 - Number.EPSILON)
        return 1;
    pp = Math.min(1 - Number.EPSILON, p * (1 + eps));
    for (ux = 0.5; ux < 1 - Number.EPSILON && (0, pnbeta_1.pnbeta)(ux, a, b, ncp, true, false) < pp; ux = 0.5 * (1 + ux));
    pp = p * (1 - eps);
    for (lx = 0.5; lx > Number.MIN_VALUE && (0, pnbeta_1.pnbeta)(lx, a, b, ncp, true, false) > pp; lx *= 0.5);
    do {
        nx = 0.5 * (lx + ux);
        if ((0, pnbeta_1.pnbeta)(nx, a, b, ncp, true, false) > p)
            ux = nx;
        else
            lx = nx;
    } while ((ux - lx) / nx > accu);
    return 0.5 * (ux + lx);
}
exports.qnbeta = qnbeta;