'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.rbetaOne = exports.expmax = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var global_rng_1 = require('../../rng/global-rng.cjs');
const printer = (0, debug_1.debug)('rbeta');
exports.expmax = r_func_1.DBL_MAX_EXP * Math.LN2;
function rbetaOne(shape1, shape2) {
    const rng = (0, global_rng_1.globalUni)();
    if (isNaN(shape1) || isNaN(shape2)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (shape1 < 0 || shape2 < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (!isFinite(shape1) && !isFinite(shape2))
        return 0.5;
    if (shape1 === 0 && shape2 === 0)
        return rng.random() < 0.5 ? 0 : 1;
    if (!isFinite(shape1) || shape2 === 0)
        return 1;
    if (!isFinite(shape2) || shape1 === 0)
        return 0;
    let r;
    let s;
    let t;
    let u1 = 0;
    let u2;
    let v = 0;
    let w = 0;
    let y;
    let z;
    let beta = 0;
    let gamma = 0;
    let delta;
    let k1 = 0;
    let k2 = 0;
    let olda = -1;
    let oldb = -1;
    const qsame = olda === shape1 && oldb === shape2;
    if (!qsame) {
        olda = shape1;
        oldb = shape2;
    }
    const a = (0, r_func_1.min)(shape1, shape2);
    const b = (0, r_func_1.max)(shape1, shape2);
    const alpha = a + b;
    function v_w_from__u1_bet(AA) {
        v = beta * (0, r_func_1.log)(u1 / (1 - u1));
        if (v <= exports.expmax) {
            w = AA * Math.exp(v);
            if (!isFinite(w)) {
                w = Number.MAX_VALUE;
            }
        } else {
            w = Number.MAX_VALUE;
        }
    }
    if (a <= 1) {
        if (!qsame) {
            beta = 1 / a;
            delta = 1 + b - a;
            k1 = delta * (0.0138889 + 0.0416667 * a) / (b * beta - 0.777778);
            k2 = 0.25 + (0.5 + 0.25 / delta) * a;
        }
        for (;;) {
            u1 = rng.random();
            u2 = rng.random();
            if (u1 < 0.5) {
                y = u1 * u2;
                z = u1 * y;
                if (0.25 * u2 + z - y >= k1)
                    continue;
            } else {
                z = u1 * u1 * u2;
                if (z <= 0.25) {
                    v_w_from__u1_bet(b);
                    break;
                }
                if (z >= k2)
                    continue;
            }
            v_w_from__u1_bet(b);
            if (alpha * ((0, r_func_1.log)(alpha / (a + w)) + v) - 1.3862944 >= (0, r_func_1.log)(z))
                break;
        }
        return shape1 === a ? a / (a + w) : w / (a + w);
    } else {
        if (!qsame) {
            beta = Math.sqrt((alpha - 2) / (2 * a * b - alpha));
            gamma = a + 1 / beta;
        }
        do {
            u1 = rng.random();
            u2 = rng.random();
            v_w_from__u1_bet(a);
            z = u1 * u1 * u2;
            r = gamma * v - 1.3862944;
            s = a + r - w;
            if (s + 2.609438 >= 5 * z)
                break;
            t = (0, r_func_1.log)(z);
            if (s > t)
                break;
        } while (r + alpha * (0, r_func_1.log)(alpha / (b + w)) < t);
        return shape1 !== a ? b / (b + w) : w / (b + w);
    }
}
exports.rbetaOne = rbetaOne;