'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dnbinom_mu = exports.dnbinom = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var dbinom_1 = require('../binomial/dbinom.cjs');
var lgammafn_sign_1 = require('../../special/gamma/lgammafn_sign.cjs');
const printer = (0, debug_1.debug)('dnbinom');
function dnbinom(x, size, prob, give_log) {
    if (isNaN(x) || isNaN(size) || isNaN(prob)) {
        return x + size + prob;
    }
    if (prob <= 0 || prob > 1 || size < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const rc = (0, r_func_1.R_D_nonint_check)(give_log, x, printer);
    if (rc !== undefined) {
        return rc;
    }
    if (x < 0) {
        return (0, r_func_1.R_D__0)(give_log);
    }
    if (x === 0 && size === 0) {
        return (0, r_func_1.R_D__1)(give_log);
    }
    x = Math.round(x);
    const ans = (0, dbinom_1.dbinom_raw)(size, x + size, prob, 1 - prob, give_log);
    const p = size / (size + x);
    return give_log ? Math.log(p) + ans : p * ans;
}
exports.dnbinom = dnbinom;
const printer_dnbinom_mu = (0, debug_1.debug)('dnbinom_mu');
function dnbinom_mu(x, size, mu, give_log) {
    let p;
    if (isNaN(x) || isNaN(size) || isNaN(mu)) {
        return x + size + mu;
    }
    if (mu < 0 || size < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_dnbinom_mu, logger_1.lineInfo4);
    }
    const rc = (0, r_func_1.R_D_nonint_check)(give_log, x, printer_dnbinom_mu);
    if (rc !== undefined) {
        return rc;
    }
    if (x < 0 || !isFinite(x)) {
        return (0, r_func_1.R_D__0)(give_log);
    }
    if (x === 0 && size === 0) {
        return (0, r_func_1.R_D__1)(give_log);
    }
    x = Math.round(x);
    if (x === 0) {
        let llogx;
        if (size < mu) {
            llogx = Math.log(size / (size + mu));
        } else {
            llogx = Math.log1p(-mu / (size + mu));
        }
        return (0, r_func_1.R_D_exp)(give_log, size * llogx);
    }
    if (x < 1e-10 * size) {
        p = size < mu ? Math.log(size / (1 + size / mu)) : Math.log(mu / (1 + mu / size));
        return (0, r_func_1.R_D_exp)(give_log, x * p - mu - (0, lgammafn_sign_1.lgammafn_sign)(x + 1) + Math.log1p(x * (x - 1) / (2 * size)));
    }
    const ans = (0, dbinom_1.dbinom_raw)(size, x + size, size / (size + mu), mu / (size + mu), give_log);
    p = size / (size + x);
    return give_log ? Math.log(p) + ans : p * ans;
}
exports.dnbinom_mu = dnbinom_mu;