'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pnbinom_mu = exports.pnbinom = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var pbeta_1 = require('../beta/pbeta.cjs');
var toms708_1 = require('../../common/toms708/toms708.cjs');
var NumberW_1 = require('../../common/toms708/NumberW.cjs');
const printer = (0, debug_1.debug)('pnbinom');
function pnbinom(x, size, prob, lowerTail, logP) {
    if (isNaN(x) || isNaN(size) || isNaN(prob))
        return x + size + prob;
    if (!isFinite(size) || !isFinite(prob)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (size < 0 || prob <= 0 || prob > 1) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (size === 0)
        return x >= 0 ? (0, r_func_1.R_DT_1)(lowerTail, logP) : (0, r_func_1.R_DT_0)(lowerTail, logP);
    if (x < 0)
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    if (!isFinite(x))
        return (0, r_func_1.R_DT_1)(lowerTail, logP);
    x = Math.floor(x + 1e-7);
    return (0, pbeta_1.pbeta)(prob, size, x + 1, lowerTail, logP);
}
exports.pnbinom = pnbinom;
const printer_pnbinom_mu = (0, debug_1.debug)('pnbinom_mu');
function pnbinom_mu(x, size, mu, lowerTail, logP) {
    if (isNaN(x) || isNaN(size) || isNaN(mu))
        return x + size + mu;
    if (!isFinite(size) || !isFinite(mu))
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pnbinom_mu, logger_1.lineInfo4);
    if (size < 0 || mu < 0)
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pnbinom_mu, logger_1.lineInfo4);
    if (size === 0)
        return x >= 0 ? (0, r_func_1.R_DT_1)(lowerTail, logP) : (0, r_func_1.R_DT_0)(lowerTail, logP);
    if (x < 0)
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    if (!isFinite(x))
        return (0, r_func_1.R_DT_1)(lowerTail, logP);
    x = Math.floor(x + 1e-7);
    {
        const ierr = new NumberW_1.NumberW(0);
        const w = new NumberW_1.NumberW(0);
        const wc = new NumberW_1.NumberW(0);
        toms708_1.Toms708.bratio(size, x + 1, size / (size + mu), mu / (size + mu), w, wc, ierr);
        if (ierr.val) {
            printer('pnbinom_mu() -> bratio() gave error code %d', ierr.val);
        }
        if (logP) {
            w.val = Math.log(w.val);
            wc.val = Math.log(wc.val);
        }
        return lowerTail ? w.val : wc.val;
    }
}
exports.pnbinom_mu = pnbinom_mu;