'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qnbinom_mu = exports.qnbinom = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var NumberW_1 = require('../../common/toms708/NumberW.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var qnorm_1 = require('../normal/qnorm.cjs');
var pnbinom_1 = require('./pnbinom.cjs');
const printer_do_search = (0, debug_1.debug)('do_search');
function do_search(y, z, p, n, pr, incr) {
    printer_do_search('start: y:%d, z:%o, p:%d, n:%d, pr:%d, incr:%d', y, z, p, n, pr, incr);
    if (z.val >= p) {
        for (;;) {
            if (y === 0 || (z.val = (0, pnbinom_1.pnbinom)(y - incr, n, pr, true, false)) < p) {
                printer_do_search('exit1');
                return y;
            }
            y = Math.max(0, y - incr);
        }
    } else {
        for (;;) {
            y = y + incr;
            if ((z.val = (0, pnbinom_1.pnbinom)(y, n, pr, true, false)) >= p) {
                printer_do_search('exit2');
                return y;
            }
        }
    }
}
const printer_qnbinom = (0, debug_1.debug)('qnbinom');
function qnbinom(p, size, prob, lower_tail, log_p) {
    let y;
    const z = new NumberW_1.NumberW(0);
    if (isNaN(p) || isNaN(size) || isNaN(prob)) {
        return NaN;
    }
    if (prob === 0 && size === 0)
        return 0;
    if (prob <= 0 || prob > 1 || size < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qnbinom, logger_1.lineInfo4);
    }
    if (prob === 1 || size === 0)
        return 0;
    const rc = (0, logger_1.R_Q_P01_boundaries)(lower_tail, log_p, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    const Q = 1 / prob;
    const P = (1 - prob) * Q;
    const mu = size * P;
    const sigma = Math.sqrt(size * P * Q);
    const gamma = (Q + P) / sigma;
    if (!lower_tail || log_p) {
        p = (0, expm1_1.R_DT_qIv)(lower_tail, log_p, p);
        const rdt0 = (0, r_func_1.R_DT_0)(lower_tail, log_p);
        const rdt1 = (0, r_func_1.R_DT_1)(lower_tail, log_p);
        if (p === rdt0)
            return 0;
        if (p === rdt1)
            return Infinity;
    }
    if (p + 1.01 * Number.EPSILON >= 1)
        return Infinity;
    z.val = (0, qnorm_1.qnorm)(p, 0, 1, true, false);
    y = Math.round(mu + sigma * (z.val + gamma * (z.val * z.val - 1) / 6));
    z.val = (0, pnbinom_1.pnbinom)(y, size, prob, true, false);
    p *= 1 - 64 * Number.EPSILON;
    if (y < 100000)
        return do_search(y, z, p, size, prob, 1);
    {
        let incr = Math.floor(y * 0.001);
        let oldincr;
        do {
            oldincr = incr;
            y = do_search(y, z, p, size, prob, incr);
            incr = Math.max(1, Math.floor(incr / 100));
        } while (oldincr > 1 && incr > y * 1e-15);
        return y;
    }
}
exports.qnbinom = qnbinom;
function qnbinom_mu(p, size, mu, lower_tail, log_p) {
    return qnbinom(p, size, size / (size + mu), lower_tail, log_p);
}
exports.qnbinom_mu = qnbinom_mu;