'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dbinom = exports.dbinom_raw = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var deviance_1 = require('../../deviance/index.cjs');
var stirling_1 = require('../../stirling/index.cjs');
const printer = (0, debug_1.debug)('dbinom');
function dbinom_raw(x, n, p, q, give_log) {
    let lc;
    if (p === 0)
        return x === 0 ? (0, r_func_1.R_D__1)(give_log) : (0, r_func_1.R_D__0)(give_log);
    if (q === 0)
        return x === n ? (0, r_func_1.R_D__1)(give_log) : (0, r_func_1.R_D__0)(give_log);
    if (x === 0) {
        if (n === 0)
            return (0, r_func_1.R_D__1)(give_log);
        const lc = p < 0.1 ? -(0, deviance_1.bd0)(n, n * q) - n * p : n * Math.log(q);
        return (0, r_func_1.R_D_exp)(give_log, lc);
    }
    if (x === n) {
        lc = q < 0.1 ? -(0, deviance_1.bd0)(n, n * p) - n * q : n * Math.log(p);
        return (0, r_func_1.R_D_exp)(give_log, lc);
    }
    if (x < 0 || x > n)
        return (0, r_func_1.R_D__0)(give_log);
    lc = (0, stirling_1.stirlerr)(n) - (0, stirling_1.stirlerr)(x) - (0, stirling_1.stirlerr)(n - x) - (0, deviance_1.bd0)(x, n * p) - (0, deviance_1.bd0)(n - x, n * q);
    const lf = r_func_1.M_LN_2PI + Math.log(x) + Math.log1p(-x / n);
    return (0, r_func_1.R_D_exp)(give_log, lc - 0.5 * lf);
}
exports.dbinom_raw = dbinom_raw;
function dbinom(x, n, prob, log = false) {
    if (isNaN(x) || isNaN(n) || isNaN(prob))
        return x + n + prob;
    if (prob < 0 || prob > 1 || (0, r_func_1.R_D_negInonint)(n))
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    const ch = (0, r_func_1.R_D_nonint_check)(log, x, printer);
    if (ch !== undefined) {
        return ch;
    }
    if (x < 0 || !isFinite(x))
        return (0, r_func_1.R_D__0)(log);
    n = Math.round(n);
    x = Math.round(x);
    return dbinom_raw(x, n, prob, 1 - prob, log);
}
exports.dbinom = dbinom;