'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pbinom = void 0;
var debug_1 = require('@mangos/debug');
var pbeta_1 = require('../beta/pbeta.cjs');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
const printer = (0, debug_1.debug)('pbinom');
function pbinom(x, n, prob, lowerTail = true, logP = false) {
    if (isNaN(x) || isNaN(n) || isNaN(prob))
        return NaN;
    if (!isFinite(n) || !isFinite(prob)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const lower_tail = lowerTail;
    const log_p = logP;
    if ((0, r_func_1.R_nonint)(n)) {
        printer('non-integer n = %d', n);
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    n = Math.round(n);
    if (n < 0 || prob < 0 || prob > 1)
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    if (x < 0)
        return (0, r_func_1.R_DT_0)(lower_tail, log_p);
    x = Math.floor(x + 1e-7);
    if (n <= x)
        return (0, r_func_1.R_DT_1)(lower_tail, log_p);
    printer('calling pbeta:(q=%d,a=%d,b=%d, l.t=%s, log=%s', prob, x + 1, n - x, !lower_tail, log_p);
    return (0, pbeta_1.pbeta)(prob, x + 1, n - x, !lower_tail, log_p);
}
exports.pbinom = pbinom;