'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qbinom = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var NumberW_1 = require('../../common/toms708/NumberW.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var qnorm_1 = require('../normal/qnorm.cjs');
var pbinom_1 = require('./pbinom.cjs');
const printer_do_search = (0, debug_1.debug)('do_search');
function do_search(y, z, p, n, pr, incr) {
    if (z.val >= p) {
        printer_do_search('new z=%o >= p = %d  --> search to left (y--) ..', z, p);
        for (;;) {
            let newz;
            if (y === 0 || (newz = (0, pbinom_1.pbinom)(y - incr, n, pr, true, false)) < p)
                return y;
            y = Math.max(0, y - incr);
            z.val = newz;
        }
    } else {
        printer_do_search('new z=%d < p = %d  --> search to right (y++) ..', z.val, p);
        for (;;) {
            y = Math.min(y + incr, n);
            if (y === n || (z.val = (0, pbinom_1.pbinom)(y, n, pr, true, false)) >= p)
                return y;
        }
    }
}
const printer_qbinom = (0, debug_1.debug)('qbinom');
function qbinom(p, size, prob, lowerTail = true, logP = false) {
    const z = new NumberW_1.NumberW(0);
    let y;
    if (isNaN(p) || isNaN(size) || isNaN(prob))
        return NaN;
    if (!isFinite(size) || !isFinite(prob)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qbinom, logger_1.lineInfo4);
    }
    if (!isFinite(p) && !logP) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qbinom, logger_1.lineInfo4);
    }
    if (!Number.isInteger(size)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qbinom, logger_1.lineInfo4);
    }
    if (prob < 0 || prob > 1 || size < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qbinom, logger_1.lineInfo4);
    }
    const rc = (0, logger_1.R_Q_P01_boundaries)(lowerTail, logP, p, 0, size);
    if (rc !== undefined) {
        return rc;
    }
    if (prob === 0 || size === 0)
        return 0;
    const q = 1 - prob;
    if (q === 0)
        return size;
    const mu = size * prob;
    const sigma = Math.sqrt(size * prob * q);
    const gamma = (q - prob) / sigma;
    printer_qbinom('qbinom(p=%d, n=%d, prob=%d, l.t.=%s, log=%s): sigm=%d, gam=%d', p, size, prob, lowerTail, logP, sigma, gamma);
    if (!lowerTail || logP) {
        p = (0, expm1_1.R_DT_qIv)(lowerTail, logP, p);
        if (p === 0)
            return 0;
        if (p === 1)
            return size;
    }
    if (1 - p < Number.EPSILON) {
        return size;
    }
    z.val = (0, qnorm_1.qnorm)(p, 0, 1, true, false);
    y = Math.floor(mu + sigma * (z.val + gamma * (z.val * z.val - 1) / 6) + 0.5);
    if (y > size) {
        y = size;
    }
    printer_qbinom('new (p,1-p)=(%d,%d), z=qnorm(..)=%d, y=%d, size=%d', p, 1 - p, z.val, y, size);
    z.val = (0, pbinom_1.pbinom)(y, size, prob, true, false);
    p *= 1 - 64 * Number.EPSILON;
    if (size < 100000) {
        return do_search(y, z, p, size, prob, 1);
    }
    let incr = Math.floor(size * 0.001);
    let oldincr;
    do {
        oldincr = incr;
        y = do_search(y, z, p, size, prob, incr);
        incr = Math.max(1, Math.floor(incr / 100));
    } while (oldincr > 1 && incr > size * 1e-15);
    return y;
}
exports.qbinom = qbinom;