'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pcauchy = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var r_func_2 = require('../../r-func.cjs');
var tanpi_1 = require('../../trigonometry/tanpi.cjs');
const printer = (0, debug_1.debug)('pcauchy');
function pcauchy(x, location = 0, scale = 1, lowerTail = true, logP = false) {
    if (isNaN(x) || isNaN(location) || isNaN(scale))
        return x + location + scale;
    if (scale <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    x = (x - location) / scale;
    if (isNaN(x)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (!isFinite(x)) {
        if (x < 0)
            return (0, r_func_1.R_DT_0)(lowerTail, logP);
        else
            return (0, r_func_1.R_DT_1)(lowerTail, logP);
    }
    if (!lowerTail)
        x = -x;
    if (Math.abs(x) > 1) {
        const y = (0, tanpi_1.atanpi)(1 / x);
        return x > 0 ? (0, r_func_2.R_D_Clog)(logP, y) : (0, r_func_1.R_D_val)(logP, -y);
    } else {
        return (0, r_func_1.R_D_val)(logP, 0.5 + (0, tanpi_1.atanpi)(x));
    }
}
exports.pcauchy = pcauchy;