'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qcauchy = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var tanpi_1 = require('../../trigonometry/tanpi.cjs');
const printer = (0, debug_1.debug)('qcauchy');
function qcauchy(p, location = 0, scale = 1, lowerTail = true, logP = false) {
    if (isNaN(p) || isNaN(location) || isNaN(scale))
        return NaN;
    let lower_tail = lowerTail;
    const rc = (0, logger_1.R_Q_P01_check)(logP, p);
    if (rc !== undefined) {
        return rc;
    }
    if (scale <= 0 || !isFinite(scale)) {
        if (scale === 0)
            return location;
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const my_INF = lower_tail ? Infinity : -Infinity;
    if (logP) {
        if (p > -1) {
            if (p === 0) {
                return my_INF;
            }
            lower_tail = !lower_tail;
            p = -Math.expm1(p);
        } else {
            p = Math.exp(p);
        }
    } else {
        if (p > 0.5) {
            if (p === 1)
                return my_INF;
            p = 1 - p;
            lower_tail = !lower_tail;
        }
    }
    if (p === 0.5)
        return location;
    if (p === 0)
        return lower_tail ? -Infinity : Infinity;
    return location + (lower_tail ? -scale : scale) / (0, tanpi_1.tanpi)(p);
}
exports.qcauchy = qcauchy;