'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dnchisq = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var dchisq_1 = require('./dchisq.cjs');
var dpois_1 = require('../poisson/dpois.cjs');
const printer_dnchisq = (0, debug_1.debug)('dnchisq');
const eps = 5e-15;
function dnchisq(x, df, ncp, give_log) {
    let i;
    let q;
    let mid;
    let dfmid;
    let imax;
    let sum;
    let term;
    if (isNaN(x) || isNaN(df) || isNaN(ncp)) {
        return x + df + ncp;
    }
    if (!isFinite(df) || !isFinite(ncp) || ncp < 0 || df < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_dnchisq, logger_1.lineInfo4);
    }
    if (x < 0) {
        return (0, r_func_1.R_D__0)(give_log);
    }
    if (x === 0 && df < 2)
        return Infinity;
    if (ncp === 0)
        return df > 0 ? (0, dchisq_1.dchisq)(x, df, give_log) : (0, r_func_1.R_D__0)(give_log);
    if (x === Infinity)
        return (0, r_func_1.R_D__0)(give_log);
    const ncp2 = 0.5 * ncp;
    imax = Math.ceil((-(2 + df) + Math.sqrt((2 - df) * (2 - df) + 4 * ncp * x)) / 4);
    if (imax < 0)
        imax = 0;
    if (isFinite(imax)) {
        dfmid = df + 2 * imax;
        mid = (0, dpois_1.dpois_raw)(imax, ncp2, false) * (0, dchisq_1.dchisq)(x, dfmid, false);
    } else {
        throw new Error(`Internal Error IE989: imax=${ imax }`);
    }
    sum = mid;
    term = mid;
    df = dfmid;
    i = imax;
    const x2 = x * ncp2;
    do {
        i++;
        q = x2 / i / df;
        df += 2;
        term *= q;
        sum += term;
    } while (q >= 1 || term * q > (1 - q) * eps || term > 1e-10 * sum);
    term = mid;
    df = dfmid;
    i = imax;
    while (i !== 0) {
        df -= 2;
        q = i * df / x2;
        i--;
        term *= q;
        sum += term;
        if (q < 1 && term * q <= (1 - q) * eps)
            break;
    }
    return (0, r_func_1.R_D_val)(give_log, sum);
}
exports.dnchisq = dnchisq;