'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qnchisq = void 0;
var debug_1 = require('@mangos/debug');
var r_func_1 = require('../../r-func.cjs');
var logger_1 = require('../../../packages/common/logger.cjs');
var qchisq_1 = require('./qchisq.cjs');
var pnchisq_1 = require('./pnchisq.cjs');
const printer = (0, debug_1.debug)('qnchisq');
const accu = 1e-13;
const racc = 4 * Number.EPSILON;
const DBL_MIN = 2.2250738585072014e-308;
const DBL_MAX = 1.7976931348623157e+308;
const Eps = 1e-11;
const rEps = 1e-10;
function qnchisq(p, df, ncp, lower_tail, log_p) {
    let ux;
    let lx;
    let ux0;
    let nx;
    let pp;
    if (isNaN(p) || isNaN(df) || isNaN(ncp)) {
        return NaN;
    }
    if (!isFinite(df)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (df < 0 || ncp < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const rc = (0, logger_1.R_Q_P01_boundaries)(lower_tail, log_p, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    pp = (0, r_func_1.R_D_qIv)(log_p, p);
    if (pp > 1 - Number.EPSILON) {
        return lower_tail ? Infinity : 0;
    }
    {
        const b = ncp * ncp / (df + 3 * ncp);
        const c = (df + 3 * ncp) / (df + 2 * ncp);
        const ff = (df + 2 * ncp) / (c * c);
        ux = b + c * (0, qchisq_1.qchisq)(p, ff, lower_tail, log_p);
        if (ux < 0)
            ux = 1;
        ux0 = ux;
    }
    if (!lower_tail && ncp >= 80) {
        if (pp < 1e-10)
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'qnchisq', printer);
        p = log_p ? -Math.expm1(pp) : 1 - pp;
        lower_tail = true;
    } else {
        p = pp;
    }
    pp = Math.min(1 - Number.EPSILON, p * (1 + Eps));
    if (lower_tail) {
        for (; ux < DBL_MAX && (0, pnchisq_1.pnchisq_raw)(ux, df, ncp, Eps, rEps, 10000, true, false) < pp; ux *= 2);
        pp = p * (1 - Eps);
        for (lx = Math.min(ux0, DBL_MAX); lx > DBL_MIN && (0, pnchisq_1.pnchisq_raw)(lx, df, ncp, Eps, rEps, 10000, true, false) > pp; lx *= 0.5);
    } else {
        for (; ux < DBL_MAX && (0, pnchisq_1.pnchisq_raw)(ux, df, ncp, Eps, rEps, 10000, false, false) > pp; ux *= 2);
        pp = p * (1 - Eps);
        for (lx = Math.min(ux0, DBL_MAX); lx > DBL_MIN && (0, pnchisq_1.pnchisq_raw)(lx, df, ncp, Eps, rEps, 10000, false, false) < pp; lx *= 0.5);
    }
    if (lower_tail) {
        do {
            nx = 0.5 * (lx + ux);
            if ((0, pnchisq_1.pnchisq_raw)(nx, df, ncp, accu, racc, 100000, true, false) > p)
                ux = nx;
            else
                lx = nx;
        } while ((ux - lx) / nx > accu);
    } else {
        do {
            nx = 0.5 * (lx + ux);
            if ((0, pnchisq_1.pnchisq_raw)(nx, df, ncp, accu, racc, 100000, false, false) < p)
                ux = nx;
            else
                lx = nx;
        } while ((ux - lx) / nx > accu);
    }
    return 0.5 * (ux + lx);
}
exports.qnchisq = qnchisq;