'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.rnchisqOne = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var rgamma_1 = require('../gamma/rgamma.cjs');
var rpois_1 = require('../poisson/rpois.cjs');
var rchisq_1 = require('./rchisq.cjs');
const printer = (0, debug_1.debug)('rnchisq');
function rnchisqOne(df, lambda) {
    if (!isFinite(df) || !isFinite(lambda) || df < 0 || lambda < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (lambda === 0) {
        return df === 0 ? 0 : (0, rgamma_1.rgamma)(df / 2, 2);
    } else {
        let r = (0, rpois_1.rpoisOne)(lambda / 2);
        if (r > 0)
            r = (0, rchisq_1.rchisqOne)(2 * r);
        if (df > 0)
            r += (0, rgamma_1.rgamma)(df / 2, 2);
        return r;
    }
}
exports.rnchisqOne = rnchisqOne;