'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.df = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var dbinom_1 = require('../binomial/dbinom.cjs');
var dgamma_1 = require('../gamma/dgamma.cjs');
const printer_df = (0, debug_1.debug)('df');
function df(x, m, n, giveLog) {
    let f;
    let dens;
    if (isNaN(x) || isNaN(m) || isNaN(n)) {
        return x + m + n;
    }
    if (m <= 0 || n <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_df, logger_1.lineInfo4);
    }
    if (x < 0) {
        return (0, r_func_1.R_D__0)(giveLog);
    }
    if (x === 0) {
        return m > 2 ? (0, r_func_1.R_D__0)(giveLog) : m === 2 ? (0, r_func_1.R_D__1)(giveLog) : Infinity;
    }
    if (!isFinite(m) && !isFinite(n)) {
        if (x === 1) {
            return Infinity;
        } else {
            return (0, r_func_1.R_D__0)(giveLog);
        }
    }
    if (!isFinite(n)) {
        return (0, dgamma_1.dgamma)(x, m / 2, 2 / m, giveLog);
    }
    if (m > 100000000000000) {
        dens = (0, dgamma_1.dgamma)(1 / x, n / 2, 2 / n, giveLog);
        return giveLog ? dens - 2 * Math.log(x) : dens / (x * x);
    }
    f = 1 / (n + x * m);
    const q = n * f;
    const p = x * m * f;
    if (m >= 2) {
        f = m * q / 2;
        dens = (0, dbinom_1.dbinom_raw)((m - 2) / 2, (m + n - 2) / 2, p, q, giveLog);
    } else {
        f = m * m * q / (2 * p * (m + n));
        dens = (0, dbinom_1.dbinom_raw)(m / 2, (m + n) / 2, p, q, giveLog);
    }
    return giveLog ? Math.log(f) + dens : f * dens;
}
exports.df = df;