'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dnf = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var dnbeta_1 = require('../beta/dnbeta.cjs');
var dnchisq_1 = require('../chi-2/dnchisq.cjs');
var dgamma_1 = require('../gamma/dgamma.cjs');
const printer = (0, debug_1.debug)('dnf');
function dnf(x, df1, df2, ncp, giveLog) {
    let z;
    let f;
    if (isNaN(x) || isNaN(df1) || isNaN(df2) || isNaN(ncp)) {
        return x + df2 + df1 + ncp;
    }
    if (df1 <= 0 || df2 <= 0 || ncp < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (x < 0) {
        return (0, r_func_1.R_D__0)(giveLog);
    }
    if (!isFinite(ncp)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (!isFinite(df1) && !isFinite(df2)) {
        if (x === 1)
            return Infinity;
        else
            return (0, r_func_1.R_D__0)(giveLog);
    }
    if (!isFinite(df2))
        return df1 * (0, dnchisq_1.dnchisq)(x * df1, df1, ncp, giveLog);
    if (df1 > 100000000000000 && ncp < 10000000) {
        f = 1 + ncp / df1;
        z = (0, dgamma_1.dgamma)(1 / x / f, df2 / 2, 2 / df2, giveLog);
        return giveLog ? z - 2 * Math.log(x) - Math.log(f) : z / (x * x) / f;
    }
    const y = df1 / df2 * x;
    z = (0, dnbeta_1.dnbeta_scalar)(y / (1 + y), df1 / 2, df2 / 2, ncp, giveLog);
    return giveLog ? z + Math.log(df1) - Math.log(df2) - 2 * Math.log1p(y) : z * (df1 / df2) / (1 + y) / (1 + y);
}
exports.dnf = dnf;