'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pf = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var pbeta_1 = require('../beta/pbeta.cjs');
var pchisq_1 = require('../chi-2/pchisq.cjs');
const printer_pf = (0, debug_1.debug)('pf');
function pf(q, df1, df2, lowerTail, logP) {
    if (isNaN(q) || isNaN(df1) || isNaN(df2))
        return q + df2 + df1;
    if (df1 <= 0 || df2 <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pf, logger_1.lineInfo4);
    }
    const rc = (0, r_func_1.R_P_bounds_01)(lowerTail, logP, q, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (df2 === Infinity) {
        if (df1 === Infinity) {
            if (q < 1)
                return (0, r_func_1.R_DT_0)(lowerTail, logP);
            if (q === 1)
                return logP ? -Math.LN2 : 0.5;
            return (0, r_func_1.R_DT_1)(lowerTail, logP);
        }
        return (0, pchisq_1.pchisq)(q * df1, df1, lowerTail, logP);
    }
    if (df1 === Infinity) {
        const _d = (0, pchisq_1.pchisq)(df2 / q, df2, !lowerTail, logP);
        return _d;
    }
    if (df1 * q > df2)
        q = (0, pbeta_1.pbeta)(df2 / (df2 + df1 * q), df2 / 2, df1 / 2, !lowerTail, logP);
    else
        q = (0, pbeta_1.pbeta)(df1 * q / (df2 + df1 * q), df1 / 2, df2 / 2, lowerTail, logP);
    return isFinite(q) ? q : NaN;
}
exports.pf = pf;