'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pnf = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var pnbeta_1 = require('../beta/pnbeta.cjs');
var pnchisq_1 = require('../chi-2/pnchisq.cjs');
const printer_pnf = (0, debug_1.debug)('pnf');
function pnf(x, df1, df2, ncp, lowerTail, logP) {
    if (isNaN(x) || isNaN(df1) || isNaN(df2) || isNaN(ncp))
        return x + df2 + df1 + ncp;
    if (df1 <= 0 || df2 <= 0 || ncp < 0)
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pnf, logger_1.lineInfo4);
    if (!isFinite(ncp))
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pnf, logger_1.lineInfo4);
    if (!isFinite(df1) && !isFinite(df2))
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pnf, logger_1.lineInfo4);
    const rc = (0, r_func_1.R_P_bounds_01)(lowerTail, logP, x, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (df2 > 100000000)
        return (0, pnchisq_1.pnchisq)(x * df1, df1, ncp, lowerTail, logP);
    const y = df1 / df2 * x;
    return (0, pnbeta_1.pnbeta2)(y / (1 + y), 1 / (1 + y), df1 / 2, df2 / 2, ncp, lowerTail, logP);
}
exports.pnf = pnf;