'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qf = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var qbeta_1 = require('../beta/qbeta.cjs');
var qchisq_1 = require('../chi-2/qchisq.cjs');
const printer = (0, debug_1.debug)('qf');
function qf(p, df1, df2, lower_tail, log_p) {
    if (isNaN(p) || isNaN(df1) || isNaN(df2))
        return p + df1 + df2;
    if (df1 <= 0 || df2 <= 0)
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    const rc = (0, logger_1.R_Q_P01_boundaries)(lower_tail, log_p, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (df1 <= df2 && df2 > 400000) {
        if (!isFinite(df1))
            return 1;
        return (0, qchisq_1.qchisq)(p, df1, lower_tail, log_p) / df1;
    }
    if (df1 > 400000) {
        return df2 / (0, qchisq_1.qchisq)(p, df2, !lower_tail, log_p);
    }
    p = (1 / (0, qbeta_1.qbeta)(p, df2 / 2, df1 / 2, !lower_tail, log_p) - 1) * (df2 / df1);
    return isFinite(p) ? p : NaN;
}
exports.qf = qf;