'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dgamma = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var dpois_1 = require('../poisson/dpois.cjs');
var r_func_1 = require('../../r-func.cjs');
const printer = (0, debug_1.debug)('dgamma');
function dgamma(x, shape, scale, aslog) {
    let pr;
    if (isNaN(x) || isNaN(shape) || isNaN(scale))
        return x + shape + scale;
    if (shape < 0 || scale <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (x < 0) {
        return (0, r_func_1.R_D__0)(aslog);
    }
    if (shape === 0) {
        return x === 0 ? Infinity : (0, r_func_1.R_D__0)(aslog);
    }
    if (x === 0) {
        if (shape < 1)
            return Infinity;
        if (shape > 1) {
            return (0, r_func_1.R_D__0)(aslog);
        }
        return aslog ? -Math.log(scale) : 1 / scale;
    }
    if (shape < 1) {
        pr = (0, dpois_1.dpois_raw)(shape, x / scale, aslog);
        return aslog ? pr + Math.log(shape / x) : pr * shape / x;
    }
    pr = (0, dpois_1.dpois_raw)(shape - 1, x / scale, aslog);
    return aslog ? pr - Math.log(scale) : pr / scale;
}
exports.dgamma = dgamma;