'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.rgamma = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var sexp_1 = require('../exp/sexp.cjs');
var global_rng_1 = require('../../rng/global-rng.cjs');
const printer_rgamma = (0, debug_1.debug)('rgamma');
const sqrt32 = 5.656854;
const exp_m1 = 0.36787944117144233;
const q1 = 0.04166669;
const q2 = 0.02083148;
const q3 = 0.00801191;
const q4 = 0.00144121;
const q5 = -0.00007388;
const q6 = 0.00024511;
const q7 = 0.0002424;
const a1 = 0.3333333;
const a2 = -0.250003;
const a3 = 0.2000062;
const a4 = -0.1662921;
const a5 = 0.1423657;
const a6 = -0.1367177;
const a7 = 0.1233795;
function rgamma(a, scale) {
    const rng = (0, global_rng_1.globalNorm)();
    let s = 0;
    let s2 = 0;
    let d = 0;
    let q0 = 0;
    let b = 0;
    let si = 0;
    let c = 0;
    let e = 0;
    let p = 0;
    let q = 0;
    let r = 0;
    let t = 0;
    let u = 0;
    let v = 0;
    let w = 0;
    let x = 0;
    let ret_val = 0;
    if (scale === 0)
        return 0;
    if (a === 0)
        return 0;
    if (!isFinite(a) || !isFinite(scale) || a < 0 || scale < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_rgamma, logger_1.lineInfo4);
    }
    if (a < 1) {
        e = 1 + exp_m1 * a;
        for (;;) {
            p = e * rng.uniform_rng.random();
            if (p >= 1) {
                x = -Math.log((e - p) / a);
                if ((0, sexp_1.exp_rand)(rng.uniform_rng) >= (1 - a) * Math.log(x))
                    break;
            } else {
                x = Math.exp(Math.log(p) / a);
                if ((0, sexp_1.exp_rand)(rng.uniform_rng) >= x)
                    break;
            }
        }
        return scale * x;
    }
    s2 = a - 0.5;
    s = Math.sqrt(s2);
    d = sqrt32 - s * 12;
    t = rng.random();
    x = s + 0.5 * t;
    ret_val = x * x;
    if (t >= 0)
        return scale * ret_val;
    u = rng.uniform_rng.random();
    if (d * u <= t * t * t)
        return scale * ret_val;
    r = 1 / a;
    q0 = ((((((q7 * r + q6) * r + q5) * r + q4) * r + q3) * r + q2) * r + q1) * r;
    if (a <= 3.686) {
        b = 0.463 + s + 0.178 * s2;
        si = 1.235;
        c = 0.195 / s - 0.079 + 0.16 * s;
    } else if (a <= 13.022) {
        b = 1.654 + 0.0076 * s2;
        si = 1.68 / s + 0.275;
        c = 0.062 / s + 0.024;
    } else {
        b = 1.77;
        si = 0.75;
        c = 0.1515 / s;
    }
    if (x > 0) {
        v = t / (s + s);
        if (Math.abs(v) <= 0.25) {
            q = q0 + 0.5 * t * t * ((((((a7 * v + a6) * v + a5) * v + a4) * v + a3) * v + a2) * v + a1) * v;
        } else {
            q = q0 - s * t + 0.25 * t * t + (s2 + s2) * Math.log(1 + v);
        }
        if (Math.log(1 - u) <= q)
            return scale * ret_val;
    }
    for (;;) {
        e = (0, sexp_1.exp_rand)(rng.uniform_rng);
        u = rng.uniform_rng.random();
        u = u + u - 1;
        if (u < 0)
            t = b - si * e;
        else
            t = b + si * e;
        if (t >= -0.71874483771719) {
            v = t / (s + s);
            if (Math.abs(v) <= 0.25)
                q = q0 + 0.5 * t * t * ((((((a7 * v + a6) * v + a5) * v + a4) * v + a3) * v + a2) * v + a1) * v;
            else
                q = q0 - s * t + 0.25 * t * t + (s2 + s2) * Math.log(1 + v);
            if (q > 0) {
                w = Math.expm1(q);
                if (c * Math.abs(u) <= w * Math.exp(e - 0.5 * t * t))
                    break;
            }
        }
    }
    x = s + 0.5 * t;
    return scale * x * x;
}
exports.rgamma = rgamma;