'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dgeom = void 0;
var debug_1 = require('@mangos/debug');
var dbinom_1 = require('../binomial/dbinom.cjs');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
const printer = (0, debug_1.debug)('dgeom');
function dgeom(x, p, log = false) {
    if (isNaN(x) || isNaN(p))
        return x + p;
    if (p <= 0 || p > 1) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const rc = (0, r_func_1.R_D_nonint_check)(log, x, printer);
    if (rc !== undefined) {
        return rc;
    }
    if (x < 0 || !isFinite(x) || p === 0) {
        return (0, r_func_1.R_D__0)(log);
    }
    x = (0, r_func_1.round)(x);
    const prob = (0, dbinom_1.dbinom_raw)(0, x, p, 1 - p, log);
    return log ? (0, r_func_1.log)(p) + prob : p * prob;
}
exports.dgeom = dgeom;