'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pgeom = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var expm1_1 = require('../exp/expm1.cjs');
const printer = (0, debug_1.debug)('pgeom');
function pgeom(q, p, lowerTail = true, logP = false) {
    if (isNaN(q) || isNaN(p))
        return NaN;
    if (p <= 0 || p > 1) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (q < 0)
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    if (!isFinite(q))
        return (0, r_func_1.R_DT_1)(lowerTail, logP);
    q = Math.floor(q + 1e-7);
    if (p === 1) {
        q = lowerTail ? 1 : 0;
        return logP ? Math.log(q) : q;
    }
    q = Math.log1p(-p) * (q + 1);
    if (logP)
        return (0, expm1_1.R_DT_Clog)(lowerTail, logP, q);
    else
        return lowerTail ? -Math.expm1(q) : Math.exp(q);
}
exports.pgeom = pgeom;