'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dhyper = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var dbinom_1 = require('../binomial/dbinom.cjs');
const printer = (0, debug_1.debug)('dhyper');
function dhyper(x, m, n, k, log = false) {
    if (isNaN(x) || isNaN(m) || isNaN(n) || isNaN(k)) {
        return NaN;
    }
    if ((0, r_func_1.R_D_negInonint)(m) || (0, r_func_1.R_D_negInonint)(n) || (0, r_func_1.R_D_negInonint)(k) || k > m + n) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (x < 0) {
        return (0, r_func_1.R_D__0)(log);
    }
    const rc = (0, r_func_1.R_D_nonint_check)(log, x, printer);
    if (rc !== undefined) {
        return rc;
    }
    x = Math.round(x);
    m = Math.round(m);
    n = Math.round(n);
    k = Math.round(k);
    if (k < x || m < x || k - x > n)
        return (0, r_func_1.R_D__0)(log);
    if (k === 0) {
        return (0, r_func_1.R_D__1)(log);
    }
    const p = k / (m + n);
    const q = (m + n - k) / (m + n);
    const p1 = (0, dbinom_1.dbinom_raw)(x, m, p, q, log);
    const p2 = (0, dbinom_1.dbinom_raw)(k - x, n, p, q, log);
    const p3 = (0, dbinom_1.dbinom_raw)(k, m + n, p, q, log);
    return log ? p1 + p2 - p3 : p1 * p2 / p3;
}
exports.dhyper = dhyper;