'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.phyper = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var dhyper_1 = require('./dhyper.cjs');
function pdhyper(x, NR, NB, n, logP) {
    let sum = 0;
    let term = 1;
    while (x > 0 && term >= Number.EPSILON * sum) {
        term *= x * (NB - n + x) / (n + 1 - x) / (NR + 1 - x);
        sum += term;
        x--;
    }
    const ss = sum;
    return logP ? Math.log1p(ss) : 1 + ss;
}
const printer_phyper = (0, debug_1.debug)('phyper');
function phyper(q, m, n, k, lowerTail = true, logP = false) {
    let lower_tail = lowerTail;
    if (isNaN(q) || isNaN(m) || isNaN(n) || isNaN(k))
        return NaN;
    q = Math.floor(q + 1e-7);
    m = Math.round(m);
    n = Math.round(n);
    k = Math.round(k);
    if (m < 0 || n < 0 || !isFinite(m + n) || k < 0 || k > m + n) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_phyper, logger_1.lineInfo4);
    }
    const ox = q;
    const onn = k;
    const onr = m;
    const onb = n;
    if (q * (m + n) > k * m) {
        const oldnb = n;
        n = m;
        m = oldnb;
        q = k - q - 1;
        lower_tail = !lower_tail;
    }
    if (q < 0 || q < k - n)
        return (0, r_func_1.R_DT_0)(lower_tail, logP);
    if (q >= m) {
        return (0, r_func_1.R_DT_1)(lower_tail, logP);
    }
    if (q >= k) {
        printer_phyper('trace q>=k q=%d m=%d n=%d k=%d', ox, onr, onb, onn);
        return (0, r_func_1.R_DT_1)(lower_tail, logP);
    }
    const d = (0, dhyper_1.dhyper)(q, m, n, k, logP);
    if (!logP && d == 0 || logP && d == -Infinity) {
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    }
    const pd = pdhyper(q, m, n, k, logP);
    return logP ? (0, expm1_1.R_DT_log)(lower_tail, logP, d + pd) : (0, r_func_1.R_D_Lval)(lower_tail, d * pd);
}
exports.phyper = phyper;