'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qhyper = exports.unRegisterBackend = exports.registerBackend = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var choose_1 = require('../../special/choose/index.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var r_func_1 = require('../../r-func.cjs');
const printer_qhyper = (0, debug_1.debug)('qhyper');
const _d = new Float64Array(7);
const ixr = 0;
const isum = 1;
const ixb = 2;
const iterm = 3;
const iNR = 4;
const iNB = 5;
let backendTinyN = cpuBackendTinyN;
let backendBigN = cpuBackendBigN;
function registerBackend(fns) {
    backendTinyN = fns.calcTinyN;
    backendBigN = fns.calcBigN;
}
exports.registerBackend = registerBackend;
function unRegisterBackend() {
    const previous = !!backendTinyN && !!backendBigN;
    backendTinyN = cpuBackendTinyN;
    backendBigN = cpuBackendBigN;
    return previous;
}
exports.unRegisterBackend = unRegisterBackend;
function cpuBackendTinyN(sum, term, p, xr, end, xb, NB, NR) {
    while (sum < p && xr < end) {
        xr++;
        NB++;
        term *= NR / xr * (xb / NB);
        sum += term;
        xb--;
        NR--;
    }
    return xr;
}
function cpuBackendBigN(sum, term, p, xr, end, xb, NB, NR) {
    while (sum < p && xr < end) {
        xr++;
        NB++;
        term += Math.log(NR / xr * (xb / NB));
        sum += Math.exp(term);
        xb--;
        NR--;
    }
    return xr;
}
function qhyper(p, m, n, k, lowerTail = true, logP = false) {
    if (isNaN(p) || isNaN(m) || isNaN(n) || isNaN(k)) {
        return NaN;
    }
    if (!isFinite(p) || !isFinite(m) || !isFinite(n) || !isFinite(k)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qhyper, logger_1.lineInfo4);
    }
    _d[iNR] = Math.round(m);
    _d[iNB] = Math.round(n);
    const N = _d[iNR] + _d[iNB];
    k = Math.round(k);
    if (_d[iNR] < 0 || _d[iNB] < 0 || k < 0 || k > N)
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qhyper, logger_1.lineInfo4);
    const xstart = Math.max(0, k - _d[iNB]);
    const xend = Math.min(k, _d[iNR]);
    const rc = (0, logger_1.R_Q_P01_boundaries)(lowerTail, logP, p, xstart, xend);
    if (rc !== undefined) {
        return rc;
    }
    _d[ixr] = xstart;
    _d[ixb] = k - _d[ixr];
    const small_N = N < 1000;
    _d[iterm] = (0, choose_1.lfastchoose)(_d[iNR], _d[ixr]) + (0, choose_1.lfastchoose)(_d[iNB], _d[ixb]) - (0, choose_1.lfastchoose)(N, k);
    if (small_N)
        _d[iterm] = Math.exp(_d[iterm]);
    _d[iNR] -= _d[ixr];
    _d[iNB] -= _d[ixb];
    if (!lowerTail || logP) {
        p = (0, expm1_1.R_DT_qIv)(lowerTail, logP, p);
    }
    p *= 1 - 1000 * r_func_1.DBL_EPSILON;
    _d[isum] = small_N ? _d[iterm] : Math.exp(_d[iterm]);
    return small_N ? backendTinyN(_d[isum], _d[iterm], p, _d[ixr], xend, _d[ixb], _d[iNB], _d[iNR]) : backendBigN(_d[isum], _d[iterm], p, _d[ixr], xend, _d[ixb], _d[iNB], _d[iNR]);
}
exports.qhyper = qhyper;