'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.rhyperOne = void 0;
var debug_1 = require('@mangos/debug');
var rbinom_1 = require('../binomial/rbinom.cjs');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var qhyper_1 = require('./qhyper.cjs');
var global_rng_1 = require('../../rng/global-rng.cjs');
const printer_afc = (0, debug_1.debug)('afc');
const al = new Float64Array([
    0,
    0,
    0.6931471805599453,
    1.791759469228055,
    3.1780538303479458,
    4.787491742782046,
    6.579251212010101,
    8.525161361065415
]);
const scale = 1e+25;
const con = 57.564627324851145;
const deltal = 0.0078;
const deltau = 0.0034;
function afc(i) {
    i = Math.trunc(i);
    if (i < 0) {
        printer_afc('rhyper.c: afc(i), i=%d < 0 -- SHOULD NOT HAPPEN!', i);
        return -1;
    }
    if (i <= 7) {
        return al[i];
    }
    const di = i;
    const i2 = di * di;
    return (di + 0.5) * Math.log(di) - di + r_func_1.M_LN_SQRT_2PI + (0.0833333333333333 - 0.00277777777777778 / i2) / di;
}
const printer_rhyper = (0, debug_1.debug)('rhyper');
const r_d = new Float64Array(14);
const d_N = 0;
const d_w = 1;
const d_a = 2;
const d_d = 3;
const d_s = 4;
const d_xl = 5;
const d_xr = 6;
const d_kl = 7;
const d_kr = 8;
const d_lamdl = 9;
const d_lamdr = 10;
const d_p1 = 11;
const d_p2 = 12;
const d_p3 = 13;
const r_i = new Int32Array(13);
const i_nn1 = 0;
const i_nn2 = 1;
const i_kk = 2;
const i_ix = 3;
const i_ks = 4;
const i_n1s = 5;
const i_n2s = 6;
const i_m = 7;
const i_minjx = 8;
const i_maxjx = 9;
const i_k = 10;
const i_n1 = 11;
const i_n2 = 12;
r_i[i_ks] = -1;
r_i[i_n1s] = -1;
r_i[i_n2s] = -1;
function L_finis() {
    if (r_i[i_kk] + r_i[i_kk] >= r_d[d_N]) {
        if (r_i[i_nn1] > r_i[i_nn2]) {
            r_i[i_ix] = r_i[i_kk] - r_i[i_nn2] + r_i[i_ix];
        } else {
            r_i[i_ix] = r_i[i_nn1] - r_i[i_ix];
        }
    } else if (r_i[i_nn1] > r_i[i_nn2]) {
        r_i[i_ix] = r_i[i_kk] - r_i[i_ix];
    }
    return r_i[i_ix];
}
function rhyperOne(m, n, k) {
    const rng = (0, global_rng_1.globalUni)();
    if (!isFinite(m) || !isFinite(n) || !isFinite(k)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_rhyper, logger_1.lineInfo4);
    }
    m = Math.round(m);
    n = Math.round(n);
    k = Math.round(k);
    if (m < 0 || n < 0 || k < 0 || k > m + n) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_rhyper, logger_1.lineInfo4);
    }
    if (m >= r_func_1.INT_MAX || n >= r_func_1.INT_MAX || k >= r_func_1.INT_MAX) {
        if (k === 1) {
            const rinv = 1 + n / m;
            const r = 1 / rinv;
            return (0, rbinom_1.rbinomOne)(k, r);
        }
        return (0, qhyper_1.qhyper)(rng.random(), m, n, k, false, false);
    }
    r_i[i_nn1] = m;
    r_i[i_nn2] = n;
    r_i[i_kk] = k;
    let setup1;
    let setup2;
    if (r_i[i_nn1] !== r_i[i_n1s] || r_i[i_nn2] !== r_i[i_n2s]) {
        setup1 = true;
        setup2 = true;
    } else if (r_i[i_kk] !== r_i[i_ks]) {
        setup1 = false;
        setup2 = true;
    } else {
        setup1 = false;
        setup2 = false;
    }
    if (setup1) {
        r_i[i_n1s] = r_i[i_nn1];
        r_i[i_n2s] = r_i[i_nn2];
        r_d[d_N] = r_i[i_nn1];
        r_d[d_N] += r_i[i_nn2];
        if (r_i[i_nn1] <= r_i[i_nn2]) {
            r_i[i_n1] = r_i[i_nn1];
            r_i[i_n2] = r_i[i_nn2];
        } else {
            r_i[i_n1] = r_i[i_nn2];
            r_i[i_n2] = r_i[i_nn1];
        }
    }
    if (setup2) {
        r_i[i_ks] = r_i[i_kk];
        if (r_i[i_kk] + r_i[i_kk] >= r_d[d_N]) {
            r_i[i_k] = r_d[d_N] - r_i[i_kk];
        } else {
            r_i[i_k] = r_i[i_kk];
        }
    }
    if (setup1 || setup2) {
        r_i[i_m] = (r_i[i_k] + 1) * (r_i[i_n1] + 1) / (r_d[d_N] + 2);
        r_i[i_minjx] = Math.max(0, r_i[i_k] - r_i[i_n2]);
        r_i[i_maxjx] = Math.min(r_i[i_n1], r_i[i_k]);
        printer_rhyper('rhyper(n1=%d, n2=%d, k=%d), setup: floor(a.mean)=: m = %d, [min,maxjx]= [%d,%d]\n', r_i[i_nn1], r_i[i_nn2], r_i[i_kk], r_i[i_m], r_i[i_minjx], r_i[i_maxjx]);
    }
    if (r_i[i_minjx] === r_i[i_maxjx]) {
        printer_rhyper('rhyper(), branch I (degenerate): ix := maxjx = %d\n', r_i[i_maxjx]);
        r_i[i_ix] = r_i[i_maxjx];
        return L_finis();
    } else if (r_i[i_m] - r_i[i_minjx] < 10) {
        if (setup1 || setup2) {
            let lw = 0;
            if (r_i[i_k] < r_i[i_n2]) {
                lw = afc(r_i[i_n2]) + afc(r_i[i_n1] + r_i[i_n2] - r_i[i_k]) - afc(r_i[i_n2] - r_i[i_k]) - afc(r_i[i_n1] + r_i[i_n2]);
            } else {
                lw = afc(r_i[i_n1]) + afc(r_i[i_k]) - afc(r_i[i_k] - r_i[i_n2]) - afc(r_i[i_n1] + r_i[i_n2]);
            }
            r_d[d_w] = Math.exp(lw + con);
        }
        printer_rhyper('rhyper(), branch II; w = %d > 0', r_d[d_w]);
        L10:
            for (;;) {
                let p = r_d[d_w];
                r_i[i_ix] = r_i[i_minjx];
                let u = rng.random() * scale;
                printer_rhyper('  _new_ u = %d', u);
                while (u > p) {
                    u -= p;
                    p *= (r_i[i_n1] - r_i[i_ix]) * (r_i[i_k] - r_i[i_ix]);
                    r_i[i_ix]++;
                    p = p / r_i[i_ix] / (r_i[i_n2] - r_i[i_k] + r_i[i_ix]);
                    printer_rhyper('       ix=%d, u=%d, p=%d (u-p=%d)\n', r_i[i_ix], u, p, u - p);
                    if (r_i[i_ix] > r_i[i_maxjx]) {
                        continue L10;
                    }
                }
                return L_finis();
            }
    } else {
        let u = 0;
        let v = 0;
        if (setup1 || setup2) {
            const val = (r_d[d_N] - r_i[i_k]) * r_i[i_k] * r_i[i_n1] * r_i[i_n2] / (r_d[d_N] - 1) / r_d[d_N] / r_d[d_N];
            r_d[d_s] = Math.sqrt(val);
            r_d[d_d] = Math.trunc(1.5 * r_d[d_s]) + 0.5;
            r_d[d_xl] = r_i[i_m] - r_d[d_d] + 0.5;
            r_d[d_xr] = r_i[i_m] + r_d[d_d] + 0.5;
            r_d[d_a] = afc(r_i[i_m]) + afc(r_i[i_n1] - r_i[i_m]) + afc(r_i[i_k] - r_i[i_m]) + afc(r_i[i_n2] - r_i[i_k] + r_i[i_m]);
            r_d[d_kl] = Math.exp(r_d[d_a] - afc(r_d[d_xl]) - afc(r_i[i_n1] - r_d[d_xl]) - afc(r_i[i_k] - r_d[d_xl]) - afc(r_i[i_n2] - r_i[i_k] + r_d[d_xl]));
            r_d[d_kr] = Math.exp(r_d[d_a] - afc(r_d[d_xr] - 1) - afc(r_i[i_n1] - r_d[d_xr] + 1) - afc(r_i[i_k] - r_d[d_xr] + 1) - afc(r_i[i_n2] - r_i[i_k] + r_d[d_xr] - 1));
            r_d[d_lamdl] = -Math.log(r_d[d_xl] * (r_i[i_n2] - r_i[i_k] + r_d[d_xl]) / (r_i[i_n1] - r_d[d_xl] + 1) / (r_i[i_k] - r_d[d_xl] + 1));
            r_d[d_lamdr] = -Math.log((r_i[i_n1] - r_d[d_xr] + 1) * (r_i[i_k] - r_d[d_xr] + 1) / r_d[d_xr] / (r_i[i_n2] - r_i[i_k] + r_d[d_xr]));
            r_d[d_p1] = r_d[d_d] + r_d[d_d];
            r_d[d_p2] = r_d[d_p1] + r_d[d_kl] / r_d[d_lamdl];
            r_d[d_p3] = r_d[d_p2] + r_d[d_kr] / r_d[d_lamdr];
        }
        printer_rhyper('rhyper(), branch III {accept/reject}: (xl,xr)= (%d,%d); (lamdl,lamdr)= (%d,%d)\n', r_d[d_xl], r_d[d_xr], r_d[d_lamdl], r_d[d_lamdr]);
        printer_rhyper('-------- p123= c(%d,%d,%d)\n', r_d[d_p1], r_d[d_p2], r_d[d_p3]);
        let n_uv = 0;
        L30:
            for (;;) {
                u = rng.random() * r_d[d_p3];
                v = rng.random();
                n_uv++;
                if (n_uv >= 10000) {
                    printer_rhyper('rhyper() branch III: giving up after %d rejections', r_i[i_nn1], r_i[i_nn2], r_i[i_kk], n_uv);
                    return (0, logger_1.ML_ERR_return_NAN2)(printer_rhyper, logger_1.lineInfo4);
                }
                printer_rhyper(' ... L30: new (u=%d, v ~ U[0,1])[%d]\n', n_uv, u, v);
                if (u < r_d[d_p1]) {
                    r_i[i_ix] = r_d[d_xl] + u;
                } else if (u <= r_d[d_p2]) {
                    r_i[i_ix] = r_d[d_xl] + Math.log(v) / r_d[d_lamdl];
                    if (r_i[i_ix] < r_i[i_minjx]) {
                        continue L30;
                    }
                    v = v * (u - r_d[d_p1]) * r_d[d_lamdl];
                } else {
                    r_i[i_ix] = r_d[d_xr] - Math.log(v) / r_d[d_lamdr];
                    if (r_i[i_ix] > r_i[i_maxjx]) {
                        continue L30;
                    }
                    v = v * (u - r_d[d_p2]) * r_d[d_lamdr];
                }
                let reject = true;
                if (r_i[i_m] < 100 || r_i[i_ix] <= 50) {
                    let i = 0;
                    let f = 1;
                    if (r_i[i_m] < r_i[i_ix]) {
                        for (i = r_i[i_m] + 1; i <= r_i[i_ix]; i++) {
                            f = f * (r_i[i_n1] - i + 1) * (r_i[i_k] - i + 1) / (r_i[i_n2] - r_i[i_k] + i) / i;
                        }
                    } else if (r_i[i_m] > r_i[i_ix]) {
                        for (i = r_i[i_ix] + 1; i <= r_i[i_m]; i++) {
                            f = f * i * (r_i[i_n2] - r_i[i_k] + i) / (r_i[i_n1] - i + 1) / (r_i[i_k] - i + 1);
                        }
                    }
                    if (v <= f) {
                        reject = false;
                    }
                } else {
                    let e = 0, g = 0, r = 0, t = 0, y = 0;
                    let de = 0, dg = 0, dr = 0, ds = 0, dt = 0, gl = 0, gu = 0, nk = 0, nm = 0, ub = 0;
                    let xk = 0, xm = 0, xn = 0, y1 = 0, ym = 0, yn = 0, yk = 0, alv = 0;
                    printer_rhyper(' ... accept/reject \'large\' case v=%d', v);
                    y = r_i[i_ix];
                    y1 = y + 1;
                    ym = y - r_i[i_m];
                    yn = r_i[i_n1] - y + 1;
                    yk = r_i[i_k] - y + 1;
                    nk = r_i[i_n2] - r_i[i_k] + y1;
                    r = -ym / y1;
                    r_d[d_s] = ym / yn;
                    t = ym / yk;
                    e = -ym / nk;
                    g = yn * yk / (y1 * nk) - 1;
                    dg = 1;
                    if (g < 0) {
                        dg = 1 + g;
                    }
                    gu = g * (1 + g * (-0.5 + g / 3));
                    gl = gu - 0.25 * (g * g * g * g) / dg;
                    xm = r_i[i_m] + 0.5;
                    xn = r_i[i_n1] - r_i[i_m] + 0.5;
                    xk = r_i[i_k] - r_i[i_m] + 0.5;
                    nm = r_i[i_n2] - r_i[i_k] + xm;
                    ub = y * gu - r_i[i_m] * gl + deltau + xm * r * (1 + r * (-0.5 + r / 3)) + xn * r_d[d_s] * (1 + r_d[d_s] * (-0.5 + r_d[d_s] / 3)) + xk * t * (1 + t * (-0.5 + t / 3)) + nm * e * (1 + e * (-0.5 + e / 3));
                    alv = Math.log(v);
                    if (alv > ub) {
                        reject = true;
                    } else {
                        dr = xm * (r * r * r * r);
                        if (r < 0) {
                            dr /= 1 + r;
                        }
                        ds = xn * (r_d[d_s] * r_d[d_s] * r_d[d_s] * r_d[d_s]);
                        if (r_d[d_s] < 0) {
                            ds /= 1 + r_d[d_s];
                        }
                        dt = xk * (t * t * t * t);
                        if (t < 0) {
                            dt /= 1 + t;
                        }
                        de = nm * (e * e * e * e);
                        if (e < 0) {
                            de /= 1 + e;
                        }
                        if (alv < ub - 0.25 * (dr + ds + dt + de) + (y + r_i[i_m]) * (gl - gu) - deltal) {
                            reject = false;
                        } else {
                            if (alv <= r_d[d_a] - afc(r_i[i_ix]) - afc(r_i[i_n1] - r_i[i_ix]) - afc(r_i[i_k] - r_i[i_ix]) - afc(r_i[i_n2] - r_i[i_k] + r_i[i_ix])) {
                                reject = false;
                            } else {
                                reject = true;
                            }
                        }
                    }
                }
                if (reject) {
                    continue L30;
                }
                break;
            }
    }
    return L_finis();
}
exports.rhyperOne = rhyperOne;