'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dmultinom = exports.dmultinomLikeR = void 0;
var debug_1 = require('@mangos/debug');
var lgammafn_sign_1 = require('../../special/gamma/lgammafn_sign.cjs');
var r_func_1 = require('../../r-func.cjs');
const printer = (0, debug_1.debug)('dmultinom');
function isZeroOrPositiveAndFinite(x) {
    return x >= 0;
}
function dmultinomLikeR(x, prob, log = false) {
    const rc = dmultinom(x, prob, log);
    if (isNaN(rc)) {
        throw new Error(`Error in dmultinom`);
    }
    return rc;
}
exports.dmultinomLikeR = dmultinomLikeR;
function dmultinom(x, prob, log = false) {
    if (x.length !== prob.length) {
        printer('x[] and prob[] must be equal length vectors.');
        return NaN;
    }
    const s = (0, r_func_1.sumfp)(prob);
    if (prob.every(isFinite) === false || prob.every(_p => _p >= 0) === false || s === 0) {
        printer('probabilities must be finite, non-negative and not all 0');
        return NaN;
    }
    prob.forEach((v, i, arr) => {
        arr[i] = v / s;
    });
    x.forEach((v, i, arr) => {
        arr[i] = (0, r_func_1.trunc)(v + 0.5);
    });
    if (x.every(isZeroOrPositiveAndFinite) === false) {
        printer('\'x\' must be non-negative (and finite)');
        return NaN;
    }
    const N = (0, r_func_1.sumfp)(x);
    let rc = (0, lgammafn_sign_1.lgammafn_sign)(N + 1);
    for (let i = 0; i < prob.length; i++) {
        if (prob[i] === 0) {
            if (x[i] !== 0) {
                return log ? -Infinity : 0;
            }
            continue;
        }
        rc += x[i] * (0, r_func_1.log)(prob[i]) - (0, lgammafn_sign_1.lgammafn_sign)(x[i] + 1);
    }
    return log ? rc : (0, r_func_1.exp)(rc);
}
exports.dmultinom = dmultinom;