'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.rmultinom = void 0;
var rbinom_1 = require('../binomial/rbinom.cjs');
var r_func_1 = require('../../r-func.cjs');
function rmultinom(n, size, prob) {
    if (n < 0) {
        throw new Error('invalid first argument "n"');
    }
    if (size < 0) {
        throw new Error('invalid second argument "size"');
    }
    if (prob.length === 0) {
        throw new Error('no positive probabilities');
    }
    if (prob.every(_p => _p >= 0) === false) {
        throw new Error('negative probability');
    }
    const s = (0, r_func_1.sumfp64)(prob);
    if (s === 0) {
        throw new Error('no positive probabilities');
    }
    if (n === 0) {
        return r_func_1.emptyFloat64Array;
    }
    if (size === 0) {
        const rc = new Float64Array(n);
        rc.fill(0);
        return rc;
    }
    const rc = new Float64Array(n * prob.length);
    let K = size;
    let pTotal = s;
    for (let i = 0; i < n * prob.length; i++) {
        const i2 = i % prob.length;
        if (i2 === 0) {
            K = size;
            pTotal = s;
        }
        if (prob[i2] === 0) {
            continue;
        }
        if (prob[i2] === pTotal) {
            rc[i] = K;
        } else {
            const pp = prob[i2] / pTotal;
            rc[i] = (0, rbinom_1.rbinomOne)(K, pp);
        }
        K -= rc[i];
        pTotal -= prob[i2];
    }
    return rc;
}
exports.rmultinom = rmultinom;