'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dnorm4 = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var r_func_2 = require('../../r-func.cjs');
const printer = (0, debug_1.debug)('dnorm');
const PRECISION_LIMIT = (0, r_func_1.sqrt)(-2 * r_func_2.M_LN2 * (r_func_2.DBL_MIN_EXP + 1 - r_func_2.DBL_MANT_DIG));
function dnorm4(x, mean = 0, sd = 1, log = false) {
    if (isNaN(x) || isNaN(mean) || isNaN(sd)) {
        return NaN;
    }
    if (!isFinite(sd)) {
        return (0, r_func_2.R_D__0)(log);
    }
    if (!isFinite(x) && mean === x) {
        return NaN;
    }
    if (sd <= 0) {
        if (sd < 0) {
            return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
        }
        return x === mean ? Infinity : (0, r_func_2.R_D__0)(log);
    }
    x = (x - mean) / sd;
    if (!isFinite(x))
        return (0, r_func_2.R_D__0)(log);
    x = (0, r_func_1.abs)(x);
    if (x >= 2 * (0, r_func_1.sqrt)(Number.MAX_VALUE)) {
        return (0, r_func_2.R_D__0)(log);
    }
    if (log) {
        return -(r_func_2.M_LN_SQRT_2PI + 0.5 * x * x + (0, r_func_1.log)(sd));
    }
    if (x < 5)
        return r_func_2.M_1_SQRT_2PI * (0, r_func_1.exp)(-0.5 * x * x) / sd;
    if (x > PRECISION_LIMIT) {
        return 0;
    }
    const x1 = (0, r_func_2.ldexp)((0, r_func_1.round)((0, r_func_2.ldexp)(x, 16)), -16);
    const x2 = x - x1;
    return r_func_2.M_1_SQRT_2PI / sd * ((0, r_func_1.exp)(-0.5 * x1 * x1) * (0, r_func_1.exp)((-0.5 * x2 - x1) * x2));
}
exports.dnorm4 = dnorm4;