'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dpois = exports.dpois_raw = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var deviance_1 = require('../../deviance/index.cjs');
var lgammafn_sign_1 = require('../../special/gamma/lgammafn_sign.cjs');
var stirling_1 = require('../../stirling/index.cjs');
const printer = (0, debug_1.debug)('dpois');
function dpois_raw(x, lambda, give_log) {
    if (lambda === 0) {
        return x === 0 ? (0, r_func_1.R_D__1)(give_log) : (0, r_func_1.R_D__0)(give_log);
    }
    if (!isFinite(lambda)) {
        return (0, r_func_1.R_D__0)(give_log);
    }
    if (x <= lambda * r_func_1.DBL_MIN) {
        return (0, r_func_1.R_D_exp)(give_log, -lambda);
    }
    if (lambda < x * r_func_1.DBL_MIN) {
        if (!isFinite(x)) {
            return (0, r_func_1.R_D__0)(give_log);
        }
        const gammafn = (0, lgammafn_sign_1.lgammafn_sign)(x + 1);
        const xlogLambda = x * (0, r_func_1.log)(lambda);
        const inp = -lambda + xlogLambda - gammafn;
        return (0, r_func_1.R_D_exp)(give_log, inp);
    }
    return (0, r_func_1.R_D_fexp)(give_log, r_func_1.M_2PI * x, -(0, stirling_1.stirlerr)(x) - (0, deviance_1.bd0)(x, lambda));
}
exports.dpois_raw = dpois_raw;
function dpois(x, lambda, log = false) {
    if (isNaN(x) || isNaN(lambda)) {
        return NaN;
    }
    if (lambda < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const rc = (0, r_func_1.R_D_nonint_check)(log, x, printer);
    if (rc !== undefined) {
        return rc;
    }
    if (x < 0 || !isFinite(x)) {
        return (0, r_func_1.R_D__0)(log);
    }
    return dpois_raw(x, lambda, log);
}
exports.dpois = dpois;