'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.ppois = void 0;
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var debug_1 = require('@mangos/debug');
var pgamma_1 = require('../gamma/pgamma.cjs');
const printer = (0, debug_1.debug)('ppois');
function ppois(q, lambda, lowerTail = true, logP = false) {
    if (isNaN(q) || isNaN(lambda))
        return q + lambda;
    if (lambda < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (q < 0) {
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    }
    if (lambda === 0) {
        return (0, r_func_1.R_DT_1)(lowerTail, logP);
    }
    if (!(0, r_func_1.isFinite)(q)) {
        return (0, r_func_1.R_DT_1)(lowerTail, logP);
    }
    q = (0, r_func_1.floor)(q + 1e-7);
    return (0, pgamma_1.pgamma)(lambda, q + 1, 1, !lowerTail, logP);
}
exports.ppois = ppois;