'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qpois = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var NumberW_1 = require('../../common/toms708/NumberW.cjs');
var ppois_1 = require('./ppois.cjs');
var qnorm_1 = require('../normal/qnorm.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var r_func_1 = require('../../r-func.cjs');
function do_search(y, z, p, lambda, incr) {
    if (z.val >= p) {
        for (;;) {
            if (y === 0 || (z.val = (0, ppois_1.ppois)(y - incr, lambda, true, false)) < p)
                return y;
            y = (0, r_func_1.max)(0, y - incr);
        }
    } else {
        for (;;) {
            y = y + incr;
            if ((z.val = (0, ppois_1.ppois)(y, lambda, true, false)) >= p)
                return y;
        }
    }
}
const printer = (0, debug_1.debug)('qpois');
function qpois(p, lambda, lowerTail = true, logP = false) {
    let y;
    const z = new NumberW_1.NumberW(0);
    if ((0, r_func_1.isNaN)(p) || (0, r_func_1.isNaN)(lambda))
        return p + lambda;
    if (!(0, r_func_1.isFinite)(lambda)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (lambda < 0)
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    if (lambda === 0)
        return 0;
    const rc = (0, logger_1.R_Q_P01_boundaries)(lowerTail, logP, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (!lowerTail || logP) {
        p = (0, expm1_1.R_DT_qIv)(lowerTail, logP, p);
        if (p === 0)
            return 0;
        if (p === 1)
            return Infinity;
    }
    if (p + 1.01 * r_func_1.DBL_EPSILON >= 1)
        return Infinity;
    const mu = lambda;
    const sigma = (0, r_func_1.sqrt)(lambda);
    const gamma = 1 / sigma;
    z.val = (0, qnorm_1.qnorm)(p, 0, 1, true, false);
    y = (0, r_func_1.round)(mu + sigma * (z.val + gamma * (z.val * z.val - 1) / 6));
    z.val = (0, ppois_1.ppois)(y, lambda, true, false);
    p *= 1 - 64 * r_func_1.DBL_EPSILON;
    if (lambda < 100000)
        return do_search(y, z, p, lambda, 1);
    {
        let incr = (0, r_func_1.floor)(y * 0.001);
        let oldincr;
        do {
            oldincr = incr;
            y = do_search(y, z, p, lambda, incr);
            incr = (0, r_func_1.max)(1, (0, r_func_1.floor)(incr / 100));
        } while (oldincr > 1 && incr > lambda * 1e-15);
        return y;
    }
}
exports.qpois = qpois;