'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dsignrank = exports.registerBackend = exports.unRegisterBackend = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var csignrank_1 = require('./csignrank.cjs');
var r_func_1 = require('../../r-func.cjs');
var csignrank_wasm_1 = require('./csignrank_wasm.cjs');
let _csignrank = csignrank_1.cpu_csignrank;
function registerBackend(fns) {
    _csignrank = fns.csignrank;
}
exports.registerBackend = registerBackend;
function unRegisterBackend() {
    _csignrank = csignrank_1.cpu_csignrank;
    return _csignrank === csignrank_1.cpu_csignrank ? false : true;
}
exports.unRegisterBackend = unRegisterBackend;
const printer = (0, debug_1.debug)('dsignrank');
function dsignrank(x, n, log = false) {
    if ((0, r_func_1.isNaN)(x) || (0, r_func_1.isNaN)(n)) {
        return x + n;
    }
    if (n <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if ((0, r_func_1.abs)(x - (0, r_func_1.round)(x)) > 1e-7) {
        return (0, r_func_1.R_D__0)(log);
    }
    n = (0, r_func_1.trunc)(n);
    x = (0, r_func_1.round)(x);
    if (x < 0 || x > n * (n + 1) / 2) {
        return (0, r_func_1.R_D__0)(log);
    }
    const u = n * (n + 1) / 2;
    const c = Math.trunc(u / 2);
    (0, csignrank_wasm_1.growMemory)(c + 1);
    new Float64Array(csignrank_wasm_1.memory.buffer).fill(0, 0, c + 1);
    const d = (0, r_func_1.R_D_exp)(log, (0, r_func_1.log)(_csignrank((0, r_func_1.trunc)(x), n, u, c)) - n * r_func_1.M_LN2);
    return d;
}
exports.dsignrank = dsignrank;