'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.psignrank = exports.registerBackend = exports.unRegisterBackend = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var csignrank_1 = require('./csignrank.cjs');
var csignrank_wasm_1 = require('./csignrank_wasm.cjs');
const printer = (0, debug_1.debug)('psignrank');
let _csignrank = csignrank_1.cpu_csignrank;
function registerBackend(fns) {
    _csignrank = fns.csignrank;
}
exports.registerBackend = registerBackend;
function unRegisterBackend() {
    _csignrank = csignrank_1.cpu_csignrank;
    return _csignrank === csignrank_1.cpu_csignrank ? false : true;
}
exports.unRegisterBackend = unRegisterBackend;
function psignrank(q, n, lowerTail = true, logP = false) {
    if ((0, r_func_1.isNaN)(q) || (0, r_func_1.isNaN)(n)) {
        return NaN;
    }
    if (!(0, r_func_1.isFinite)(n) || n <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    q = (0, r_func_1.round)(q + 1e-7);
    if (q < 0) {
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    }
    if (q >= n * (n + 1) / 2) {
        return (0, r_func_1.R_DT_1)(lowerTail, logP);
    }
    n = (0, r_func_1.round)(n);
    const u = n * (n + 1) / 2;
    const c = (0, r_func_1.trunc)(u / 2);
    (0, csignrank_wasm_1.growMemory)(c + 1);
    new Float64Array(csignrank_wasm_1.memory.buffer).fill(0, 0, c + 1);
    const f = (0, r_func_1.exp)(-n * r_func_1.M_LN2);
    let p = 0;
    if (q <= u / 2) {
        for (let i = 0; i <= q; i++) {
            p += _csignrank(i, n, u, c) * f;
        }
    } else {
        q = n * (n + 1) / 2 - q;
        for (let i = 0; i < q; i++) {
            p += _csignrank(i, n, u, c) * f;
        }
        lowerTail = !lowerTail;
    }
    return (0, r_func_1.R_DT_val)(lowerTail, logP, p);
}
exports.psignrank = psignrank;