'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qsignrank = exports.registerBackend = exports.unRegisterBackend = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var csignrank_1 = require('./csignrank.cjs');
var r_func_1 = require('../../r-func.cjs');
var csignrank_wasm_1 = require('./csignrank_wasm.cjs');
const printer = (0, debug_1.debug)('qsignrank');
const PRECISION_LOWER_LIMIT = -r_func_1.DBL_MIN_VALUE_LN / r_func_1.M_LN2;
let _csignrank = csignrank_1.cpu_csignrank;
function registerBackend(fns) {
    _csignrank = fns.csignrank;
}
exports.registerBackend = registerBackend;
function unRegisterBackend() {
    _csignrank = csignrank_1.cpu_csignrank;
    return _csignrank === csignrank_1.cpu_csignrank ? false : true;
}
exports.unRegisterBackend = unRegisterBackend;
function qsignrank(p, n, lowerTail = true, logP = false) {
    if ((0, r_func_1.isNaN)(p) || (0, r_func_1.isNaN)(n)) {
        return NaN;
    }
    if (logP) {
        if (p === -Infinity) {
            p = 0;
            logP = false;
        } else if (p === 0) {
            p = 1;
            logP = false;
        }
    }
    if (!(0, r_func_1.isFinite)(p) || !(0, r_func_1.isFinite)(n)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (n > PRECISION_LOWER_LIMIT) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_UNDERFLOW, 'n > 1074', printer);
        return NaN;
    }
    const rc = (0, logger_1.R_Q_P01_check)(logP, p);
    if (rc !== undefined) {
        return rc;
    }
    if (n <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (p === (0, r_func_1.R_DT_0)(lowerTail, logP)) {
        return 0;
    }
    n = (0, r_func_1.round)(n);
    const u = n * (n + 1) / 2;
    const c = (0, r_func_1.trunc)(u / 2);
    if (p === (0, r_func_1.R_DT_1)(lowerTail, logP)) {
        return u;
    }
    if (logP || !lowerTail) {
        p = (0, expm1_1.R_DT_qIv)(lowerTail, logP, p);
    }
    (0, csignrank_wasm_1.growMemory)(c + 1);
    const f = (0, r_func_1.exp)(-n * r_func_1.M_LN2);
    let _p = 0;
    let q = 0;
    if (p <= 0.5) {
        p = p - 10 * r_func_1.DBL_EPSILON;
        for (;;) {
            _p += _csignrank(q, n, u, c) * f;
            if (_p >= p)
                break;
            q++;
        }
    } else {
        p = 1 - p + 10 * r_func_1.DBL_EPSILON;
        for (;;) {
            _p += _csignrank(q, n, u, c) * f;
            if (_p > p) {
                q = (0, r_func_1.trunc)(u - q);
                break;
            }
            q++;
        }
    }
    return q;
}
exports.qsignrank = qsignrank;