'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dnt = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var gamma_1 = require('../../special/gamma/index.cjs');
var normal_1 = require('../normal/index.cjs');
var dt_1 = require('./dt.cjs');
var pnt_1 = require('./pnt.cjs');
const printer_dnt = (0, debug_1.debug)('dnt');
function dnt(x, df, ncp = 0, giveLog = false) {
    if (isNaN(x) || isNaN(df) || isNaN(ncp)) {
        return x + df + ncp;
    }
    if (df <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_dnt, logger_1.lineInfo4);
    }
    if (!isFinite(x)) {
        return (0, r_func_1.R_D__0)(giveLog);
    }
    if (ncp === 0) {
        return (0, dt_1._dt)(x, df, giveLog);
    }
    if (df > 100000000) {
        return (0, normal_1.dnorm)(x, ncp, 1, giveLog);
    }
    let u;
    if ((0, r_func_1.abs)(x) > (0, r_func_1.sqrt)(df * r_func_1.DBL_EPSILON)) {
        printer_dnt('abs(x:%d)>sqrt(df*espsilon):%d', (0, r_func_1.abs)(x), (0, r_func_1.sqrt)(df * r_func_1.DBL_EPSILON));
        u = (0, r_func_1.log)(df) - (0, r_func_1.log)((0, r_func_1.abs)(x)) + (0, r_func_1.log)((0, r_func_1.abs)((0, pnt_1.pnt)(x * (0, r_func_1.sqrt)((df + 2) / df), df + 2, ncp, true, false) - (0, pnt_1.pnt)(x, df, ncp, true, false)));
    } else {
        printer_dnt('abs(x:%d)<=sqrt(df*espsilon):%d', (0, r_func_1.abs)(x), (0, r_func_1.sqrt)(df * r_func_1.DBL_EPSILON));
        u = (0, gamma_1.lgammafn_sign)((df + 1) / 2) - (0, gamma_1.lgammafn_sign)(df / 2) - (r_func_1.M_LN_SQRT_PI + 0.5 * ((0, r_func_1.log)(df) + ncp * ncp));
    }
    printer_dnt('u=%d, giveLog=%s', u, giveLog);
    return giveLog ? u : (0, r_func_1.exp)(u);
}
exports.dnt = dnt;