'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports._dt = void 0;
var r_func_1 = require('../../r-func.cjs');
var deviance_1 = require('../../deviance/index.cjs');
var normal_1 = require('../normal/index.cjs');
var stirling_1 = require('../../stirling/index.cjs');
function _dt(x, n, giveLog = false) {
    if (!isFinite(n)) {
        return (0, normal_1.dnorm)(x, 0, 1, giveLog);
    }
    let u;
    const t = -(0, deviance_1.bd0)(n / 2, (n + 1) / 2) + (0, stirling_1.stirlerr)((n + 1) / 2) - (0, stirling_1.stirlerr)(n / 2);
    const x2n = x * x / n;
    let ax = 0;
    let l_x2n;
    const lrg_x2n = x2n > 1 / r_func_1.DBL_EPSILON;
    if (lrg_x2n) {
        ax = (0, r_func_1.abs)(x);
        l_x2n = (0, r_func_1.log)(ax) - (0, r_func_1.log)(n) / 2;
        u = n * l_x2n;
    } else if (x2n > 0.2) {
        l_x2n = (0, r_func_1.log)(1 + x2n) / 2;
        u = n * l_x2n;
    } else {
        l_x2n = (0, r_func_1.log1p)(x2n) / 2;
        u = -(0, deviance_1.bd0)(n / 2, (n + x * x) / 2) + x * x / 2;
    }
    if (giveLog)
        return t - u - (r_func_1.M_LN_SQRT_2PI + l_x2n);
    const I_sqrt_ = lrg_x2n ? (0, r_func_1.sqrt)(n) / ax : (0, r_func_1.exp)(-l_x2n);
    return (0, r_func_1.exp)(t - u) * r_func_1.M_1_SQRT_2PI * I_sqrt_;
}
exports._dt = _dt;