'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pnt = void 0;
var debug_1 = require('@mangos/debug');
var pbeta_1 = require('../beta/pbeta.cjs');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var gamma_1 = require('../../special/gamma/index.cjs');
var pnorm_1 = require('../normal/pnorm.cjs');
var pt_1 = require('./pt.cjs');
const printer_pnt = (0, debug_1.debug)('pnt');
const DBL_MIN_EXP = -1021;
const itrmax = 1000;
const errmax = 1e-12;
function finis(tnc, del, lower_tail, negdel, log_p) {
    const pn = (0, pnorm_1.pnorm5)(-del, 0, 1, true, false);
    printer_pnt('pnorm(%d,0,1)=%d', -del, pn);
    tnc += pn;
    printer_pnt('tnc=%d', tnc);
    lower_tail = lower_tail !== negdel;
    if (tnc > 1 - 1e-10 && lower_tail) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'pnt{final}', printer_pnt);
    }
    const rc = (0, r_func_1.R_DT_val)(lower_tail, log_p, (0, r_func_1.min)(tnc, 1));
    printer_pnt('rc:%d, tnc:%d, log_p:%s, lower_tail:%s', rc, tnc, log_p, lower_tail);
    return rc;
}
function pnt(t, df, ncp, lower_tail, log_p) {
    let errbd;
    let rxb;
    let x;
    let geven;
    let godd;
    let q;
    let s;
    let tnc = 0;
    let xeven;
    let xodd;
    if (isNaN(t) || isNaN(df) || isNaN(ncp)) {
        return t + df + ncp;
    }
    if (df <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pnt, logger_1.lineInfo4);
    }
    if (!(0, r_func_1.isFinite)(t)) {
        return t < 0 ? (0, r_func_1.R_DT_0)(lower_tail, log_p) : (0, r_func_1.R_DT_1)(lower_tail, log_p);
    }
    if (ncp === 0) {
        return (0, pt_1.pt)(t, df, lower_tail, log_p);
    }
    let negdel;
    let del;
    let tt;
    if (t >= 0) {
        negdel = false;
        tt = t;
        del = ncp;
    } else {
        if (ncp > 40 && (!log_p || !lower_tail)) {
            printer_pnt('if x <=0 and solution for edge ncp > 40');
            return (0, r_func_1.R_DT_0)(lower_tail, log_p);
        }
        negdel = true;
        tt = -t;
        del = -ncp;
    }
    if (df > 400000 || del * del > 2 * r_func_1.M_LN2 * -DBL_MIN_EXP) {
        printer_pnt('Abramowitz & Stegun 26.7.10 ncp:%d, del:%d, ncp2:%d, del2:%d, D:%d', ncp, del, ncp * ncp, del * del, 2 * r_func_1.M_LN2 * -DBL_MIN_EXP);
        const _s = 1 / (4 * df);
        return (0, pnorm_1.pnorm5)(tt * (1 - _s), del, (0, r_func_1.sqrt)(1 + tt * tt * 2 * _s), lower_tail !== negdel, log_p);
    }
    x = t * t;
    rxb = df / (x + df);
    x = x / (x + df);
    printer_pnt('pnt(t=%d, df=%d, ncp=%d, rxb=%d) ==> x=%d', t, df, ncp, rxb, x);
    if (x > 0) {
        printer_pnt('x > 0 branch');
        const lambda = del * del;
        let p = 0.5 * (0, r_func_1.exp)(-0.5 * lambda);
        printer_pnt('p=%d', p);
        if (p === 0) {
            printer_pnt('p=%d, underflow protection', p);
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_UNDERFLOW, 'pnt', printer_pnt);
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'pnt', printer_pnt);
            return (0, r_func_1.R_DT_0)(lower_tail, log_p);
        }
        printer_pnt('it  1e5*(godd,   geven)|          p           q           s' + '        pnt(*)     errbd');
        q = r_func_1.M_SQRT_2dPI * p * del;
        s = 0.5 - p;
        if (s < 1e-7) {
            s = -0.5 * (0, r_func_1.expm1)(-0.5 * lambda);
        }
        let a = 0.5;
        const b = 0.5 * df;
        rxb = (0, r_func_1.pow)(rxb, b);
        const albeta = r_func_1.M_LN_SQRT_PI + (0, gamma_1.lgamma)(b) - (0, gamma_1.lgamma)(0.5 + b);
        printer_pnt('%d = lgamma(%d)-lgamma(0.5+%d)+0.572364942924700087071713675677', albeta, b, b);
        xodd = (0, pbeta_1.pbeta)(x, a, b, true, false);
        printer_pnt('return from pbeta:%d', xodd);
        godd = 2 * rxb * (0, r_func_1.exp)(a * (0, r_func_1.log)(x) - albeta);
        tnc = b * x;
        xeven = tnc < r_func_1.DBL_EPSILON ? tnc : 1 - rxb;
        geven = tnc * rxb;
        tnc = p * xodd + q * xeven;
        for (let it = 1; it <= itrmax; it++) {
            a += 1;
            xodd -= godd;
            xeven -= geven;
            godd *= x * (a + b - 1) / a;
            geven *= x * (a + b - 0.5) / (a + 0.5);
            p *= lambda / (2 * it);
            q *= lambda / (2 * it + 1);
            tnc += p * xodd + q * xeven;
            s -= p;
            if (s < -1e-10) {
                (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'pnt', printer_pnt);
                printer_pnt('goto:true, s = %d < 0 !!! ---> non-convergence!!', s);
                return finis(tnc, del, lower_tail, negdel, log_p);
            }
            if (s <= 0 && it > 1) {
                printer_pnt('goto:true, s:%d < 0 && it:%d>1', s, it);
                return finis(tnc, del, lower_tail, negdel, log_p);
            }
            errbd = 2 * s * (xodd - godd);
            printer_pnt('%d %d %d|%d %d %d %d %d', it, 100000 * godd, 100000 * geven, p, q, s, tnc, errbd);
            if ((0, r_func_1.abs)(errbd) < errmax) {
                printer_pnt('goto:true, errbd:%d < errmax:%d', errbd, errmax);
                return finis(tnc, del, lower_tail, negdel, log_p);
            }
        }
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_NOCONV, 'pnt', printer_pnt);
    } else {
        tnc = 0;
    }
    return finis(tnc, del, lower_tail, negdel, log_p);
}
exports.pnt = pnt;