'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pt = void 0;
var lbeta_1 = require('../../special/beta/lbeta.cjs');
var pbeta_1 = require('../beta/pbeta.cjs');
var r_func_1 = require('../../r-func.cjs');
var pnorm_1 = require('../normal/pnorm.cjs');
function pt(x, n, lower_tail, log_p) {
    let val;
    if (!isFinite(n)) {
        return (0, pnorm_1.pnorm5)(x, 0, 1, lower_tail, log_p);
    }
    const nx = 1 + x / n * x;
    if (nx > 1e+100) {
        const lb = (0, lbeta_1.default)(0.5 * n, 0.5);
        const l05n = (0, r_func_1.log)(0.5 * n);
        const labsx = (0, r_func_1.log)((0, r_func_1.abs)(x));
        const logn = (0, r_func_1.log)(n);
        const lval = -0.5 * n * (2 * labsx - logn) - lb - l05n;
        val = log_p ? lval : (0, r_func_1.exp)(lval);
    } else {
        val = n > x * x ? (0, pbeta_1.pbeta)(x * x / (n + x * x), 0.5, n / 2, false, log_p) : (0, pbeta_1.pbeta)(1 / nx, n / 2, 0.5, true, log_p);
    }
    if (x <= 0) {
        lower_tail = !lower_tail;
    }
    if (log_p) {
        if (lower_tail) {
            return (0, r_func_1.log1p)(-0.5 * (0, r_func_1.exp)(val));
        } else {
            return val - r_func_1.M_LN2;
        }
    } else {
        val /= 2;
        return (0, r_func_1.R_D_Cval)(lower_tail, val);
    }
}
exports.pt = pt;