'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qnt = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var r_func_1 = require('../../r-func.cjs');
var qnorm_1 = require('../normal/qnorm.cjs');
var pnt_1 = require('./pnt.cjs');
var qt_1 = require('./qt.cjs');
const printer = (0, debug_1.debug)('qnt');
const accu = 1e-13;
const Eps = 1e-11;
function qnt(p, df, ncp, lower_tail, log_p) {
    let ux;
    let lx;
    let nx;
    let pp;
    if (isNaN(p) || isNaN(df) || isNaN(ncp)) {
        return p + df + ncp;
    }
    if (df <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (ncp === 0 && df >= 1) {
        return (0, qt_1.qt)(p, df, lower_tail, log_p);
    }
    const rc = (0, logger_1.R_Q_P01_boundaries)(lower_tail, log_p, p, -Infinity, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (!isFinite(df)) {
        return (0, qnorm_1.qnorm)(p, ncp, 1, lower_tail, log_p);
    }
    p = (0, expm1_1.R_DT_qIv)(lower_tail, log_p, p);
    if (p > 1 - r_func_1.DBL_EPSILON) {
        return Infinity;
    }
    pp = (0, r_func_1.min)(1 - r_func_1.DBL_EPSILON, p * (1 + Eps));
    for (ux = (0, r_func_1.max)(1, ncp); ux < r_func_1.DBL_MAX && (0, pnt_1.pnt)(ux, df, ncp, true, false) < pp; ux *= 2);
    pp = p * (1 - Eps);
    for (lx = (0, r_func_1.min)(-1, -ncp); lx > -r_func_1.DBL_MAX && (0, pnt_1.pnt)(lx, df, ncp, true, false) > pp; lx *= 2);
    do {
        nx = 0.5 * (lx + ux);
        if ((0, pnt_1.pnt)(nx, df, ncp, true, false) > p) {
            ux = nx;
        } else {
            lx = nx;
        }
    } while (ux - lx > accu * (0, r_func_1.max)((0, r_func_1.abs)(lx), (0, r_func_1.abs)(ux)));
    return 0.5 * (lx + ux);
}
exports.qnt = qnt;