'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qt = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var qnorm_1 = require('../normal/qnorm.cjs');
var tanpi_1 = require('../../trigonometry/tanpi.cjs');
var dt_1 = require('./dt.cjs');
var pt_1 = require('./pt.cjs');
const printer = (0, debug_1.debug)('qt');
const accu = 1e-13;
const Eps = 1e-11;
const eps = 1e-12;
function qt(p, ndf, lower_tail, log_p) {
    let P;
    let q;
    if (isNaN(p) || isNaN(ndf)) {
        return p + ndf;
    }
    const rc = (0, logger_1.R_Q_P01_boundaries)(lower_tail, log_p, p, -Infinity, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (ndf <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (ndf < 1) {
        let ux;
        let lx;
        let nx;
        let pp;
        let iter = 0;
        p = (0, expm1_1.R_DT_qIv)(lower_tail, log_p, p);
        if (p > 1 - r_func_1.DBL_EPSILON) {
            return Infinity;
        }
        pp = (0, r_func_1.min)(1 - r_func_1.DBL_EPSILON, p * (1 + Eps));
        for (ux = 1; ux < r_func_1.DBL_MAX && (0, pt_1.pt)(ux, ndf, true, false) < pp; ux *= 2);
        pp = p * (1 - Eps);
        for (lx = -1; lx > -r_func_1.DBL_MAX && (0, pt_1.pt)(lx, ndf, true, false) > pp; lx *= 2);
        do {
            nx = 0.5 * (lx + ux);
            if ((0, pt_1.pt)(nx, ndf, true, false) > p) {
                ux = nx;
            } else {
                lx = nx;
            }
        } while ((ux - lx) / (0, r_func_1.abs)(nx) > accu && ++iter < 1000);
        if (iter >= 1000) {
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'qt', printer);
        }
        return 0.5 * (lx + ux);
    }
    if (ndf > 100000000000000000000) {
        return (0, qnorm_1.qnorm)(p, 0, 1, lower_tail, log_p);
    }
    P = (0, r_func_1.R_D_qIv)(log_p, p);
    const neg = (!lower_tail || P < 0.5) && (lower_tail || P > 0.5);
    const is_neg_lower = lower_tail === neg;
    if (neg) {
        if (log_p) {
            if (lower_tail) {
                P = 2 * P;
            } else {
                P = 2 * -(0, r_func_1.expm1)(p);
            }
        } else {
            P = 2 * (0, r_func_1.R_D_Lval)(lower_tail, p);
        }
    } else {
        if (log_p) {
            if (lower_tail) {
                P = 2 * -(0, r_func_1.expm1)(p);
            } else {
                P = 2 * P;
            }
        } else {
            P = 2 * (0, r_func_1.R_D_Cval)(lower_tail, p);
        }
    }
    if ((0, r_func_1.abs)(ndf - 2) < eps) {
        if (P > r_func_1.DBL_MIN) {
            if (3 * P < r_func_1.DBL_EPSILON) {
                q = 1 / (0, r_func_1.sqrt)(P);
            } else if (P > 0.9) {
                q = (1 - P) * (0, r_func_1.sqrt)(2 / (P * (2 - P)));
            } else {
                q = (0, r_func_1.sqrt)(2 / (P * (2 - P)) - 2);
            }
        } else {
            if (log_p) {
                q = is_neg_lower ? (0, r_func_1.exp)(-p / 2) / r_func_1.SQRT2 : 1 / (0, r_func_1.sqrt)(-(0, r_func_1.expm1)(p));
            } else {
                q = Infinity;
            }
        }
    } else if (ndf < 1 + eps) {
        if (P === 1) {
            q = 0;
        } else if (P > r_func_1.DBL_EPSILON / 2) {
            q = 1 / (0, tanpi_1.tanpi)(P / 2);
        } else {
            if (log_p) {
                q = is_neg_lower ? r_func_1.M_1_PI * (0, r_func_1.exp)(-p) : -1 / (r_func_1.PI * (0, r_func_1.expm1)(p));
            } else {
                q = Infinity;
            }
        }
    } else {
        let x = 0;
        let y = 0;
        let log_P2 = 0;
        const a = 1 / (ndf - 0.5);
        const b = 48 / (a * a);
        let c = ((20700 * a / b - 98) * a - 16) * a + 96.36;
        const d = ((94.5 / (b + c) - 3) / b + 1) * (0, r_func_1.sqrt)(a * r_func_1.M_PI_2) * ndf;
        const P_ok1 = P > r_func_1.DBL_MIN || !log_p;
        let P_ok = P_ok1;
        if (P_ok1) {
            y = (0, r_func_1.pow)(d * P, 2 / ndf);
            P_ok = y >= r_func_1.DBL_EPSILON;
        }
        if (!P_ok) {
            if (is_neg_lower) {
                log_P2 = (0, r_func_1.R_D_log)(log_p, p);
            } else {
                log_P2 = (0, expm1_1.R_D_LExp)(log_p, p);
            }
            x = ((0, r_func_1.log)(d) + r_func_1.LN2 + log_P2) / ndf;
            y = (0, r_func_1.exp)(2 * x);
        }
        if (ndf < 2.1 && P > 0.5 || y > 0.05 + a) {
            if (P_ok) {
                x = (0, qnorm_1.qnorm)(0.5 * P, 0, 1, false, false);
            } else {
                x = (0, qnorm_1.qnorm)(log_P2, 0, 1, lower_tail, true);
            }
            y = x * x;
            if (ndf < 5) {
                c += 0.3 * (ndf - 4.5) * (x + 0.6);
            }
            c = (((0.05 * d * x - 5) * x - 7) * x - 2) * x + b + c;
            y = (((((0.4 * y + 6.3) * y + 36) * y + 94.5) / c - y - 3) / b + 1) * x;
            y = (0, r_func_1.expm1)(a * y * y);
            q = (0, r_func_1.sqrt)(ndf * y);
        } else if (!P_ok && x < -r_func_1.LN2 * r_func_1.DBL_MANT_DIG) {
            q = (0, r_func_1.sqrt)(ndf) * (0, r_func_1.exp)(-x);
        } else {
            y = ((1 / (((ndf + 6) / (ndf * y) - 0.089 * d - 0.822) * (ndf + 2) * 3) + 0.5 / (ndf + 4)) * y - 1) * (ndf + 1) / (ndf + 2) + 1 / y;
            q = (0, r_func_1.sqrt)(ndf * y);
        }
        if (P_ok1) {
            let it = 0;
            while (it++ < 10 && (y = (0, dt_1._dt)(q, ndf, false)) > 0 && isFinite(x = ((0, pt_1.pt)(q, ndf, false, false) - P / 2) / y) && (0, r_func_1.abs)(x) > 1e-14 * (0, r_func_1.abs)(q)) {
                q += x * (1 + x * q * (ndf + 1) / (2 * (q * q + ndf)));
            }
        }
    }
    if (neg) {
        q = -q;
    }
    return q;
}
exports.qt = qt;