'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.ptukey = void 0;
var debug_1 = require('@mangos/debug');
var wprob_1 = require('./wprob.cjs');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var gamma_1 = require('../../special/gamma/index.cjs');
const printer_ptukey = (0, debug_1.debug)('ptukey');
const nlegq = 16;
const ihalfq = 8;
const eps1 = -30;
const eps2 = 1e-14;
const dhaf = 100;
const dquar = 800;
const deigh = 5000;
const dlarg = 25000;
const ulen1 = 1;
const ulen2 = 0.5;
const ulen3 = 0.25;
const ulen4 = 0.125;
const xlegq = [
    0.9894009349916499,
    0.9445750230732326,
    0.8656312023878318,
    0.755404408355003,
    0.6178762444026438,
    0.45801677765722737,
    0.2816035507792589,
    0.09501250983763744
];
const alegq = [
    0.027152459411754096,
    0.062253523938647894,
    0.09515851168249279,
    0.12462897125553388,
    0.14959598881657674,
    0.16915651939500254,
    0.18260341504492358,
    0.1894506104550685
];
function ptukey(q, nmeans, df, nrnages = 1, lowerTail = true, logP = false) {
    let ans;
    let f2lf;
    let otsum;
    let qsqz;
    let rotsum;
    let twa1;
    let ulen;
    let wprb;
    if (isNaN(q) || isNaN(nrnages) || isNaN(nmeans) || isNaN(df)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_ptukey, logger_1.lineInfo4);
    }
    if (q <= 0) {
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    }
    if (df < 2 || nrnages < 1 || nmeans < 2) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_ptukey, logger_1.lineInfo4);
    }
    if (!isFinite(q)) {
        return (0, r_func_1.R_DT_1)(lowerTail, logP);
    }
    if (df > dlarg) {
        return (0, r_func_1.R_DT_val)(lowerTail, logP, (0, wprob_1.wprob)(q, nrnages, nmeans));
    }
    const f2 = df * 0.5;
    f2lf = f2 * (0, r_func_1.log)(df) - df * r_func_1.M_LN2 - (0, gamma_1.lgammafn_sign)(f2);
    const f21 = f2 - 1;
    const ff4 = df * 0.25;
    if (df <= dhaf) {
        ulen = ulen1;
    } else if (df <= dquar) {
        ulen = ulen2;
    } else if (df <= deigh) {
        ulen = ulen3;
    } else {
        ulen = ulen4;
    }
    f2lf += (0, r_func_1.log)(ulen);
    ans = 0;
    otsum = 0;
    for (let i = 1; i <= 50; i++) {
        otsum = 0;
        twa1 = (2 * i - 1) * ulen;
        for (let jj = 1; jj <= nlegq; jj++) {
            const j = ihalfq < jj ? jj - ihalfq - 1 : jj - 1;
            const t1 = ihalfq < jj ? f2lf + f21 * (0, r_func_1.log)(twa1 + xlegq[j] * ulen) - (xlegq[j] * ulen + twa1) * ff4 : f2lf + f21 * (0, r_func_1.log)(twa1 - xlegq[j] * ulen) + (xlegq[j] * ulen - twa1) * ff4;
            if (t1 >= eps1) {
                if (ihalfq < jj) {
                    qsqz = q * (0, r_func_1.sqrt)((xlegq[j] * ulen + twa1) * 0.5);
                } else {
                    qsqz = q * (0, r_func_1.sqrt)((-(xlegq[j] * ulen) + twa1) * 0.5);
                }
                wprb = (0, wprob_1.wprob)(qsqz, nrnages, nmeans);
                rotsum = wprb * alegq[j] * (0, r_func_1.exp)(t1);
                otsum += rotsum;
            }
        }
        if (i * ulen >= 1 && otsum <= eps2) {
            break;
        }
        ans += otsum;
    }
    if (otsum > eps2) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'ptukey', printer_ptukey);
    }
    ans = (0, r_func_1.min)(1, ans);
    return (0, r_func_1.R_DT_val)(lowerTail, logP, ans);
}
exports.ptukey = ptukey;