'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qtukey = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var ptukey_1 = require('./ptukey.cjs');
var r_func_1 = require('../../r-func.cjs');
const p0 = 0.322232421088;
const q0 = 0.099348462606;
const p1 = -1;
const q1 = 0.588581570495;
const p2 = -0.342242088547;
const q2 = 0.531103462366;
const p3 = -0.204231210125;
const q3 = 0.10353775285;
const p4 = -0.0000453642210148;
const q4 = 0.0038560700634;
const c1 = 0.8832;
const c2 = 0.2368;
const c3 = 1.214;
const c4 = 1.208;
const c5 = 1.4142;
const vmax = 120;
const printer = (0, debug_1.debug)('qtukey');
function qinv(p, c, v) {
    let q;
    let t;
    const ps = 0.5 - 0.5 * p;
    const yi = (0, r_func_1.sqrt)((0, r_func_1.log)(1 / (ps * ps)));
    t = yi + ((((yi * p4 + p3) * yi + p2) * yi + p1) * yi + p0) / ((((yi * q4 + q3) * yi + q2) * yi + q1) * yi + q0);
    if (v < vmax) {
        t += (t * t * t + t) / v / 4;
    }
    q = c1 - c2 * t;
    if (v < vmax) {
        q += -c3 / v + c4 * t / v;
    }
    return t * (q * (0, r_func_1.log)(c - 1) + c5);
}
function qtukey(p, nmeans, df, nranges = 1, lowerTail = true, logP = false) {
    const eps = 0.0001;
    const maxiter = 50;
    let ans = 0;
    let valx0;
    let valx1;
    let x0;
    let x1;
    let xabs;
    let iter;
    if (isNaN(p) || isNaN(nranges) || isNaN(nmeans) || isNaN(df)) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_DOMAIN, 'qtukey', printer);
        return NaN;
    }
    if (df < 2 || nranges < 1 || nmeans < 2) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const rc = (0, logger_1.R_Q_P01_boundaries)(lowerTail, logP, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    p = (0, expm1_1.R_DT_qIv)(lowerTail, logP, p);
    x0 = qinv(p, nmeans, df);
    valx0 = (0, ptukey_1.ptukey)(x0, nmeans, df, nranges, true, false) - p;
    if (valx0 > 0) {
        x1 = (0, r_func_1.max)(0, x0 - 1);
    } else {
        x1 = x0 + 1;
    }
    valx1 = (0, ptukey_1.ptukey)(x1, nmeans, df, nranges, true, false) - p;
    for (iter = 1; iter < maxiter; iter++) {
        ans = x1 - valx1 * (x1 - x0) / (valx1 - valx0);
        valx0 = valx1;
        x0 = x1;
        if (ans < 0) {
            ans = 0;
            valx1 = -p;
        }
        valx1 = (0, ptukey_1.ptukey)(ans, nmeans, df, nranges, true, false) - p;
        x1 = ans;
        xabs = (0, r_func_1.abs)(x1 - x0);
        if (xabs < eps) {
            return ans;
        }
    }
    (0, logger_1.ML_ERROR2)(logger_1.ME.ME_NOCONV, 'qtukey', printer);
    return ans;
}
exports.qtukey = qtukey;