'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.dweibull = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
const printer = (0, debug_1.debug)('dweibull');
function dweibull(x, shape, scale = 1, log = false) {
    if (isNaN(x) || isNaN(shape) || isNaN(scale)) {
        return x + shape + scale;
    }
    if (shape <= 0 || scale <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (x < 0) {
        return (0, r_func_1.R_D__0)(log);
    }
    if (!isFinite(x)) {
        return (0, r_func_1.R_D__0)(log);
    }
    if (x === 0 && shape < 1) {
        return Infinity;
    }
    const tmp1 = (0, r_func_1.pow)(x / scale, shape - 1);
    const tmp2 = tmp1 * (x / scale);
    return log ? -tmp2 + (0, r_func_1.log)(shape * tmp1 / scale) : shape * tmp1 * (0, r_func_1.exp)(-tmp2) / scale;
}
exports.dweibull = dweibull;