'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pweibull = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var expm1_1 = require('../exp/expm1.cjs');
const printer = (0, debug_1.debug)('pweibull');
function pweibull(q, shape, scale = 1, lowerTail = true, logP = false) {
    if (isNaN(q) || isNaN(shape) || isNaN(scale)) {
        return q + shape + scale;
    }
    if (shape <= 0 || scale <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    if (q <= 0) {
        return (0, r_func_1.R_DT_0)(lowerTail, logP);
    }
    q = -(0, r_func_1.pow)(q / scale, shape);
    return lowerTail ? logP ? (0, expm1_1.R_Log1_Exp)(q) : -(0, r_func_1.expm1)(q) : (0, r_func_1.R_D_exp)(logP, q);
}
exports.pweibull = pweibull;