'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qweibull = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var r_func_1 = require('../../r-func.cjs');
const printer = (0, debug_1.debug)('qweibull');
function qweibull(p, shape, scale = 1, lowerTail = true, logP = false) {
    if (isNaN(p) || isNaN(shape) || isNaN(scale)) {
        return p + shape + scale;
    }
    if (shape <= 0 || scale <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer, logger_1.lineInfo4);
    }
    const rc = (0, logger_1.R_Q_P01_boundaries)(lowerTail, logP, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    return scale * (0, r_func_1.pow)(-(0, expm1_1.R_DT_Clog)(lowerTail, logP, p), 1 / shape);
}
exports.qweibull = qweibull;