'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.pwilcox = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var choose_1 = require('../../special/choose/index.cjs');
var WilcoxonCache_1 = require('./WilcoxonCache.cjs');
var cwilcox_1 = require('./cwilcox.cjs');
const {
    round: R_forceint,
    floor
} = Math;
const {
    isNaN: ISNAN,
    isFinite: R_FINITE
} = Number;
const printer_pwilcox = (0, debug_1.debug)('pwilcox');
function pwilcox(q, m, n, lowerTail = true, logP = false) {
    m = R_forceint(m);
    n = R_forceint(n);
    const w = new WilcoxonCache_1.WilcoxonCache();
    let lower_tail = lowerTail;
    if (ISNAN(q) || ISNAN(m) || ISNAN(n)) {
        return q + m + n;
    }
    if (!R_FINITE(m) || !R_FINITE(n)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pwilcox, logger_1.lineInfo4);
    }
    if (m <= 0 || n <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_pwilcox, logger_1.lineInfo4);
    }
    q = floor(q + 1e-7);
    if (q < 0) {
        return (0, r_func_1.R_DT_0)(lower_tail, logP);
    }
    if (q >= m * n) {
        return (0, r_func_1.R_DT_1)(lower_tail, logP);
    }
    const c = (0, choose_1.choose)(m + n, n);
    let p = 0;
    if (q <= m * n / 2) {
        for (let i = 0; i <= q; i++) {
            p += (0, cwilcox_1.cwilcox)(i, m, n, w) / c;
        }
    } else {
        q = m * n - q;
        for (let i = 0; i < q; i++) {
            p += (0, cwilcox_1.cwilcox)(i, m, n, w) / c;
        }
        lower_tail = !lower_tail;
    }
    return (0, r_func_1.R_DT_val)(lower_tail, logP, p);
}
exports.pwilcox = pwilcox;