'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.qwilcox = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var expm1_1 = require('../exp/expm1.cjs');
var cwilcox_1 = require('./cwilcox.cjs');
var WilcoxonCache_1 = require('./WilcoxonCache.cjs');
var choose_1 = require('../../special/choose/index.cjs');
const printer_qwilcox = (0, debug_1.debug)('qwilcox');
function qwilcox(x, m, n, lowerTail = true, logP = false) {
    m = Math.round(m);
    n = Math.round(n);
    const w = new WilcoxonCache_1.WilcoxonCache();
    if (isNaN(x) || isNaN(m) || isNaN(n)) {
        return x + m + n;
    }
    if (!isFinite(x) || !isFinite(m) || !isFinite(n)) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qwilcox, logger_1.lineInfo4);
    }
    const rc = (0, logger_1.R_Q_P01_check)(logP, x);
    if (rc !== undefined) {
        return rc;
    }
    if (m <= 0 || n <= 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_qwilcox, logger_1.lineInfo4);
    }
    if (x === (0, r_func_1.R_DT_0)(lowerTail, logP)) {
        return 0;
    }
    if (x === (0, r_func_1.R_DT_1)(lowerTail, logP)) {
        return m * n;
    }
    x = (0, expm1_1.R_DT_qIv)(lowerTail, logP, x);
    const c = (0, choose_1.choose)(m + n, n);
    let p = 0;
    let q = 0;
    if (x <= 0.5) {
        x = x - 10 * r_func_1.DBL_EPSILON;
        for (;;) {
            p += (0, cwilcox_1.cwilcox)(q, m, n, w) / c;
            if (p >= x)
                break;
            q++;
        }
    } else {
        x = 1 - x + 10 * r_func_1.DBL_EPSILON;
        for (;;) {
            p += (0, cwilcox_1.cwilcox)(q, m, n, w) / c;
            if (p > x) {
                q = Math.trunc(m * n - q);
                break;
            }
            q++;
        }
    }
    return q;
}
exports.qwilcox = qwilcox;