'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.rwilcoxOne = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../packages/common/logger.cjs');
var r_func_1 = require('../../r-func.cjs');
var utils_1 = require('../../rng/utils.cjs');
var global_rng_1 = require('../../rng/global-rng.cjs');
const printer_rwilcox = (0, debug_1.debug)('rwilcox');
const MAXSIZE = 800000000;
function rwilcoxOne(m, n) {
    const rng = (0, global_rng_1.globalUni)();
    const sampleKind = (0, global_rng_1.globalSampleKind)();
    if (isNaN(m) || isNaN(n)) {
        return m + n;
    }
    m = (0, r_func_1.round)(m);
    n = (0, r_func_1.round)(n);
    if (m < 0 || n < 0) {
        return (0, logger_1.ML_ERR_return_NAN2)(printer_rwilcox, logger_1.lineInfo4);
    }
    if (m === 0 || n === 0) {
        return 0;
    }
    let k = (0, r_func_1.trunc)(m + n);
    if (k >= MAXSIZE) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_DOMAIN, 'k > MAXSIZE(=2**32)', printer_rwilcox);
        return (0, logger_1.ML_ERR_return_NAN2)(printer_rwilcox, logger_1.lineInfo4);
    }
    const x = new Uint32Array(k);
    for (let i = 0; i < k; i++) {
        x[i] = i;
    }
    let r = 0;
    printer_rwilcox(`------v`);
    for (let i = 0; i < n; i++) {
        const j = (0, r_func_1.floor)((0, utils_1.R_unif_index)(k, rng, sampleKind));
        r += x[j];
        x[j] = x[--k];
        printer_rwilcox('i:%d,\tn:%d\tj:%d\tk:%d\tr:%d\tx:%o', i, n, j, k, x);
    }
    return r - n * (n - 1) / 2;
}
exports.rwilcoxOne = rwilcoxOne;