'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.randomSeed = exports.RNGkind = exports.setSeed = exports.globalSampleKind = exports.globalNorm = exports.globalUni = void 0;
var knuth_taocp_1 = require('./knuth-taocp/index.cjs');
var knuth_taocp_2002_1 = require('./knuth-taocp-2002/index.cjs');
var lecuyer_cmrg_1 = require('./lecuyer-cmrg/index.cjs');
var marsaglia_multicarry_1 = require('./marsaglia-multicarry/index.cjs');
var mersenne_twister_1 = require('./mersenne-twister/index.cjs');
var super_duper_1 = require('./super-duper/index.cjs');
var wichmann_hill_1 = require('./wichmann-hill/index.cjs');
var ahrens_dieter_1 = require('./normal/ahrens-dieter/index.cjs');
var box_muller_1 = require('./normal/box-muller/index.cjs');
var buggy_kinderman_ramage_1 = require('./normal/buggy-kinderman-ramage/index.cjs');
var inversion_1 = require('./normal/inversion/index.cjs');
var kinderman_ramage_1 = require('./normal/kinderman-ramage/index.cjs');
const uniformMap = {
    KNUTH_TAOCP: knuth_taocp_1.KnuthTAOCP,
    KNUTH_TAOCP2002: knuth_taocp_2002_1.KnuthTAOCP2002,
    LECUYER_CMRG: lecuyer_cmrg_1.LecuyerCMRG,
    MARSAGLIA_MULTICARRY: marsaglia_multicarry_1.MarsagliaMultiCarry,
    MERSENNE_TWISTER: mersenne_twister_1.MersenneTwister,
    SUPER_DUPER: super_duper_1.SuperDuper,
    WICHMANN_HILL: wichmann_hill_1.WichmannHill
};
const normalMap = {
    AHRENS_DIETER: ahrens_dieter_1.AhrensDieter,
    BOX_MULLER: box_muller_1.BoxMuller,
    BUGGY_KINDERMAN_RAMAGE: buggy_kinderman_ramage_1.BuggyKindermanRamage,
    INVERSION: inversion_1.Inversion,
    KINDERMAN_RAMAGE: kinderman_ramage_1.KindermanRamage
};
const uniformKeys = {
    KNUTH_TAOCP: 'KNUTH_TAOCP',
    KNUTH_TAOCP2002: 'KNUTH_TAOCP2002',
    LECUYER_CMRG: 'LECUYER_CMRG',
    MARSAGLIA_MULTICARRY: 'MARSAGLIA_MULTICARRY',
    MERSENNE_TWISTER: 'MERSENNE_TWISTER',
    SUPER_DUPER: 'SUPER_DUPER',
    WICHMANN_HILL: 'WICHMANN_HILL'
};
const normalKeys = {
    AHRENS_DIETER: 'AHRENS_DIETER',
    BOX_MULLER: 'BOX_MULLER',
    BUGGY_KINDERMAN_RAMAGE: 'BUGGY_KINDERMAN_RAMAGE',
    INVERSION: 'INVERSION',
    KINDERMAN_RAMAGE: 'KINDERMAN_RAMAGE'
};
const sampleKind = {
    ROUNDING: 'ROUNDING',
    REJECTION: 'REJECTION'
};
const symRNG = Symbol.for('rngUNIFORM');
const symRNGNormal = Symbol.for('rngNORMAL');
const symSampleKind = Symbol.for('sample.kind');
function globalUni(d) {
    if (d) {
        globalThis[symRNG] = d;
    }
    return globalThis[symRNG];
}
exports.globalUni = globalUni;
function globalNorm(d) {
    if (d) {
        globalThis[symRNGNormal] = d;
    }
    return globalThis[symRNGNormal];
}
exports.globalNorm = globalNorm;
function globalSampleKind(d) {
    if (d) {
        globalThis[symSampleKind] = d;
    }
    return globalThis[symSampleKind];
}
exports.globalSampleKind = globalSampleKind;
function setSeed(seed, randomSet) {
    if (typeof seed !== 'number') {
        throw new Error('Seed needs to bea number');
    }
    if (isNaN(seed) || !isFinite(seed)) {
        throw new Error('Seed needs to be a finite number');
    }
    if (randomSet) {
        RNGkind(randomSet);
    }
    const gu = globalUni();
    const s = new Uint32Array([0]);
    s[0] = seed;
    if (s[0] !== seed) {
        throw new Error(`Seed needs to be an unsigned 32 bit integer, it is ${ seed }`);
    }
    gu.init(seed);
}
exports.setSeed = setSeed;
function RNGkind(opt = {}) {
    let gu = globalUni();
    let no = globalNorm();
    let sk = globalSampleKind();
    function testAndSetUniform(u) {
        const tu = uniformMap[u];
        if (tu) {
            if (tu.kind !== gu.constructor.kind) {
                no.unregister();
                gu = globalUni(new tu());
                no.register(gu);
            }
            return true;
        }
        return false;
    }
    function testAndSetNormal(n) {
        const tn = normalMap[n];
        if (tn) {
            if (tn.kind !== no.constructor.kind) {
                no.unregister();
                no = globalNorm(new tn(gu));
                no.register(gu);
            }
            return true;
        }
        return false;
    }
    function testAndSetSampleKind(s) {
        if (s === 'REJECTION' || s === 'ROUNDING') {
            if (s !== sk) {
                sk = globalSampleKind(s);
            }
            return true;
        }
        return false;
    }
    if (opt.uniform) {
        testAndSetUniform(opt.uniform);
    }
    if (opt.normal) {
        testAndSetNormal(opt.normal);
    }
    if (opt.sampleKind) {
        testAndSetSampleKind(opt.sampleKind);
    }
    return {
        uniform: gu.constructor.kind,
        normal: no.constructor.kind,
        sampleKind: sk
    };
}
exports.RNGkind = RNGkind;
RNGkind.uniform = uniformKeys;
RNGkind.normal = normalKeys;
RNGkind.sampleKind = sampleKind;
function randomSeed(internalState) {
    const gu = globalUni();
    if (internalState === undefined) {
        return gu.seed;
    }
    const state = gu.seed;
    if (state.constructor.name === internalState.constructor.name && state.length === internalState.length) {
        gu.seed = internalState;
    }
    throw new Error(`the internal state of ${ gu.name } is a ${ state.constructor.name } of length ${ state.length }`);
}
exports.randomSeed = randomSeed;
globalThis[symRNG] = new mersenne_twister_1.MersenneTwister();
globalThis[symRNGNormal] = new inversion_1.Inversion(globalThis[symRNG]);
globalThis[symSampleKind] = 'ROUNDING';