'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.KnuthTAOCP2002 = void 0;
var fixup_1 = require('../fixup.cjs');
var irng_1 = require('../irng.cjs');
var seed_1 = require('../seed.cjs');
var seedcheck_1 = require('../seedcheck.cjs');
const QUALITY = 1009;
const SEED_LEN = 101;
const LL = 37;
const KK = 100;
const TT = 70;
const MM = 1073741824;
function mod_diff(x, y) {
    const d = new Uint32Array(3);
    d[0] = x;
    d[1] = y;
    d[2] = d[0] - d[1] & MM - 1;
    return d[2];
}
function is_odd(x) {
    return x % 2 === 1;
}
class KnuthTAOCP2002 extends irng_1.IRNG {
    constructor(_seed = (0, seed_1.default)()) {
        super('Knuth-TAOCP-2002');
        this.qualityBuffer = new ArrayBuffer(QUALITY * 4);
        this.ran_arr_buf = new Uint32Array(this.qualityBuffer);
        this.m_seed = new Uint32Array(SEED_LEN);
        this.ran_x = this.m_seed;
        this.init(_seed);
    }
    get KT_pos() {
        return this.m_seed[100];
    }
    set KT_pos(v) {
        this.m_seed[100] = v;
    }
    ran_array(aa, n) {
        let i;
        let j;
        if (!aa.length) {
            return;
        }
        for (j = 0; j < KK; j++) {
            aa[j] = this.ran_x[j];
        }
        for (; j < n; j++) {
            aa[j] = mod_diff(aa[j - KK], aa[j - LL]);
        }
        for (i = 0; i < LL; i++, j++) {
            this.ran_x[i] = mod_diff(aa[j - KK], aa[j - LL]);
        }
        for (; i < KK; i++, j++) {
            this.ran_x[i] = mod_diff(aa[j - KK], this.ran_x[i - LL]);
        }
    }
    ran_arr_cycle() {
        this.ran_array(this.ran_arr_buf, QUALITY);
        this.ran_arr_buf[KK] = -1;
    }
    ran_start(_seed) {
        let t;
        let j;
        const x = new Uint32Array(KK + KK - 1);
        const ss = new Uint32Array(1);
        const se = new Uint32Array([_seed]);
        ss[0] = se[0] + 2 & MM - 2;
        for (j = 0; j < KK; j++) {
            x[j] = ss[0];
            ss[0] = ss[0] << 1;
            if (ss[0] >= MM) {
                ss[0] = ss[0] - (MM - 2);
            }
        }
        x[1]++;
        for (ss[0] = se[0] & MM - 1, t = TT - 1; t;) {
            for (j = KK - 1; j > 0; j--) {
                x[j + j] = x[j];
                x[j + j - 1] = 0;
            }
            for (j = KK + KK - 2; j >= KK; j--) {
                x[j - (KK - LL)] = mod_diff(x[j - (KK - LL)], x[j]);
                x[j - KK] = mod_diff(x[j - KK], x[j]);
            }
            if (is_odd(ss[0])) {
                for (j = KK; j > 0; j--) {
                    x[j] = x[j - 1];
                }
                x[0] = x[KK];
                x[LL] = mod_diff(x[LL], x[KK]);
            }
            if (ss[0]) {
                ss[0] = ss[0] >>> 1;
            } else {
                t--;
            }
        }
        for (j = 0; j < LL; j++) {
            this.ran_x[j + KK - LL] = x[j];
        }
        for (; j < KK; j++) {
            this.ran_x[j - LL] = x[j];
        }
        for (j = 0; j < 10; j++) {
            this.ran_array(x, KK + KK - 1);
        }
    }
    RNG_Init_KT2(_seed) {
        this.ran_start(_seed % 1073741821);
        this.KT_pos = 100;
    }
    KT_next() {
        if (this.KT_pos >= 100) {
            this.ran_arr_cycle();
            this.KT_pos = 0;
        }
        return this.ran_x[this.KT_pos++];
    }
    random() {
        const KT = 9.31322574615479e-10;
        return (0, fixup_1.fixup)(this.KT_next() * KT);
    }
    init(_seed = (0, seed_1.default)()) {
        const s = new Uint32Array([0]);
        s[0] = _seed;
        for (let j = 0; j < 50; j++) {
            s[0] = 69069 * s[0] + 1;
        }
        this.RNG_Init_KT2(s[0]);
        super.init(_seed);
    }
    set seed(_seed) {
        (0, seedcheck_1.seedCheck)(this.name, _seed, SEED_LEN);
        this.m_seed.set(_seed);
    }
    get seed() {
        return this.m_seed.slice();
    }
    get cut() {
        return 2 ** 25 - 1;
    }
}
exports.KnuthTAOCP2002 = KnuthTAOCP2002;
KnuthTAOCP2002.kind = 'KNUTH_TAOCP2002';