'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.LecuyerCMRG = void 0;
var irng_1 = require('../irng.cjs');
var seed_1 = require('../seed.cjs');
var seedcheck_1 = require('../seedcheck.cjs');
var r_func_1 = require('../../r-func.cjs');
const SEED_LEN = 6;
const a12 = 1403580;
const a13n = 810728;
const m2 = 4294944443;
const m1 = 4294967087;
const normc = 2.328306549295728e-10;
const a21 = 527612;
const a23n = 1370589;
class LecuyerCMRG extends irng_1.IRNG {
    constructor(_seed = (0, seed_1.default)()) {
        super('L\'Ecuyer-CMRG');
        this.m_seed = new Int32Array(SEED_LEN);
        this.init(_seed);
    }
    init(_seed = (0, seed_1.default)()) {
        const s = new Int32Array([0]);
        s[0] = _seed;
        for (let j = 0; j < 50; j++) {
            s[0] = 69069 * s[0] + 1;
        }
        for (let j = 0; j < this.m_seed.length; j++) {
            s[0] = 69069 * s[0] + 1;
            while (s[0] >= m2) {
                s[0] = 69069 * s[0] + 1;
            }
            this.m_seed[j] = s[0];
        }
        super.init(_seed);
    }
    random() {
        const II = this.m_seed;
        let k;
        let p1;
        let p2;
        p1 = a12 * new Uint32Array([II[1]])[0] - a13n * new Uint32Array([II[0]])[0];
        k = new Int32Array([p1 / m1])[0];
        p1 -= k * m1;
        if (p1 < 0)
            p1 += m1;
        II[0] = II[1];
        II[1] = II[2];
        II[2] = new Int32Array([p1])[0];
        p2 = a21 * new Uint32Array([II[5]])[0] - a23n * new Uint32Array([II[3]])[0];
        k = new Int32Array([p2 / m2])[0];
        p2 -= k * m2;
        if (p2 < 0)
            p2 += m2;
        II[3] = II[4];
        II[4] = II[5];
        II[5] = new Int32Array([p2])[0];
        return (p1 > p2 ? p1 - p2 : p1 - p2 + m1) * normc;
    }
    set seed(_seed) {
        (0, seedcheck_1.seedCheck)(this.name, _seed, SEED_LEN);
        this.m_seed.set(_seed);
    }
    get seed() {
        return this.m_seed.slice();
    }
    get cut() {
        return r_func_1.INT_MAX;
    }
}
exports.LecuyerCMRG = LecuyerCMRG;
LecuyerCMRG.kind = 'LECUYER_CMRG';