'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.MarsagliaMultiCarry = void 0;
var r_func_1 = require('../../r-func.cjs');
var fixup_1 = require('../fixup.cjs');
var irng_1 = require('../irng.cjs');
var seed_1 = require('../seed.cjs');
var seedcheck_1 = require('../seedcheck.cjs');
const SEED_LEN = 2;
class MarsagliaMultiCarry extends irng_1.IRNG {
    constructor(_seed = (0, seed_1.default)()) {
        super('Marsaglia-MultiCarry');
        this.m_seed = new Int32Array(SEED_LEN);
        this.init(_seed);
    }
    fixupSeeds() {
        const s = this.m_seed;
        if (s[0] === 0)
            s[0] = 1;
        if (s[1] === 0)
            s[1] = 1;
        return;
    }
    init(_seed = (0, seed_1.default)()) {
        const s = new Int32Array([_seed]);
        for (let j = 0; j < 50; j++) {
            s[0] = 69069 * s[0] + 1;
        }
        for (let j = 0; j < this.m_seed.length; j++) {
            s[0] = 69069 * s[0] + 1;
            this.m_seed[j] = s[0];
        }
        this.fixupSeeds();
        super.init(_seed);
    }
    random() {
        const s = this.m_seed;
        s[0] = 36969 * (s[0] & 65535) + (s[0] >>> 16);
        s[1] = 18000 * (s[1] & 65535) + (s[1] >>> 16);
        const un = new Uint32Array(SEED_LEN);
        un[0] = s[0] << 16;
        un[1] = s[1] & 65535;
        un[0] = un[0] ^ un[1];
        return (0, fixup_1.fixup)(un[0] * fixup_1.i2_32m1);
    }
    set seed(_seed) {
        (0, seedcheck_1.seedCheck)(this.name, _seed, SEED_LEN);
        this.m_seed.set(_seed);
        this.fixupSeeds();
    }
    get seed() {
        return this.m_seed.slice();
    }
    get cut() {
        return r_func_1.INT_MAX;
    }
}
exports.MarsagliaMultiCarry = MarsagliaMultiCarry;
MarsagliaMultiCarry.kind = 'MARSAGLIA_MULTICARRY';