'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.MersenneTwister = void 0;
var fixup_1 = require('../fixup.cjs');
var irng_1 = require('../irng.cjs');
var seed_1 = require('../seed.cjs');
var seedcheck_1 = require('../seedcheck.cjs');
var r_func_1 = require('../../r-func.cjs');
const N = 624;
const M = 397;
const MATRIX_A = 2567483615;
const UPPER_MASK = 2147483648;
const LOWER_MASK = 2147483647;
const TEMPERING_MASK_B = 2636928640;
const TEMPERING_MASK_C = 4022730752;
const SEED_LEN = 625;
class MersenneTwister extends irng_1.IRNG {
    constructor(_seed = (0, seed_1.default)()) {
        super('Mersenne-Twister');
        const buf = new ArrayBuffer(SEED_LEN * 4);
        this.m_seed = new Int32Array(buf);
        this.mt = new Int32Array(buf, 4);
        this.mti = N + 1;
        this.init(_seed);
    }
    MT_sgenrand(_seed) {
        for (let i = 0; i < N; i++) {
            this.mt[i] = _seed & 4294901760;
            _seed = 69069 * _seed + 1 << 0;
            this.mt[i] |= (_seed & 4294901760) >>> 16;
            _seed = 69069 * _seed + 1 << 0;
        }
        this.mti = N;
    }
    MT_genrand() {
        const y = new Int32Array(1);
        const mag01 = new Int32Array([
            0,
            MATRIX_A
        ]);
        const dummy = this.m_seed;
        this.mti = dummy[0];
        if (this.mti >= N) {
            let kk;
            if (this.mti === N + 1)
                this.MT_sgenrand(4357);
            for (kk = 0; kk < N - M; kk++) {
                y[0] = this.mt[kk] & UPPER_MASK | this.mt[kk + 1] & LOWER_MASK;
                this.mt[kk] = this.mt[kk + M] ^ y[0] >>> 1 ^ mag01[y[0] & 1];
            }
            for (; kk < N - 1; kk++) {
                y[0] = this.mt[kk] & UPPER_MASK | this.mt[kk + 1] & LOWER_MASK;
                this.mt[kk] = this.mt[kk + (M - N)] ^ y[0] >>> 1 ^ mag01[y[0] & 1];
            }
            y[0] = this.mt[N - 1] & UPPER_MASK | this.mt[0] & LOWER_MASK;
            this.mt[N - 1] = this.mt[M - 1] ^ y[0] >>> 1 ^ mag01[y[0] & 1];
            this.mti = 0;
        }
        y[0] = this.mt[this.mti++];
        y[0] ^= y[0] >>> 11;
        y[0] ^= y[0] << 7 & TEMPERING_MASK_B;
        y[0] ^= y[0] << 15 & TEMPERING_MASK_C;
        y[0] ^= y[0] >>> 18;
        dummy[0] = this.mti;
        return new Uint32Array(y.buffer)[0] * 2.3283064365386963e-10;
    }
    fixupSeeds() {
        const s = this.m_seed;
        if (s[0] <= 0)
            s[0] = 624;
        if (this.mt.find(v => !!v) === undefined) {
            this.init();
        }
        return;
    }
    random() {
        const rc = this.MT_genrand();
        return (0, fixup_1.fixup)(rc);
    }
    init(_seed = (0, seed_1.default)()) {
        const s = new Uint32Array([0]);
        s[0] = _seed;
        for (let j = 0; j < 50; j++) {
            s[0] = 69069 * s[0] + 1;
        }
        for (let j = 0; j < this.m_seed.length; j++) {
            s[0] = 69069 * s[0] + 1;
            this.m_seed[j] = s[0];
        }
        this.m_seed[0] = 624;
        this.fixupSeeds();
        super.init(_seed);
    }
    set seed(_seed) {
        (0, seedcheck_1.seedCheck)(this.name, _seed, SEED_LEN);
        this.m_seed.set(_seed);
        this.fixupSeeds();
    }
    get seed() {
        return this.m_seed.slice();
    }
    get cut() {
        return r_func_1.INT_MAX;
    }
}
exports.MersenneTwister = MersenneTwister;
MersenneTwister.kind = 'MERSENNE_TWISTER';