'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.BoxMuller = void 0;
var normal_rng_1 = require('../normal-rng.cjs');
const {log, sqrt, cos, sin} = Math;
const DBL_MIN = 2.22507e-308;
const M_PI = 3.141592653589793;
class BoxMuller extends normal_rng_1.IRNGNormal {
    constructor(_rng) {
        super(_rng, 'Box-Muller');
        this.BM_norm_keep = 0;
    }
    reset(rng) {
        super.reset(rng);
        this.BM_norm_keep = 0;
    }
    random() {
        let s = 0;
        let theta = 0;
        if (this.BM_norm_keep !== 0) {
            s = this.BM_norm_keep;
            this.BM_norm_keep = 0;
            return s;
        } else {
            theta = 2 * M_PI * this._rng.random();
            const R = sqrt(-2 * log(this._rng.random())) + 10 * DBL_MIN;
            this.BM_norm_keep = R * sin(theta);
            return R * cos(theta);
        }
    }
}
exports.BoxMuller = BoxMuller;
BoxMuller.kind = 'BOX_MULLER';