'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.IRNGNormal = void 0;
var irng_1 = require('../irng.cjs');
class IRNGNormal {
    constructor(_rng, name) {
        if (this.constructor.name === 'IRNGNormal') {
            throw new TypeError(`Cannot instantiate class "IRNGNormal" directly`);
        }
        this._name = name;
        this.register = this.register.bind(this);
        this.unregister = this.unregister.bind(this);
        this.register(_rng);
    }
    reset(rng) {
        if (rng && rng.name !== this._rng.name) {
            this._rng = rng;
        }
    }
    unregister() {
        this._rng.unregister(irng_1.MessageType.INIT, this.reset);
    }
    register(_rng) {
        this._rng = _rng;
        this.random = this.random.bind(this);
        this.randoms = this.randoms.bind(this);
        this.reset = this.reset.bind(this);
        this._rng.register(irng_1.MessageType.INIT, this.reset);
    }
    randoms(n) {
        n = !n || n < 0 ? 1 : n;
        const rc = new Float32Array(n);
        for (let i = 0; i < n; i++) {
            rc[i] = this.random();
        }
        return rc;
    }
    random() {
        throw new Error(`override this function in ${ this.constructor.name }`);
    }
    get name() {
        return this._name;
    }
    get uniform_rng() {
        return this._rng;
    }
}
exports.IRNGNormal = IRNGNormal;