'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.WichmannHill = void 0;
var r_func_1 = require('../../r-func.cjs');
var fixup_1 = require('../fixup.cjs');
var irng_1 = require('../irng.cjs');
var seed_1 = require('../seed.cjs');
var seedcheck_1 = require('../seedcheck.cjs');
var r_func_2 = require('../../r-func.cjs');
const SEED_LEN = 3;
class WichmannHill extends irng_1.IRNG {
    constructor(_seed = (0, seed_1.default)()) {
        super('Wichmann-Hill');
        this.m_seed = new Uint32Array(SEED_LEN);
        this.init(_seed);
    }
    random() {
        const s = this.m_seed;
        s[0] = s[0] * 171 % 30269;
        s[1] = s[1] * 172 % 30307;
        s[2] = s[2] * 170 % 30323;
        const value = s[0] / 30269 + s[1] / 30307 + s[2] / 30323;
        return (0, fixup_1.fixup)((0, r_func_2.frac)(value));
    }
    fixupSeeds() {
        const s = this.m_seed;
        s[0] = s[0] % 30269;
        s[1] = s[1] % 30307;
        s[2] = s[2] % 30323;
        if (s[0] === 0)
            s[0] = 1;
        if (s[1] === 0)
            s[1] = 1;
        if (s[2] === 0)
            s[2] = 1;
        return;
    }
    init(_seed = (0, seed_1.default)()) {
        const s = new Uint32Array([_seed]);
        for (let j = 0; j < 50; j++) {
            s[0] = 69069 * s[0] + 1;
        }
        for (let j = 0; j < this.m_seed.length; j++) {
            s[0] = 69069 * s[0] + 1;
            this.m_seed[j] = s[0];
        }
        this.fixupSeeds();
        super.init(_seed);
    }
    set seed(_seed) {
        (0, seedcheck_1.seedCheck)(this.name, _seed, SEED_LEN);
        this.m_seed.set(_seed);
        this.fixupSeeds();
    }
    get seed() {
        return this.m_seed.slice();
    }
    get cut() {
        return r_func_1.INT_MAX;
    }
}
exports.WichmannHill = WichmannHill;
WichmannHill.kind = 'WICHMANN_HILL';