'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.I_bessel = void 0;
var debug_1 = require('@mangos/debug');
var gamma_cody_1 = require('../gamma_cody.cjs');
var bessel_constants_1 = require('../bessel-constants.cjs');
const {
    sqrt,
    pow,
    min,
    max: fmax,
    exp,
    trunc
} = Math;
const {POSITIVE_INFINITY: ML_POSINF} = Number;
const printer = (0, debug_1.debug)('I_bessel');
function I_bessel(x, alpha, nb, ize) {
    const const__ = 1.585;
    let nend;
    let intx;
    let nbmx;
    let k;
    let l;
    let n;
    let nstart;
    let pold;
    let test;
    let p;
    let em;
    let en;
    let empal;
    let emp2al;
    let halfx;
    let aa;
    let bb;
    let cc;
    let psave;
    let plast;
    let tover;
    let psavel;
    let sum;
    let ncalc = nb;
    const bi2 = new Array(nb).fill(0);
    const nu = alpha;
    const twonu = nu + nu;
    if (nb > 0 && x >= 0 && 0 <= nu && nu < 1 && 1 <= ize && ize <= 2) {
        ncalc = nb;
        if (ize === 1 && x > bessel_constants_1.exparg_BESS) {
            return {
                x: ML_POSINF,
                nb,
                ncalc
            };
        }
        if (ize === 2 && x > bessel_constants_1.xlrg_BESS_IJ) {
            return {
                x: 0,
                nb,
                ncalc
            };
        }
        intx = trunc(x);
        if (x >= bessel_constants_1.rtnsig_BESS) {
            nbmx = nb - intx;
            n = intx + 1;
            en = n + n + twonu;
            plast = 1;
            p = en / x;
            test = bessel_constants_1.ensig_BESS + bessel_constants_1.ensig_BESS;
            if (intx * 2 > bessel_constants_1.nsig_BESS * 5) {
                test = sqrt(test * p);
            } else {
                test /= pow(const__, intx);
            }
            let gotoL120 = false;
            if (nbmx >= 3) {
                tover = bessel_constants_1.enten_BESS / bessel_constants_1.ensig_BESS;
                nstart = intx + 2;
                nend = nb - 1;
                for (k = nstart; k <= nend; ++k) {
                    n = k;
                    en += 2;
                    pold = plast;
                    plast = p;
                    p = en * plast / x + pold;
                    if (p > tover) {
                        tover = bessel_constants_1.enten_BESS;
                        p /= tover;
                        plast /= tover;
                        psave = p;
                        psavel = plast;
                        nstart = n + 1;
                        do {
                            ++n;
                            en += 2;
                            pold = plast;
                            plast = p;
                            p = en * plast / x + pold;
                        } while (p <= 1);
                        bb = en / x;
                        test = pold * plast / bessel_constants_1.ensig_BESS;
                        test *= 0.5 - 0.5 / (bb * bb);
                        p = plast * tover;
                        --n;
                        en -= 2;
                        nend = min(nb, n);
                        for (l = nstart; l <= nend; ++l) {
                            ncalc = l - 1;
                            pold = psavel;
                            psavel = psave;
                            psave = en * psavel / x + pold;
                            if (psave * psavel > test) {
                                break;
                            }
                            ncalc = nend;
                        }
                        gotoL120 = true;
                        break;
                    }
                }
                if (gotoL120 === false) {
                    n = nend;
                    en = n + n + twonu;
                    test = fmax(test, sqrt(plast * bessel_constants_1.ensig_BESS) * sqrt(p + p));
                }
            }
            if (gotoL120 === false) {
                do {
                    ++n;
                    en += 2;
                    pold = plast;
                    plast = p;
                    p = en * plast / x + pold;
                } while (p < test);
            }
            ++n;
            en += 2;
            bb = 0;
            aa = 1 / p;
            em = n - 1;
            empal = em + nu;
            emp2al = em - 1 + twonu;
            sum = aa * empal * emp2al / em;
            nend = n - nb;
            let gotoL230 = false;
            for (let cnt = 0; cnt < 1; cnt++) {
                if (nend < 0) {
                    bi2[n - 1] = aa;
                    nend = -nend;
                    for (l = 1; l <= nend; ++l) {
                        bi2[n + l - 1] = 0;
                    }
                } else {
                    if (nend > 0) {
                        for (l = 1; l <= nend; ++l) {
                            --n;
                            en -= 2;
                            cc = bb;
                            bb = aa;
                            if (nend > 100 && aa > 1e+200) {
                                const pow05_to_900 = pow(2, -900);
                                cc = cc * pow05_to_900;
                                bb = bb * pow05_to_900;
                                sum = sum * pow05_to_900;
                            }
                            aa = en * bb / x + cc;
                            em -= 1;
                            emp2al -= 1;
                            if (n === 1) {
                                break;
                            }
                            if (n === 2) {
                                emp2al = 1;
                            }
                            empal -= 1;
                            sum = (sum + aa * empal) * emp2al / em;
                        }
                    }
                    bi2[n - 1] = aa;
                    if (nb <= 1) {
                        sum = sum + sum + aa;
                        gotoL230 = true;
                        break;
                    }
                    --n;
                    en -= 2;
                    bi2[n - 1] = en * aa / x + bb;
                    if (n === 1) {
                        break;
                    }
                    em -= 1;
                    if (n === 2)
                        emp2al = 1;
                    else
                        emp2al -= 1;
                    empal -= 1;
                    sum = (sum + bi2[n - 1] * empal) * emp2al / em;
                }
                nend = n - 2;
                if (nend > 0) {
                    for (l = 1; l <= nend; ++l) {
                        --n;
                        en -= 2;
                        bi2[n - 1] = en * bi2[n] / x + bi2[n + 1];
                        em -= 1;
                        if (n === 2)
                            emp2al = 1;
                        else
                            emp2al -= 1;
                        empal -= 1;
                        sum = (sum + bi2[n - 1] * empal) * emp2al / em;
                    }
                }
                bi2[0] = 2 * empal * bi2[1] / x + bi2[2];
            }
            if (gotoL230 === false) {
                sum = sum + sum + bi2[0];
            }
            if (nu !== 0)
                sum *= (0, gamma_cody_1.Rf_gamma_cody)(1 + nu) * pow(x * 0.5, -nu);
            if (ize === 1)
                sum *= exp(-x);
            aa = bessel_constants_1.enmten_BESS;
            if (sum > 1)
                aa *= sum;
            for (n = 1; n <= nb; ++n) {
                if (bi2[n - 1] < aa)
                    bi2[n - 1] = 0;
                else
                    bi2[n - 1] /= sum;
            }
            const rc = {
                x: bi2[nb - 1],
                nb,
                ncalc
            };
            printer('normalize, devide all  Bi[N] by sum, result:%o', rc);
            return rc;
        } else {
            aa = 1;
            empal = 1 + nu;
            halfx = 0.5 * x;
            if (nu !== 0)
                aa = pow(halfx, nu) / (0, gamma_cody_1.Rf_gamma_cody)(empal);
            if (ize === 2)
                aa *= exp(-x);
            bb = halfx * halfx;
            bi2[0] = aa + aa * bb / empal;
            if (x !== 0 && bi2[0] === 0)
                ncalc = 0;
            if (nb > 1) {
                if (x === 0) {
                    for (n = 2; n <= nb; ++n)
                        bi2[n - 1] = 0;
                } else {
                    cc = halfx;
                    tover = (bessel_constants_1.enmten_BESS + bessel_constants_1.enmten_BESS) / x;
                    if (bb !== 0)
                        tover = bessel_constants_1.enmten_BESS / bb;
                    for (n = 2; n <= nb; ++n) {
                        aa /= empal;
                        empal += 1;
                        aa *= cc;
                        if (aa <= tover * empal)
                            bi2[n - 1] = aa = 0;
                        else
                            bi2[n - 1] = aa + aa * bb / empal;
                        if (bi2[n - 1] === 0 && ncalc > n)
                            ncalc = n - 1;
                    }
                }
            }
        }
    } else {
        ncalc = min(nb, 0) - 1;
    }
    const rc = {
        x: bi2[nb - 1],
        nb,
        ncalc
    };
    printer('drop off, result:%o', rc);
    return rc;
}
exports.I_bessel = I_bessel;