'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../../packages/common/logger.cjs');
var sinpi_1 = require('../../../trigonometry/sinpi.cjs');
var besselK_1 = require('../besselK/index.cjs');
var IBessel_1 = require('./IBessel.cjs');
var r_func_1 = require('../../../r-func.cjs');
const printer = (0, debug_1.debug)('besselI');
function BesselI(x, nu, exponScaled = false) {
    if (isNaN(x) || isNaN(nu))
        return x + nu;
    if (x < 0) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'bessel_i', printer);
        return NaN;
    }
    const ize = exponScaled ? 2 : 1;
    const na = (0, r_func_1.floor)(nu);
    if (nu < 0) {
        return BesselI(x, -nu, exponScaled) + (nu === na ? 0 : (0, besselK_1.default)(x, -nu, exponScaled) * (ize === 1 ? 2 : 2 * (0, r_func_1.exp)(-2 * x)) / r_func_1.PI * (0, sinpi_1.sinpi)(-nu));
    }
    const nb = 1 + (0, r_func_1.trunc)(na);
    nu -= nb - 1;
    const rc = (0, IBessel_1.I_bessel)(x, nu, nb, ize);
    if (rc.ncalc !== rc.nb) {
        if (rc.ncalc < 0)
            printer('bessel_i(%d): ncalc (=%d) != nb (=%d); nu=%d. Arg. out of range?', x, rc.ncalc, rc.nb, nu);
        else
            printer('bessel_i(%d,nu=%d): precision lost in result\n', rc.x, nu + rc.nb - 1);
    }
    x = rc.x;
    return x;
}
exports.default = BesselI;