'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../../packages/common/logger.cjs');
var cospi_1 = require('../../../trigonometry/cospi.cjs');
var sinpi_1 = require('../../../trigonometry/sinpi.cjs');
var besselY_1 = require('../besselY/index.cjs');
var Jbessel_1 = require('./Jbessel.cjs');
var r_func_1 = require('../../../r-func.cjs');
const printer = (0, debug_1.debug)('BesselJ');
function BesselJ(x, nu) {
    if (isNaN(x) || isNaN(nu))
        return x + nu;
    if (x < 0) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'BesselJ', printer);
        return NaN;
    }
    const na = (0, r_func_1.floor)(nu);
    if (nu < 0) {
        let rc;
        if (nu - na === 0.5) {
            rc = 0;
        } else {
            rc = BesselJ(x, -nu) * (0, cospi_1.cospi)(nu);
        }
        if (nu !== na) {
            rc += (0, besselY_1.default)(x, -nu) * (0, sinpi_1.sinpi)(nu);
        }
        return rc;
    } else if (nu > 10000000) {
        printer('besselJ(x, nu): nu=%d too large for bessel_j() algorithm', nu);
        return NaN;
    }
    const nb = 1 + na;
    nu -= na;
    const rc = (0, Jbessel_1.J_bessel)(x, nu, nb);
    printer('debug (nu=%d, na=%d, nb=%d, rc=%j', nu, na, nb, rc);
    if (rc.ncalc !== nb) {
        if (rc.ncalc < 0) {
            printer('bessel_j(%d): ncalc (=%d) != nb (=%d); nu=%d. Arg. out of range?', x, rc.ncalc, rc.nb, nu);
        } else {
            printer('bessel_j(%d,nu=%d): precision lost in result', x, nu + nb - 1);
        }
    }
    return rc.x;
}
exports.default = BesselJ;