'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.K_bessel = void 0;
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../../packages/common/logger.cjs');
var bessel_constants_1 = require('../bessel-constants.cjs');
const M_SQRT_2dPI = 0.7978845608028654;
var r_func_1 = require('../../../r-func.cjs');
const printer = (0, debug_1.debug)('K_bessel');
function K_bessel(x, alpha, nb, ize) {
    const a = 0.11593151565841245;
    const p = [
        0.8056298756904329,
        20.404550020536515,
        157.7056051066762,
        536.6711164692075,
        900.3827592912887,
        730.9238866506604,
        229.29930150942513,
        0.8224670334241132
    ];
    const q = [
        29.460198624785043,
        277.5778685102212,
        1206.7032559102745,
        2762.9144415979154,
        3443.740505065646,
        2210.6319011337864,
        572.2673383598922
    ];
    const r = [
        -0.48672575865218404,
        13.079485869097804,
        -101.96490580880537,
        347.65409106507815,
        0.0003495898124521935
    ];
    const s = [
        -25.57910550997646,
        212.57260432226545,
        -610.6901868494411,
        422.6966880577776
    ];
    const t = [
        1.6125990452916364e-10,
        2.5051878502858254e-8,
        0.0000027557319615147965,
        0.00019841269840928374,
        0.008333333333333475,
        0.16666666666666666
    ];
    const estm = [
        52.0583,
        5.7607,
        2.7782,
        14.4303,
        185.3004,
        9.3715
    ];
    const estf = [
        41.8341,
        7.1075,
        6.4306,
        42.511,
        1.35633,
        84.5096,
        20
    ];
    let iend;
    let i;
    let j;
    let k;
    let m;
    let ii;
    let mplus1;
    let x2by4;
    let twox;
    let c;
    let blpha;
    let ratio;
    let wminf;
    let d1;
    let d2;
    let d3;
    let f0;
    let f1;
    let f2;
    let p0;
    let q0;
    let t1;
    let t2;
    let twonu;
    let dm;
    let bk1;
    let bk2;
    let nu;
    const bk = new Array(nb).fill(0);
    ii = 0;
    const ex = x;
    nu = alpha;
    let ncalc = (0, r_func_1.min)(nb, 0) - 2;
    if (nb > 0 && 0 <= nu && nu < 1 && 1 <= ize && ize <= 2) {
        for (let cnt = 0; cnt < 1; cnt++) {
            if (ex <= 0 || ize === 1 && ex > bessel_constants_1.xmax_BESS_K) {
                if (ex <= 0) {
                    if (ex < 0)
                        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'K_bessel', printer);
                    bk[0] = Infinity;
                } else
                    bk[0] = 0;
                ncalc = nb;
                return {
                    x: bk[0],
                    nb,
                    ncalc
                };
            }
            k = 0;
            if (nu < bessel_constants_1.sqxmin_BESS_K) {
                nu = 0;
            } else if (nu > 0.5) {
                k = 1;
                nu -= 1;
            }
            twonu = nu + nu;
            iend = nb + k - 1;
            c = nu * nu;
            d3 = -c;
            if (ex <= 1) {
                d1 = 0;
                d2 = p[0];
                t1 = 1;
                t2 = q[0];
                for (i = 2; i <= 7; i += 2) {
                    d1 = c * d1 + p[i - 1];
                    d2 = c * d2 + p[i];
                    t1 = c * t1 + q[i - 1];
                    t2 = c * t2 + q[i];
                }
                d1 = nu * d1;
                t1 = nu * t1;
                f1 = (0, r_func_1.log)(ex);
                f0 = a + nu * (p[7] - nu * (d1 + d2) / (t1 + t2)) - f1;
                q0 = (0, r_func_1.exp)(-nu * (a - nu * (p[7] + nu * (d1 - d2) / (t1 - t2)) - f1));
                f1 = nu * f0;
                p0 = (0, r_func_1.exp)(f1);
                d1 = r[4];
                t1 = 1;
                for (i = 0; i < 4; ++i) {
                    d1 = c * d1 + r[i];
                    t1 = c * t1 + s[i];
                }
                if ((0, r_func_1.abs)(f1) <= 0.5) {
                    f1 *= f1;
                    d2 = 0;
                    for (i = 0; i < 6; ++i) {
                        d2 = f1 * d2 + t[i];
                    }
                    d2 = f0 + f0 * f1 * d2;
                } else {
                    d2 = (0, r_func_1.sinh)(f1) / nu;
                }
                f0 = d2 - nu * d1 / (t1 * p0);
                if (ex <= 1e-10) {
                    bk[0] = f0 + ex * f0;
                    if (ize === 1) {
                        bk[0] -= ex * bk[0];
                    }
                    ratio = p0 / f0;
                    c = ex * r_func_1.DBL_MAX;
                    if (k !== 0) {
                        ncalc = -1;
                        if (bk[0] >= c / ratio) {
                            return {
                                x: bk[nb - 1],
                                nb,
                                ncalc
                            };
                        }
                        bk[0] = ratio * bk[0] / ex;
                        twonu += 2;
                        ratio = twonu;
                    }
                    ncalc = 1;
                    if (nb === 1)
                        return {
                            x: bk[nb - 1],
                            nb,
                            ncalc
                        };
                    ncalc = -1;
                    for (i = 1; i < nb; ++i) {
                        if (ratio >= c)
                            return {
                                x: bk[nb - 1],
                                nb,
                                ncalc
                            };
                        bk[i] = ratio / ex;
                        twonu += 2;
                        ratio = twonu;
                    }
                    ncalc = 1;
                    break;
                } else {
                    c = 1;
                    x2by4 = ex * ex / 4;
                    p0 = 0.5 * p0;
                    q0 = 0.5 * q0;
                    d1 = -1;
                    d2 = 0;
                    bk1 = 0;
                    bk2 = 0;
                    f1 = f0;
                    f2 = p0;
                    do {
                        d1 += 2;
                        d2 += 1;
                        d3 = d1 + d3;
                        c = x2by4 * c / d2;
                        f0 = (d2 * f0 + p0 + q0) / d3;
                        p0 /= d2 - nu;
                        q0 /= d2 + nu;
                        t1 = c * f0;
                        t2 = c * (p0 - d2 * f0);
                        bk1 += t1;
                        bk2 += t2;
                    } while ((0, r_func_1.abs)(t1 / (f1 + bk1)) > r_func_1.DBL_EPSILON || (0, r_func_1.abs)(t2 / (f2 + bk2)) > r_func_1.DBL_EPSILON);
                    bk1 = f1 + bk1;
                    bk2 = 2 * (f2 + bk2) / ex;
                    if (ize === 2) {
                        d1 = (0, r_func_1.exp)(ex);
                        bk1 *= d1;
                        bk2 *= d1;
                    }
                    wminf = estf[0] * ex + estf[1];
                }
            } else if (r_func_1.DBL_EPSILON * ex > 1) {
                ncalc = nb;
                bk1 = 1 / (M_SQRT_2dPI * (0, r_func_1.sqrt)(ex));
                return {
                    x: bk1,
                    nb,
                    ncalc
                };
            } else {
                twox = ex + ex;
                blpha = 0;
                ratio = 0;
                if (ex <= 4) {
                    d2 = (0, r_func_1.trunc)(estm[0] / ex + estm[1]);
                    m = (0, r_func_1.trunc)(d2);
                    d1 = d2 + d2;
                    d2 -= 0.5;
                    d2 *= d2;
                    for (i = 2; i <= m; ++i) {
                        d1 -= 2;
                        d2 -= d1;
                        ratio = (d3 + d2) / (twox + d1 - ratio);
                    }
                    d2 = (0, r_func_1.trunc)(estm[2] * ex + estm[3]);
                    m = (0, r_func_1.trunc)(d2);
                    c = (0, r_func_1.abs)(nu);
                    d3 = c + c;
                    d1 = d3 - 1;
                    f1 = r_func_1.DBL_MIN;
                    f0 = (2 * (c + d2) / ex + 0.5 * ex / (c + d2 + 1)) * r_func_1.DBL_MIN;
                    for (i = 3; i <= m; ++i) {
                        d2 -= 1;
                        f2 = (d3 + d2 + d2) * f0;
                        blpha = (1 + d1 / d2) * (f2 + blpha);
                        f2 = f2 / ex + f1;
                        f1 = f0;
                        f0 = f2;
                    }
                    f1 = (d3 + 2) * f0 / ex + f1;
                    d1 = 0;
                    t1 = 1;
                    for (i = 1; i <= 7; ++i) {
                        d1 = c * d1 + p[i - 1];
                        t1 = c * t1 + q[i - 1];
                    }
                    p0 = (0, r_func_1.exp)(c * (a + c * (p[7] - c * d1 / t1) - (0, r_func_1.log)(ex))) / ex;
                    f2 = (c + 0.5 - ratio) * f1 / ex;
                    bk1 = p0 + (d3 * f0 - f2 + f0 + blpha) / (f2 + f1 + f0) * p0;
                    if (ize === 1) {
                        bk1 *= (0, r_func_1.exp)(-ex);
                    }
                    wminf = estf[2] * ex + estf[3];
                } else {
                    dm = (0, r_func_1.trunc)(estm[4] / ex + estm[5]);
                    m = (0, r_func_1.trunc)(dm);
                    d2 = dm - 0.5;
                    d2 *= d2;
                    d1 = dm + dm;
                    for (i = 2; i <= m; ++i) {
                        dm -= 1;
                        d1 -= 2;
                        d2 -= d1;
                        ratio = (d3 + d2) / (twox + d1 - ratio);
                        blpha = (ratio + ratio * blpha) / dm;
                    }
                    bk1 = 1 / ((M_SQRT_2dPI + M_SQRT_2dPI * blpha) * (0, r_func_1.sqrt)(ex));
                    if (ize === 1)
                        bk1 *= (0, r_func_1.exp)(-ex);
                    wminf = estf[4] * (ex - (0, r_func_1.abs)(ex - estf[6])) + estf[5];
                }
                bk2 = bk1 + bk1 * (nu + 0.5 - ratio) / ex;
            }
            ncalc = nb;
            bk[0] = bk1;
            if (iend === 0)
                return {
                    x: bk[nb - 1],
                    nb,
                    ncalc
                };
            j = 1 - k;
            if (j >= 0)
                bk[j] = bk2;
            if (iend === 1)
                return {
                    x: bk[nb - 1],
                    nb,
                    ncalc
                };
            m = (0, r_func_1.min)((0, r_func_1.trunc)(wminf - nu), iend);
            for (i = 2; i <= m; ++i) {
                t1 = bk1;
                bk1 = bk2;
                twonu += 2;
                if (ex < 1) {
                    if (bk1 >= r_func_1.DBL_MAX / twonu * ex)
                        break;
                } else {
                    if (bk1 / ex >= r_func_1.DBL_MAX / twonu)
                        break;
                }
                bk2 = twonu / ex * bk1 + t1;
                ii = i;
                ++j;
                if (j >= 0) {
                    bk[j] = bk2;
                }
            }
            m = ii;
            if (m === iend) {
                return {
                    x: bk[nb - 1],
                    nb,
                    ncalc
                };
            }
            ratio = bk2 / bk1;
            mplus1 = m + 1;
            ncalc = -1;
            for (i = mplus1; i <= iend; ++i) {
                twonu += 2;
                ratio = twonu / ex + 1 / ratio;
                ++j;
                if (j >= 1) {
                    bk[j] = ratio;
                } else {
                    if (bk2 >= r_func_1.DBL_MAX / ratio)
                        return {
                            x: bk[nb - 1],
                            nb,
                            ncalc
                        };
                    bk2 *= ratio;
                }
            }
            ncalc = (0, r_func_1.max)(1, mplus1 - k);
            if (ncalc === 1)
                bk[0] = bk2;
            if (nb === 1) {
                return {
                    x: bk2,
                    nb,
                    ncalc
                };
            }
        }
        for (i = ncalc; i < nb; ++i) {
            bk[i] *= bk[i - 1];
            ncalc++;
        }
    }
    return {
        x: bk[nb - 1],
        nb,
        ncalc
    };
}
exports.K_bessel = K_bessel;