'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../../packages/common/logger.cjs');
var r_func_1 = require('../../../r-func.cjs');
var Kbessel_1 = require('./Kbessel.cjs');
const printer = (0, debug_1.debug)('BesselK');
function BesselK(x, nu, exponScaled = false) {
    if ((0, r_func_1.isNaN)(x) || (0, r_func_1.isNaN)(nu))
        return x + nu;
    if (x < 0) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'bessel_k', printer);
        return NaN;
    }
    const ize = exponScaled ? 2 : 1;
    if (nu < 0)
        nu = -nu;
    const nb = 1 + (0, r_func_1.floor)(nu);
    nu -= nb - 1;
    const rc = (0, Kbessel_1.K_bessel)(x, nu, nb, ize);
    if (rc.ncalc !== rc.nb) {
        if (rc.ncalc < 0)
            printer('bessel_k(%d): ncalc (=%d) != nb (=%d); nu=%d. Arg. out of range?\n', rc.x, rc.ncalc, rc.nb, nu);
        else
            printer('bessel_k(%d,nu=%d): precision lost in result\n', rc.x, nu + rc.nb - 1);
    }
    x = rc.x;
    return x;
}
exports.default = BesselK;