'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.Y_bessel = void 0;
var debug_1 = require('@mangos/debug');
var cospi_1 = require('../../../trigonometry/cospi.cjs');
var sinpi_1 = require('../../../trigonometry/sinpi.cjs');
var bessel_constants_1 = require('../bessel-constants.cjs');
const M_SQRT_2dPI = 0.7978845608028654;
const M_1_PI = 0.3183098861837907;
const M_PI_2 = 1.5707963267948966;
const {
    min,
    trunc,
    sqrt,
    sin,
    cos,
    log,
    abs,
    pow,
    PI: M_PI
} = Math;
const {
    MIN_VALUE: DBL_MIN,
    EPSILON: DBL_EPSILON,
    NEGATIVE_INFINITY: ML_NEGINF,
    MAX_VALUE: DBL_MAX
} = Number;
const printer = (0, debug_1.debug)('Y_bessel');
function Y_bessel(x, alpha, nb) {
    const fivpi = 15.707963267948966;
    const pim5 = 0.7079632679489662;
    const ch = [
        -6.773524182239884e-24,
        -6.145518011604988e-23,
        2.9017595056104746e-21,
        1.36394179190731e-19,
        2.3826220476859634e-18,
        -9.064290795755071e-18,
        -1.4943667065169001e-15,
        -3.391907830536221e-14,
        -1.702377664251273e-13,
        9.160975093876865e-12,
        2.42309579004827e-10,
        1.7451364971382985e-9,
        -3.3126119768180853e-8,
        -8.659207996139126e-7,
        -0.0000049717367041957395,
        0.00007630959758590813,
        0.0012719271366545622,
        0.0017063050710955563,
        -0.07685284084478668,
        -0.28387654227602355,
        0.9218702936504527
    ];
    let i;
    let k;
    let na;
    let ncalc;
    let alfa;
    let div;
    let ddiv;
    let even;
    let gamma;
    let term;
    let cosmu;
    let sinmu;
    let b;
    let c;
    let d;
    let e;
    let f;
    let g;
    let h;
    let p;
    let q;
    let r;
    let s;
    let d1;
    let d2;
    let q0;
    let pa;
    let pa1;
    let qa;
    let qa1;
    let en;
    let ya;
    let ya1;
    let twobyx;
    let den;
    let odd;
    let aye;
    let dmu;
    let x2;
    let en1 = ya = ya1 = 0;
    const ex = x;
    let nu = alpha;
    const by = new Float64Array(nb).fill(0);
    if (!(nb > 0 && 0 <= nu && nu < 1)) {
        printer('violaton: abort nb=%d, nu=%d', nb, nu);
        by[0] = 0;
        ncalc = min(nb, 0) - 1;
        return {
            x: 0,
            nb,
            ncalc
        };
    }
    if (ex < DBL_MIN || ex > bessel_constants_1.xlrg_BESS_Y) {
        printer('range issue: x < DBL_MIN but still bigger then 1e8 x=(%d)', ex);
        ncalc = nb;
        if (ex > bessel_constants_1.xlrg_BESS_Y)
            by[0] = 0;
        else if (ex < DBL_MIN)
            by[0] = ML_NEGINF;
        return {
            x: by[0],
            nb,
            ncalc
        };
    }
    const xna = trunc(nu + 0.5);
    na = trunc(xna);
    if (na === 1) {
        nu -= xna;
    }
    if (nu === -0.5) {
        p = M_SQRT_2dPI / sqrt(ex);
        ya = p * sin(ex);
        ya1 = -p * cos(ex);
    } else if (ex < 3) {
        b = ex * 0.5;
        d = -log(b);
        f = nu * d;
        e = pow(b, -nu);
        if (abs(nu) < bessel_constants_1.M_eps_sinc)
            c = M_1_PI;
        else
            c = nu / (0, sinpi_1.sinpi)(nu);
        if (abs(f) < 1) {
            x2 = f * f;
            en = 19;
            s = 1;
            for (i = 1; i <= 9; ++i) {
                s = s * x2 / en / (en - 1) + 1;
                en -= 2;
            }
        } else {
            s = (e - 1 / e) * 0.5 / f;
        }
        x2 = nu * nu * 8;
        aye = ch[0];
        even = 0;
        alfa = ch[1];
        odd = 0;
        for (i = 3; i <= 19; i += 2) {
            even = -(aye + aye + even);
            aye = -even * x2 - aye + ch[i - 1];
            odd = -(alfa + alfa + odd);
            alfa = -odd * x2 - alfa + ch[i];
        }
        even = (even * 0.5 + aye) * x2 - aye + ch[20];
        odd = (odd + alfa) * 2;
        gamma = odd * nu + even;
        g = e * gamma;
        e = (e + 1 / e) * 0.5;
        f = 2 * c * (odd * e + even * s * d);
        e = nu * nu;
        p = g * c;
        q = M_1_PI / g;
        c = nu * M_PI_2;
        if (abs(c) < bessel_constants_1.M_eps_sinc)
            r = 1;
        else
            r = (0, sinpi_1.sinpi)(nu / 2) / c;
        r = M_PI * c * r * r;
        c = 1;
        d = -b * b;
        h = 0;
        ya = f + r * q;
        ya1 = p;
        en = 1;
        while (abs(g / (1 + abs(ya))) + abs(h / (1 + abs(ya1))) > DBL_EPSILON) {
            f = (f * en + p + q) / (en * en - e);
            c *= d / en;
            p /= en - nu;
            q /= en + nu;
            g = c * (f + r * q);
            h = c * p - en * g;
            ya += g;
            ya1 += h;
            en += 1;
        }
        ya = -ya;
        ya1 = -ya1 / b;
    } else if (ex < bessel_constants_1.thresh_BESS_Y) {
        c = (0.5 - nu) * (0.5 + nu);
        b = ex + ex;
        e = ex * M_1_PI * (0, cospi_1.cospi)(nu) / DBL_EPSILON;
        e *= e;
        p = 1;
        q = -ex;
        r = 1 + ex * ex;
        s = r;
        en = 2;
        while (r * en * en < e) {
            en1 = en + 1;
            d = (en - 1 + c / en) / s;
            p = (en + en - p * d) / en1;
            q = (-b + q * d) / en1;
            s = p * p + q * q;
            r *= s;
            en = en1;
        }
        f = p / s;
        p = f;
        g = -q / s;
        q = g;
        for (;;) {
            en -= 1;
            if (en > 0) {
                r = en1 * (2 - p) - 2;
                s = b + en1 * q;
                d = (en - 1 + c / en) / (r * r + s * s);
                p = d * r;
                q = d * s;
                e = f + 1;
                f = p * e - g * q;
                g = q * e + p * g;
                en1 = en;
                continue;
            }
            break;
        }
        f = 1 + f;
        d = f * f + g * g;
        pa = f / d;
        qa = -g / d;
        d = nu + 0.5 - p;
        q += ex;
        pa1 = (pa * q - qa * d) / ex;
        qa1 = (qa * q + pa * d) / ex;
        b = ex - M_PI_2 * (nu + 0.5);
        c = cos(b);
        s = sin(b);
        d = M_SQRT_2dPI / sqrt(ex);
        ya = d * (pa * s + qa * c);
        ya1 = d * (qa1 * s - pa1 * c);
    } else {
        na = 0;
        d1 = trunc(ex / fivpi);
        i = trunc(d1);
        dmu = ex - 15 * d1 - d1 * pim5 - (alpha + 0.5) * M_PI_2;
        if (i - (i / 2 << 1) === 0) {
            cosmu = cos(dmu);
            sinmu = sin(dmu);
        } else {
            cosmu = -cos(dmu);
            sinmu = -sin(dmu);
        }
        ddiv = 8 * ex;
        dmu = alpha;
        den = sqrt(ex);
        for (k = 1; k <= 2; ++k) {
            p = cosmu;
            cosmu = sinmu;
            sinmu = -p;
            d1 = (2 * dmu - 1) * (2 * dmu + 1);
            d2 = 0;
            div = ddiv;
            p = 0;
            q = 0;
            q0 = d1 / div;
            term = q0;
            for (i = 2; i <= 20; ++i) {
                d2 += 8;
                d1 -= d2;
                div += ddiv;
                term = -term * d1 / div;
                p += term;
                d2 += 8;
                d1 -= d2;
                div += ddiv;
                term *= d1 / div;
                q += term;
                if (abs(term) <= DBL_EPSILON) {
                    break;
                }
            }
            p += 1;
            q += q0;
            if (k === 1)
                ya = M_SQRT_2dPI * (p * cosmu - q * sinmu) / den;
            else
                ya1 = M_SQRT_2dPI * (p * cosmu - q * sinmu) / den;
            dmu += 1;
        }
    }
    if (na === 1) {
        h = 2 * (nu + 1) / ex;
        if (h > 1) {
            if (abs(ya1) > DBL_MAX / h) {
                h = 0;
                ya = 0;
            }
        }
        h = h * ya1 - ya;
        ya = ya1;
        ya1 = h;
    }
    by[0] = ya;
    ncalc = 1;
    if (nb > 1) {
        by[1] = ya1;
        if (ya1 !== 0) {
            aye = 1 + alpha;
            twobyx = 2 / ex;
            ncalc = 2;
            for (i = 2; i < nb; ++i) {
                if (twobyx < 1) {
                    if (abs(by[i - 1]) * twobyx >= DBL_MAX / aye)
                        break;
                } else {
                    if (abs(by[i - 1]) >= DBL_MAX / aye / twobyx)
                        break;
                }
                by[i] = twobyx * aye * by[i - 1] - by[i - 2];
                aye += 1;
                ++ncalc;
            }
        }
    }
    return {
        x: by[ncalc - 1],
        nb,
        ncalc
    };
}
exports.Y_bessel = Y_bessel;