'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
var debug_1 = require('@mangos/debug');
var logger_1 = require('../../../../packages/common/logger.cjs');
var cospi_1 = require('../../../trigonometry/cospi.cjs');
var sinpi_1 = require('../../../trigonometry/sinpi.cjs');
var besselJ_1 = require('../besselJ/index.cjs');
var Ybessel_1 = require('./Ybessel.cjs');
var r_func_1 = require('../../../r-func.cjs');
const printer = (0, debug_1.debug)('BesselY');
function BesselY(x, nu) {
    if (isNaN(x) || isNaN(nu))
        return x + nu;
    if (x < 0) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'BesselY', printer);
        return NaN;
    }
    const na = (0, r_func_1.floor)(nu);
    if (nu < 0) {
        return (nu - na === 0.5 ? 0 : BesselY(x, -nu) * (0, cospi_1.cospi)(nu)) - (nu === na ? 0 : (0, besselJ_1.default)(x, -nu) * (0, sinpi_1.sinpi)(nu));
    } else if (nu > 10000000) {
        printer('besselY(x, nu): nu=%d too large for bessel_y() algorithm', nu);
        return NaN;
    }
    const nb = 1 + na;
    nu -= nb - 1;
    const rc = (0, Ybessel_1.Y_bessel)(x, nu, nb);
    if (rc.ncalc !== nb) {
        if (rc.ncalc === -1) {
            return Infinity;
        } else if (rc.ncalc < -1) {
            printer('bessel_y(%d): ncalc (=%d) != nb (=%d); nu=%d. Arg. out of range?\n', rc.x, rc.ncalc, nb, nu);
        } else {
            printer('bessel_y(%d,nu=%d): precision lost in result\n', rc.x, nu + nb - 1);
        }
    }
    return rc.x;
}
exports.default = BesselY;