'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.choose = exports.lchoose = exports.lfastchoose = void 0;
var debug_1 = require('@mangos/debug');
var r_func_1 = require('../../r-func.cjs');
var lbeta_1 = require('../beta/lbeta.cjs');
var lgammafn_sign_1 = require('../gamma/lgammafn_sign.cjs');
function lfastchoose(n, k) {
    return -(0, r_func_1.log)(n + 1) - (0, lbeta_1.default)(n - k + 1, k + 1);
}
exports.lfastchoose = lfastchoose;
function lfastchoose2(n, k, sChoose) {
    const r = (0, lgammafn_sign_1.lgammafn_sign)(n - k + 1, sChoose);
    return (0, lgammafn_sign_1.lgammafn_sign)(n + 1) - (0, lgammafn_sign_1.lgammafn_sign)(k + 1) - r;
}
const printer_lchoose = (0, debug_1.debug)('lchoose');
function lchoose(n, k) {
    const k0 = k;
    k = Math.round(k);
    if (isNaN(n) || isNaN(k))
        return n + k;
    if ((0, r_func_1.abs)(k - k0) > 1e-7)
        printer_lchoose('"k" (%d) must be integer, rounded to %d', k0, k);
    if (k < 2) {
        if (k < 0)
            return -Infinity;
        if (k === 0)
            return 0;
        return (0, r_func_1.log)((0, r_func_1.abs)(n));
    }
    if (n < 0) {
        return lchoose(-n + k - 1, k);
    } else if ((0, r_func_1.isInteger)(n)) {
        n = (0, r_func_1.round)(n);
        if (n < k)
            return -Infinity;
        if (n - k < 2)
            return lchoose(n, n - k);
        return lfastchoose(n, k);
    }
    if (n < k - 1) {
        return lfastchoose2(n, k);
    }
    return lfastchoose(n, k);
}
exports.lchoose = lchoose;
const k_small_max = 30;
const printer_choose = (0, debug_1.debug)('choose');
function choose(n, k) {
    let r;
    const k0 = k;
    k = (0, r_func_1.round)(k);
    if (isNaN(n) || isNaN(k)) {
        return n + k;
    }
    if ((0, r_func_1.abs)(k - k0) > 1e-7) {
        printer_choose('k (%d) must be integer, rounded to %d', k0, k);
    }
    if (k < k_small_max) {
        let j;
        if (n - k < k && n >= 0 && (0, r_func_1.isInteger)(n)) {
            k = n - k;
        }
        if (k < 0) {
            return 0;
        }
        if (k === 0) {
            return 1;
        }
        r = n;
        for (j = 2; j <= k; j++) {
            r *= (n - j + 1) / j;
        }
        return (0, r_func_1.isInteger)(n) ? (0, r_func_1.round)(r) : r;
    }
    if (n < 0) {
        r = choose(-n + k - 1, k);
        if ((0, r_func_1.isOdd)(k)) {
            r = -r;
        }
        return r;
    } else if ((0, r_func_1.isInteger)(n)) {
        n = (0, r_func_1.round)(n);
        if (n < k)
            return 0;
        if (n - k < k_small_max)
            return choose(n, n - k);
        return (0, r_func_1.round)((0, r_func_1.exp)(lfastchoose(n, k)));
    }
    if (n < k - 1) {
        const schoose = new Int32Array(1);
        r = lfastchoose2(n, k, schoose);
        return schoose[0] * (0, r_func_1.exp)(r);
    }
    return (0, r_func_1.exp)(lfastchoose(n, k));
}
exports.choose = choose;