'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.gammafn = void 0;
var chebyshev_1 = require('../../chebyshev/chebyshev.cjs');
var logger_1 = require('../../../packages/common/logger.cjs');
var stirling_1 = require('../../stirling/index.cjs');
var sinpi_1 = require('../../trigonometry/sinpi.cjs');
var r_func_1 = require('../../r-func.cjs');
var debug_1 = require('@mangos/debug');
const printer = (0, debug_1.debug)('gammafn');
const gamcs = [
    +0.00857119559098933,
    +0.004415381324841007,
    +0.05685043681599363,
    -0.00421983539641856,
    +0.0013268081812124603,
    -0.00018930245297988805,
    +0.00003606925327441245,
    -0.000006056761904460864,
    +0.0000010558295463022833,
    -1.811967365542384e-7,
    +3.117724964715322e-8,
    -5.354219639019687e-9,
    +9.193275519859589e-10,
    -1.5779412802883398e-10,
    +2.7079806229349544e-11,
    -4.64681865382573e-12,
    +7.97335019200742e-13,
    -1.368078209830916e-13,
    +2.3473194865638007e-14,
    -4.027432614949067e-15,
    +6.910051747372101e-16,
    -1.185584500221993e-16,
    +2.034148542496374e-17,
    -3.490054341717406e-18,
    +5.987993856485306e-19,
    -1.027378057872228e-19,
    +1.7627028160605298e-20,
    -3.024320653735306e-21,
    +5.188914660218398e-22,
    -8.902770842456576e-23,
    +1.5274740684933426e-23,
    -2.620731256187363e-24,
    +4.496464047830539e-25,
    -7.714712731336878e-26,
    +1.323635453126044e-26,
    -2.2709994129429287e-27,
    +3.8964189980039913e-28,
    -6.685198115125953e-29,
    +1.1469986631400244e-29,
    -1.9679385863451348e-30,
    +3.376448816585338e-31,
    -5.793070335782136e-32
];
const ngam = 22;
const xmin = -170.5674972726612;
const xmax = 171.61447887182297;
const xsml = 2.2474362225598545e-308;
const dxrel = 1.4901161193847656e-8;
function gammafn(x) {
    let i;
    let n;
    let y;
    let sinpiy;
    let value;
    if (isNaN(x))
        return x;
    if (x === 0 || x < 0 && x === (0, r_func_1.round)(x)) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_DOMAIN, 'gammafn', printer);
        return NaN;
    }
    y = (0, r_func_1.abs)(x);
    if (y <= 10) {
        n = x >> 0;
        if (x < 0)
            --n;
        y = x - n;
        --n;
        value = (0, chebyshev_1.chebyshev_eval)(y * 2 - 1, gamcs, ngam) + 0.9375;
        if (n === 0)
            return value;
        if (n >= 0) {
            for (i = 1; i <= n; i++) {
                value *= y + i;
            }
            return value;
        }
        if (x < -0.5 && (0, r_func_1.abs)((x - (0, r_func_1.trunc)(x - 0.5)) / x) < dxrel) {
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'gammafn', printer);
        }
        if (y < xsml) {
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'gammafn', printer);
            return Infinity;
        }
        n = -n;
        for (i = 0; i < n; i++) {
            value /= x + i;
        }
        return value;
    } else {
        if (x > xmax) {
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'gammafn', printer);
            return Infinity;
        }
        if (x < xmin) {
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_UNDERFLOW, 'gammafn', printer);
            return 0;
        }
        if (y <= 50 && y === (0, r_func_1.trunc)(y)) {
            value = 1;
            for (i = 2; i < y; i++)
                value *= i;
        } else {
            value = (0, r_func_1.exp)((y - 0.5) * (0, r_func_1.log)(y) - y + r_func_1.M_LN_SQRT_2PI + (0, stirling_1.stirlerr)(y));
        }
        if (x > 0)
            return value;
        if ((0, r_func_1.abs)((x - (0, r_func_1.trunc)(x - 0.5)) / x) < dxrel) {
            (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'gammafn', printer);
        }
        sinpiy = (0, sinpi_1.sinpi)(y);
        return -r_func_1.PI / (y * sinpiy * value);
    }
}
exports.gammafn = gammafn;