'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.lgammafn_sign = void 0;
var debug_1 = require('@mangos/debug');
var r_func_1 = require('../../r-func.cjs');
var logger_1 = require('../../../packages/common/logger.cjs');
var sinpi_1 = require('../../trigonometry/sinpi.cjs');
var lgammacor_1 = require('./lgammacor.cjs');
var gamma_fn_1 = require('./gamma_fn.cjs');
const printer_sign = (0, debug_1.debug)('lgammafn_sign');
const xmax = 2.5327372760800758e+305;
const dxrel = 1.4901161193847656e-8;
const ML_POSINF = Infinity;
function lgammafn_sign(x, sgn) {
    if (isNaN(x))
        return x;
    if (sgn !== undefined && sgn !== null) {
        sgn[0] = 1;
        if (x < 0 && (0, r_func_1.fmod)((0, r_func_1.floor)(-x), 2) === 0) {
            sgn[0] = -1;
        }
    }
    if (x <= 0 && x === (0, r_func_1.trunc)(x)) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'lgamma', printer_sign);
        return ML_POSINF;
    }
    const y = (0, r_func_1.abs)(x);
    if (y < 1e-306)
        return -(0, r_func_1.log)(y);
    if (y <= 10)
        return (0, r_func_1.log)((0, r_func_1.abs)((0, gamma_fn_1.gammafn)(x)));
    if (y > xmax) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_RANGE, 'lgamma', printer_sign);
        return ML_POSINF;
    }
    if (x > 0) {
        if (x > 100000000000000000)
            return x * ((0, r_func_1.log)(x) - 1);
        else if (x > 4934720)
            return r_func_1.M_LN_SQRT_2PI + (x - 0.5) * (0, r_func_1.log)(x) - x;
        else
            return r_func_1.M_LN_SQRT_2PI + (x - 0.5) * (0, r_func_1.log)(x) - x + (0, lgammacor_1.lgammacor)(x);
    }
    const sinpiy = (0, r_func_1.abs)((0, sinpi_1.sinpi)(y));
    const ans = r_func_1.M_LN_SQRT_PId2 + (x - 0.5) * (0, r_func_1.log)(y) - x - (0, r_func_1.log)(sinpiy) - (0, lgammacor_1.lgammacor)(y);
    if ((0, r_func_1.abs)((x - (0, r_func_1.trunc)(x - 0.5)) * ans / x) < dxrel) {
        (0, logger_1.ML_ERROR2)(logger_1.ME.ME_PRECISION, 'lgamma', printer_sign);
    }
    return ans;
}
exports.lgammafn_sign = lgammafn_sign;