'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.R_Q_P01_check = exports.R_Q_P01_boundaries = exports.ML_ERR_return_NAN2 = exports.ML_ERROR2 = exports.mapErr = exports.ME = exports.lineInfo4 = exports.createLineInfo = void 0;
var debug_1 = require('@mangos/debug');
var debug_2 = require('@mangos/debug');
const debug_R_Q_P01_boundaries = (0, debug_1.debug)('R_Q_P01_boundaries');
const debug_R_Q_P01_check = (0, debug_1.debug)('R_Q_P01_check');
function createLineInfo(n) {
    return function () {
        const info = (0, debug_2.getLineInfo)(n);
        return `${ info.fnName }, line:${ info.line }, col:${ info.column }`;
    };
}
exports.createLineInfo = createLineInfo;
const lineInfo4 = createLineInfo(4);
exports.lineInfo4 = lineInfo4;
var ME;
(function (ME) {
    ME[ME['ME_NONE'] = 0] = 'ME_NONE';
    ME[ME['ME_DOMAIN'] = 1] = 'ME_DOMAIN';
    ME[ME['ME_RANGE'] = 2] = 'ME_RANGE';
    ME[ME['ME_NOCONV'] = 4] = 'ME_NOCONV';
    ME[ME['ME_PRECISION'] = 8] = 'ME_PRECISION';
    ME[ME['ME_UNDERFLOW'] = 16] = 'ME_UNDERFLOW';
}(ME = exports.ME || (exports.ME = {})));
exports.mapErr = new Map([
    [
        ME.ME_NONE,
        'No error'
    ],
    [
        ME.ME_DOMAIN,
        'argument out of domain in \'%s\''
    ],
    [
        ME.ME_RANGE,
        'argument out of range in \'%s\''
    ],
    [
        ME.ME_NOCONV,
        'convergence failed in \'%s\''
    ],
    [
        ME.ME_PRECISION,
        'full precision may not have been achieved in \'%s\''
    ],
    [
        ME.ME_UNDERFLOW,
        'underflow occurred in \'%s\''
    ]
]);
function ML_ERROR2(x, s, printer) {
    if (!printer.enabled) {
        return;
    }
    const str = exports.mapErr.get(x);
    const val = typeof s === 'function' ? s() : s;
    if (str) {
        printer(str, val);
    }
}
exports.ML_ERROR2 = ML_ERROR2;
function ML_ERR_return_NAN2(printer, getExtraInfo) {
    if (printer.enabled) {
        ML_ERROR2(ME.ME_DOMAIN, getExtraInfo(), printer);
    }
    return NaN;
}
exports.ML_ERR_return_NAN2 = ML_ERR_return_NAN2;
function R_Q_P01_boundaries(lower_tail, log_p, p, _LEFT_, _RIGHT_) {
    if (log_p) {
        if (p > 0) {
            return ML_ERR_return_NAN2(debug_R_Q_P01_boundaries, lineInfo4);
        }
        if (p === 0)
            return lower_tail ? _RIGHT_ : _LEFT_;
        if (p === -Infinity)
            return lower_tail ? _LEFT_ : _RIGHT_;
    } else {
        if (p < 0 || p > 1) {
            return ML_ERR_return_NAN2(debug_R_Q_P01_boundaries, lineInfo4);
        }
        if (p === 0)
            return lower_tail ? _LEFT_ : _RIGHT_;
        if (p === 1)
            return lower_tail ? _RIGHT_ : _LEFT_;
    }
    return undefined;
}
exports.R_Q_P01_boundaries = R_Q_P01_boundaries;
function R_Q_P01_check(logP, p) {
    if (logP && p > 0 || !logP && (p < 0 || p > 1)) {
        return ML_ERR_return_NAN2(debug_R_Q_P01_check, lineInfo4);
    }
    return undefined;
}
exports.R_Q_P01_check = R_Q_P01_check;