import { debug } from '@mangos/debug';
import { NumberW } from './NumberW.mjs';
import {
    trunc,
    log,
    abs,
    min,
    pow
} from '../../r-func.mjs';
const printer_bratio = debug('Toms708.bratio');
const aVec = new Float64Array([
    0.5,
    0.5,
    0.5,
    1,
    1,
    1,
    1,
    1,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    5.5,
    10,
    10,
    10,
    10,
    20,
    20,
    20,
    20,
    20,
    30,
    30,
    40,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    2,
    3,
    4,
    5
]);
const bVec = new Float64Array([
    0.5,
    0.5,
    0.5,
    0.5,
    0.5,
    0.5,
    0.5,
    1,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    5,
    0.5,
    5,
    5,
    10,
    5,
    10,
    10,
    20,
    20,
    10,
    10,
    20,
    0.5,
    0.5,
    0.5,
    0.5,
    2,
    3,
    4,
    5,
    2,
    2,
    2,
    2
]);
const fxVec = new Float64Array([
    0.06376856085851985,
    0.2048327646991335,
    1,
    0,
    0.005012562893380045,
    0.0513167019494862,
    0.2928932188134525,
    0.5,
    0.028,
    0.104,
    0.216,
    0.352,
    0.5,
    0.648,
    0.784,
    0.896,
    0.972,
    0.4361908850559777,
    0.1516409096347099,
    0.08978271484375,
    1,
    0.5,
    0.4598773297575791,
    0.2146816102371739,
    0.9507364826957875,
    0.5,
    0.8979413687105918,
    0.2241297491808366,
    0.7586405487192086,
    0.7001783247477069,
    0.0513167019494862,
    0.1055728090000841,
    0.1633399734659245,
    0.2254033307585166,
    0.36,
    0.488,
    0.5904,
    0.67232,
    0.216,
    0.0837,
    0.03078,
    0.010935
]);
const xVec = new Float64Array([
    0.01,
    0.1,
    1,
    0,
    0.01,
    0.1,
    0.5,
    0.5,
    0.1,
    0.2,
    0.3,
    0.4,
    0.5,
    0.6,
    0.7,
    0.8,
    0.9,
    0.5,
    0.9,
    0.5,
    1,
    0.5,
    0.8,
    0.6,
    0.8,
    0.5,
    0.6,
    0.7,
    0.8,
    0.7,
    0.1,
    0.2,
    0.3,
    0.4,
    0.2,
    0.2,
    0.2,
    0.2,
    0.3,
    0.3,
    0.3,
    0.3
]);
export class Toms708 {
    static alnrel(a) {
        const p3 = -0.0178874546012214;
        const p2 = 0.405303492862024;
        const p1 = -1.29418923021993;
        const q3 = -0.0845104217945565;
        const q2 = 0.747811014037616;
        const q1 = -1.62752256355323;
        let t = 0;
        let w = 0;
        let x = 0;
        let t2 = 0;
        if (Math.abs(a) > 0.375) {
            x = 1 + a;
            return log(x);
        }
        t = a / (a + 2);
        t2 = t * t;
        w = (((p3 * t2 + p2) * t2 + p1) * t2 + 1) / (((q3 * t2 + q2) * t2 + q1) * t2 + 1);
        return 2 * t * w;
    }
    static algdiv(a, b) {
        const c5 = -0.00165322962780713;
        const c4 = 0.000837308034031215;
        const c3 = -0.00059520293135187;
        const c2 = 0.00079365066682539;
        const c1 = -0.00277777777760991;
        const c0 = 0.0833333333333333;
        let s11 = 0;
        let c = 0;
        let d = 0;
        let h = 0;
        let t = 0;
        let u = 0;
        let v = 0;
        let w = 0;
        let x = 0;
        let s3 = 0;
        let s5 = 0;
        let s7 = 0;
        let x2 = 0;
        let s9 = 0;
        if (a <= b) {
            h = a / b;
            c = h / (1 + h);
            x = 1 / (1 + h);
            d = b + (a - 0.5);
        } else {
            h = b / a;
            c = 1 / (1 + h);
            x = h / (1 + h);
            d = a + (b - 0.5);
        }
        x2 = x * x;
        s3 = 1 + x + x2;
        s5 = 1 + x + x2 * s3;
        s7 = 1 + x + x2 * s5;
        s9 = 1 + x + x2 * s7;
        s11 = 1 + x + x2 * s9;
        t = Math.pow(1 / b, 2);
        w = ((((c5 * s11 * t + c4 * s9) * t + c3 * s7) * t + c2 * s5) * t + c1 * s3) * t + c0;
        w *= c / b;
        u = d * Toms708.alnrel(a / b);
        v = a * (log(b) - 1);
        if (u <= v)
            return w - u - v;
        return w - v - u;
    }
    static apser(a, b, x, eps) {
        const g = 0.577215664901533;
        let j = 0;
        let bx = 0;
        let c = 0;
        let s = 0;
        let t = 0;
        let tol = 0;
        let aj = 0;
        bx = b * x;
        t = x - bx;
        if (b * eps > 0.02) {
            c = log(bx) + g + t;
        } else {
            c = log(x) + Toms708.psi(b) + g + t;
        }
        tol = 5 * eps * Math.abs(c);
        j = 1;
        s = 0;
        do {
            j++;
            t *= x - bx / j;
            aj = t / j;
            s += aj;
        } while (Math.abs(aj) > tol);
        return -(a * (c + s));
    }
    static basym(a, b, lambda, eps) {
        const a0 = new Array(21).fill(0);
        const b0 = new Array(21).fill(0);
        const c = new Array(21).fill(0);
        const d = new Array(21).fill(0);
        const num = 20;
        const e0 = 1.12837916709551;
        const e1 = 0.353553390593274;
        let h;
        let r0;
        let r1;
        let w0;
        if (a <= b) {
            h = a / b;
            r0 = 1 / (1 + h);
            r1 = (b - a) / b;
            w0 = 1 / Math.sqrt(a * (1 + h));
        } else {
            h = b / a;
            r0 = 1 / (1 + h);
            r1 = (b - a) / a;
            w0 = 1 / Math.sqrt(b * (1 + h));
        }
        const f = a * Toms708.rlog1(-lambda / a) + b * Toms708.rlog1(lambda / b);
        const t = Math.exp(-f);
        if (t === 0)
            return 0;
        const z0 = Math.sqrt(f);
        const z = 0.5 * (z0 / e1);
        const z2 = f + f;
        a0[0] = 2 / 3 * r1;
        c[0] = -0.5 * a0[0];
        d[0] = -c[0];
        let j0 = 0.5 / e0 * Toms708.erfc1(1, z0);
        let j1 = e1;
        let sum = j0 + d[0] * w0 * j1;
        let s = 1;
        const h2 = h * h;
        let hn = 1;
        let w = w0;
        let znm1 = z;
        let zn = z2;
        for (let n = 2; n <= num; n += 2) {
            hn = h2 * hn;
            a0[n] = 2 * r0 * (1 + h * hn) / (n + 2);
            const np1 = n + 1;
            s = s + hn;
            a0[np1] = 2 * r1 * s / (n + 3);
            for (let i = n; i <= np1; i++) {
                const r = -0.5 * (i + 1);
                b0[1] = r * a0[1];
                for (let m = 2; m <= i; m++) {
                    let bsum = 0;
                    const mm1 = m - 1;
                    for (let j = 1; j <= mm1; j++) {
                        const mmj = m - j;
                        bsum = bsum + (j * r - mmj) * a0[j] * b0[mmj];
                    }
                    b0[m] = r * a0[m] + bsum / m;
                }
                c[i] = b0[i] / (i + 1);
                let dsum = 0;
                const im1 = i - 1;
                for (let j = 1; j <= im1; j++) {
                    const imj = i - j;
                    dsum = dsum + d[imj] * c[j];
                }
                d[i] = -(dsum + c[i]);
            }
            j0 = e1 * znm1 + (n - 1) * j0;
            j1 = e1 * zn + n * j1;
            znm1 = z2 * znm1;
            zn = z2 * zn;
            w = w0 * w;
            const t0 = d[n] * w * j0;
            w = w0 * w;
            const t1 = d[np1] * w * j1;
            sum = sum + (t0 + t1);
            if (Math.abs(t0) + Math.abs(t1) <= eps * sum)
                break;
        }
        const u = Math.exp(-Toms708.bcorr(a, b));
        return e0 * t * u * sum;
    }
    static bcorr(a0, b0) {
        const c5 = -0.00165322962780713;
        const c4 = 0.000837308034031215;
        const c3 = -0.00059520293135187;
        const c2 = 0.00079365066682539;
        const c1 = -0.00277777777760991;
        const c0 = 0.0833333333333333;
        let s11 = 0;
        let a = 0;
        let b = 0;
        let c = 0;
        let h = 0;
        let t = 0;
        let w = 0;
        let x = 0;
        let s3 = 0;
        let s5 = 0;
        let s7 = 0;
        let x2 = 0;
        let s9 = 0;
        a = Math.min(a0, b0);
        b = Math.max(a0, b0);
        h = a / b;
        c = h / (1 + h);
        x = 1 / (1 + h);
        x2 = x * x;
        s3 = 1 + (x + x2);
        s5 = 1 + (x + x2 * s3);
        s7 = 1 + (x + x2 * s5);
        s9 = 1 + (x + x2 * s7);
        s11 = 1 + (x + x2 * s9);
        t = Math.pow(1 / b, 2);
        w = ((((c5 * s11 * t + c4 * s9) * t + c3 * s7) * t + c2 * s5) * t + c1 * s3) * t + c0;
        w = w * (c / b);
        t = Math.pow(1 / a, 2);
        return (((((c5 * t + c4) * t + c3) * t + c2) * t + c1) * t + c0) / a + w;
    }
    static beta_cdf_values(nData, a, b, x, fx) {
        const nMax = 42;
        if (nData.val < 0)
            nData.val = 0;
        nData.val++;
        if (nData.val > nMax) {
            nData.val = 0;
            a.val = b.val = x.val = fx.val = 0;
        } else {
            a.val = aVec[nData.val - 1];
            b.val = bVec[nData.val - 1];
            x.val = xVec[nData.val - 1];
            fx.val = fxVec[nData.val - 1];
        }
    }
    static betaln(a0, b0) {
        const e = 0.918938533204673;
        let a = 0;
        let b = 0;
        let c = 0;
        let h = 0;
        let i = 0;
        let n = 0;
        let u = 0;
        let v = 0;
        let w = 0;
        let z = 0;
        a = Math.min(a0, b0);
        b = Math.max(a0, b0);
        if (a < 1) {
            if (b < 8)
                return Toms708.gamln(a) + Toms708.algdiv(a, b);
            else
                return Toms708.gamln(a) + Toms708.gamln(b) - Toms708.gamln(a + b);
        } else if (a >= 1 && a < 8) {
            if (a < 2 && b < 2) {
                return Toms708.gamln(a) + Toms708.gamln(b) - Toms708.gsumln(a, b);
            } else if (b > 2) {
                w = 0;
                if (b < 8) {
                    n = trunc(b - 1);
                    z = 1;
                    for (i = 1; i <= n; i++) {
                        b--;
                        z *= b / (a + b);
                    }
                    return w + log(z) + (Toms708.gamln(a) + Toms708.gamln(b) - Toms708.gsumln(a, b));
                } else
                    return Toms708.gamln(a) + Toms708.algdiv(a, b);
            } else if (a > 2) {
                if (b > 1000) {
                    n = trunc(a - 1);
                    w = 1;
                    for (i = 1; i <= n; i++) {
                        a--;
                        w = w * (a / (1 + a / b));
                    }
                    return log(w) - n * log(b) + (Toms708.gamln(a) + Toms708.algdiv(a, b));
                }
                n = trunc(a - 1);
                w = 1;
                for (i = 1; i <= n; i++) {
                    a--;
                    h = a / b;
                    w *= h / (1 + h);
                }
                w = log(w);
                if (b < 8) {
                    n = trunc(b - 1);
                    z = 1;
                    for (i = 1; i <= n; i++) {
                        b--;
                        z *= b / (a + b);
                    }
                    return w + log(z) + (Toms708.gamln(a) + Toms708.gamln(b) - Toms708.gsumln(a, b));
                } else
                    return w + Toms708.gamln(a) + Toms708.algdiv(a, b);
            }
        }
        w = Toms708.bcorr(a, b);
        h = a / b;
        c = h / (1 + h);
        u = -((a - 0.5) * log(c));
        v = b * Toms708.alnrel(h);
        if (u > v)
            return -0.5 * log(b) + e + w - u - v;
        else
            return -0.5 * log(b) + e + w - v - u;
    }
    static bfrac(a, b, x, y, lambda, eps) {
        let n = 0;
        let yp1 = 0;
        let alpha = 0;
        let c = 0;
        let e = 0;
        let p = 0;
        let r = 0;
        let s = 0;
        let t = 0;
        let w = 0;
        let c0 = 0;
        let c1 = 0;
        let anp1 = 0;
        let bnp1 = 0;
        let r0 = 0;
        let beta = 0;
        let an = 0;
        let bn = 0;
        let dResult = Toms708.brcomp(a, b, x, y);
        if (dResult === 0)
            return 0;
        c = 1 + lambda;
        c0 = b / a;
        c1 = 1 + 1 / a;
        yp1 = y + 1;
        n = 0;
        p = 1;
        s = a + 1;
        an = 0;
        bn = 1;
        anp1 = 1;
        bnp1 = c / c1;
        r = c1 / c;
        for (;;) {
            n++;
            t = n / a;
            w = n * (b - n) * x;
            e = a / s;
            alpha = p * (p + c0) * e * e * (w * x);
            e = (1 + t) / (c1 + t + t);
            beta = n + w / s + e * (c + n * yp1);
            p = 1 + t;
            s = s + 2;
            t = alpha * an + beta * anp1;
            an = anp1;
            anp1 = t;
            t = alpha * bn + beta * bnp1;
            bn = bnp1;
            bnp1 = t;
            r0 = r;
            r = anp1 / bnp1;
            if (Math.abs(r - r0) <= eps * r)
                break;
            an = an / bnp1;
            bn = bn / bnp1;
            anp1 = r;
            bnp1 = 1;
        }
        dResult *= r;
        return dResult;
    }
    static bgrat(a, b, x, y, w, eps, ierr) {
        let j = 0;
        let l = 0;
        let lnx = 0;
        let nu = 0;
        let n2 = 0;
        const c = new Array(30);
        const d = new Array(30);
        let bm1 = 0;
        let nm1 = 0;
        let i = 0;
        let n = 0;
        let q = 0;
        let r = 0;
        let s = 0;
        let t = 0;
        let u = 0;
        let v = 0;
        let z = 0;
        let sum = 0;
        let bp2n = 0;
        let t2 = 0;
        let coef = 0;
        let dj = 0;
        let cn = 0;
        bm1 = b - 0.5 - 0.5;
        nu = a + 0.5 * bm1;
        if (y <= 0.375) {
            lnx = Toms708.alnrel(-y);
        } else {
            lnx = log(x);
        }
        z = -(nu * lnx);
        if (b * z !== 0) {
            r = b * (1 + Toms708.gam1(b)) * Math.exp(b * log(z));
            r *= Math.exp(a * lnx) * Math.exp(0.5 * bm1 * lnx);
            u = Toms708.algdiv(b, a) + b * log(nu);
            u = r * Math.exp(-u);
            if (u === 0) {
                ierr.val = 1;
                return;
            }
            const p1 = new NumberW(0);
            const q1 = new NumberW(0);
            Toms708.grat1(b, z, r, p1, q1, eps);
            q = q1.val;
            v = 0.25 * Math.pow(1 / nu, 2);
            t2 = 0.25 * lnx * lnx;
            l = w.val / u;
            j = q / r;
            sum = j;
            t = 1;
            cn = 1;
            n2 = 0;
            for (n = 1; n <= 30; n++) {
                bp2n = b + n2;
                j = (bp2n * (bp2n + 1) * j + (z + bp2n + 1) * t) * v;
                n2 = n2 + 2;
                t *= t2;
                cn /= n2 * (n2 + 1);
                c[n - 1] = cn;
                s = 0;
                if (n !== 1) {
                    nm1 = n - 1;
                    coef = b - n;
                    for (i = 1; i <= nm1; i++) {
                        s += coef * c[i - 1] * d[n - i - 1];
                        coef += b;
                    }
                }
                d[n - 1] = bm1 * cn + s / n;
                dj = d[n - 1] * j;
                sum += dj;
                if (sum <= 0) {
                    ierr.val = 1;
                    return;
                }
                if (Math.abs(dj) <= eps * (sum + l)) {
                    break;
                }
            }
            ierr.val = 0;
            w.val += u * sum;
            return;
        } else {
            ierr.val = 1;
            return;
        }
    }
    static brcomp(a, b, x, y) {
        const Const = 0.398942280401433;
        if (x === 0 || y === 0)
            return 0;
        let b0;
        const a0 = Math.min(a, b);
        let lnx;
        let lny;
        let c;
        if (a0 < 8) {
            if (x <= 0.375) {
                lnx = log(x);
                lny = Toms708.alnrel(-x);
            } else {
                if (y <= 0.375) {
                    lnx = Toms708.alnrel(-y);
                    lny = log(y);
                } else {
                    lnx = log(x);
                    lny = log(y);
                }
                let z = a * lnx + b * lny;
                if (a0 >= 1) {
                    z -= Toms708.betaln(a, b);
                    return Math.exp(z);
                }
                b0 = Math.max(a, b);
                if (b0 < 8) {
                    if (b0 <= 1) {
                        let dResult = Math.exp(z);
                        if (dResult === 0)
                            return 0;
                        const apb = a + b;
                        if (apb <= 1) {
                            z = 1 + Toms708.gam1(apb);
                        } else {
                            const u = a + b - 1;
                            z = (1 + Toms708.gam1(u)) / apb;
                        }
                        c = (1 + Toms708.gam1(a)) * (1 + Toms708.gam1(b)) / z;
                        dResult *= a0 * c / (1 + a0 / b0);
                        return dResult;
                    }
                } else {
                    let u = Toms708.gamln1(a0);
                    const n = b0 - 1;
                    if (n >= 1) {
                        c = 1;
                        for (let i = 1; i <= n; i++) {
                            b0--;
                            c *= b0 / (a0 + b0);
                        }
                        u = log(c) + u;
                    }
                    z -= u;
                    b0--;
                    const apb = a0 + b0;
                    let t;
                    if (apb <= 1) {
                        t = 1 + Toms708.gam1(apb);
                    } else {
                        u = a0 + b0 - 1;
                        t = (1 + Toms708.gam1(u)) / apb;
                    }
                    return a0 * Math.exp(z) * (1 + Toms708.gam1(b0)) / t;
                }
                const u = Toms708.gamln1(a0) + Toms708.algdiv(a0, b0);
                return a0 * Math.exp(z - u);
            }
        }
        let x0;
        let y0;
        let lambda;
        let e;
        let h;
        let u;
        let v;
        if (a <= b) {
            h = a / b;
            x0 = h / (1 + h);
            y0 = 1 / (1 + h);
            lambda = a - (a + b) * x;
        } else {
            h = b / a;
            x0 = 1 / (1 + h);
            y0 = h / (1 + h);
            lambda = (a + b) * y - b;
        }
        e = -lambda / a;
        if (Math.abs(e) <= 0.6) {
            u = Toms708.rlog1(e);
        } else {
            u = e - log(x / x0);
        }
        e = lambda / b;
        if (Math.abs(e) > 0.6) {
            v = Toms708.rlog1(e);
        } else {
            v = e - log(y / y0);
        }
        const z = Math.exp(-(a * u + b * v));
        return Const * Math.sqrt(b * x0) * z * Math.exp(-Toms708.bcorr(a, b));
    }
    static brcmp1(mu, a, b, x, y) {
        const invSqrtPi = 0.398942280401433;
        let c;
        let u;
        let apb;
        const a0 = Math.min(a, b);
        if (a0 < 8) {
            let lnx;
            let lny;
            if (x <= 0.375) {
                lnx = log(x);
                lny = Toms708.alnrel(-x);
            } else {
                if (y <= 0.375) {
                    lnx = Toms708.alnrel(-y);
                    lny = log(y);
                } else {
                    lnx = log(x);
                    lny = log(y);
                }
            }
            let z = a * lnx + b * lny;
            if (a0 >= 1) {
                z -= Toms708.betaln(a, b);
                return Toms708.esum(mu, z);
            }
            let b0 = Math.max(a, b);
            if (b0 >= 8) {
                u = Toms708.gamln1(a0) + Toms708.algdiv(a0, b0);
                return a0 * Toms708.esum(mu, z - u);
            }
            if (b0 <= 1) {
                let dResult = Toms708.esum(mu, z);
                if (dResult === 0)
                    return 0;
                apb = a + b;
                if (apb <= 1) {
                    z = 1 + Toms708.gam1(apb);
                } else {
                    u = a + b - 1;
                    z = (1 + Toms708.gam1(u)) / apb;
                }
                c = (1 + Toms708.gam1(a)) * (1 + Toms708.gam1(b)) / z;
                dResult *= a0 * c / (1 + a0 / b0);
                return dResult;
            }
            u = Toms708.gamln1(a0);
            const n = trunc(b0 - 1);
            if (n >= 1) {
                c = 1;
                for (let i = 1; i <= n; i++) {
                    b0 -= 1;
                    c = c * (b0 / (a0 + b0));
                }
                u = log(c) + u;
            }
            z -= u;
            b0 -= 1;
            apb = a0 + b0;
            let t;
            if (apb <= 1) {
                t = 1 + Toms708.gam1(apb);
            } else {
                u = a0 + b0 - 1;
                t = (1 + Toms708.gam1(u)) / apb;
            }
            return a0 * Toms708.esum(mu, z) * (1 + Toms708.gam1(b0)) / t;
        }
        let h;
        let x0;
        let y0;
        let lambda;
        if (a <= b) {
            h = a / b;
            x0 = h / (1 + h);
            y0 = 1 / (1 + h);
            lambda = a - (a + b) * x;
        } else {
            h = b / a;
            x0 = 1 / (1 + h);
            y0 = h / (1 + h);
            lambda = (a + b) * y - b;
        }
        let e = -lambda / a;
        if (Math.abs(e) <= 0.6) {
            u = Toms708.rlog1(e);
        } else {
            u = e - log(x / x0);
        }
        e = lambda / b;
        let v;
        if (Math.abs(e) <= 0.6) {
            v = Toms708.rlog1(e);
        } else {
            v = e - log(y / y0);
        }
        const z = Toms708.esum(mu, -(a * u + b * v));
        return invSqrtPi * Math.sqrt(b * x0) * z * Math.exp(-Toms708.bcorr(a, b));
    }
    static bpser(a, b, x, eps) {
        let n = 0;
        let apb = 0;
        let c = 0;
        let i = 0;
        let m = 0;
        let t = 0;
        let u = 0;
        let w = 0;
        let z = 0;
        let tol = 0;
        let sum = 0;
        let a0 = 0;
        let b0 = 0;
        let dResult = 0;
        if (x === 0) {
            return 0;
        }
        a0 = Math.min(a, b);
        if (a0 >= 1) {
            z = a * log(x) - Toms708.betaln(a, b);
            dResult = Math.exp(z) / a;
        } else {
            b0 = Math.max(a, b);
            if (b0 < 8) {
                if (b0 <= 1) {
                    dResult = Math.pow(x, a);
                    if (dResult === 0) {
                        return 0;
                    }
                    apb = a + b;
                    if (apb <= 1) {
                        z = 1 + Toms708.gam1(apb);
                    } else {
                        u = a + b - 1;
                        z = (1 + Toms708.gam1(u)) / apb;
                    }
                    c = (1 + Toms708.gam1(a)) * (1 + Toms708.gam1(b)) / z;
                    dResult *= c * (b / apb);
                } else {
                    u = Toms708.gamln1(a0);
                    m = trunc(b0 - 1);
                    if (m >= 1) {
                        c = 1;
                        for (i = 1; i <= m; i++) {
                            b0--;
                            c *= b0 / (a0 + b0);
                        }
                        u = log(c) + u;
                    }
                    z = a * log(x) - u;
                    b0--;
                    apb = a0 + b0;
                    if (apb <= 1) {
                        t = 1 + Toms708.gam1(apb);
                    } else {
                        u = a0 + b0 - 1;
                        t = (1 + Toms708.gam1(u)) / apb;
                    }
                    dResult = Math.exp(z) * (a0 / a) * (1 + Toms708.gam1(b0)) / t;
                }
            } else {
                u = Toms708.gamln1(a0) + Toms708.algdiv(a0, b0);
                z = a * log(x) - u;
                dResult = a0 / a * Math.exp(z);
            }
        }
        if (dResult === 0 || a <= 0.1 * eps) {
            return dResult;
        }
        sum = 0;
        n = 0;
        c = 1;
        tol = eps / a;
        do {
            n++;
            c *= (0.5 + (0.5 - b / n)) * x;
            w = c / (a + n);
            sum += w;
        } while (Math.abs(w) > tol);
        dResult *= 1 + a * sum;
        return dResult;
    }
    static bratio(a, b, x, y, w, w1, ierr) {
        let lambda = 0;
        let ind = 0;
        let n = 0;
        let t = 0;
        let eps = 0;
        let z = 0;
        let a0 = 0;
        let b0 = 0;
        let x0 = 0;
        let y0 = 0;
        eps = Toms708.spmpar(1);
        printer_bratio('eps is: %d', eps);
        w.val = 0;
        w1.val = 0;
        if (a < 0 || b < 0) {
            ierr.val = 1;
            return;
        }
        if (a === 0 && b === 0) {
            ierr.val = 2;
            return;
        }
        if (x < 0 || x > 1) {
            ierr.val = 3;
            return;
        }
        if (y < 0 || y > 1) {
            ierr.val = 4;
            return;
        }
        z = x + y - 0.5 - 0.5;
        if (abs(z) > 3 * eps) {
            ierr.val = 5;
            return;
        }
        ierr.val = 0;
        if (x === 0) {
            if (a === 0) {
                ierr.val = 6;
                return;
            } else {
                w.val = 0;
                w1.val = 1;
                return;
            }
        }
        if (y === 0) {
            if (b === 0) {
                ierr.val = 7;
                return;
            } else {
                w.val = 1;
                w1.val = 0;
                return;
            }
        }
        if (a === 0) {
            w.val = 1;
            w1.val = 0;
            return;
        }
        if (b === 0) {
            w.val = 0;
            w1.val = 1;
            return;
        }
        eps = Math.max(eps, 1e-15);
        if (Math.max(a, b) < 0.001 * eps) {
            w.val = b / (a + b);
            w1.val = a / (a + b);
            return;
        }
        ind = 0;
        a0 = a;
        b0 = b;
        x0 = x;
        y0 = y;
        if (min(a0, b0) <= 1) {
            if (x > 0.5) {
                ind = 1;
                a0 = b;
                b0 = a;
                x0 = y;
                y0 = x;
            }
            if (b0 < min(eps, eps * a0)) {
                w.val = Toms708.fpser(a0, b0, x0, eps);
                w1.val = 0.5 + (0.5 - w.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            if (a0 < min(eps, eps * b0) && b0 * x0 <= 1) {
                w1.val = Toms708.apser(a0, b0, x0, eps);
                w.val = 0.5 + (0.5 - w1.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            if (Math.max(a0, b0) <= 1) {
                if (a0 >= Math.min(0.2, b0) || Math.pow(x0, a0) <= 0.9) {
                    w.val = Toms708.bpser(a0, b0, x0, eps);
                    w1.val = 0.5 + (0.5 - w.val);
                    if (ind === 0) {
                        return;
                    } else {
                        t = w.val;
                        w.val = w1.val;
                        w1.val = t;
                        return;
                    }
                }
                if (x0 >= 0.3) {
                    w1.val = Toms708.bpser(b0, a0, y0, eps);
                    w.val = 0.5 + (0.5 - w1.val);
                    if (ind === 0) {
                        return;
                    } else {
                        t = w.val;
                        w.val = w1.val;
                        w1.val = t;
                        return;
                    }
                }
                n = 20;
                w1.val = Toms708.bup(b0, a0, y0, x0, n, eps);
                b0 += n;
                const ierr1 = new NumberW(0);
                Toms708.bgrat(b0, a0, y0, x0, w1, 15 * eps, ierr1);
                w.val = 0.5 + (0.5 - w1.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            if (b0 <= 1) {
                w.val = Toms708.bpser(a0, b0, x0, eps);
                w1.val = 0.5 + (0.5 - w.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            if (x0 >= 0.3) {
                w1.val = Toms708.bpser(b0, a0, y0, eps);
                w.val = 0.5 + (0.5 - w1.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            if (x0 < 0.1) {
                if (pow(x0 * b0, a0) <= 0.7) {
                    w.val = Toms708.bpser(a0, b0, x0, eps);
                    w1.val = 0.5 + (0.5 - w.val);
                    if (ind === 0) {
                        return;
                    } else {
                        t = w.val;
                        w.val = w1.val;
                        w1.val = t;
                        return;
                    }
                }
            }
            if (b0 > 15) {
                const ierr1 = new NumberW(0);
                Toms708.bgrat(b0, a0, y0, x0, w1, 15 * eps, ierr1);
                w.val = 0.5 + (0.5 - w1.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            n = 20;
            w1.val = Toms708.bup(b0, a0, y0, x0, n, eps);
            b0 += n;
            const ierr1 = new NumberW(0);
            Toms708.bgrat(b0, a0, y0, x0, w1, 15 * eps, ierr1);
            w.val = 0.5 + (0.5 - w1.val);
            if (ind === 0) {
                return;
            } else {
                t = w.val;
                w.val = w1.val;
                w1.val = t;
                return;
            }
        }
        if (a <= b) {
            lambda = a - (a + b) * x;
        } else {
            lambda = (a + b) * y - b;
        }
        if (lambda < 0) {
            ind = 1;
            a0 = b;
            b0 = a;
            x0 = y;
            y0 = x;
            lambda = abs(lambda);
        }
        if (b0 < 40 && b0 * x0 <= 0.7) {
            w.val = Toms708.bpser(a0, b0, x0, eps);
            w1.val = 0.5 + (0.5 - w.val);
            if (ind === 0) {
                return;
            } else {
                t = w.val;
                w.val = w1.val;
                w1.val = t;
                return;
            }
        }
        if (b0 < 40) {
            n = Math.trunc(b0);
            b0 -= n;
            if (b0 === 0) {
                n--;
                b0 = 1;
            }
            w.val = Toms708.bup(b0, a0, y0, x0, n, eps);
            if (x0 <= 0.7) {
                w.val += Toms708.bpser(a0, b0, x0, eps);
                w1.val = 0.5 + (0.5 - w.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            if (a0 <= 15) {
                n = 20;
                w.val += Toms708.bup(a0, b0, x0, y0, n, eps);
                a0 += n;
            }
            const ierr1 = new NumberW(0);
            Toms708.bgrat(a0, b0, x0, y0, w, 15 * eps, ierr1);
            w1.val = 0.5 + (0.5 - w.val);
            if (ind === 0) {
                return;
            } else {
                t = w.val;
                w.val = w1.val;
                w1.val = t;
                return;
            }
        }
        if (a0 <= b0) {
            if (a0 <= 100) {
                w.val = Toms708.bfrac(a0, b0, x0, y0, lambda, 15 * eps);
                w1.val = 0.5 + (0.5 - w.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            if (lambda > 0.03 * a0) {
                w.val = Toms708.bfrac(a0, b0, x0, y0, lambda, 15 * eps);
                w1.val = 0.5 + (0.5 - w.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            w.val = Toms708.basym(a0, b0, lambda, 100 * eps);
            w1.val = 0.5 + (0.5 - w.val);
            if (ind === 0) {
                return;
            } else {
                t = w.val;
                w.val = w1.val;
                w1.val = t;
                return;
            }
        } else {
            if (b0 <= 100 || lambda > 0.03 * b0) {
                w.val = Toms708.bfrac(a0, b0, x0, y0, lambda, 15 * eps);
                w1.val = 0.5 + (0.5 - w.val);
                if (ind === 0) {
                    return;
                } else {
                    t = w.val;
                    w.val = w1.val;
                    w1.val = t;
                    return;
                }
            }
            w.val = Toms708.basym(a0, b0, lambda, 100 * eps);
            w1.val = 0.5 + (0.5 - w.val);
            if (ind === 0) {
                return;
            } else {
                t = w.val;
                w.val = w1.val;
                w1.val = t;
                return;
            }
        }
    }
    static bup(a, b, x, y, n, eps) {
        let mu = 0;
        let d = 1;
        let t = 0;
        let r = 0;
        let w = 0;
        let k;
        let l;
        const apb = a + b;
        const ap1 = a + 1;
        if (n !== 1 && a > 1) {
            if (apb >= 1.1 * ap1) {
                mu = trunc(Math.abs(Toms708.exparg(1)));
                k = trunc(Toms708.exparg(0));
                if (k < mu) {
                    mu = k;
                }
                t = mu;
                d = Math.exp(-t);
            }
        }
        const dResult = Toms708.brcmp1(mu, a, b, x, y) / a;
        if (n === 1 || dResult === 0) {
            return dResult;
        }
        const nm1 = n - 1;
        w = d;
        k = 0;
        if (b <= 1) {
            const kp1 = k + 1;
            for (let i = kp1; i <= nm1; i++) {
                l = i - 1;
                d = (apb + l) / (ap1 + l) * x * d;
                w += d;
                if (d <= eps * w)
                    break;
            }
            return dResult * w;
        }
        if (y > 0.0001) {
            r = (b - 1) * x / y - a;
            if (r < 1) {
                const kp1 = k + 1;
                for (let i = kp1; i <= nm1; i++) {
                    l = i - 1;
                    d = (apb + l) / (ap1 + l) * x * d;
                    w += d;
                    if (d <= eps * w)
                        break;
                }
                return dResult * w;
            }
            k = nm1;
            t = nm1;
            if (r < t) {
                k = trunc(r);
            }
        } else {
            k = nm1;
        }
        for (let i = 1; i <= k; i++) {
            l = i - 1;
            d = (apb + l) / (ap1 + l) * x * d;
            w += d;
        }
        if (k === nm1) {
            return dResult * w;
        }
        const kp1 = k + 1;
        for (let i = kp1; i <= nm1; i++) {
            l = i - 1;
            d = (apb + l) / (ap1 + l) * x * d;
            w += d;
            if (d <= eps * w)
                break;
        }
        return dResult * w;
    }
    static erf(x) {
        const c = 0.564189583547756;
        const a = [
            0.000077105849500132,
            -0.00133733772997339,
            0.0323076579225834,
            0.0479137145607681,
            0.128379167095513
        ];
        const b = [
            0.00301048631703895,
            0.0538971687740286,
            0.375795757275549
        ];
        const p = [
            -1.36864857382717e-7,
            0.564195517478974,
            7.21175825088309,
            43.1622272220567,
            152.98928504694,
            339.320816734344,
            451.918953711873,
            300.459261020162
        ];
        const q = [
            1,
            12.7827273196294,
            77.0001529352295,
            277.585444743988,
            638.980264465631,
            931.35409485061,
            790.950925327898,
            300.459260956983
        ];
        const r = [
            2.10144126479064,
            26.2370141675169,
            21.3688200555087,
            4.6580782871847,
            0.282094791773523
        ];
        const s = [
            94.153775055546,
            187.11481179959,
            99.0191814623914,
            18.0124575948747
        ];
        let ax = 0;
        let bot = 0;
        let t = 0;
        let top = 0;
        let x2 = 0;
        ax = Math.abs(x);
        if (ax <= 0.5) {
            t = x * x;
            top = (((a[0] * t + a[1]) * t + a[2]) * t + a[3]) * t + a[4] + 1;
            bot = ((b[0] * t + b[1]) * t + b[2]) * t + 1;
            return x * (top / bot);
        } else {
            if (ax <= 4) {
                top = ((((((p[0] * ax + p[1]) * ax + p[2]) * ax + p[3]) * ax + p[4]) * ax + p[5]) * ax + p[6]) * ax + p[7];
                bot = ((((((q[0] * ax + q[1]) * ax + q[2]) * ax + q[3]) * ax + q[4]) * ax + q[5]) * ax + q[6]) * ax + q[7];
                return x < 0 ? -(0.5 + (0.5 - Math.exp(-x * x) * top / bot)) : 0.5 + (0.5 - Math.exp(-x * x) * top / bot);
            } else {
                if (ax < 5.8) {
                    x2 = x * x;
                    t = 1 / x2;
                    top = (((r[0] * t + r[1]) * t + r[2]) * t + r[3]) * t + r[4];
                    bot = (((s[0] * t + s[2]) * t + s[3]) * t + s[3]) * t + 1;
                    let dResult = (c - top / (x2 * bot)) / ax;
                    dResult = 0.5 + (0.5 - Math.exp(-x2) * dResult);
                    return x < 0 ? -dResult : dResult;
                } else {
                    return Toms708.sign(1, x);
                }
            }
        }
    }
    static erfc1(ind, x) {
        const c = 0.564189583547756;
        const a = [
            0.000077105849500132,
            -0.00133733772997339,
            0.0323076579225834,
            0.0479137145607681,
            0.128379167095513
        ];
        const b = [
            0.00301048631703895,
            0.0538971687740286,
            0.375795757275549
        ];
        const p = [
            -1.36864857382717e-7,
            0.564195517478974,
            7.21175825088309,
            43.1622272220567,
            152.98928504694,
            339.320816734344,
            451.918953711873,
            300.459261020162
        ];
        const q = [
            1,
            12.7827273196294,
            77.0001529352295,
            277.585444743988,
            638.980264465631,
            931.35409485061,
            790.950925327898,
            300.459260956983
        ];
        const r = [
            2.10144126479064,
            26.2370141675169,
            21.3688200555087,
            4.6580782871847,
            0.282094791773523
        ];
        const s = [
            94.153775055546,
            187.11481179959,
            99.0191814623914,
            18.0124575948747
        ];
        let dResult = 0;
        let w = 0;
        let ax = 0;
        let e = 0;
        let bot = 0;
        let t = 0;
        let top = 0;
        ax = Math.abs(x);
        if (ax < 0.5) {
            t = x * x;
            top = (((a[0] * t + a[1]) * t + a[2]) * t + a[3]) * t + a[4] + 1;
            bot = ((b[0] * t + b[1]) * t + b[2]) * t + 1;
            dResult = 0.5 + (0.5 - x * (top / bot));
            if (ind !== 0)
                dResult *= Math.exp(t);
            return dResult;
        } else if (ax <= 4 && ax >= 0.5) {
            top = ((((((p[0] * ax + p[1]) * ax + p[2]) * ax + p[3]) * ax + p[4]) * ax + p[5]) * ax + p[6]) * ax + p[7];
            bot = ((((((q[0] * ax + q[1]) * ax + q[2]) * ax + q[3]) * ax + q[4]) * ax + q[5]) * ax + q[6]) * ax + q[7];
            dResult = top / bot;
        } else {
            if (x <= -5.6) {
                if (ind !== 0)
                    return 2 * Math.exp(x * x);
                else
                    return 2;
            }
            if (ind !== 0) {
                t = Math.pow(1 / x, 2);
                top = (((r[0] * t + r[1]) * t + r[2]) * t + r[3]) * t + r[4];
                bot = (((s[0] * t + s[2]) * t + s[3]) * t + s[3]) * t + 1;
                dResult = (c - t * top / bot) / ax;
            } else {
                if (x > 100 || x * x > -Toms708.exparg(1)) {
                    return 0;
                }
            }
        }
        if (ind !== 0) {
            if (x < 0)
                dResult = 2 * Math.exp(x * x) - dResult;
            return dResult;
        }
        w = x * x;
        t = w;
        e = w - t;
        dResult *= (0.5 + (0.5 - e)) * Math.exp(-t);
        if (x < 0)
            dResult = 2 - dResult;
        return dResult;
    }
    static esum(mu, x) {
        let w = 0;
        if (x <= 0) {
            if (mu < 0) {
                w = mu;
                return Math.exp(w) * Math.exp(x);
            }
            w = mu + x;
            if (w > 0) {
                w = mu;
                return Math.exp(w) * Math.exp(x);
            }
            return Math.exp(w);
        }
        if (mu > 0) {
            w = mu;
            return Math.exp(w) * Math.exp(x);
        }
        w = mu + x;
        if (w < 0) {
            w = mu;
            return Math.exp(w) * Math.exp(x);
        }
        return Math.exp(w);
    }
    static erf_values(nData, x, fx) {
        const nmax = 21;
        const bvec = [
            0,
            0.112462916,
            0.2227025892,
            0.3286267595,
            0.428392355,
            0.5204998778,
            0.6038560908,
            0.6778011938,
            0.7421009647,
            0.7969082124,
            0.8427007929,
            0.8802050696,
            0.9103139782,
            0.9340079449,
            0.9522851198,
            0.9661051465,
            0.9763483833,
            0.9837904586,
            0.9890905016,
            0.9927904292,
            0.995322265
        ];
        const xvec = [
            0,
            0.1,
            0.2,
            0.3,
            0.4,
            0.5,
            0.6,
            0.7,
            0.8,
            0.9,
            1,
            1.1,
            1.2,
            1.3,
            1.4,
            1.5,
            1.6,
            1.7,
            1.8,
            1.9,
            2
        ];
        if (nData.val < 0)
            nData.val = 0;
        nData.val++;
        if (nmax < nData.val) {
            nData.val = 0;
            x.val = 0;
            fx.val = 0;
        } else {
            x.val = xvec[nData.val - 1];
            fx.val = bvec[nData.val - 1];
        }
    }
    static exparg(l) {
        let lnb = 0;
        let m = 0;
        const b = Toms708.ipmpar(4);
        switch (b) {
        case 2:
            lnb = 0.69314718055995;
            break;
        case 8:
            lnb = 2.0794415416798;
            break;
        case 16:
            lnb = 2.7725887222398;
            break;
        default:
            lnb = log(b);
            break;
        }
        if (l !== 0) {
            m = Toms708.ipmpar(6);
            return 0.99999 * m * lnb;
        } else {
            m = Toms708.ipmpar(7);
            return 0.99999 * m * lnb;
        }
    }
    static fpser(a, b, x, eps) {
        let c = 0;
        let s = 0;
        let t = 0;
        let tol = 0;
        let an = 0;
        let dResult = 1;
        if (a > 0.001 * eps) {
            dResult = 0;
            t = a * log(x);
            if (t < Toms708.exparg(1)) {
                return dResult;
            }
            dResult = Math.exp(t);
        }
        dResult = b / a * dResult;
        tol = eps / a;
        an = a + 1;
        t = x;
        s = t / an;
        do {
            an = an + 1;
            t = x * t;
            c = t / an;
            s = s + c;
        } while (Math.abs(c) > tol);
        return dResult * (1 + a * s);
    }
    static gam1(a) {
        const p = [
            0.577215664901533,
            -0.409078193005776,
            -0.230975380857675,
            0.0597275330452234,
            0.0076696818164949,
            -0.00514889771323592,
            0.000589597428611429
        ];
        const q = [
            1,
            0.427569613095214,
            0.158451672430138,
            0.0261132021441447,
            0.00423244297896961
        ];
        const r = [
            -0.422784335098468,
            -0.771330383816272,
            -0.244757765222226,
            0.118378989872749,
            0.000930357293360349,
            -0.0118290993445146,
            0.00223047661158249,
            0.000266505979058923,
            -0.000132674909766242
        ];
        const s2 = 0.0559398236957378;
        const s1 = 0.273076135303957;
        let d = 0;
        let bot = 0;
        let t = 0;
        let w = 0;
        let top = 0;
        t = a;
        d = a - 0.5;
        if (d > 0) {
            t = d - 0.5;
        }
        if (t === 0) {
            return 0;
        } else if (t > 0) {
            top = (((((p[6] * t + p[5]) * t + p[4]) * t + p[3]) * t + p[2]) * t + p[1]) * t + p[0];
            bot = (((q[4] * t + q[3]) * t + q[2]) * t + q[1]) * t + 1;
            w = top / bot;
            if (d <= 0) {
                return a * w;
            } else {
                return t / a * (w - 0.5 - 0.5);
            }
        } else {
            top = (((((((r[8] * t + r[7]) * t + r[6]) * t + r[5]) * t + r[4]) * t + r[3]) * t + r[2]) * t + r[1]) * t + r[0];
            bot = (s2 * t + s1) * t + 1;
            w = top / bot;
            if (d <= 0) {
                return a * (w + 0.5 + 0.5);
            } else {
                return t * w / a;
            }
        }
    }
    static grat1(a, x, r, p, q, eps) {
        let j = 0;
        let l = 0;
        let am0 = 0;
        let an0 = 0;
        let a2n = 0;
        let b2n = 0;
        let cma = 0;
        let c = 0;
        let g = 0;
        let h = 0;
        let t = 0;
        let w = 0;
        let z = 0;
        let tol = 0;
        let sum = 0;
        let a2nm1 = 0;
        let b2nm1 = 0;
        let an = 0;
        if (a * x === 0) {
            if (x <= a) {
                p.val = 0;
                q.val = 1;
                return;
            } else {
                p.val = 1;
                q.val = 0;
                return;
            }
        }
        if (a === 0.5) {
            if (x < 0.25) {
                p.val = Toms708.erf(Math.sqrt(x));
                q.val = 0.5 + (0.5 - p.val);
                return;
            } else {
                q.val = Toms708.erfc1(0, Math.sqrt(x));
                p.val = 0.5 + (0.5 - q.val);
                return;
            }
        }
        if (x < 1.1) {
            an = 3;
            c = x;
            sum = x / (a + 3);
            tol = 0.1 * eps / (a + 1);
            do {
                an++;
                c = -c * (x / an);
                t = c / (a + an);
                sum = sum + t;
            } while (Math.abs(t) > tol);
            j = a * x * ((sum / 6 - 0.5 / (a + 2)) * x + 1 / (a + 1));
            z = a * log(x);
            h = Toms708.gam1(a);
            g = 1 + h;
            if (x < 0.25) {
                if (z > -0.13394) {
                    l = Toms708.rexp(z);
                    w = 0.5 + (0.5 + l);
                    q.val = (w * j - l) * g - h;
                    if (q.val < 0) {
                        p.val = 1;
                        q.val = 0;
                        return;
                    }
                    p.val = 0.5 + (0.5 - q.val);
                    return;
                } else {
                    w = Math.exp(z);
                    p.val = w * g * (0.5 + (0.5 - j));
                    q.val = 0.5 + (0.5 - p.val);
                    return;
                }
            } else {
                w = Math.exp(z);
                p.val = w * g * (0.5 + (0.5 - j));
                q.val = 0.5 + (0.5 - p.val);
                return;
            }
        } else {
            a2nm1 = 1;
            a2n = 1;
            b2nm1 = x;
            b2n = x + (1 - a);
            c = 1;
            do {
                a2nm1 = x * a2n + c * a2nm1;
                b2nm1 = x * b2n + c * b2nm1;
                am0 = a2nm1 / b2nm1;
                c++;
                cma = c - a;
                a2n = a2nm1 + cma * a2n;
                b2n = b2nm1 + cma * b2n;
                an0 = a2n / b2n;
            } while (Math.abs(an0 - am0) >= eps * an0);
            q.val = r * an0;
            p.val = 0.5 + (0.5 - q.val);
            return;
        }
    }
    static ipmpar(i) {
        const imach = [
            2,
            31,
            2147483647,
            2,
            24,
            -125,
            128,
            53,
            -1021,
            1024
        ];
        return imach[i - 1];
    }
    static gamma_inc_values(nData, a, x, fx) {
        const nMax = 20;
        const aVec = [
            0.1,
            0.1,
            0.1,
            0.5,
            0.5,
            0.5,
            1,
            1,
            1,
            1.1,
            1.1,
            1.1,
            2,
            2,
            2,
            6,
            6,
            11,
            26,
            41
        ];
        const fxVec = [
            0.7382350532339351,
            0.9083579897300343,
            0.9886559833621947,
            0.3014646416966613,
            0.7793286380801532,
            0.9918490284064972,
            0.09516258196404043,
            0.6321205588285577,
            0.9932620530009145,
            0.07205974576054322,
            0.5891809618706485,
            0.9915368159845525,
            0.01018582711118352,
            0.4421745996289254,
            0.9927049442755639,
            0.04202103819530612,
            0.9796589705830716,
            0.9226039842296428,
            0.4470785799755852,
            0.7444549220718699
        ];
        const xVec = [
            0.03,
            0.3,
            1.5,
            0.075,
            0.75,
            3.5,
            0.1,
            1,
            5,
            0.1,
            1,
            5,
            0.15,
            1.5,
            7,
            2.5,
            12,
            16,
            25,
            45
        ];
        if (nData.val < 0)
            nData.val = 0;
        nData.val++;
        if (nMax < nData.val) {
            nData.val = 0;
            a.val = 0;
            x.val = 0;
            fx.val = 0;
        } else {
            a.val = aVec[nData.val - 1];
            x.val = xVec[nData.val - 1];
            fx.val = fxVec[nData.val - 1];
        }
    }
    static gamma_log_values(nData, x, fx) {
        const nmax = 18;
        const bvec = [
            1.524064183,
            0.7966780066,
            0.3982337117,
            0.1520599127,
            0,
            -0.04987246543,
            -0.08537410945,
            -0.1081747934,
            -0.119612895,
            -0.120782204,
            -0.1125917658,
            -0.09580771625,
            -0.07108385116,
            -0.0389842838,
            0,
            12.80182743,
            39.33988571,
            71.25704193
        ];
        const xvec = [
            0.2,
            0.4,
            0.6,
            0.8,
            1,
            1.1,
            1.2,
            1.3,
            1.4,
            1.5,
            1.6,
            1.7,
            1.8,
            1.9,
            2,
            10,
            20,
            30
        ];
        if (nData.val < 0)
            nData.val = 0;
        nData.val++;
        if (nmax < nData.val) {
            nData.val = 0;
            x.val = 0;
            fx.val = 0;
        } else {
            x.val = xvec[nData.val - 1];
            fx.val = bvec[nData.val - 1];
        }
    }
    static gamln(a) {
        const d = 0.418938533204673;
        const c5 = -0.00165322962780713;
        const c4 = 0.000837308034031215;
        const c3 = -0.00059520293135187;
        const c2 = 0.00079365066682539;
        const c1 = -0.00277777777760991;
        const c0 = 0.0833333333333333;
        let i = 0;
        let n = 0;
        let t = 0;
        let w = 0;
        if (a <= 0.8)
            return Toms708.gamln1(a) - log(a);
        if (a <= 2.25) {
            t = a - 0.5 - 0.5;
            return Toms708.gamln1(t);
        }
        if (a < 10) {
            n = trunc(a - 1.25);
            t = a;
            w = 1;
            for (i = 1; i <= n; i++) {
                t--;
                w *= t;
            }
            return Toms708.gamln1(t - 1) + log(w);
        }
        t = Math.pow(1 / a, 2);
        w = (((((c5 * t + c4) * t + c3) * t + c2) * t + c1) * t + c0) / a;
        return d + w + (a - 0.5) * (log(a) - 1);
    }
    static gamln1(a) {
        const p6 = -0.00271935708322958;
        const p5 = -0.0673562214325671;
        const p4 = -0.402055799310489;
        const p3 = -0.780427615533591;
        const p2 = -0.168860593646662;
        const p1 = 0.844203922187225;
        const p0 = 0.577215664901533;
        const q6 = 0.000667465618796164;
        const q5 = 0.0325038868253937;
        const q4 = 0.361951990101499;
        const q3 = 1.56875193295039;
        const q2 = 3.12755088914843;
        const q1 = 2.88743195473681;
        const r5 = 0.000497958207639485;
        const r4 = 0.017050248402265;
        const r3 = 0.156513060486551;
        const r2 = 0.565221050691933;
        const r1 = 0.848044614534529;
        const r0 = 0.422784335098467;
        const s5 = 0.000116165475989616;
        const s4 = 0.00713309612391;
        const s3 = 0.10155218743983;
        const s2 = 0.548042109832463;
        const s1 = 1.24313399877507;
        let w = 0;
        let x = 0;
        if (a < 0.6) {
            w = ((((((p6 * a + p5) * a + p4) * a + p3) * a + p2) * a + p1) * a + p0) / ((((((q6 * a + q5) * a + q4) * a + q3) * a + q2) * a + q1) * a + 1);
            return -a * w;
        }
        x = a - 0.5 - 0.5;
        w = (((((r5 * x + r4) * x + r3) * x + r2) * x + r1) * x + r0) / (((((s5 * x + s4) * x + s3) * x + s2) * x + s1) * x + 1);
        return x * w;
    }
    static gsumln(a, b) {
        const x = a + b - 2;
        if (x <= 0.25)
            return Toms708.gamln1(1 + x);
        if (x <= 1.25)
            return Toms708.gamln1(x) + Toms708.alnrel(x);
        return Toms708.gamln1(x - 1) + log(x * (1 + x));
    }
    static psi(xx) {
        const piov4 = 0.785398163397448;
        const dx0 = 1.4616321449683622;
        const p1 = [
            0.0089538502298197,
            4.77762828042627,
            142.441585084029,
            1186.45200713425,
            3633.51846806499,
            4138.10161269013,
            1305.60269827897
        ];
        const q1 = [
            44.8452573429826,
            520.752771467162,
            2210.0079924783,
            3641.27349079381,
            1908.310765963,
            0.00000691091682714533
        ];
        const p2 = [
            -2.12940445131011,
            -7.01677227766759,
            -4.48616543918019,
            -0.648157123766197
        ];
        const q2 = [
            32.2703493791143,
            89.2920700481861,
            54.6117738103215,
            7.77788548522962
        ];
        let nq = 0;
        let xsmall = 0;
        let xmax1 = 0;
        let den = 0;
        let i = 0;
        let aug = 0;
        let m = 0;
        let n = 0;
        let sgn = 0;
        let w = 0;
        let x = 0;
        let z = 0;
        let upper = 0;
        let xmx0 = 0;
        xmax1 = Toms708.ipmpar(3);
        xmax1 = Math.min(xmax1, 1 / Toms708.spmpar(1));
        xsmall = 1e-9;
        x = xx;
        aug = 0;
        if (x < 0.5) {
            if (Math.abs(x) <= xsmall) {
                if (x === 0)
                    return 0;
                aug = -1 / x;
            } else {
                w = -x;
                sgn = piov4;
                if (w <= 0) {
                    w = -w;
                    sgn = -sgn;
                }
                if (w >= xmax1)
                    return 0;
                nq = trunc(w);
                w = w - nq;
                nq = trunc(w * 4);
                w = 4 * (w - nq * 0.25);
                n = nq / 2;
                if (n + n !== nq) {
                    w = 1 - w;
                }
                z = piov4 * w;
                m = n / 2;
                if (m + m !== n) {
                    sgn = -sgn;
                }
                n = (nq + 1) / 2;
                m = n / 2;
                m = m + m;
                if (m === n) {
                    if (z === 0)
                        return 0;
                    aug = sgn * (Math.cos(z) / Math.sin(z) * 4);
                } else {
                    aug = sgn * (Math.sin(z) / Math.cos(z) * 4);
                }
            }
            x = 1 - x;
        }
        if (x <= 3) {
            den = x;
            upper = p1[1] * x;
            for (i = 0; i < 5; i++) {
                den = (den + q1[i]) * x;
                upper = (upper + p1[i + 1]) * x;
            }
            den = (upper + p1[6]) / (den + q1[5]);
            xmx0 = x - dx0;
            return den * xmx0 + aug;
        }
        if (x < xmax1) {
            w = 1 / (x * x);
            den = w;
            upper = p2[0] * w;
            for (i = 0; i < 3; i++) {
                den = (den + q2[i]) * w;
                upper = (upper + p2[i + 1]) * w;
            }
            aug = upper / (den + q2[3]) - 0.5 / x + aug;
        }
        return aug + log(x);
    }
    static psi_values(n, x, fx) {
        const nmax = 11;
        const fxvec = [
            -0.5772156649,
            -0.4237549404,
            -0.2890398966,
            -0.1691908889,
            -0.0613845446,
            -0.036489974,
            0.1260474528,
            0.2085478749,
            0.2849914333,
            0.3561841612,
            0.4227843351
        ];
        const xvec = [
            1,
            1.1,
            1.2,
            1.3,
            1.4,
            1.5,
            1.6,
            1.7,
            1.8,
            1.9,
            2
        ];
        if (n.val < 0)
            n.val = 0;
        n.val++;
        if (nmax < n.val) {
            n.val = 0;
            x.val = 0;
            fx.val = 0;
        } else {
            x.val = xvec[n.val - 1];
            fx.val = fxvec[n.val - 1];
        }
    }
    static r4_epsilon() {
        let r = 1;
        let rTest = 1 + r / 2;
        while (1 < rTest) {
            r = r / 2;
            rTest = 1 + r / 2;
        }
        return r;
    }
    static rexp(x) {
        const q4 = 0.000595130811860248;
        const q3 = -0.0119041179760821;
        const q2 = 0.107141568980644;
        const q1 = -0.499999999085958;
        const p2 = 0.0238082361044469;
        const p1 = 9.14041914819518e-10;
        let w = 0;
        if (Math.abs(x) <= 0.15) {
            return x * (((p2 * x + p1) * x + 1) / (((q4 * x + q3) * x + q2) * x + q1) * x + 1);
        } else {
            w = Math.exp(x);
            if (x <= 0) {
                return w - 0.5 - 0.5;
            } else {
                return w * (0.5 + (0.5 - 1 / w));
            }
        }
    }
    static rlog1(x) {
        const a = 0.0566749439387324;
        const b = 0.0456512608815524;
        const p2 = 0.00620886815375787;
        const p1 = -0.224696413112536;
        const p0 = 0.333333333333333;
        const q2 = 0.354508718369557;
        const q1 = -1.27408923933623;
        let h = 0;
        let r = 0;
        let t = 0;
        let w = 0;
        let w1 = 0;
        if (x < -0.39 || x > 0.57) {
            w = x + 0.5 + 0.5;
            return x - log(w);
        }
        if (x < -0.18) {
            h = x + 0.3;
            h = h / 0.7;
            w1 = a - h * 0.3;
        } else if (x > 0.18) {
            h = 0.75 * x - 0.25;
            w1 = b + h / 3;
        } else {
            h = x;
            w1 = 0;
        }
        r = h / (h + 2);
        t = r * r;
        w = ((p2 * t + p1) * t + p0) / ((q2 * t + q1) * t + 1);
        return 2 * t * (1 / (1 - r) - r * w) + w1;
    }
    static sign(a1, a2) {
        return a2 >= 0 ? Math.abs(a1) : -Math.abs(a1);
    }
    static spmpar(i) {
        let emin = 0;
        let emax = 0;
        let binv = 0;
        let bm1 = 0;
        let ibeta = 0;
        let b = 0;
        let m = 0;
        let one = 0;
        let w = 0;
        let z = 0;
        if (i > 1) {
            if (i > 2) {
                ibeta = Toms708.ipmpar(4);
                m = Toms708.ipmpar(5);
                emax = Toms708.ipmpar(7);
                b = ibeta;
                bm1 = ibeta - 1;
                one = 1;
                z = Math.pow(b, m - 1);
                w = ((z - one) * b + bm1) / (b * z);
                z = Math.pow(b, emax - 2);
                return w * z * b * b;
            } else {
                b = Toms708.ipmpar(4);
                emin = Toms708.ipmpar(6);
                one = 1;
                binv = one / b;
                w = Math.pow(b, emin + 2);
                return w * binv * binv * binv;
            }
        }
        b = Toms708.ipmpar(4);
        m = Toms708.ipmpar(5);
        return Math.pow(b, 1 - m);
    }
}