import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D__0,
    R_D_exp,
    R_D_val,
    log,
    log1p
} from '../../r-func.mjs';
import { dbinom_raw } from '../binomial/dbinom.mjs';
import { lbeta } from '../../special/beta/index.mjs';
const printer = debug('dbeta');
export function dbeta_scalar(x, a, b, asLog) {
    if (isNaN(x) || isNaN(a) || isNaN(b))
        return x + a + b;
    if (a < 0 || b < 0)
        return ML_ERR_return_NAN2(printer, lineInfo4);
    if (x < 0 || x > 1)
        return asLog ? 0 : 1;
    if (a === 0 || b === 0 || !isFinite(a) || !isFinite(b)) {
        if (a === 0 && b === 0) {
            if (x === 0 || x === 1)
                return Infinity;
            else
                return R_D__0(asLog);
        }
        if (a === 0 || a / b === 0) {
            if (x === 0)
                return Infinity;
            else
                return R_D__0(asLog);
        }
        if (b === 0 || b / a === 0) {
            if (x === 1)
                return Infinity;
            else
                return R_D__0(asLog);
        }
        if (x === 0.5)
            return Infinity;
        else
            return R_D__0(asLog);
    }
    if (x === 0) {
        if (a > 1)
            return R_D__0(asLog);
        if (a < 1)
            return Infinity;
        return R_D_val(asLog, b);
    }
    if (x === 1) {
        if (b > 1)
            return R_D__0(asLog);
        if (b < 1)
            return Infinity;
        return R_D_val(asLog, a);
    }
    let lval;
    if (a <= 2 || b <= 2)
        lval = (a - 1) * log(x) + (b - 1) * log1p(-x) - lbeta(a, b);
    else {
        lval = log(a + b - 1) + dbinom_raw(a - 1, a + b - 2, x, 1 - x, true);
    }
    return R_D_exp(asLog, lval);
}