import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D__0,
    R_D_exp,
    ceil,
    sqrt
} from '../../r-func.mjs';
import { dpois_raw } from '../poisson/dpois.mjs';
import { dbeta_scalar } from './dbeta.mjs';
const printer = debug('dnbeta');
const eps = 1e-15;
export function dnbeta_scalar(x, a, b, ncp, give_log) {
    let kMax;
    let k;
    let D;
    let term;
    let sum;
    let p_k;
    let q;
    if (isNaN(x) || isNaN(a) || isNaN(b) || isNaN(ncp))
        return x + a + b + ncp;
    if (ncp < 0 || a <= 0 || b <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (!isFinite(a) || !isFinite(b) || !isFinite(ncp)) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (x < 0 || x > 1) {
        return R_D__0(give_log);
    }
    const ncp2 = 0.5 * ncp;
    const dx2 = ncp2 * x;
    const d = (dx2 - a - 1) / 2;
    D = d * d + dx2 * (a + b) - a;
    if (D <= 0) {
        kMax = 0;
    } else {
        D = ceil(d + sqrt(D));
        kMax = D > 0 ? D : 0;
    }
    term = dbeta_scalar(x, a + kMax, b, true);
    p_k = dpois_raw(kMax, ncp2, true);
    if (x === 0 || !isFinite(term) || !isFinite(p_k)) {
        return R_D_exp(give_log, p_k + term);
    }
    p_k += term;
    sum = term = 1;
    k = kMax;
    while (k > 0 && term > sum * eps) {
        k--;
        q = (k + 1) * (k + a) / (k + a + b) / dx2;
        term *= q;
        sum += term;
    }
    term = 1;
    k = kMax;
    do {
        q = dx2 * (k + a + b) / (k + a) / (k + 1);
        k++;
        term *= q;
        sum += term;
    } while (term > sum * eps);
    return R_D_exp(give_log, p_k + Math.log(sum));
}