import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    ME,
    ML_ERROR2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_P_bounds_01,
    floor,
    exp,
    sqrt,
    log,
    log1p,
    max
} from '../../r-func.mjs';
import { lgammafn_sign } from '../../special/gamma/lgammafn_sign.mjs';
import { Toms708 } from '../../common/toms708/toms708.mjs';
import { NumberW } from '../../common/toms708/NumberW.mjs';
const printer = debug('pnbeta_raw');
const printer_pnbeta2 = debug('pnbeta2');
const errmax = 1e-9;
const itrmax = 10000;
function pnbeta_raw(x, o_x, a, b, ncp) {
    let errbd;
    const temp = new NumberW(0);
    const tmp_c = new NumberW(0);
    const ierr = new NumberW(0);
    let ans;
    let ax;
    let gx;
    let q;
    let sumq;
    if (ncp < 0 || a <= 0 || b <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    const c = ncp / 2;
    const x0 = floor(max(c - 7 * sqrt(c), 0));
    const a0 = a + x0;
    const lbeta = lgammafn_sign(a0) + lgammafn_sign(b) - lgammafn_sign(a0 + b);
    Toms708.bratio(a0, b, x, o_x, temp, tmp_c, ierr);
    gx = exp(a0 * log(x) + b * (x < 0.5 ? log1p(-x) : log(o_x)) - lbeta - log(a0));
    if (a0 > a)
        q = exp(-c + x0 * log(c) - lgammafn_sign(x0 + 1));
    else
        q = exp(-c);
    sumq = 1 - q;
    ans = ax = q * temp.val;
    let j = floor(x0);
    do {
        j++;
        temp.val -= gx;
        gx *= x * (a + b + j - 1) / (a + j);
        q *= c / j;
        sumq -= q;
        ax = temp.val * q;
        ans += ax;
        errbd = (temp.val - gx) * sumq;
    } while (errbd > errmax && j < itrmax + x0);
    if (errbd > errmax)
        ML_ERROR2(ME.ME_PRECISION, 'pnbeta', printer);
    if (j >= itrmax + x0)
        ML_ERROR2(ME.ME_NOCONV, 'pnbeta', printer);
    return ans;
}
export function pnbeta2(x, o_x, a, b, ncp, lower_tail, log_p) {
    let ans = pnbeta_raw(x, o_x, a, b, ncp);
    if (isNaN(ans))
        return NaN;
    if (lower_tail) {
        return log_p ? log(ans) : ans;
    } else {
        if (ans > 1 - 1e-10)
            ML_ERROR2(ME.ME_PRECISION, 'pnbeta', printer_pnbeta2);
        if (ans > 1)
            ans = 1;
        return log_p ? log1p(-ans) : 1 - ans;
    }
}
export function pnbeta(x, a, b, ncp, lower_tail, log_p) {
    if (isNaN(x) || isNaN(a) || isNaN(b) || isNaN(ncp))
        return x + a + b + ncp;
    const rc = R_P_bounds_01(lower_tail, log_p, x, 0, 1);
    if (rc !== undefined) {
        return rc;
    }
    return pnbeta2(x, 1 - x, a, b, ncp, lower_tail, log_p);
}