import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    ME,
    ML_ERROR2
} from '../../../packages/common/logger.mjs';
import {
    DBL_MANT_DIG,
    M_LN2,
    R_D_half,
    R_DT_0,
    R_DT_1,
    R_pow_di,
    DBL_MIN_EXP,
    abs as fabs,
    log
} from '../../r-func.mjs';
import {
    R_DT_CIv,
    R_DT_Clog,
    R_DT_log,
    R_DT_qIv,
    R_Log1_Exp
} from '../exp/expm1.mjs';
import { lbeta } from '../../special/beta/index.mjs';
import { pbeta_raw } from './pbeta.mjs';
const USE_LOG_X_CUTOFF = -5;
const n_NEWTON_FREE = 4;
const MLOGICAL_NA = -1;
const printer_qbeta = debug('qbeta');
export function qbeta(p, shape1, shape2, lower_tail, log_p) {
    if (isNaN(shape1) || isNaN(shape2) || isNaN(p))
        return shape1 + shape2 + p;
    if (shape1 < 0 || shape2 < 0) {
        return ML_ERR_return_NAN2(printer_qbeta, lineInfo4);
    }
    const qbet = new Float64Array(2);
    qbeta_raw(p, shape1, shape2, lower_tail, log_p, MLOGICAL_NA, USE_LOG_X_CUTOFF, n_NEWTON_FREE, qbet);
    return qbet[0];
}
const DBL_very_MIN = Number.MIN_VALUE / 4;
const DBL_log_v_MIN = M_LN2 * (DBL_MIN_EXP - 2);
const DBL_1__eps = 1 - 2 ** -53;
const fpu = 3e-308;
const acu_min = 1e-300;
const p_lo = fpu;
const p_hi = 1 - 2.22e-16;
const const1 = 2.30753;
const const2 = 0.27061;
const const3 = 0.99229;
const const4 = 0.04481;
const log_eps_c = M_LN2 * (1 - DBL_MANT_DIG);
function return_q_0(_give_log_q, qb) {
    qb[0] = 0;
    qb[1] = 1;
    return;
}
function return_q_1(_give_log_q, qb) {
    qb[0] = 1;
    qb[1] = 0;
    return;
}
function return_q_half(_give_log_q, qb) {
    qb[0] = qb[1] = 0.5;
    return;
}
const printer_qbeta_raw = debug('qbeta_raw');
const R_ifDEBUG_printf = printer_qbeta_raw;
function qbeta_raw(alpha, p, q, lower_tail, log_p, swap_01, log_q_cut, n_N, qb) {
    const swap_choose = swap_01 === MLOGICAL_NA;
    let swap_tail;
    let log_;
    const give_log_q = false;
    let use_log_x = give_log_q;
    let warned = false;
    let add_N_step = true;
    let i_pb;
    let i_inn;
    let a;
    let la;
    let g;
    let h;
    let pp;
    let qq;
    let r;
    let s;
    let t;
    let w;
    let y = -1;
    let u = 0;
    let xinbta = 0;
    let u_n = 0;
    let wprev;
    let prev;
    let adj;
    if (alpha === R_DT_0(lower_tail, log_p)) {
        return return_q_0(give_log_q, qb);
    }
    if (alpha === R_DT_1(lower_tail, log_p)) {
        return return_q_1(give_log_q, qb);
    }
    if (log_p && alpha > 0 || !log_p && (alpha < 0 || alpha > 1)) {
        printer_qbeta_raw('qbeta(alpha=%d, %d, %d, .., log_p=%d): %s%s', alpha, p, q, log_p, 'alpha not in ', log_p ? '[-Inf, 0]' : '[0,1]');
        ML_ERROR2(ME.ME_DOMAIN, lineInfo4, printer_qbeta_raw);
        qb[0] = qb[1] = NaN;
        return;
    }
    if (p === 0 || q === 0 || !isFinite(p) || !isFinite(q)) {
        printer_qbeta_raw('qbeta(%d, %d, %d, lower_t=%d, log_p=%d): (p,q)-boundary: trivial', alpha, p, q, lower_tail, log_p);
        if (p === 0 && q === 0) {
            if (alpha < R_D_half(log_p)) {
                return return_q_0(give_log_q, qb);
            }
            if (alpha > R_D_half(log_p)) {
                return return_q_1(give_log_q, qb);
            }
            return return_q_half(give_log_q, qb);
        } else if (p === 0 || p / q === 0) {
            return return_q_0(give_log_q, qb);
        } else if (q === 0 || q / p === 0) {
            return return_q_1(give_log_q, qb);
        }
        return return_q_half(give_log_q, qb);
    }
    const p_ = R_DT_qIv(lower_tail, log_p, alpha);
    const logbeta = lbeta(p, q);
    swap_tail = swap_choose ? p_ > 0.5 : !!swap_01;
    if (swap_tail) {
        a = R_DT_CIv(lower_tail, log_p, alpha);
        la = R_DT_Clog(lower_tail, log_p, alpha);
        pp = q;
        qq = p;
    } else {
        a = p_;
        la = R_DT_log(lower_tail, log_p, alpha);
        pp = p;
        qq = q;
    }
    const acu = Math.max(acu_min, Math.pow(10, -13 - 2.5 / (pp * pp) - 0.5 / (a * a)));
    let tx = 0;
    const u0 = (la + Math.log(pp) + logbeta) / pp;
    r = pp * (1 - qq) / (pp + 1);
    t = 0.2;
    printer_qbeta_raw('qbeta(%d, %d, %d, lower_t=%d, log_p=%d):%s   swap_tail=%d, la=%d, u0=%d (bnd: %d (%d)) ', alpha, p, q, lower_tail, log_p, log_p && (p_ === 0 || p_ === 1) ? p_ === 0 ? ' p_=0' : ' p_=1' : '', swap_tail, la, u0, (t * log_eps_c - Math.log(Math.abs(pp * (1 - qq) * (2 - qq) / (2 * (pp + 2))))) / 2, t * log_eps_c - Math.log(Math.abs(r)));
    const L_return = () => {
        if (use_log_x) {
            if (add_N_step) {
                xinbta = Math.exp(u_n);
                y = pbeta_raw(xinbta, pp, qq, true, log_p);
                w = log_p ? (y - la) * Math.exp(y + logbeta + r * Math.log(xinbta) + t * Math.log1p(-xinbta)) : (y - a) * Math.exp(logbeta + r * Math.log(xinbta) + t * Math.log1p(-xinbta));
                tx = xinbta - w;
                R_ifDEBUG_printf('Final Newton correction(non-log scale): xinbta=%.16g, y=%g, w=%g. => new tx=%.16g\n', xinbta, y, w, tx);
            } else {
                if (swap_tail) {
                    qb[0] = -Math.expm1(u_n);
                    qb[1] = Math.exp(u_n);
                } else {
                    qb[0] = Math.exp(u_n);
                    qb[1] = -Math.expm1(u_n);
                }
                return;
            }
        }
        if (swap_tail) {
            qb[0] = 1 - tx;
            qb[1] = tx;
        } else {
            qb[0] = tx;
            qb[1] = 1 - tx;
        }
    };
    const L_converged = () => {
        log_ = log_p || use_log_x;
        R_ifDEBUG_printf(' %s: Final delta(y) = %g%s\n', warned ? '_NO_ convergence' : 'converged', y - (log_ ? la : a), log_ ? ' (log_)' : '');
        if (log_ && y === -Infinity || !log_ && y === 0) {
            w = pbeta_raw(DBL_very_MIN, pp, qq, true, log_);
            if (log_ || Math.abs(w - a) <= Math.abs(y - a)) {
                tx = DBL_very_MIN;
                u_n = DBL_log_v_MIN;
            }
            add_N_step = false;
        } else if (!warned && (log_ ? Math.abs(y - la) > 3 : Math.abs(y - a) > 0.0001)) {
            if (!(log_ && y === -Infinity && pbeta_raw(DBL_1__eps, pp, qq, true, true) > la + 2))
                printer_qbeta_raw('qbeta(a, *) =: x0 with |pbeta(x0,* %s) - alpha| = %d is not accurate', log_ ? ', log_' : '', Math.abs(y - (log_ ? la : a)));
        }
    };
    const L_Newton = () => {
        r = 1 - pp;
        t = 1 - qq;
        wprev = 0;
        prev = 1;
        adj = 1;
        if (use_log_x) {
            for (i_pb = 0; i_pb < 1000; i_pb++) {
                y = pbeta_raw(xinbta, pp, qq, true, true);
                w = y === -Infinity ? 0 : (y - la) * Math.exp(y - u + logbeta + r * u + t * R_Log1_Exp(u));
                if (!isFinite(w))
                    break;
                if (i_pb >= n_N && w * wprev <= 0)
                    prev = Math.max(Math.abs(adj), fpu);
                R_ifDEBUG_printf('N(i=%2d): u=%#20.16g, pb(e^u)=%#12.6g, w=%#15.9g, %s prev=%11g,', i_pb, u, y, w, w * wprev <= 0 ? 'new' : 'old', prev);
                g = 1;
                for (i_inn = 0; i_inn < 1000; i_inn++) {
                    adj = g * w;
                    if (i_pb < n_N || Math.abs(adj) < prev) {
                        u_n = u - adj;
                        if (u_n <= 0) {
                            if (prev <= acu || Math.abs(w) <= acu) {
                                R_ifDEBUG_printf(' it{in}=%d, -adj=%g, %s <= acu  ==> convergence\n', i_inn, -adj, prev <= acu ? 'prev' : '|w|');
                                return;
                            }
                            break;
                        }
                    }
                    g /= 3;
                }
                const D = Math.min(Math.abs(adj), Math.abs(u_n - u));
                R_ifDEBUG_printf(' it{in}=%d, delta(u)=%9.3g, D/|.|=%.3g\n', i_inn, u_n - u, D / Math.abs(u_n + u));
                if (D <= 4e-16 * Math.abs(u_n + u))
                    return;
                u = u_n;
                xinbta = Math.exp(u);
                wprev = w;
            }
        } else
            for (i_pb = 0; i_pb < 1000; i_pb++) {
                y = pbeta_raw(xinbta, pp, qq, true, log_p);
                if (!isFinite(y) && !(log_p && y === -Infinity)) {
                    ML_ERROR2(ME.ME_DOMAIN, lineInfo4, printer_qbeta_raw);
                    qb[0] = qb[1] = NaN;
                    return;
                }
                w = log_p ? (y - la) * Math.exp(y + logbeta + r * Math.log(xinbta) + t * Math.log1p(-xinbta)) : (y - a) * Math.exp(logbeta + r * Math.log(xinbta) + t * Math.log1p(-xinbta));
                if (i_pb >= n_N && w * wprev <= 0)
                    prev = Math.max(Math.abs(adj), fpu);
                R_ifDEBUG_printf('N(i=%2d): x0=%d, pb(x0)=%d, w=%d, %s prev=%d,', i_pb, xinbta, y, w, w * wprev <= 0 ? 'new' : 'old', prev);
                g = 1;
                for (i_inn = 0; i_inn < 1000; i_inn++) {
                    adj = g * w;
                    if (i_pb < n_N || Math.abs(adj) < prev) {
                        tx = xinbta - adj;
                        if (0 <= tx && tx <= 1) {
                            if (prev <= acu || Math.abs(w) <= acu) {
                                R_ifDEBUG_printf(' it{in}=%d, delta(x)=%g, %s <= acu  ==> convergence\n', i_inn, -adj, prev <= acu ? 'prev' : '|w|');
                                return;
                            }
                            if (tx !== 0 && tx !== 1)
                                break;
                        }
                    }
                    g /= 3;
                }
                R_ifDEBUG_printf(' it{in}=%d, delta(x)=%g\n', i_inn, tx - xinbta);
                if (Math.abs(tx - xinbta) <= 4e-16 * (tx + xinbta))
                    return;
                xinbta = tx;
                if (tx === 0)
                    break;
                wprev = w;
            }
        warned = true;
        ML_ERROR2(ME.ME_PRECISION, 'qbeta', printer_qbeta_raw);
    };
    if (M_LN2 * DBL_MIN_EXP < u0 && u0 < -0.01 && u0 < (t * log_eps_c - log(fabs(pp * (1 - qq) * (2 - qq) / (2 * (pp + 2))))) / 2) {
        r = r * Math.exp(u0);
        if (r > -1) {
            u = u0 - Math.log1p(r) / pp;
            printer_qbeta_raw('u1-u0=%d --> choosing u = u1', u - u0);
        } else {
            u = u0;
            printer_qbeta_raw('cannot cheaply improve u0');
        }
        tx = xinbta = Math.exp(u);
        use_log_x = true;
        L_Newton();
        L_converged();
        L_return();
        return;
    }
    r = Math.sqrt(-2 * la);
    y = r - (const1 + const2 * r) / (1 + (const3 + const4 * r) * r);
    if (pp > 1 && qq > 1) {
        r = (y * y - 3) / 6;
        s = 1 / (pp + pp - 1);
        t = 1 / (qq + qq - 1);
        h = 2 / (s + t);
        w = y * Math.sqrt(h + r) / h - (t - s) * (r + 5 / 6 - 2 / (3 * h));
        printer_qbeta_raw('p,q > 1 => w=%d', w);
        if (w > 300) {
            t = w + w + Math.log(qq) - Math.log(pp);
            u = t <= 18 ? -Math.log1p(Math.exp(t)) : -t - Math.exp(-t);
            xinbta = Math.exp(u);
        } else {
            xinbta = pp / (pp + qq * Math.exp(w + w));
            u = -Math.log1p(qq / pp * Math.exp(w + w));
        }
    } else {
        r = qq + qq;
        t = 1 / (3 * Math.sqrt(qq));
        t = r * R_pow_di(1 + t * (-t + y), 3);
        s = 4 * pp + r - 2;
        R_ifDEBUG_printf('min(p,q) <= 1: t=%g', t);
        if (t === 0 || t < 0 && s >= t) {
            let l1ma;
            if (swap_tail)
                l1ma = R_DT_log(lower_tail, log_p, alpha);
            else
                l1ma = R_DT_Clog(lower_tail, log_p, alpha);
            R_ifDEBUG_printf(' t <= 0 : Math.log1p(-a)=%.15g, better l1ma=%.15g\n', Math.log1p(-a), l1ma);
            const xx = (l1ma + Math.log(qq) + logbeta) / qq;
            if (xx <= 0) {
                xinbta = -Math.expm1(xx);
                u = R_Log1_Exp(xx);
            } else {
                R_ifDEBUG_printf(' xx=%g > 0: xinbta:= 1-e^xx < 0\n', xx);
                xinbta = 0;
                u = -Infinity;
            }
        } else {
            t = s / t;
            R_ifDEBUG_printf(' t > 0 or s < t < 0:  new t = %g ( > 1 ?)\n', t);
            if (t <= 1) {
                u = (la + Math.log(pp) + logbeta) / pp;
                xinbta = Math.exp(u);
            } else {
                xinbta = 1 - 2 / (t + 1);
                u = Math.log1p(-2 / (t + 1));
            }
        }
    }
    if (swap_choose && (swap_tail && u >= -Math.exp(log_q_cut) || !swap_tail && u >= -Math.exp(4 * log_q_cut) && pp / qq < 1000)) {
        swap_tail = !swap_tail;
        R_ifDEBUG_printf(' u = %g (e^u = xinbta = %.16g) ==> ', u, xinbta);
        if (swap_tail) {
            a = R_DT_CIv(lower_tail, log_p, alpha);
            la = R_DT_Clog(lower_tail, log_p, alpha);
            pp = q;
            qq = p;
        } else {
            a = p_;
            la = R_DT_log(lower_tail, log_p, alpha);
            pp = p;
            qq = q;
        }
        R_ifDEBUG_printf('"%s\'; la = %g\n', swap_tail ? 'swap now' : 'swap back', la);
        u = R_Log1_Exp(u);
        xinbta = Math.exp(u);
    }
    if (!use_log_x)
        use_log_x = u < log_q_cut;
    const bad_u = !isFinite(u);
    const bad_init = bad_u || xinbta > p_hi;
    R_ifDEBUG_printf(' -> u = %g, e^u = xinbta = %.16g, (Newton acu=%g%s)\n', u, xinbta, acu, bad_u ? ', ** bad u **' : use_log_x ? ', on u = Math.log(x) scale' : '');
    u_n = 1;
    tx = xinbta;
    if (bad_u || u < log_q_cut) {
        w = pbeta_raw(DBL_very_MIN, pp, qq, true, log_p);
        if (w > (log_p ? la : a)) {
            R_ifDEBUG_printf(' quantile is left of smallest positive number; "convergence"\n');
            if (log_p || Math.abs(w - a) < Math.abs(0 - a)) {
                tx = DBL_very_MIN;
                u_n = DBL_log_v_MIN;
            } else {
                tx = 0;
                u_n = -Infinity;
            }
            use_log_x = log_p;
            add_N_step = false;
            L_return();
            return;
        } else {
            R_ifDEBUG_printf(' pbeta(smallest pos.) = %g <= %g  --> continuing\n', w, log_p ? la : a);
            if (u < DBL_log_v_MIN) {
                u = DBL_log_v_MIN;
                xinbta = DBL_very_MIN;
            }
        }
    }
    if (bad_init && !(use_log_x && tx > 0)) {
        if (u === -Infinity) {
            R_ifDEBUG_printf('  u = -Inf;');
            u = M_LN2 * DBL_MIN_EXP;
            xinbta = Number.MIN_VALUE;
        } else {
            R_ifDEBUG_printf(' bad_init: u=%g, xinbta=%g;', u, xinbta);
            xinbta = xinbta > 1.1 ? 0.5 : xinbta < p_lo ? Math.exp(u) : p_hi;
            if (bad_u)
                u = Math.log(xinbta);
        }
        R_ifDEBUG_printf(' -> (partly)new u=%g, xinbta=%g\n', u, xinbta);
    }
    L_Newton();
    L_converged();
    L_return();
    return;
}