import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    R_Q_P01_boundaries
} from '../../../packages/common/logger.mjs';
import { R_DT_qIv } from '../exp/expm1.mjs';
import { pnbeta } from './pnbeta.mjs';
const printer_qnbeta = debug('qnbeta');
const accu = 1e-15;
const eps = 1e-14;
export function qnbeta(p, a, b, ncp, lower_tail, log_p) {
    let ux;
    let lx;
    let nx;
    let pp;
    if (isNaN(p) || isNaN(a) || isNaN(b) || isNaN(ncp))
        return p + a + b + ncp;
    if (!isFinite(a))
        return ML_ERR_return_NAN2(printer_qnbeta, lineInfo4);
    if (ncp < 0 || a <= 0 || b <= 0)
        return ML_ERR_return_NAN2(printer_qnbeta, lineInfo4);
    const rc = R_Q_P01_boundaries(lower_tail, log_p, p, 0, 1);
    if (rc !== undefined) {
        return rc;
    }
    p = R_DT_qIv(lower_tail, log_p, p);
    if (p > 1 - Number.EPSILON)
        return 1;
    pp = Math.min(1 - Number.EPSILON, p * (1 + eps));
    for (ux = 0.5; ux < 1 - Number.EPSILON && pnbeta(ux, a, b, ncp, true, false) < pp; ux = 0.5 * (1 + ux));
    pp = p * (1 - eps);
    for (lx = 0.5; lx > Number.MIN_VALUE && pnbeta(lx, a, b, ncp, true, false) > pp; lx *= 0.5);
    do {
        nx = 0.5 * (lx + ux);
        if (pnbeta(nx, a, b, ncp, true, false) > p)
            ux = nx;
        else
            lx = nx;
    } while ((ux - lx) / nx > accu);
    return 0.5 * (ux + lx);
}