import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D__0,
    R_D__1,
    R_D_exp,
    R_D_nonint_check
} from '../../r-func.mjs';
import { dbinom_raw } from '../binomial/dbinom.mjs';
import { lgammafn_sign } from '../../special/gamma/lgammafn_sign.mjs';
const printer = debug('dnbinom');
export function dnbinom(x, size, prob, give_log) {
    if (isNaN(x) || isNaN(size) || isNaN(prob)) {
        return x + size + prob;
    }
    if (prob <= 0 || prob > 1 || size < 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    const rc = R_D_nonint_check(give_log, x, printer);
    if (rc !== undefined) {
        return rc;
    }
    if (x < 0) {
        return R_D__0(give_log);
    }
    if (x === 0 && size === 0) {
        return R_D__1(give_log);
    }
    x = Math.round(x);
    const ans = dbinom_raw(size, x + size, prob, 1 - prob, give_log);
    const p = size / (size + x);
    return give_log ? Math.log(p) + ans : p * ans;
}
const printer_dnbinom_mu = debug('dnbinom_mu');
export function dnbinom_mu(x, size, mu, give_log) {
    let p;
    if (isNaN(x) || isNaN(size) || isNaN(mu)) {
        return x + size + mu;
    }
    if (mu < 0 || size < 0) {
        return ML_ERR_return_NAN2(printer_dnbinom_mu, lineInfo4);
    }
    const rc = R_D_nonint_check(give_log, x, printer_dnbinom_mu);
    if (rc !== undefined) {
        return rc;
    }
    if (x < 0 || !isFinite(x)) {
        return R_D__0(give_log);
    }
    if (x === 0 && size === 0) {
        return R_D__1(give_log);
    }
    x = Math.round(x);
    if (x === 0) {
        let llogx;
        if (size < mu) {
            llogx = Math.log(size / (size + mu));
        } else {
            llogx = Math.log1p(-mu / (size + mu));
        }
        return R_D_exp(give_log, size * llogx);
    }
    if (x < 1e-10 * size) {
        p = size < mu ? Math.log(size / (1 + size / mu)) : Math.log(mu / (1 + mu / size));
        return R_D_exp(give_log, x * p - mu - lgammafn_sign(x + 1) + Math.log1p(x * (x - 1) / (2 * size)));
    }
    const ans = dbinom_raw(size, x + size, size / (size + mu), mu / (size + mu), give_log);
    p = size / (size + x);
    return give_log ? Math.log(p) + ans : p * ans;
}