import {
    dnbinom as _dnb,
    dnbinom_mu
} from './dnbinom.mjs';
import {
    pnbinom as _pnb,
    pnbinom_mu
} from './pnbinom.mjs';
import {
    qnbinom as _qnb,
    qnbinom_mu
} from './qnbinom.mjs';
import {
    rnbinomOne as _rnbinomOne,
    rnbinom_muOne
} from './rnbinom.mjs';
import { repeatedCall64 } from '../../r-func.mjs';
const probAndMuBoth = '"prob" and "mu" both specified';
const probMis = 'argument "prob" is missing, with no default';
export function dnbinom(x, size, prob, mu, log = false) {
    if (mu !== undefined && prob !== undefined) {
        throw new TypeError(probAndMuBoth);
    }
    if (mu !== undefined) {
        return dnbinom_mu(x, size, mu, log);
    }
    if (prob === undefined) {
        throw new TypeError(probMis);
    }
    return _dnb(x, size, prob, log);
}
export function pnbinom(q, size, prob, mu, lowerTail = true, logP = false) {
    if (mu !== undefined && prob !== undefined) {
        throw new TypeError(probAndMuBoth);
    }
    if (mu !== undefined) {
        return pnbinom_mu(q, size, mu, lowerTail, logP);
    }
    if (prob === undefined) {
        throw new TypeError(probMis);
    }
    return _pnb(q, size, prob, lowerTail, logP);
}
export function qnbinom(p, size, prob, mu, lowerTail = true, logP = false) {
    if (mu !== undefined && prob !== undefined) {
        throw new TypeError(probAndMuBoth);
    }
    if (mu !== undefined) {
        return qnbinom_mu(p, size, mu, lowerTail, logP);
    }
    if (prob === undefined) {
        throw new TypeError(probMis);
    }
    return _qnb(p, size, prob, lowerTail, logP);
}
export function rnbinom(n, size, prob, mu) {
    if (mu !== undefined && prob !== undefined) {
        throw new TypeError(probAndMuBoth);
    }
    if (mu !== undefined) {
        return repeatedCall64(n, rnbinom_muOne, size, mu);
    }
    if (prob === undefined) {
        throw new TypeError(probMis);
    }
    return repeatedCall64(n, _rnbinomOne, size, prob);
}
export function rnbinomOne(size, prob, mu) {
    if (mu !== undefined && prob !== undefined) {
        throw new TypeError(probAndMuBoth);
    }
    if (mu !== undefined) {
        return rnbinom_muOne(size, mu);
    }
    if (prob === undefined) {
        throw new TypeError(probMis);
    }
    return _rnbinomOne(size, prob);
}