import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_DT_0,
    R_DT_1
} from '../../r-func.mjs';
import { pbeta } from '../beta/pbeta.mjs';
import { Toms708 } from '../../common/toms708/toms708.mjs';
import { NumberW } from '../../common/toms708/NumberW.mjs';
const printer = debug('pnbinom');
export function pnbinom(x, size, prob, lowerTail, logP) {
    if (isNaN(x) || isNaN(size) || isNaN(prob))
        return x + size + prob;
    if (!isFinite(size) || !isFinite(prob)) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (size < 0 || prob <= 0 || prob > 1) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (size === 0)
        return x >= 0 ? R_DT_1(lowerTail, logP) : R_DT_0(lowerTail, logP);
    if (x < 0)
        return R_DT_0(lowerTail, logP);
    if (!isFinite(x))
        return R_DT_1(lowerTail, logP);
    x = Math.floor(x + 1e-7);
    return pbeta(prob, size, x + 1, lowerTail, logP);
}
const printer_pnbinom_mu = debug('pnbinom_mu');
export function pnbinom_mu(x, size, mu, lowerTail, logP) {
    if (isNaN(x) || isNaN(size) || isNaN(mu))
        return x + size + mu;
    if (!isFinite(size) || !isFinite(mu))
        return ML_ERR_return_NAN2(printer_pnbinom_mu, lineInfo4);
    if (size < 0 || mu < 0)
        return ML_ERR_return_NAN2(printer_pnbinom_mu, lineInfo4);
    if (size === 0)
        return x >= 0 ? R_DT_1(lowerTail, logP) : R_DT_0(lowerTail, logP);
    if (x < 0)
        return R_DT_0(lowerTail, logP);
    if (!isFinite(x))
        return R_DT_1(lowerTail, logP);
    x = Math.floor(x + 1e-7);
    {
        const ierr = new NumberW(0);
        const w = new NumberW(0);
        const wc = new NumberW(0);
        Toms708.bratio(size, x + 1, size / (size + mu), mu / (size + mu), w, wc, ierr);
        if (ierr.val) {
            printer('pnbinom_mu() -> bratio() gave error code %d', ierr.val);
        }
        if (logP) {
            w.val = Math.log(w.val);
            wc.val = Math.log(wc.val);
        }
        return lowerTail ? w.val : wc.val;
    }
}