import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import { R_pow_di } from '../../r-func.mjs';
import { globalUni } from '../../rng/global-rng.mjs';
import { qbinom } from './qbinom.mjs';
const printer_rbinom = debug('rbinom');
export function rbinomOne(size, prob) {
    const rng = globalUni();
    let c = 0;
    let fm = 0;
    let npq = 0;
    let p1 = 0;
    let p2 = 0;
    let p3 = 0;
    let p4 = 0;
    let qn = 0;
    let xl = 0;
    let xll = 0;
    let xlr = 0;
    let xm = 0;
    let xr = 0;
    let psave = -1;
    let m = 0;
    let f;
    let f1;
    let f2;
    let u;
    let v;
    let w;
    let w2;
    let x;
    let x1;
    let x2;
    let z;
    let z2;
    let r;
    let al;
    let alv;
    let amaxp;
    let ffm;
    let ynorm;
    let i;
    let ix = 0;
    let k;
    if (!isFinite(size))
        return ML_ERR_return_NAN2(printer_rbinom, lineInfo4);
    r = Math.round(size);
    if (r !== size)
        return ML_ERR_return_NAN2(printer_rbinom, lineInfo4);
    if (!isFinite(prob) || r < 0 || prob < 0 || prob > 1) {
        return ML_ERR_return_NAN2(printer_rbinom, lineInfo4);
    }
    if (r === 0 || prob === 0)
        return 0;
    if (prob === 1)
        return r;
    if (r >= 2147483647) {
        const _p = rng.random();
        printer_rbinom('Evade overflow:%d > MAX_SAFE_INTEGER', r);
        const retv = qbinom(_p, r, prob, false, false);
        return retv;
    }
    const n = Math.trunc(r);
    const p = Math.min(prob, 1 - prob);
    const q = 1 - p;
    const np = n * p;
    r = p / q;
    const g = r * (n + 1);
    const finis = () => {
        if (psave > 0.5) {
            ix = n - ix;
        }
        return ix;
    };
    const L_np_small = () => {
        for (;;) {
            ix = 0;
            f = qn;
            u = rng.random();
            for (;;) {
                if (u < f) {
                    return;
                }
                if (ix > 110)
                    break;
                u -= f;
                ix++;
                f *= g / ix - r;
            }
        }
    };
    psave = prob;
    if (np < 30) {
        qn = R_pow_di(q, n);
        L_np_small();
        return finis();
    } else {
        ffm = np + p;
        m = Math.trunc(ffm);
        fm = m;
        npq = np * q;
        p1 = Math.trunc(2.195 * Math.sqrt(npq) - 4.6 * q) + 0.5;
        xm = fm + 0.5;
        xl = xm - p1;
        xr = xm + p1;
        c = 0.134 + 20.5 / (15.3 + fm);
        al = (ffm - xl) / (ffm - xl * p);
        xll = al * (1 + 0.5 * al);
        al = (xr - ffm) / (xr * q);
        xlr = al * (1 + 0.5 * al);
        p2 = p1 * (1 + c + c);
        p3 = p2 + c / xll;
        p4 = p3 + c / xlr;
    }
    for (;;) {
        u = rng.random() * p4;
        v = rng.random();
        if (u <= p1) {
            ix = Math.trunc(xm - p1 * v + u);
            return finis();
        }
        if (u <= p2) {
            x = xl + (u - p1) / c;
            v = v * c + 1 - Math.abs(xm - x) / p1;
            if (v > 1 || v <= 0)
                continue;
            ix = Math.trunc(x);
        } else {
            if (u > p3) {
                ix = Math.trunc(xr - Math.log(v) / xlr);
                if (ix > n)
                    continue;
                v = v * (u - p3) * xlr;
            } else {
                ix = Math.trunc(xl + Math.log(v) / xll);
                if (ix < 0)
                    continue;
                v = v * (u - p2) * xll;
            }
        }
        k = Math.abs(ix - m);
        if (k <= 20 || k >= npq / 2 - 1) {
            f = 1;
            if (m < ix) {
                for (i = m + 1; i <= ix; i++)
                    f *= g / i - r;
            } else if (m !== ix) {
                for (i = ix + 1; i <= m; i++)
                    f /= g / i - r;
            }
            if (v <= f) {
                return finis();
            }
        } else {
            amaxp = k / npq * ((k * (k / 3 + 0.625) + 0.1666666666666) / npq + 0.5);
            ynorm = -k * k / (2 * npq);
            alv = Math.log(v);
            if (alv < ynorm - amaxp) {
                return finis();
            }
            if (alv <= ynorm + amaxp) {
                x1 = ix + 1;
                f1 = fm + 1;
                z = n + 1 - fm;
                w = n - ix + 1;
                z2 = z * z;
                x2 = x1 * x1;
                f2 = f1 * f1;
                w2 = w * w;
                if (alv <= xm * Math.log(f1 / x1) + (n - m + 0.5) * Math.log(z / w) + (ix - m) * Math.log(w * p / (x1 * q)) + (13860 - (462 - (132 - (99 - 140 / f2) / f2) / f2) / f2) / f1 / 166320 + (13860 - (462 - (132 - (99 - 140 / z2) / z2) / z2) / z2) / z / 166320 + (13860 - (462 - (132 - (99 - 140 / x2) / x2) / x2) / x2) / x1 / 166320 + (13860 - (462 - (132 - (99 - 140 / w2) / w2) / w2) / w2) / w / 166320) {
                    return finis();
                }
            }
        }
    }
}