import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D_val,
    R_DT_0,
    R_DT_1
} from '../../r-func.mjs';
import { R_D_Clog } from '../../r-func.mjs';
import { atanpi } from '../../trigonometry/tanpi.mjs';
const printer = debug('pcauchy');
export function pcauchy(x, location = 0, scale = 1, lowerTail = true, logP = false) {
    if (isNaN(x) || isNaN(location) || isNaN(scale))
        return x + location + scale;
    if (scale <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    x = (x - location) / scale;
    if (isNaN(x)) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (!isFinite(x)) {
        if (x < 0)
            return R_DT_0(lowerTail, logP);
        else
            return R_DT_1(lowerTail, logP);
    }
    if (!lowerTail)
        x = -x;
    if (Math.abs(x) > 1) {
        const y = atanpi(1 / x);
        return x > 0 ? R_D_Clog(logP, y) : R_D_val(logP, -y);
    } else {
        return R_D_val(logP, 0.5 + atanpi(x));
    }
}