import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    R_Q_P01_check
} from '../../../packages/common/logger.mjs';
import { tanpi } from '../../trigonometry/tanpi.mjs';
const printer = debug('qcauchy');
export function qcauchy(p, location = 0, scale = 1, lowerTail = true, logP = false) {
    if (isNaN(p) || isNaN(location) || isNaN(scale))
        return NaN;
    let lower_tail = lowerTail;
    const rc = R_Q_P01_check(logP, p);
    if (rc !== undefined) {
        return rc;
    }
    if (scale <= 0 || !isFinite(scale)) {
        if (scale === 0)
            return location;
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    const my_INF = lower_tail ? Infinity : -Infinity;
    if (logP) {
        if (p > -1) {
            if (p === 0) {
                return my_INF;
            }
            lower_tail = !lower_tail;
            p = -Math.expm1(p);
        } else {
            p = Math.exp(p);
        }
    } else {
        if (p > 0.5) {
            if (p === 1)
                return my_INF;
            p = 1 - p;
            lower_tail = !lower_tail;
        }
    }
    if (p === 0.5)
        return location;
    if (p === 0)
        return lower_tail ? -Infinity : Infinity;
    return location + (lower_tail ? -scale : scale) / tanpi(p);
}