import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D__0,
    R_D_val
} from '../../r-func.mjs';
import { dchisq } from './dchisq.mjs';
import { dpois_raw } from '../poisson/dpois.mjs';
const printer_dnchisq = debug('dnchisq');
const eps = 5e-15;
export function dnchisq(x, df, ncp, give_log) {
    let i;
    let q;
    let mid;
    let dfmid;
    let imax;
    let sum;
    let term;
    if (isNaN(x) || isNaN(df) || isNaN(ncp)) {
        return x + df + ncp;
    }
    if (!isFinite(df) || !isFinite(ncp) || ncp < 0 || df < 0) {
        return ML_ERR_return_NAN2(printer_dnchisq, lineInfo4);
    }
    if (x < 0) {
        return R_D__0(give_log);
    }
    if (x === 0 && df < 2)
        return Infinity;
    if (ncp === 0)
        return df > 0 ? dchisq(x, df, give_log) : R_D__0(give_log);
    if (x === Infinity)
        return R_D__0(give_log);
    const ncp2 = 0.5 * ncp;
    imax = Math.ceil((-(2 + df) + Math.sqrt((2 - df) * (2 - df) + 4 * ncp * x)) / 4);
    if (imax < 0)
        imax = 0;
    if (isFinite(imax)) {
        dfmid = df + 2 * imax;
        mid = dpois_raw(imax, ncp2, false) * dchisq(x, dfmid, false);
    } else {
        throw new Error(`Internal Error IE989: imax=${ imax }`);
    }
    sum = mid;
    term = mid;
    df = dfmid;
    i = imax;
    const x2 = x * ncp2;
    do {
        i++;
        q = x2 / i / df;
        df += 2;
        term *= q;
        sum += term;
    } while (q >= 1 || term * q > (1 - q) * eps || term > 1e-10 * sum);
    term = mid;
    df = dfmid;
    i = imax;
    while (i !== 0) {
        df -= 2;
        q = i * df / x2;
        i--;
        term *= q;
        sum += term;
        if (q < 1 && term * q <= (1 - q) * eps)
            break;
    }
    return R_D_val(give_log, sum);
}