import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D__0,
    R_D__1
} from '../../r-func.mjs';
import { dbinom_raw } from '../binomial/dbinom.mjs';
import { dgamma } from '../gamma/dgamma.mjs';
const printer_df = debug('df');
export function df(x, m, n, giveLog) {
    let f;
    let dens;
    if (isNaN(x) || isNaN(m) || isNaN(n)) {
        return x + m + n;
    }
    if (m <= 0 || n <= 0) {
        return ML_ERR_return_NAN2(printer_df, lineInfo4);
    }
    if (x < 0) {
        return R_D__0(giveLog);
    }
    if (x === 0) {
        return m > 2 ? R_D__0(giveLog) : m === 2 ? R_D__1(giveLog) : Infinity;
    }
    if (!isFinite(m) && !isFinite(n)) {
        if (x === 1) {
            return Infinity;
        } else {
            return R_D__0(giveLog);
        }
    }
    if (!isFinite(n)) {
        return dgamma(x, m / 2, 2 / m, giveLog);
    }
    if (m > 100000000000000) {
        dens = dgamma(1 / x, n / 2, 2 / n, giveLog);
        return giveLog ? dens - 2 * Math.log(x) : dens / (x * x);
    }
    f = 1 / (n + x * m);
    const q = n * f;
    const p = x * m * f;
    if (m >= 2) {
        f = m * q / 2;
        dens = dbinom_raw((m - 2) / 2, (m + n - 2) / 2, p, q, giveLog);
    } else {
        f = m * m * q / (2 * p * (m + n));
        dens = dbinom_raw(m / 2, (m + n) / 2, p, q, giveLog);
    }
    return giveLog ? Math.log(f) + dens : f * dens;
}