import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import { R_D__0 } from '../../r-func.mjs';
import { dnbeta_scalar } from '../beta/dnbeta.mjs';
import { dnchisq } from '../chi-2/dnchisq.mjs';
import { dgamma } from '../gamma/dgamma.mjs';
const printer = debug('dnf');
export function dnf(x, df1, df2, ncp, giveLog) {
    let z;
    let f;
    if (isNaN(x) || isNaN(df1) || isNaN(df2) || isNaN(ncp)) {
        return x + df2 + df1 + ncp;
    }
    if (df1 <= 0 || df2 <= 0 || ncp < 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (x < 0) {
        return R_D__0(giveLog);
    }
    if (!isFinite(ncp)) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (!isFinite(df1) && !isFinite(df2)) {
        if (x === 1)
            return Infinity;
        else
            return R_D__0(giveLog);
    }
    if (!isFinite(df2))
        return df1 * dnchisq(x * df1, df1, ncp, giveLog);
    if (df1 > 100000000000000 && ncp < 10000000) {
        f = 1 + ncp / df1;
        z = dgamma(1 / x / f, df2 / 2, 2 / df2, giveLog);
        return giveLog ? z - 2 * Math.log(x) - Math.log(f) : z / (x * x) / f;
    }
    const y = df1 / df2 * x;
    z = dnbeta_scalar(y / (1 + y), df1 / 2, df2 / 2, ncp, giveLog);
    return giveLog ? z + Math.log(df1) - Math.log(df2) - 2 * Math.log1p(y) : z * (df1 / df2) / (1 + y) / (1 + y);
}