import { rchisqOne } from '../chi-2/rchisq.mjs';
import { rnchisqOne } from '../chi-2/rnchisq.mjs';
import { repeatedCall64 } from '../../r-func.mjs';
import { df as _df } from './df.mjs';
import { dnf } from './dnf.mjs';
import { pf as _pf } from './pf.mjs';
import { pnf } from './pnf.mjs';
import { qf as _qf } from './qf.mjs';
import { qnf } from './qnf.mjs';
import { rfOne as _rfOne } from './rf.mjs';
export function df(x, df1, df2, ncp, log = false) {
    if (ncp === undefined) {
        return _df(x, df1, df2, log);
    }
    return dnf(x, df1, df2, ncp, log);
}
export function pf(q, df1, df2, ncp, lowerTail = true, logP = false) {
    if (ncp === undefined) {
        return _pf(q, df1, df2, lowerTail, logP);
    }
    return pnf(q, df1, df2, ncp, lowerTail, logP);
}
export function qf(p, df1, df2, ncp, lowerTail = true, logP = false) {
    if (ncp === undefined) {
        return _qf(p, df1, df2, lowerTail, logP);
    }
    return qnf(p, df1, df2, ncp, lowerTail, logP);
}
export function rf(n, df1, df2, ncp) {
    if (ncp === undefined) {
        return repeatedCall64(n, _rfOne, df1, df2);
    }
    if (isNaN(ncp) || isNaN(df1) || isNaN(df2) || !isFinite(df1) || !isFinite(df2)) {
        return repeatedCall64(n, () => NaN);
    }
    const noms = repeatedCall64(n, () => rnchisqOne(df1, ncp) / df1);
    for (let i = 0; i < noms.length; i++) {
        const dom = rchisqOne(df2) / df2;
        noms[i] = noms[i] / dom;
    }
    return noms;
}
export function rfOne(df1, df2, ncp) {
    if (ncp === undefined) {
        return _rfOne(df1, df2);
    }
    if (isNaN(ncp) || isNaN(df1) || isNaN(df2) || !isFinite(df1) || !isFinite(df2)) {
        return NaN;
    }
    const nom = rnchisqOne(df1, ncp) / df1;
    const dom = rchisqOne(df2) / df2;
    return nom / dom;
}