import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_DT_0,
    R_DT_1,
    R_P_bounds_01
} from '../../r-func.mjs';
import { pbeta } from '../beta/pbeta.mjs';
import { pchisq } from '../chi-2/pchisq.mjs';
const printer_pf = debug('pf');
export function pf(q, df1, df2, lowerTail, logP) {
    if (isNaN(q) || isNaN(df1) || isNaN(df2))
        return q + df2 + df1;
    if (df1 <= 0 || df2 <= 0) {
        return ML_ERR_return_NAN2(printer_pf, lineInfo4);
    }
    const rc = R_P_bounds_01(lowerTail, logP, q, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (df2 === Infinity) {
        if (df1 === Infinity) {
            if (q < 1)
                return R_DT_0(lowerTail, logP);
            if (q === 1)
                return logP ? -Math.LN2 : 0.5;
            return R_DT_1(lowerTail, logP);
        }
        return pchisq(q * df1, df1, lowerTail, logP);
    }
    if (df1 === Infinity) {
        const _d = pchisq(df2 / q, df2, !lowerTail, logP);
        return _d;
    }
    if (df1 * q > df2)
        q = pbeta(df2 / (df2 + df1 * q), df2 / 2, df1 / 2, !lowerTail, logP);
    else
        q = pbeta(df1 * q / (df2 + df1 * q), df1 / 2, df2 / 2, lowerTail, logP);
    return isFinite(q) ? q : NaN;
}