import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import { R_P_bounds_01 } from '../../r-func.mjs';
import { pnbeta2 } from '../beta/pnbeta.mjs';
import { pnchisq } from '../chi-2/pnchisq.mjs';
const printer_pnf = debug('pnf');
export function pnf(x, df1, df2, ncp, lowerTail, logP) {
    if (isNaN(x) || isNaN(df1) || isNaN(df2) || isNaN(ncp))
        return x + df2 + df1 + ncp;
    if (df1 <= 0 || df2 <= 0 || ncp < 0)
        return ML_ERR_return_NAN2(printer_pnf, lineInfo4);
    if (!isFinite(ncp))
        return ML_ERR_return_NAN2(printer_pnf, lineInfo4);
    if (!isFinite(df1) && !isFinite(df2))
        return ML_ERR_return_NAN2(printer_pnf, lineInfo4);
    const rc = R_P_bounds_01(lowerTail, logP, x, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (df2 > 100000000)
        return pnchisq(x * df1, df1, ncp, lowerTail, logP);
    const y = df1 / df2 * x;
    return pnbeta2(y / (1 + y), 1 / (1 + y), df1 / 2, df2 / 2, ncp, lowerTail, logP);
}