import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    R_Q_P01_boundaries
} from '../../../packages/common/logger.mjs';
import { qbeta } from '../beta/qbeta.mjs';
import { qchisq } from '../chi-2/qchisq.mjs';
const printer = debug('qf');
export function qf(p, df1, df2, lower_tail, log_p) {
    if (isNaN(p) || isNaN(df1) || isNaN(df2))
        return p + df1 + df2;
    if (df1 <= 0 || df2 <= 0)
        return ML_ERR_return_NAN2(printer, lineInfo4);
    const rc = R_Q_P01_boundaries(lower_tail, log_p, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (df1 <= df2 && df2 > 400000) {
        if (!isFinite(df1))
            return 1;
        return qchisq(p, df1, lower_tail, log_p) / df1;
    }
    if (df1 > 400000) {
        return df2 / qchisq(p, df2, !lower_tail, log_p);
    }
    p = (1 / qbeta(p, df2 / 2, df1 / 2, !lower_tail, log_p) - 1) * (df2 / df1);
    return isFinite(p) ? p : NaN;
}