import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    R_Q_P01_boundaries
} from '../../../packages/common/logger.mjs';
import { qnbeta } from '../beta/qnbeta.mjs';
import { qnchisq } from '../chi-2/qnchisq.mjs';
const printer = debug('qnf');
export function qnf(p, df1, df2, ncp, lowerTail, logP) {
    if (isNaN(p) || isNaN(df1) || isNaN(df2) || isNaN(ncp))
        return p + df1 + df2 + ncp;
    if (df1 <= 0 || df2 <= 0 || ncp < 0 || !isFinite(ncp) || !isFinite(df1) && !isFinite(df2)) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    const rc = R_Q_P01_boundaries(lowerTail, logP, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (df2 > 100000000)
        return qnchisq(p, df1, ncp, lowerTail, logP) / df1;
    const y = qnbeta(p, df1 / 2, df2 / 2, ncp, lowerTail, logP);
    return y / (1 - y) * (df2 / df1);
}