import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import { dpois_raw } from '../poisson/dpois.mjs';
import { R_D__0 } from '../../r-func.mjs';
const printer = debug('dgamma');
export function dgamma(x, shape, scale, aslog) {
    let pr;
    if (isNaN(x) || isNaN(shape) || isNaN(scale))
        return x + shape + scale;
    if (shape < 0 || scale <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (x < 0) {
        return R_D__0(aslog);
    }
    if (shape === 0) {
        return x === 0 ? Infinity : R_D__0(aslog);
    }
    if (x === 0) {
        if (shape < 1)
            return Infinity;
        if (shape > 1) {
            return R_D__0(aslog);
        }
        return aslog ? -Math.log(scale) : 1 / scale;
    }
    if (shape < 1) {
        pr = dpois_raw(shape, x / scale, aslog);
        return aslog ? pr + Math.log(shape / x) : pr * shape / x;
    }
    pr = dpois_raw(shape - 1, x / scale, aslog);
    return aslog ? pr - Math.log(scale) : pr / scale;
}