import { dgamma as _dgamma } from './dgamma.mjs';
import { pgamma as _pgamma } from './pgamma.mjs';
import { qgamma as _qgamma } from './qgamma.mjs';
import { rgamma as _rgamma } from './rgamma.mjs';
import { repeatedCall64 } from '../../r-func.mjs';
function gammaNormalizeParams(rate, scale) {
    if (scale === undefined) {
        if (rate === undefined) {
            return 1;
        }
        return 1 / rate;
    }
    if (rate === undefined) {
        return scale;
    }
    throw new TypeError('specify \'rate\' or \'scale\' but not both');
}
export function dgamma(x, shape, rate, scale, log = false) {
    const _scale = gammaNormalizeParams(rate, scale);
    return _dgamma(x, shape, _scale, log);
}
export function qgamma(p, shape, rate, scale, lowerTail = true, logP = false) {
    const _scale = gammaNormalizeParams(rate, scale);
    return _qgamma(p, shape, _scale, lowerTail, logP);
}
export function pgamma(q, shape, rate, scale, lowerTail = true, logP = false) {
    const _scale = gammaNormalizeParams(rate, scale);
    return _pgamma(q, shape, _scale, lowerTail, logP);
}
export function rgamma(n, shape, rate, scale) {
    const _scale = gammaNormalizeParams(rate, scale);
    return repeatedCall64(n, _rgamma, shape, _scale);
}
export function rgammaOne(shape, rate, scale) {
    const _scale = gammaNormalizeParams(rate, scale);
    return _rgamma(shape, _scale);
}