import { debug } from '@mangos/debug';
import { dbinom_raw } from '../binomial/dbinom.mjs';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D__0,
    R_D_nonint_check,
    log as _log,
    round
} from '../../r-func.mjs';
const printer = debug('dgeom');
export function dgeom(x, p, log = false) {
    if (isNaN(x) || isNaN(p))
        return x + p;
    if (p <= 0 || p > 1) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    const rc = R_D_nonint_check(log, x, printer);
    if (rc !== undefined) {
        return rc;
    }
    if (x < 0 || !isFinite(x) || p === 0) {
        return R_D__0(log);
    }
    x = round(x);
    const prob = dbinom_raw(0, x, p, 1 - p, log);
    return log ? _log(p) + prob : p * prob;
}