import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_DT_0,
    R_DT_1
} from '../../r-func.mjs';
import { R_DT_Clog } from '../exp/expm1.mjs';
const printer = debug('pgeom');
export function pgeom(q, p, lowerTail = true, logP = false) {
    if (isNaN(q) || isNaN(p))
        return NaN;
    if (p <= 0 || p > 1) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (q < 0)
        return R_DT_0(lowerTail, logP);
    if (!isFinite(q))
        return R_DT_1(lowerTail, logP);
    q = Math.floor(q + 1e-7);
    if (p === 1) {
        q = lowerTail ? 1 : 0;
        return logP ? Math.log(q) : q;
    }
    q = Math.log1p(-p) * (q + 1);
    if (logP)
        return R_DT_Clog(lowerTail, logP, q);
    else
        return lowerTail ? -Math.expm1(q) : Math.exp(q);
}