import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D_Lval,
    R_DT_0,
    R_DT_1
} from '../../r-func.mjs';
import { R_DT_log } from '../exp/expm1.mjs';
import { dhyper } from './dhyper.mjs';
function pdhyper(x, NR, NB, n, logP) {
    let sum = 0;
    let term = 1;
    while (x > 0 && term >= Number.EPSILON * sum) {
        term *= x * (NB - n + x) / (n + 1 - x) / (NR + 1 - x);
        sum += term;
        x--;
    }
    const ss = sum;
    return logP ? Math.log1p(ss) : 1 + ss;
}
const printer_phyper = debug('phyper');
export function phyper(q, m, n, k, lowerTail = true, logP = false) {
    let lower_tail = lowerTail;
    if (isNaN(q) || isNaN(m) || isNaN(n) || isNaN(k))
        return NaN;
    q = Math.floor(q + 1e-7);
    m = Math.round(m);
    n = Math.round(n);
    k = Math.round(k);
    if (m < 0 || n < 0 || !isFinite(m + n) || k < 0 || k > m + n) {
        return ML_ERR_return_NAN2(printer_phyper, lineInfo4);
    }
    const ox = q;
    const onn = k;
    const onr = m;
    const onb = n;
    if (q * (m + n) > k * m) {
        const oldnb = n;
        n = m;
        m = oldnb;
        q = k - q - 1;
        lower_tail = !lower_tail;
    }
    if (q < 0 || q < k - n)
        return R_DT_0(lower_tail, logP);
    if (q >= m) {
        return R_DT_1(lower_tail, logP);
    }
    if (q >= k) {
        printer_phyper('trace q>=k q=%d m=%d n=%d k=%d', ox, onr, onb, onn);
        return R_DT_1(lower_tail, logP);
    }
    const d = dhyper(q, m, n, k, logP);
    if (!logP && d == 0 || logP && d == -Infinity) {
        return R_DT_0(lowerTail, logP);
    }
    const pd = pdhyper(q, m, n, k, logP);
    return logP ? R_DT_log(lower_tail, logP, d + pd) : R_D_Lval(lower_tail, d * pd);
}