import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    R_Q_P01_boundaries
} from '../../../packages/common/logger.mjs';
import { lfastchoose } from '../../special/choose/index.mjs';
import { R_DT_qIv } from '../exp/expm1.mjs';
import { DBL_EPSILON } from '../../r-func.mjs';
const printer_qhyper = debug('qhyper');
const _d = new Float64Array(7);
const ixr = 0;
const isum = 1;
const ixb = 2;
const iterm = 3;
const iNR = 4;
const iNB = 5;
let backendTinyN = cpuBackendTinyN;
let backendBigN = cpuBackendBigN;
export function registerBackend(fns) {
    backendTinyN = fns.calcTinyN;
    backendBigN = fns.calcBigN;
}
export function unRegisterBackend() {
    const previous = !!backendTinyN && !!backendBigN;
    backendTinyN = cpuBackendTinyN;
    backendBigN = cpuBackendBigN;
    return previous;
}
function cpuBackendTinyN(sum, term, p, xr, end, xb, NB, NR) {
    while (sum < p && xr < end) {
        xr++;
        NB++;
        term *= NR / xr * (xb / NB);
        sum += term;
        xb--;
        NR--;
    }
    return xr;
}
function cpuBackendBigN(sum, term, p, xr, end, xb, NB, NR) {
    while (sum < p && xr < end) {
        xr++;
        NB++;
        term += Math.log(NR / xr * (xb / NB));
        sum += Math.exp(term);
        xb--;
        NR--;
    }
    return xr;
}
export function qhyper(p, m, n, k, lowerTail = true, logP = false) {
    if (isNaN(p) || isNaN(m) || isNaN(n) || isNaN(k)) {
        return NaN;
    }
    if (!isFinite(p) || !isFinite(m) || !isFinite(n) || !isFinite(k)) {
        return ML_ERR_return_NAN2(printer_qhyper, lineInfo4);
    }
    _d[iNR] = Math.round(m);
    _d[iNB] = Math.round(n);
    const N = _d[iNR] + _d[iNB];
    k = Math.round(k);
    if (_d[iNR] < 0 || _d[iNB] < 0 || k < 0 || k > N)
        return ML_ERR_return_NAN2(printer_qhyper, lineInfo4);
    const xstart = Math.max(0, k - _d[iNB]);
    const xend = Math.min(k, _d[iNR]);
    const rc = R_Q_P01_boundaries(lowerTail, logP, p, xstart, xend);
    if (rc !== undefined) {
        return rc;
    }
    _d[ixr] = xstart;
    _d[ixb] = k - _d[ixr];
    const small_N = N < 1000;
    _d[iterm] = lfastchoose(_d[iNR], _d[ixr]) + lfastchoose(_d[iNB], _d[ixb]) - lfastchoose(N, k);
    if (small_N)
        _d[iterm] = Math.exp(_d[iterm]);
    _d[iNR] -= _d[ixr];
    _d[iNB] -= _d[ixb];
    if (!lowerTail || logP) {
        p = R_DT_qIv(lowerTail, logP, p);
    }
    p *= 1 - 1000 * DBL_EPSILON;
    _d[isum] = small_N ? _d[iterm] : Math.exp(_d[iterm]);
    return small_N ? backendTinyN(_d[isum], _d[iterm], p, _d[ixr], xend, _d[ixb], _d[iNB], _d[iNR]) : backendBigN(_d[isum], _d[iterm], p, _d[ixr], xend, _d[ixb], _d[iNB], _d[iNR]);
}