import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    R_Q_P01_boundaries
} from '../../../packages/common/logger.mjs';
import { log } from '../../r-func.mjs';
import { R_Log1_Exp } from '../exp/expm1.mjs';
const printer_qlogis = debug('qlogis');
export function qlogis(p, location = 0, scale = 1, lowerTail = true, logP = false) {
    if (isNaN(p) || isNaN(location) || isNaN(scale))
        return NaN;
    const rc = R_Q_P01_boundaries(lowerTail, logP, p, -Infinity, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (scale < 0) {
        return ML_ERR_return_NAN2(printer_qlogis, lineInfo4);
    }
    if (scale === 0)
        return location;
    if (logP) {
        if (lowerTail)
            p = p - R_Log1_Exp(p);
        else
            p = R_Log1_Exp(p) - p;
    } else
        p = log(lowerTail ? p / (1 - p) : (1 - p) / p);
    return location + scale * p;
}