import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    M_1_SQRT_2PI,
    M_LN_SQRT_2PI,
    R_D__0,
    log as _log
} from '../../r-func.mjs';
const printer = debug('dlnorm');
export function dlnorm(x, meanlog = 0, sdlog = 1, log = false) {
    if (isNaN(x) || isNaN(meanlog) || isNaN(sdlog)) {
        return x + meanlog + sdlog;
    }
    if (sdlog <= 0) {
        if (sdlog < 0) {
            return ML_ERR_return_NAN2(printer, lineInfo4);
        }
        return Math.log(x) === meanlog ? Infinity : R_D__0(log);
    }
    if (x <= 0) {
        return R_D__0(log);
    }
    const y = (_log(x) - meanlog) / sdlog;
    return log ? -(M_LN_SQRT_2PI + 0.5 * y * y + _log(x * sdlog)) : M_1_SQRT_2PI * Math.exp(-0.5 * y * y) / (x * sdlog);
}