import { debug } from '@mangos/debug';
import { lgammafn_sign } from '../../special/gamma/lgammafn_sign.mjs';
import {
    sumfp,
    log as _log,
    exp,
    trunc
} from '../../r-func.mjs';
const printer = debug('dmultinom');
function isZeroOrPositiveAndFinite(x) {
    return x >= 0;
}
export function dmultinomLikeR(x, prob, log = false) {
    const rc = dmultinom(x, prob, log);
    if (isNaN(rc)) {
        throw new Error(`Error in dmultinom`);
    }
    return rc;
}
export function dmultinom(x, prob, log = false) {
    if (x.length !== prob.length) {
        printer('x[] and prob[] must be equal length vectors.');
        return NaN;
    }
    const s = sumfp(prob);
    if (prob.every(isFinite) === false || prob.every(_p => _p >= 0) === false || s === 0) {
        printer('probabilities must be finite, non-negative and not all 0');
        return NaN;
    }
    prob.forEach((v, i, arr) => {
        arr[i] = v / s;
    });
    x.forEach((v, i, arr) => {
        arr[i] = trunc(v + 0.5);
    });
    if (x.every(isZeroOrPositiveAndFinite) === false) {
        printer('\'x\' must be non-negative (and finite)');
        return NaN;
    }
    const N = sumfp(x);
    let rc = lgammafn_sign(N + 1);
    for (let i = 0; i < prob.length; i++) {
        if (prob[i] === 0) {
            if (x[i] !== 0) {
                return log ? -Infinity : 0;
            }
            continue;
        }
        rc += x[i] * _log(prob[i]) - lgammafn_sign(x[i] + 1);
    }
    return log ? rc : exp(rc);
}