import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    exp,
    round,
    log as _log,
    sqrt,
    abs
} from '../../r-func.mjs';
import {
    DBL_MANT_DIG,
    DBL_MIN_EXP,
    ldexp,
    M_1_SQRT_2PI,
    M_LN2,
    M_LN_SQRT_2PI,
    R_D__0
} from '../../r-func.mjs';
const printer = debug('dnorm');
const PRECISION_LIMIT = sqrt(-2 * M_LN2 * (DBL_MIN_EXP + 1 - DBL_MANT_DIG));
export function dnorm4(x, mean = 0, sd = 1, log = false) {
    if (isNaN(x) || isNaN(mean) || isNaN(sd)) {
        return NaN;
    }
    if (!isFinite(sd)) {
        return R_D__0(log);
    }
    if (!isFinite(x) && mean === x) {
        return NaN;
    }
    if (sd <= 0) {
        if (sd < 0) {
            return ML_ERR_return_NAN2(printer, lineInfo4);
        }
        return x === mean ? Infinity : R_D__0(log);
    }
    x = (x - mean) / sd;
    if (!isFinite(x))
        return R_D__0(log);
    x = abs(x);
    if (x >= 2 * sqrt(Number.MAX_VALUE)) {
        return R_D__0(log);
    }
    if (log) {
        return -(M_LN_SQRT_2PI + 0.5 * x * x + _log(sd));
    }
    if (x < 5)
        return M_1_SQRT_2PI * exp(-0.5 * x * x) / sd;
    if (x > PRECISION_LIMIT) {
        return 0;
    }
    const x1 = ldexp(round(ldexp(x, 16)), -16);
    const x2 = x - x1;
    return M_1_SQRT_2PI / sd * (exp(-0.5 * x1 * x1) * exp((-0.5 * x2 - x1) * x2));
}