'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D__0,
    R_D__1,
    R_D_exp,
    R_D_fexp,
    R_D_nonint_check,
    DBL_MIN,
    M_2PI,
    log
} from '../../r-func.mjs';
import { bd0 } from '../../deviance/index.mjs';
import { lgammafn_sign as lgammafn } from '../../special/gamma/lgammafn_sign.mjs';
import { stirlerr } from '../../stirling/index.mjs';
const printer = debug('dpois');
export function dpois_raw(x, lambda, give_log) {
    if (lambda === 0) {
        return x === 0 ? R_D__1(give_log) : R_D__0(give_log);
    }
    if (!isFinite(lambda)) {
        return R_D__0(give_log);
    }
    if (x <= lambda * DBL_MIN) {
        return R_D_exp(give_log, -lambda);
    }
    if (lambda < x * DBL_MIN) {
        if (!isFinite(x)) {
            return R_D__0(give_log);
        }
        const gammafn = lgammafn(x + 1);
        const xlogLambda = x * log(lambda);
        const inp = -lambda + xlogLambda - gammafn;
        return R_D_exp(give_log, inp);
    }
    return R_D_fexp(give_log, M_2PI * x, -stirlerr(x) - bd0(x, lambda));
}
export function dpois(x, lambda, log = false) {
    if (isNaN(x) || isNaN(lambda)) {
        return NaN;
    }
    if (lambda < 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    const rc = R_D_nonint_check(log, x, printer);
    if (rc !== undefined) {
        return rc;
    }
    if (x < 0 || !isFinite(x)) {
        return R_D__0(log);
    }
    return dpois_raw(x, lambda, log);
}