'use strict';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_DT_0,
    R_DT_1,
    isFinite,
    floor
} from '../../r-func.mjs';
import { debug } from '@mangos/debug';
import { pgamma } from '../gamma/pgamma.mjs';
const printer = debug('ppois');
export function ppois(q, lambda, lowerTail = true, logP = false) {
    if (isNaN(q) || isNaN(lambda))
        return q + lambda;
    if (lambda < 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (q < 0) {
        return R_DT_0(lowerTail, logP);
    }
    if (lambda === 0) {
        return R_DT_1(lowerTail, logP);
    }
    if (!isFinite(q)) {
        return R_DT_1(lowerTail, logP);
    }
    q = floor(q + 1e-7);
    return pgamma(lambda, q + 1, 1, !lowerTail, logP);
}