'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    R_Q_P01_boundaries
} from '../../../packages/common/logger.mjs';
import { NumberW } from '../../common/toms708/NumberW.mjs';
import { ppois } from './ppois.mjs';
import { qnorm } from '../normal/qnorm.mjs';
import { R_DT_qIv } from '../exp/expm1.mjs';
import {
    max,
    sqrt,
    floor,
    round as nearbyint,
    isNaN,
    DBL_EPSILON,
    isFinite
} from '../../r-func.mjs';
function do_search(y, z, p, lambda, incr) {
    if (z.val >= p) {
        for (;;) {
            if (y === 0 || (z.val = ppois(y - incr, lambda, true, false)) < p)
                return y;
            y = max(0, y - incr);
        }
    } else {
        for (;;) {
            y = y + incr;
            if ((z.val = ppois(y, lambda, true, false)) >= p)
                return y;
        }
    }
}
const printer = debug('qpois');
export function qpois(p, lambda, lowerTail = true, logP = false) {
    let y;
    const z = new NumberW(0);
    if (isNaN(p) || isNaN(lambda))
        return p + lambda;
    if (!isFinite(lambda)) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (lambda < 0)
        return ML_ERR_return_NAN2(printer, lineInfo4);
    if (lambda === 0)
        return 0;
    const rc = R_Q_P01_boundaries(lowerTail, logP, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (!lowerTail || logP) {
        p = R_DT_qIv(lowerTail, logP, p);
        if (p === 0)
            return 0;
        if (p === 1)
            return Infinity;
    }
    if (p + 1.01 * DBL_EPSILON >= 1)
        return Infinity;
    const mu = lambda;
    const sigma = sqrt(lambda);
    const gamma = 1 / sigma;
    z.val = qnorm(p, 0, 1, true, false);
    y = nearbyint(mu + sigma * (z.val + gamma * (z.val * z.val - 1) / 6));
    z.val = ppois(y, lambda, true, false);
    p *= 1 - 64 * DBL_EPSILON;
    if (lambda < 100000)
        return do_search(y, z, p, lambda, 1);
    {
        let incr = floor(y * 0.001);
        let oldincr;
        do {
            oldincr = incr;
            y = do_search(y, z, p, lambda, incr);
            incr = max(1, floor(incr / 100));
        } while (oldincr > 1 && incr > lambda * 1e-15);
        return y;
    }
}