'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import { globalNorm } from '../../rng/global-rng.mjs';
import {
    imax2,
    imin2,
    M_1_SQRT_2PI,
    trunc,
    log,
    abs,
    pow,
    exp,
    floor,
    sqrt,
    isFinite
} from '../../r-func.mjs';
import { fsign } from './fsign.mjs';
import { exp_rand } from '../exp/sexp.mjs';
const a0 = -0.5;
const a1 = 0.3333333;
const a2 = -0.2500068;
const a3 = 0.2000118;
const a4 = -0.1661269;
const a5 = 0.1421878;
const a6 = -0.1384794;
const a7 = 0.125006;
const one_7 = 0.14285714285714285;
const one_12 = 0.08333333333333333;
const one_24 = 0.041666666666666664;
const printer_rpois = debug('rpois');
const fact = [
    1,
    1,
    2,
    6,
    24,
    120,
    720,
    5040,
    40320,
    362880
];
export function rpoisOne(lambda) {
    const rng = globalNorm();
    let l = 0;
    let m = 0;
    const pp = new Float32Array(36);
    let b1 = 0;
    let b2 = 0;
    let c = 0;
    let c0 = 0;
    let c1 = 0;
    let c2 = 0;
    let c3 = 0;
    let p0 = 0;
    let p = 0;
    let q = 0;
    let s = 0;
    let d = 0;
    let omega = 0;
    let big_l = 0;
    let del;
    let difmuk = 0;
    let E = 0;
    let fk = 0;
    let fx;
    let fy;
    let px;
    let py;
    let t = 0;
    let u = 0;
    let v;
    let x;
    let pois = -1;
    let k;
    let kflag = 0;
    if (!isFinite(lambda) || lambda < 0) {
        return ML_ERR_return_NAN2(printer_rpois, lineInfo4);
    }
    if (lambda === 0)
        return 0;
    const big_mu = lambda >= 10;
    if (big_mu) {
        s = sqrt(lambda);
        d = 6 * lambda * lambda;
        big_l = floor(lambda - 1.1484);
    } else {
        m = imax2(1, trunc(lambda));
        l = 0;
        q = p0 = p = exp(-lambda);
        for (;;) {
            u = rng.uniform_rng.random();
            if (u <= p0)
                return 0;
            if (l !== 0) {
                for (k = u <= 0.458 ? 1 : imin2(l, m); k <= l; k++)
                    if (u <= pp[k])
                        return k;
                if (l === 35)
                    continue;
            }
            l++;
            for (k = l; k <= 35; k++) {
                p *= lambda / k;
                q += p;
                pp[k] = q;
                if (u <= q) {
                    l = k;
                    return k;
                }
            }
            l = 35;
        }
    }
    const g = lambda + s * rng.random();
    if (g >= 0) {
        pois = floor(g);
        if (pois >= big_l)
            return pois;
        fk = pois;
        difmuk = lambda - fk;
        u = rng.uniform_rng.random();
        if (d * u >= difmuk * difmuk * difmuk)
            return pois;
    }
    if (big_mu) {
        omega = M_1_SQRT_2PI / s;
        b1 = one_24 / lambda;
        b2 = 0.3 * b1 * b1;
        c3 = one_7 * b1 * b2;
        c2 = b2 - 15 * c3;
        c1 = b1 - 6 * b2 + 45 * c3;
        c0 = 1 - b1 + 3 * b2 - 15 * c3;
        c = 0.1069 / lambda;
    }
    let gotoStepF = false;
    let once = true;
    for (;;) {
        if (once) {
            once = false;
            if (g >= 0) {
                kflag = 0;
                gotoStepF = true;
            }
        }
        if (!gotoStepF) {
            E = exp_rand(rng.uniform_rng);
            u = 2 * rng.uniform_rng.random() - 1;
            t = 1.8 + fsign(E, u >= 0);
        }
        if (t > -0.6744 || gotoStepF) {
            if (!gotoStepF) {
                pois = floor(lambda + s * t);
                fk = pois;
                difmuk = lambda - fk;
                kflag = 1;
            }
            gotoStepF = false;
            if (pois < 10) {
                px = -lambda;
                py = pow(lambda, pois) / fact[trunc(pois)];
            } else {
                del = one_12 / fk;
                del = del * (1 - 4.8 * del * del);
                v = difmuk / fk;
                if (abs(v) <= 0.25)
                    px = fk * v * v * (((((((a7 * v + a6) * v + a5) * v + a4) * v + a3) * v + a2) * v + a1) * v + a0) - del;
                else
                    px = fk * log(1 + v) - difmuk - del;
                py = M_1_SQRT_2PI / sqrt(fk);
            }
            x = (0.5 - difmuk) / s;
            x *= x;
            fx = -0.5 * x;
            fy = omega * (((c3 * x + c2) * x + c1) * x + c0);
            if (kflag > 0) {
                if (c * abs(u) <= py * exp(px + E) - fy * exp(fx + E)) {
                    break;
                }
            } else if (fy - u * fy <= py * exp(px - fx)) {
                break;
            }
        }
    }
    return pois;
}