'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import { cpu_csignrank } from './csignrank.mjs';
import {
    R_D__0,
    R_D_exp,
    isNaN,
    trunc,
    abs,
    log as _log,
    M_LN2,
    round
} from '../../r-func.mjs';
import {
    growMemory,
    memory
} from './csignrank_wasm.mjs';
let _csignrank = cpu_csignrank;
function registerBackend(fns) {
    _csignrank = fns.csignrank;
}
function unRegisterBackend() {
    _csignrank = cpu_csignrank;
    return _csignrank === cpu_csignrank ? false : true;
}
export {
    unRegisterBackend,
    registerBackend
};
const printer = debug('dsignrank');
export function dsignrank(x, n, log = false) {
    if (isNaN(x) || isNaN(n)) {
        return x + n;
    }
    if (n <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (abs(x - round(x)) > 1e-7) {
        return R_D__0(log);
    }
    n = trunc(n);
    x = round(x);
    if (x < 0 || x > n * (n + 1) / 2) {
        return R_D__0(log);
    }
    const u = n * (n + 1) / 2;
    const c = Math.trunc(u / 2);
    growMemory(c + 1);
    new Float64Array(memory.buffer).fill(0, 0, c + 1);
    const d = R_D_exp(log, _log(_csignrank(trunc(x), n, u, c)) - n * M_LN2);
    return d;
}