'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_DT_0,
    R_DT_1,
    R_DT_val,
    round,
    trunc,
    M_LN2,
    exp,
    isNaN,
    isFinite
} from '../../r-func.mjs';
import { cpu_csignrank } from './csignrank.mjs';
import {
    growMemory,
    memory
} from './csignrank_wasm.mjs';
const printer = debug('psignrank');
let _csignrank = cpu_csignrank;
function registerBackend(fns) {
    _csignrank = fns.csignrank;
}
function unRegisterBackend() {
    _csignrank = cpu_csignrank;
    return _csignrank === cpu_csignrank ? false : true;
}
export {
    unRegisterBackend,
    registerBackend
};
export function psignrank(q, n, lowerTail = true, logP = false) {
    if (isNaN(q) || isNaN(n)) {
        return NaN;
    }
    if (!isFinite(n) || n <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    q = round(q + 1e-7);
    if (q < 0) {
        return R_DT_0(lowerTail, logP);
    }
    if (q >= n * (n + 1) / 2) {
        return R_DT_1(lowerTail, logP);
    }
    n = round(n);
    const u = n * (n + 1) / 2;
    const c = trunc(u / 2);
    growMemory(c + 1);
    new Float64Array(memory.buffer).fill(0, 0, c + 1);
    const f = exp(-n * M_LN2);
    let p = 0;
    if (q <= u / 2) {
        for (let i = 0; i <= q; i++) {
            p += _csignrank(i, n, u, c) * f;
        }
    } else {
        q = n * (n + 1) / 2 - q;
        for (let i = 0; i < q; i++) {
            p += _csignrank(i, n, u, c) * f;
        }
        lowerTail = !lowerTail;
    }
    return R_DT_val(lowerTail, logP, p);
}