'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    M_LN_SQRT_PI,
    R_D__0,
    sqrt,
    abs,
    DBL_EPSILON,
    exp,
    log
} from '../../r-func.mjs';
import { lgammafn_sign as lgamma } from '../../special/gamma/index.mjs';
import { dnorm } from '../normal/index.mjs';
import { _dt } from './dt.mjs';
import { pnt } from './pnt.mjs';
const printer_dnt = debug('dnt');
export function dnt(x, df, ncp = 0, giveLog = false) {
    if (isNaN(x) || isNaN(df) || isNaN(ncp)) {
        return x + df + ncp;
    }
    if (df <= 0) {
        return ML_ERR_return_NAN2(printer_dnt, lineInfo4);
    }
    if (!isFinite(x)) {
        return R_D__0(giveLog);
    }
    if (ncp === 0) {
        return _dt(x, df, giveLog);
    }
    if (df > 100000000) {
        return dnorm(x, ncp, 1, giveLog);
    }
    let u;
    if (abs(x) > sqrt(df * DBL_EPSILON)) {
        printer_dnt('abs(x:%d)>sqrt(df*espsilon):%d', abs(x), sqrt(df * DBL_EPSILON));
        u = log(df) - log(abs(x)) + log(abs(pnt(x * sqrt((df + 2) / df), df + 2, ncp, true, false) - pnt(x, df, ncp, true, false)));
    } else {
        printer_dnt('abs(x:%d)<=sqrt(df*espsilon):%d', abs(x), sqrt(df * DBL_EPSILON));
        u = lgamma((df + 1) / 2) - lgamma(df / 2) - (M_LN_SQRT_PI + 0.5 * (log(df) + ncp * ncp));
    }
    printer_dnt('u=%d, giveLog=%s', u, giveLog);
    return giveLog ? u : exp(u);
}