'use strict';
import {
    M_1_SQRT_2PI,
    M_LN_SQRT_2PI,
    abs,
    log1p,
    log,
    sqrt,
    exp,
    DBL_EPSILON
} from '../../r-func.mjs';
import { bd0 } from '../../deviance/index.mjs';
import { dnorm } from '../normal/index.mjs';
import { stirlerr } from '../../stirling/index.mjs';
export function _dt(x, n, giveLog = false) {
    if (!isFinite(n)) {
        return dnorm(x, 0, 1, giveLog);
    }
    let u;
    const t = -bd0(n / 2, (n + 1) / 2) + stirlerr((n + 1) / 2) - stirlerr(n / 2);
    const x2n = x * x / n;
    let ax = 0;
    let l_x2n;
    const lrg_x2n = x2n > 1 / DBL_EPSILON;
    if (lrg_x2n) {
        ax = abs(x);
        l_x2n = log(ax) - log(n) / 2;
        u = n * l_x2n;
    } else if (x2n > 0.2) {
        l_x2n = log(1 + x2n) / 2;
        u = n * l_x2n;
    } else {
        l_x2n = log1p(x2n) / 2;
        u = -bd0(n / 2, (n + x * x) / 2) + x * x / 2;
    }
    if (giveLog)
        return t - u - (M_LN_SQRT_2PI + l_x2n);
    const I_sqrt_ = lrg_x2n ? sqrt(n) / ax : exp(-l_x2n);
    return exp(t - u) * M_1_SQRT_2PI * I_sqrt_;
}