'use strict';
import { rchisq } from '../chi-2/index.mjs';
import { rnorm } from '../normal/index.mjs';
import { rtOne } from './rt.mjs';
import { dnt } from './dnt.mjs';
import { pnt } from './pnt.mjs';
import { qnt } from './qnt.mjs';
import { qt as _qt } from './qt.mjs';
import {
    repeatedCall64,
    sqrt
} from '../../r-func.mjs';
export function dt(x, df, ncp = 0, log = false) {
    return dnt(x, df, ncp, log);
}
export function pt(q, df, ncp = 0, lowerTail = true, logP = false) {
    return pnt(q, df, ncp, lowerTail, logP);
}
export function qt(p, df, ncp, lowerTail = true, logP = false) {
    return ncp === undefined ? _qt(p, df, lowerTail, logP) : qnt(p, df, ncp, lowerTail, logP);
}
export function rt(n, df, ncp) {
    if (ncp === undefined) {
        return repeatedCall64(n, rtOne, df);
    } else {
        const norm = rnorm(n, ncp, 1);
        const chisq = rchisq(n, df, undefined);
        for (let i = 0; i < n; i++) {
            chisq[i] /= df;
            chisq[i] = sqrt(chisq[i]);
            norm[i] /= chisq[i];
        }
        return norm;
    }
}
export {
    rtOne
};