'use strict';
import { debug } from '@mangos/debug';
import { pbeta } from '../beta/pbeta.mjs';
import {
    ME,
    ML_ERR_return_NAN2,
    lineInfo4,
    ML_ERROR2
} from '../../../packages/common/logger.mjs';
import {
    M_LN_SQRT_PI,
    M_SQRT_2dPI,
    R_DT_0,
    R_DT_1,
    R_DT_val,
    isFinite,
    DBL_EPSILON,
    sqrt,
    exp,
    pow,
    log,
    expm1,
    abs,
    min,
    M_LN2
} from '../../r-func.mjs';
import { lgamma } from '../../special/gamma/index.mjs';
import { pnorm5 as pnorm } from '../normal/pnorm.mjs';
import { pt } from './pt.mjs';
const printer_pnt = debug('pnt');
const DBL_MIN_EXP = -1021;
const itrmax = 1000;
const errmax = 1e-12;
function finis(tnc, del, lower_tail, negdel, log_p) {
    const pn = pnorm(-del, 0, 1, true, false);
    printer_pnt('pnorm(%d,0,1)=%d', -del, pn);
    tnc += pn;
    printer_pnt('tnc=%d', tnc);
    lower_tail = lower_tail !== negdel;
    if (tnc > 1 - 1e-10 && lower_tail) {
        ML_ERROR2(ME.ME_PRECISION, 'pnt{final}', printer_pnt);
    }
    const rc = R_DT_val(lower_tail, log_p, min(tnc, 1));
    printer_pnt('rc:%d, tnc:%d, log_p:%s, lower_tail:%s', rc, tnc, log_p, lower_tail);
    return rc;
}
export function pnt(t, df, ncp, lower_tail, log_p) {
    let errbd;
    let rxb;
    let x;
    let geven;
    let godd;
    let q;
    let s;
    let tnc = 0;
    let xeven;
    let xodd;
    if (isNaN(t) || isNaN(df) || isNaN(ncp)) {
        return t + df + ncp;
    }
    if (df <= 0) {
        return ML_ERR_return_NAN2(printer_pnt, lineInfo4);
    }
    if (!isFinite(t)) {
        return t < 0 ? R_DT_0(lower_tail, log_p) : R_DT_1(lower_tail, log_p);
    }
    if (ncp === 0) {
        return pt(t, df, lower_tail, log_p);
    }
    let negdel;
    let del;
    let tt;
    if (t >= 0) {
        negdel = false;
        tt = t;
        del = ncp;
    } else {
        if (ncp > 40 && (!log_p || !lower_tail)) {
            printer_pnt('if x <=0 and solution for edge ncp > 40');
            return R_DT_0(lower_tail, log_p);
        }
        negdel = true;
        tt = -t;
        del = -ncp;
    }
    if (df > 400000 || del * del > 2 * M_LN2 * -DBL_MIN_EXP) {
        printer_pnt('Abramowitz & Stegun 26.7.10 ncp:%d, del:%d, ncp2:%d, del2:%d, D:%d', ncp, del, ncp * ncp, del * del, 2 * M_LN2 * -DBL_MIN_EXP);
        const _s = 1 / (4 * df);
        return pnorm(tt * (1 - _s), del, sqrt(1 + tt * tt * 2 * _s), lower_tail !== negdel, log_p);
    }
    x = t * t;
    rxb = df / (x + df);
    x = x / (x + df);
    printer_pnt('pnt(t=%d, df=%d, ncp=%d, rxb=%d) ==> x=%d', t, df, ncp, rxb, x);
    if (x > 0) {
        printer_pnt('x > 0 branch');
        const lambda = del * del;
        let p = 0.5 * exp(-0.5 * lambda);
        printer_pnt('p=%d', p);
        if (p === 0) {
            printer_pnt('p=%d, underflow protection', p);
            ML_ERROR2(ME.ME_UNDERFLOW, 'pnt', printer_pnt);
            ML_ERROR2(ME.ME_RANGE, 'pnt', printer_pnt);
            return R_DT_0(lower_tail, log_p);
        }
        printer_pnt('it  1e5*(godd,   geven)|          p           q           s' + '        pnt(*)     errbd');
        q = M_SQRT_2dPI * p * del;
        s = 0.5 - p;
        if (s < 1e-7) {
            s = -0.5 * expm1(-0.5 * lambda);
        }
        let a = 0.5;
        const b = 0.5 * df;
        rxb = pow(rxb, b);
        const albeta = M_LN_SQRT_PI + lgamma(b) - lgamma(0.5 + b);
        printer_pnt('%d = lgamma(%d)-lgamma(0.5+%d)+0.572364942924700087071713675677', albeta, b, b);
        xodd = pbeta(x, a, b, true, false);
        printer_pnt('return from pbeta:%d', xodd);
        godd = 2 * rxb * exp(a * log(x) - albeta);
        tnc = b * x;
        xeven = tnc < DBL_EPSILON ? tnc : 1 - rxb;
        geven = tnc * rxb;
        tnc = p * xodd + q * xeven;
        for (let it = 1; it <= itrmax; it++) {
            a += 1;
            xodd -= godd;
            xeven -= geven;
            godd *= x * (a + b - 1) / a;
            geven *= x * (a + b - 0.5) / (a + 0.5);
            p *= lambda / (2 * it);
            q *= lambda / (2 * it + 1);
            tnc += p * xodd + q * xeven;
            s -= p;
            if (s < -1e-10) {
                ML_ERROR2(ME.ME_PRECISION, 'pnt', printer_pnt);
                printer_pnt('goto:true, s = %d < 0 !!! ---> non-convergence!!', s);
                return finis(tnc, del, lower_tail, negdel, log_p);
            }
            if (s <= 0 && it > 1) {
                printer_pnt('goto:true, s:%d < 0 && it:%d>1', s, it);
                return finis(tnc, del, lower_tail, negdel, log_p);
            }
            errbd = 2 * s * (xodd - godd);
            printer_pnt('%d %d %d|%d %d %d %d %d', it, 100000 * godd, 100000 * geven, p, q, s, tnc, errbd);
            if (abs(errbd) < errmax) {
                printer_pnt('goto:true, errbd:%d < errmax:%d', errbd, errmax);
                return finis(tnc, del, lower_tail, negdel, log_p);
            }
        }
        ML_ERROR2(ME.ME_NOCONV, 'pnt', printer_pnt);
    } else {
        tnc = 0;
    }
    return finis(tnc, del, lower_tail, negdel, log_p);
}