'use strict';
import lbeta from '../../special/beta/lbeta.mjs';
import { pbeta } from '../beta/pbeta.mjs';
import {
    M_LN2,
    R_D_Cval,
    log,
    log1p,
    exp,
    abs
} from '../../r-func.mjs';
import { pnorm5 as pnorm } from '../normal/pnorm.mjs';
export function pt(x, n, lower_tail, log_p) {
    let val;
    if (!isFinite(n)) {
        return pnorm(x, 0, 1, lower_tail, log_p);
    }
    const nx = 1 + x / n * x;
    if (nx > 1e+100) {
        const lb = lbeta(0.5 * n, 0.5);
        const l05n = log(0.5 * n);
        const labsx = log(abs(x));
        const logn = log(n);
        const lval = -0.5 * n * (2 * labsx - logn) - lb - l05n;
        val = log_p ? lval : exp(lval);
    } else {
        val = n > x * x ? pbeta(x * x / (n + x * x), 0.5, n / 2, false, log_p) : pbeta(1 / nx, n / 2, 0.5, true, log_p);
    }
    if (x <= 0) {
        lower_tail = !lower_tail;
    }
    if (log_p) {
        if (lower_tail) {
            return log1p(-0.5 * exp(val));
        } else {
            return val - M_LN2;
        }
    } else {
        val /= 2;
        return R_D_Cval(lower_tail, val);
    }
}