'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    R_Q_P01_boundaries
} from '../../../packages/common/logger.mjs';
import { R_DT_qIv } from '../exp/expm1.mjs';
import {
    DBL_EPSILON,
    min,
    max,
    abs,
    DBL_MAX
} from '../../r-func.mjs';
import { qnorm } from '../normal/qnorm.mjs';
import { pnt } from './pnt.mjs';
import { qt } from './qt.mjs';
const printer = debug('qnt');
const accu = 1e-13;
const Eps = 1e-11;
export function qnt(p, df, ncp, lower_tail, log_p) {
    let ux;
    let lx;
    let nx;
    let pp;
    if (isNaN(p) || isNaN(df) || isNaN(ncp)) {
        return p + df + ncp;
    }
    if (df <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (ncp === 0 && df >= 1) {
        return qt(p, df, lower_tail, log_p);
    }
    const rc = R_Q_P01_boundaries(lower_tail, log_p, p, -Infinity, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    if (!isFinite(df)) {
        return qnorm(p, ncp, 1, lower_tail, log_p);
    }
    p = R_DT_qIv(lower_tail, log_p, p);
    if (p > 1 - DBL_EPSILON) {
        return Infinity;
    }
    pp = min(1 - DBL_EPSILON, p * (1 + Eps));
    for (ux = max(1, ncp); ux < DBL_MAX && pnt(ux, df, ncp, true, false) < pp; ux *= 2);
    pp = p * (1 - Eps);
    for (lx = min(-1, -ncp); lx > -DBL_MAX && pnt(lx, df, ncp, true, false) > pp; lx *= 2);
    do {
        nx = 0.5 * (lx + ux);
        if (pnt(nx, df, ncp, true, false) > p) {
            ux = nx;
        } else {
            lx = nx;
        }
    } while (ux - lx > accu * max(abs(lx), abs(ux)));
    return 0.5 * (lx + ux);
}