'use strict';
import { debug } from '@mangos/debug';
import { wprob } from './wprob.mjs';
import {
    ME,
    ML_ERR_return_NAN2,
    lineInfo4,
    ML_ERROR2
} from '../../../packages/common/logger.mjs';
import {
    R_DT_val,
    R_DT_0,
    R_DT_1,
    log,
    M_LN2,
    sqrt,
    exp,
    min
} from '../../r-func.mjs';
import { lgammafn_sign } from '../../special/gamma/index.mjs';
const printer_ptukey = debug('ptukey');
const nlegq = 16;
const ihalfq = 8;
const eps1 = -30;
const eps2 = 1e-14;
const dhaf = 100;
const dquar = 800;
const deigh = 5000;
const dlarg = 25000;
const ulen1 = 1;
const ulen2 = 0.5;
const ulen3 = 0.25;
const ulen4 = 0.125;
const xlegq = [
    0.9894009349916499,
    0.9445750230732326,
    0.8656312023878318,
    0.755404408355003,
    0.6178762444026438,
    0.45801677765722737,
    0.2816035507792589,
    0.09501250983763744
];
const alegq = [
    0.027152459411754096,
    0.062253523938647894,
    0.09515851168249279,
    0.12462897125553388,
    0.14959598881657674,
    0.16915651939500254,
    0.18260341504492358,
    0.1894506104550685
];
export function ptukey(q, nmeans, df, nrnages = 1, lowerTail = true, logP = false) {
    let ans;
    let f2lf;
    let otsum;
    let qsqz;
    let rotsum;
    let twa1;
    let ulen;
    let wprb;
    if (isNaN(q) || isNaN(nrnages) || isNaN(nmeans) || isNaN(df)) {
        return ML_ERR_return_NAN2(printer_ptukey, lineInfo4);
    }
    if (q <= 0) {
        return R_DT_0(lowerTail, logP);
    }
    if (df < 2 || nrnages < 1 || nmeans < 2) {
        return ML_ERR_return_NAN2(printer_ptukey, lineInfo4);
    }
    if (!isFinite(q)) {
        return R_DT_1(lowerTail, logP);
    }
    if (df > dlarg) {
        return R_DT_val(lowerTail, logP, wprob(q, nrnages, nmeans));
    }
    const f2 = df * 0.5;
    f2lf = f2 * log(df) - df * M_LN2 - lgammafn_sign(f2);
    const f21 = f2 - 1;
    const ff4 = df * 0.25;
    if (df <= dhaf) {
        ulen = ulen1;
    } else if (df <= dquar) {
        ulen = ulen2;
    } else if (df <= deigh) {
        ulen = ulen3;
    } else {
        ulen = ulen4;
    }
    f2lf += log(ulen);
    ans = 0;
    otsum = 0;
    for (let i = 1; i <= 50; i++) {
        otsum = 0;
        twa1 = (2 * i - 1) * ulen;
        for (let jj = 1; jj <= nlegq; jj++) {
            const j = ihalfq < jj ? jj - ihalfq - 1 : jj - 1;
            const t1 = ihalfq < jj ? f2lf + f21 * log(twa1 + xlegq[j] * ulen) - (xlegq[j] * ulen + twa1) * ff4 : f2lf + f21 * log(twa1 - xlegq[j] * ulen) + (xlegq[j] * ulen - twa1) * ff4;
            if (t1 >= eps1) {
                if (ihalfq < jj) {
                    qsqz = q * sqrt((xlegq[j] * ulen + twa1) * 0.5);
                } else {
                    qsqz = q * sqrt((-(xlegq[j] * ulen) + twa1) * 0.5);
                }
                wprb = wprob(qsqz, nrnages, nmeans);
                rotsum = wprb * alegq[j] * exp(t1);
                otsum += rotsum;
            }
        }
        if (i * ulen >= 1 && otsum <= eps2) {
            break;
        }
        ans += otsum;
    }
    if (otsum > eps2) {
        ML_ERROR2(ME.ME_PRECISION, 'ptukey', printer_ptukey);
    }
    ans = min(1, ans);
    return R_DT_val(lowerTail, logP, ans);
}