'use strict';
import { debug } from '@mangos/debug';
import {
    ME,
    ML_ERR_return_NAN2,
    lineInfo4,
    ML_ERROR2,
    R_Q_P01_boundaries
} from '../../../packages/common/logger.mjs';
import { R_DT_qIv } from '../exp/expm1.mjs';
import { ptukey } from './ptukey.mjs';
import {
    abs,
    max,
    sqrt,
    log
} from '../../r-func.mjs';
const p0 = 0.322232421088;
const q0 = 0.099348462606;
const p1 = -1;
const q1 = 0.588581570495;
const p2 = -0.342242088547;
const q2 = 0.531103462366;
const p3 = -0.204231210125;
const q3 = 0.10353775285;
const p4 = -0.0000453642210148;
const q4 = 0.0038560700634;
const c1 = 0.8832;
const c2 = 0.2368;
const c3 = 1.214;
const c4 = 1.208;
const c5 = 1.4142;
const vmax = 120;
const printer = debug('qtukey');
function qinv(p, c, v) {
    let q;
    let t;
    const ps = 0.5 - 0.5 * p;
    const yi = sqrt(log(1 / (ps * ps)));
    t = yi + ((((yi * p4 + p3) * yi + p2) * yi + p1) * yi + p0) / ((((yi * q4 + q3) * yi + q2) * yi + q1) * yi + q0);
    if (v < vmax) {
        t += (t * t * t + t) / v / 4;
    }
    q = c1 - c2 * t;
    if (v < vmax) {
        q += -c3 / v + c4 * t / v;
    }
    return t * (q * log(c - 1) + c5);
}
export function qtukey(p, nmeans, df, nranges = 1, lowerTail = true, logP = false) {
    const eps = 0.0001;
    const maxiter = 50;
    let ans = 0;
    let valx0;
    let valx1;
    let x0;
    let x1;
    let xabs;
    let iter;
    if (isNaN(p) || isNaN(nranges) || isNaN(nmeans) || isNaN(df)) {
        ML_ERROR2(ME.ME_DOMAIN, 'qtukey', printer);
        return NaN;
    }
    if (df < 2 || nranges < 1 || nmeans < 2) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    const rc = R_Q_P01_boundaries(lowerTail, logP, p, 0, Infinity);
    if (rc !== undefined) {
        return rc;
    }
    p = R_DT_qIv(lowerTail, logP, p);
    x0 = qinv(p, nmeans, df);
    valx0 = ptukey(x0, nmeans, df, nranges, true, false) - p;
    if (valx0 > 0) {
        x1 = max(0, x0 - 1);
    } else {
        x1 = x0 + 1;
    }
    valx1 = ptukey(x1, nmeans, df, nranges, true, false) - p;
    for (iter = 1; iter < maxiter; iter++) {
        ans = x1 - valx1 * (x1 - x0) / (valx1 - valx0);
        valx0 = valx1;
        x0 = x1;
        if (ans < 0) {
            ans = 0;
            valx1 = -p;
        }
        valx1 = ptukey(ans, nmeans, df, nranges, true, false) - p;
        x1 = ans;
        xabs = abs(x1 - x0);
        if (xabs < eps) {
            return ans;
        }
    }
    ML_ERROR2(ME.ME_NOCONV, 'qtukey', printer);
    return ans;
}