'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D__0,
    pow,
    log as _log,
    exp
} from '../../r-func.mjs';
const printer = debug('dweibull');
export function dweibull(x, shape, scale = 1, log = false) {
    if (isNaN(x) || isNaN(shape) || isNaN(scale)) {
        return x + shape + scale;
    }
    if (shape <= 0 || scale <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (x < 0) {
        return R_D__0(log);
    }
    if (!isFinite(x)) {
        return R_D__0(log);
    }
    if (x === 0 && shape < 1) {
        return Infinity;
    }
    const tmp1 = pow(x / scale, shape - 1);
    const tmp2 = tmp1 * (x / scale);
    return log ? -tmp2 + _log(shape * tmp1 / scale) : shape * tmp1 * exp(-tmp2) / scale;
}