'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_D_exp,
    R_DT_0,
    pow,
    expm1
} from '../../r-func.mjs';
import { R_Log1_Exp } from '../exp/expm1.mjs';
const printer = debug('pweibull');
export function pweibull(q, shape, scale = 1, lowerTail = true, logP = false) {
    if (isNaN(q) || isNaN(shape) || isNaN(scale)) {
        return q + shape + scale;
    }
    if (shape <= 0 || scale <= 0) {
        return ML_ERR_return_NAN2(printer, lineInfo4);
    }
    if (q <= 0) {
        return R_DT_0(lowerTail, logP);
    }
    q = -pow(q / scale, shape);
    return lowerTail ? logP ? R_Log1_Exp(q) : -expm1(q) : R_D_exp(logP, q);
}