'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import {
    R_DT_0,
    R_DT_1,
    R_DT_val
} from '../../r-func.mjs';
import { choose } from '../../special/choose/index.mjs';
import { WilcoxonCache } from './WilcoxonCache.mjs';
import { cwilcox } from './cwilcox.mjs';
const {
    round: R_forceint,
    floor
} = Math;
const {
    isNaN: ISNAN,
    isFinite: R_FINITE
} = Number;
const printer_pwilcox = debug('pwilcox');
export function pwilcox(q, m, n, lowerTail = true, logP = false) {
    m = R_forceint(m);
    n = R_forceint(n);
    const w = new WilcoxonCache();
    let lower_tail = lowerTail;
    if (ISNAN(q) || ISNAN(m) || ISNAN(n)) {
        return q + m + n;
    }
    if (!R_FINITE(m) || !R_FINITE(n)) {
        return ML_ERR_return_NAN2(printer_pwilcox, lineInfo4);
    }
    if (m <= 0 || n <= 0) {
        return ML_ERR_return_NAN2(printer_pwilcox, lineInfo4);
    }
    q = floor(q + 1e-7);
    if (q < 0) {
        return R_DT_0(lower_tail, logP);
    }
    if (q >= m * n) {
        return R_DT_1(lower_tail, logP);
    }
    const c = choose(m + n, n);
    let p = 0;
    if (q <= m * n / 2) {
        for (let i = 0; i <= q; i++) {
            p += cwilcox(i, m, n, w) / c;
        }
    } else {
        q = m * n - q;
        for (let i = 0; i < q; i++) {
            p += cwilcox(i, m, n, w) / c;
        }
        lower_tail = !lower_tail;
    }
    return R_DT_val(lower_tail, logP, p);
}