'use strict';
import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4,
    ML_ERROR2,
    ME
} from '../../../packages/common/logger.mjs';
import {
    round,
    trunc,
    floor
} from '../../r-func.mjs';
import { R_unif_index } from '../../rng/utils.mjs';
import {
    globalUni,
    globalSampleKind
} from '../../rng/global-rng.mjs';
const printer_rwilcox = debug('rwilcox');
const MAXSIZE = 800000000;
export function rwilcoxOne(m, n) {
    const rng = globalUni();
    const sampleKind = globalSampleKind();
    if (isNaN(m) || isNaN(n)) {
        return m + n;
    }
    m = round(m);
    n = round(n);
    if (m < 0 || n < 0) {
        return ML_ERR_return_NAN2(printer_rwilcox, lineInfo4);
    }
    if (m === 0 || n === 0) {
        return 0;
    }
    let k = trunc(m + n);
    if (k >= MAXSIZE) {
        ML_ERROR2(ME.ME_DOMAIN, 'k > MAXSIZE(=2**32)', printer_rwilcox);
        return ML_ERR_return_NAN2(printer_rwilcox, lineInfo4);
    }
    const x = new Uint32Array(k);
    for (let i = 0; i < k; i++) {
        x[i] = i;
    }
    let r = 0;
    printer_rwilcox(`------v`);
    for (let i = 0; i < n; i++) {
        const j = floor(R_unif_index(k, rng, sampleKind));
        r += x[j];
        x[j] = x[--k];
        printer_rwilcox('i:%d,\tn:%d\tj:%d\tk:%d\tr:%d\tx:%o', i, n, j, k, x);
    }
    return r - n * (n - 1) / 2;
}