const {isInteger, parseFloat, parseInt, MIN_VALUE, MAX_VALUE, isNaN, isFinite, EPSILON, MAX_SAFE_INTEGER, NEGATIVE_INFINITY, POSITIVE_INFINITY} = Number;
export {
    isInteger,
    parseFloat,
    parseInt,
    MIN_VALUE,
    MAX_VALUE,
    isNaN,
    isFinite,
    EPSILON,
    MAX_SAFE_INTEGER,
    NEGATIVE_INFINITY,
    POSITIVE_INFINITY
};
const {abs, cbrt, ceil, exp, expm1, floor, fround, imul, log, log10, log1p, log2, max, min, pow, random, round, sign, sqrt, PI, trunc, SQRT2, LN2, sinh, cos, sin, tan, atan} = Math;
export {
    abs,
    cbrt,
    ceil,
    exp,
    expm1,
    floor,
    fround,
    imul,
    log,
    log10,
    log1p,
    log2,
    max,
    min,
    pow,
    random,
    round,
    sign,
    sqrt,
    PI,
    trunc,
    SQRT2,
    LN2,
    sinh,
    cos,
    sin,
    tan,
    atan
};
export function sum(x) {
    const rc = x.reduce((pv, v) => pv + v, 0);
    return rc;
}
export var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel['ERROR'] = 0] = 'ERROR';
    LogLevel[LogLevel['WARN'] = 1] = 'WARN';
    LogLevel[LogLevel['INFO'] = 2] = 'INFO';
    LogLevel[LogLevel['DEBUG'] = 3] = 'DEBUG';
}(LogLevel || (LogLevel = {})));
export function Welch_Satterthwaite(s, n) {
    const elts = s.map((_s, i) => _s * _s / n[i]);
    const dom = elts.map((e, i) => e * e / (n[i] - 1));
    return pow(sum(elts), 2) / sum(dom);
}
export function repeatedCall(n, fn, ...arg) {
    let result;
    if (n === 0) {
        result = emptyFloat32Array;
    } else if (n > 0 && isFinite(n)) {
        result = new Float32Array(n);
    } else {
        throw new TypeError(`"n" argument is not a finite number or negative`);
    }
    for (let i = 0; i < result.length; i++) {
        result[i] = fn(...arg);
    }
    return result;
}
export function repeatedCall64(n, fn, ...args) {
    let result;
    if (n === 0) {
        result = emptyFloat64Array;
    } else if (n > 0 && isFinite(n)) {
        result = new Float64Array(n);
    } else {
        throw new TypeError(`"n=${ n }" is not a positive finite number`);
    }
    for (let i = 0; i < result.length; i++) {
        result[i] = fn(...args);
    }
    return result;
}
export const emptyFloat32Array = new Float32Array(0);
export const emptyFloat64Array = new Float64Array(0);
export const nanFloat32Array = new Float32Array([NaN]);
export function isArray(x) {
    return Array.isArray(x) || x instanceof Float32Array || x instanceof Float64Array;
}
export function isEmptyArray(x) {
    return isArray(x) && x.length === 0;
}
export const M_SQRT2 = 1.4142135623730951;
export function frac(x) {
    return x - trunc(x);
}
export const M_SQRT_32 = 5.656854249492381;
export const DBL_EPSILON = 2.220446049250313e-16;
export const DBL_MANT_DIG = 18;
export const DBL_MIN = 2.2250738585072014e-308;
export const DBL_MAX = 1.7976931348623157e+308;
export const M_LN2 = 0.6931471805599453;
export const M_1_SQRT_2PI = 0.3989422804014327;
export const M_2PI = 6.283185307179586;
export const M_LN_2PI = 1.8378770664093456;
export const M_1_PI = 0.3183098861837907;
export const M_PI_2 = PI / 2;
export const M_LN_SQRT_PI = 0.5723649429247001;
export const INT_MAX = 2 ** 31 - 1;
export const M_LN_SQRT_2PI = 0.9189385332046728;
export const M_LN_SQRT_PId2 = 0.22579135264472744;
export const M_SQRT_2dPI = 0.7978845608028654;
export const M_LOG10_2 = 0.3010299956639812;
export const DBL_MAX_EXP = log2(MAX_VALUE);
export const DBL_MIN_VALUE_LN = log(MIN_VALUE);
export const DBL_MIN_EXP = log2(MIN_VALUE);
export function R_D__1(logP) {
    return logP ? 0 : 1;
}
export const R_D__0 = logP => {
    return logP ? -Infinity : 0;
};
export const R_DT_0 = (lower_tail, log_p) => {
    return lower_tail ? R_D__0(log_p) : R_D__1(log_p);
};
export const R_DT_1 = (lower_tail, log_p) => {
    return lower_tail ? R_D__1(log_p) : R_D__0(log_p);
};
export function R_D_val(log_p, x) {
    return log_p ? log(x) : x;
}
export function R_D_Clog(log_p, p) {
    return log_p ? log1p(-p) : 0.5 - p + 0.5;
}
export function R_DT_val(lower_tail, log_p, x) {
    return lower_tail ? R_D_val(log_p, x) : R_D_Clog(log_p, x);
}
export function imin2(x, y) {
    return min(trunc(x), trunc(y));
}
export function R_D_Lval(lowerTail, p) {
    return lowerTail ? p : 1 - p;
}
export function R_D_Cval(lowerTail, p) {
    return lowerTail ? 1 - p : p;
}
export function R_P_bounds_Inf_01(lowerTail, log_p, x) {
    if (!isFinite(x)) {
        if (x > 0) {
            return R_DT_1(lowerTail, log_p);
        }
        return R_DT_0(lowerTail, log_p);
    }
    return undefined;
}
export function R_D_half(log_p) {
    return log_p ? -M_LN2 : 0.5;
}
export function R_P_bounds_01(lower_tail, log_p, x, x_min, x_max) {
    if (x <= x_min)
        return R_DT_0(lower_tail, log_p);
    if (x >= x_max)
        return R_DT_1(lower_tail, log_p);
    return undefined;
}
export const R_D_exp = (log_p, x) => {
    return log_p ? x : exp(x);
};
export function R_D_nonint_check(log, x, printer) {
    if (R_nonint(x)) {
        printer('non-integer x = %d', x);
        return R_D__0(log);
    }
    return undefined;
}
export function fmod(x, y) {
    return x % y;
}
export function imax2(x, y) {
    return max(trunc(x), trunc(y));
}
export function isOdd(k) {
    return floor(k / 2) * 2 !== k;
}
export function R_D_negInonint(x) {
    return x < 0 || R_nonint(x);
}
export function R_nonint(x) {
    return !isInteger(x);
}
export function R_D_fexp(give_log, f, x) {
    return give_log ? -0.5 * log(f) + x : exp(x) / sqrt(f);
}
export function R_pow_di(x, n) {
    let pow = 1;
    if (isNaN(x))
        return x;
    if (n !== 0) {
        if (!isFinite(x))
            return R_pow(x, n);
        if (n < 0) {
            n = -n;
            x = 1 / x;
        }
        for (;;) {
            if (n & 1)
                pow *= x;
            if (n >>= 1)
                x *= x;
            else
                break;
        }
    }
    return pow;
}
export function R_pow(x, y) {
    if (y === 2)
        return x * x;
    if (x === 1 || y === 0)
        return 1;
    if (x === 0) {
        if (y > 0)
            return 0;
        else if (y < 0)
            return Infinity;
        else
            return y;
    }
    if (isFinite(x) && isFinite(y)) {
        return pow(x, y);
    }
    if (isNaN(x) || isNaN(y))
        return x + y;
    if (!isFinite(x)) {
        if (x > 0)
            return y < 0 ? 0 : Infinity;
        else {
            if (isFinite(y) && y === floor(y))
                return y < 0 ? 0 : myfmod(y, 2) ? x : -x;
        }
    }
    if (!isFinite(y)) {
        if (x >= 0) {
            if (y > 0)
                return x >= 1 ? Infinity : 0;
            else
                return x < 1 ? Infinity : 0;
        }
    }
    return NaN;
}
export function R_isnancpp(x) {
    return isNaN(x);
}
export function myfmod(x1, x2) {
    const q = x1 / x2;
    return x1 - floor(q) * x2;
}
export function ldexp(x, y) {
    if (isNaN(x) || isNaN(y)) {
        return x + y;
    }
    if (!isFinite(x) || !isFinite(y)) {
        return Infinity;
    }
    return x * pow(2, y);
}
export function R_D_log(log_p, p) {
    return log_p ? p : log(p);
}
export function R_D_qIv(logP, p) {
    return logP ? exp(p) : p;
}
export function sumfp(x) {
    return x.reduce((pv, v) => pv + v, 0);
}
export function sumfp64(x) {
    return x.reduce((pv, v) => pv + v, 0);
}