export var MessageType;
(function (MessageType) {
    MessageType['INIT'] = '@@INIT@@';
}(MessageType || (MessageType = {})));
export class IRNG {
    constructor(name) {
        this._name = name;
        this.notify = new Map();
        this.emit = this.emit.bind(this);
    }
    get name() {
        return this._name;
    }
    randoms(n) {
        n = !n || n < 0 ? 1 : n;
        const rc = new Float32Array(n);
        for (let i = 0; i < n; i++) {
            rc[i] = this.random();
        }
        return rc;
    }
    random() {
        throw new Error(`override this function in ${ this.constructor.name }`);
    }
    emit(event, ...args) {
        const handlers = this.notify.get(event);
        if (handlers !== undefined) {
            handlers.forEach(h => h(...args));
        }
    }
    register(event, handler) {
        const handlers = this.notify.get(event) || [];
        this.notify.set(event, handlers);
        handlers.push(handler);
    }
    unregister(event, handler) {
        if (!handler) {
            this.notify.delete(event);
            return;
        }
        const handlers = this.notify.get(event);
        if (!handlers) {
            return;
        }
        const idx = handlers.indexOf(handler);
        if (idx >= 0) {
            handlers.splice(idx, 1);
            if (handlers.length === 0) {
                this.notify.delete(event);
            }
        }
    }
    init(seed) {
        this.emit(MessageType.INIT, this, seed);
    }
}