'use strict';
import { fixup } from '../fixup.mjs';
import { IRNG } from '../irng.mjs';
import seed from '../seed.mjs';
import { TAOCP1997init } from './taocp-1997-init.mjs';
import { seedCheck } from '../seedcheck.mjs';
const MM = 1 << 30;
const KK = 100;
const LL = 37;
const QUALITY = 1009;
const qualityBuffer = new ArrayBuffer(QUALITY * 4);
const ran_arr_buf = new Uint32Array(qualityBuffer);
const KT = 9.31322574615479e-10;
const mod_diff = (x, y) => x - y & MM - 1;
const SEED_LEN = 101;
export class KnuthTAOCP extends IRNG {
    constructor(_seed = seed()) {
        super('Knuth-TAOCP');
        this.m_seed = new Uint32Array(SEED_LEN);
        this.init(_seed);
    }
    get KT_pos() {
        return this.m_seed[100];
    }
    set KT_pos(v) {
        this.m_seed[100] = v;
    }
    fixupSeeds() {
        if (this.KT_pos <= 0)
            this.KT_pos = 100;
        const s = this.m_seed.slice(0, 100);
        if (s.find(v => !!v) === undefined)
            this.init(seed());
        return;
    }
    KT_next() {
        const s = this.m_seed;
        if (this.KT_pos >= 100) {
            this.ran_arr_cycle();
            this.KT_pos = 0;
        }
        return s[this.KT_pos++];
    }
    RNG_Init_R_KT(_seed) {
        this.m_seed.set(TAOCP1997init(_seed % 1073741821));
        this.KT_pos = 100;
        this.fixupSeeds();
    }
    ran_arr_cycle() {
        this.ran_array(ran_arr_buf, QUALITY);
        ran_arr_buf[KK] = -1;
        return ran_arr_buf[0];
    }
    ran_array(aa, n) {
        let i;
        let j;
        const ran_x = this.m_seed;
        for (j = 0; j < KK; j++)
            aa[j] = ran_x[j];
        for (; j < n; j++)
            aa[j] = mod_diff(aa[j - KK], aa[j - LL]);
        for (i = 0; i < LL; i++, j++)
            ran_x[i] = mod_diff(aa[j - KK], aa[j - LL]);
        for (; i < KK; i++, j++)
            ran_x[i] = mod_diff(aa[j - KK], ran_x[i - LL]);
    }
    init(_seed = seed()) {
        const s = new Uint32Array([0]);
        s[0] = _seed;
        for (let j = 0; j < 50; j++) {
            s[0] = 69069 * s[0] + 1;
        }
        this.RNG_Init_R_KT(s[0]);
        super.init(_seed);
    }
    set seed(_seed) {
        seedCheck(this.name, _seed, SEED_LEN);
        this.m_seed.set(_seed);
    }
    get seed() {
        return this.m_seed.slice();
    }
    random() {
        return fixup(this.KT_next() * KT);
    }
    get cut() {
        return 2 ** 25 - 1;
    }
}
KnuthTAOCP.kind = 'KNUTH_TAOCP';