'use strict';
import { INT_MAX } from '../../r-func.mjs';
import {
    fixup,
    i2_32m1
} from '../fixup.mjs';
import { IRNG } from '../irng.mjs';
import seed from '../seed.mjs';
import { seedCheck } from '../seedcheck.mjs';
const SEED_LEN = 2;
export class MarsagliaMultiCarry extends IRNG {
    constructor(_seed = seed()) {
        super('Marsaglia-MultiCarry');
        this.m_seed = new Int32Array(SEED_LEN);
        this.init(_seed);
    }
    fixupSeeds() {
        const s = this.m_seed;
        if (s[0] === 0)
            s[0] = 1;
        if (s[1] === 0)
            s[1] = 1;
        return;
    }
    init(_seed = seed()) {
        const s = new Int32Array([_seed]);
        for (let j = 0; j < 50; j++) {
            s[0] = 69069 * s[0] + 1;
        }
        for (let j = 0; j < this.m_seed.length; j++) {
            s[0] = 69069 * s[0] + 1;
            this.m_seed[j] = s[0];
        }
        this.fixupSeeds();
        super.init(_seed);
    }
    random() {
        const s = this.m_seed;
        s[0] = 36969 * (s[0] & 65535) + (s[0] >>> 16);
        s[1] = 18000 * (s[1] & 65535) + (s[1] >>> 16);
        const un = new Uint32Array(SEED_LEN);
        un[0] = s[0] << 16;
        un[1] = s[1] & 65535;
        un[0] = un[0] ^ un[1];
        return fixup(un[0] * i2_32m1);
    }
    set seed(_seed) {
        seedCheck(this.name, _seed, SEED_LEN);
        this.m_seed.set(_seed);
        this.fixupSeeds();
    }
    get seed() {
        return this.m_seed.slice();
    }
    get cut() {
        return INT_MAX;
    }
}
MarsagliaMultiCarry.kind = 'MARSAGLIA_MULTICARRY';