'use strict';
import { MessageType } from '../../irng.mjs';
import { IRNGNormal } from '../normal-rng.mjs';
const {
    log,
    sqrt,
    min: fmin2,
    max: fmax2,
    abs: fabs,
    exp
} = Math;
export class BuggyKindermanRamage extends IRNGNormal {
    constructor(_rng) {
        super(_rng, 'Buggy-Kinderman-Ramage');
        this._rng.unregister(MessageType.INIT, this.reset.bind(this));
    }
    random() {
        const A = 2.216035867166471;
        const C1 = 0.398942280401433;
        const C2 = 0.180025191068563;
        const g = x => C1 * exp(-x * x / 2) - C2 * (A - x);
        const u1 = this._rng.random();
        let u2;
        let u3;
        let tt;
        if (u1 < 0.884070402298758) {
            const u2 = this._rng.random();
            return A * (1.1311316354418 * u1 + u2 - 1);
        }
        if (u1 >= 0.973310954173898) {
            for (;;) {
                u2 = this._rng.random();
                u3 = this._rng.random();
                tt = A * A - 2 * log(u3);
                if (u2 * u2 < A * A / tt)
                    return u1 < 0.986655477086949 ? sqrt(tt) : -sqrt(tt);
            }
        }
        if (u1 >= 0.958720824790463) {
            for (;;) {
                u2 = this._rng.random();
                u3 = this._rng.random();
                tt = A - 0.63083480192196 * fmin2(u2, u3);
                if (fmax2(u2, u3) <= 0.755591531667601)
                    return u2 < u3 ? tt : -tt;
                if (0.034240503750111 * fabs(u2 - u3) <= g(tt))
                    return u2 < u3 ? tt : -tt;
            }
        }
        if (u1 >= 0.911312780288703) {
            for (;;) {
                u2 = this._rng.random();
                u3 = this._rng.random();
                tt = 0.479727404222441 + 1.10547366102207 * fmin2(u2, u3);
                if (fmax2(u2, u3) <= 0.87283497667179)
                    return u2 < u3 ? tt : -tt;
                if (0.049264496373128 * fabs(u2 - u3) <= g(tt))
                    return u2 < u3 ? tt : -tt;
            }
        }
        for (;;) {
            u2 = this._rng.random();
            u3 = this._rng.random();
            tt = 0.479727404222441 - 0.59550713801594 * fmin2(u2, u3);
            if (fmax2(u2, u3) <= 0.805577924423817)
                return u2 < u3 ? tt : -tt;
        }
    }
}
BuggyKindermanRamage.kind = 'BUGGY_KINDERMAN_RAMAGE';