'use strict';
import { INT_MAX } from '../../r-func.mjs';
import { fixup } from '../fixup.mjs';
import { IRNG } from '../irng.mjs';
import seed from '../seed.mjs';
import { seedCheck } from '../seedcheck.mjs';
import { frac } from '../../r-func.mjs';
const SEED_LEN = 3;
export class WichmannHill extends IRNG {
    constructor(_seed = seed()) {
        super('Wichmann-Hill');
        this.m_seed = new Uint32Array(SEED_LEN);
        this.init(_seed);
    }
    random() {
        const s = this.m_seed;
        s[0] = s[0] * 171 % 30269;
        s[1] = s[1] * 172 % 30307;
        s[2] = s[2] * 170 % 30323;
        const value = s[0] / 30269 + s[1] / 30307 + s[2] / 30323;
        return fixup(frac(value));
    }
    fixupSeeds() {
        const s = this.m_seed;
        s[0] = s[0] % 30269;
        s[1] = s[1] % 30307;
        s[2] = s[2] % 30323;
        if (s[0] === 0)
            s[0] = 1;
        if (s[1] === 0)
            s[1] = 1;
        if (s[2] === 0)
            s[2] = 1;
        return;
    }
    init(_seed = seed()) {
        const s = new Uint32Array([_seed]);
        for (let j = 0; j < 50; j++) {
            s[0] = 69069 * s[0] + 1;
        }
        for (let j = 0; j < this.m_seed.length; j++) {
            s[0] = 69069 * s[0] + 1;
            this.m_seed[j] = s[0];
        }
        this.fixupSeeds();
        super.init(_seed);
    }
    set seed(_seed) {
        seedCheck(this.name, _seed, SEED_LEN);
        this.m_seed.set(_seed);
        this.fixupSeeds();
    }
    get seed() {
        return this.m_seed.slice();
    }
    get cut() {
        return INT_MAX;
    }
}
WichmannHill.kind = 'WICHMANN_HILL';