import { debug } from '@mangos/debug';
import {
    ME,
    ML_ERROR2
} from '../../../../packages/common/logger.mjs';
import { sinpi } from '../../../trigonometry/sinpi.mjs';
import BesselK from '../besselK/index.mjs';
import { I_bessel } from './IBessel.mjs';
import {
    exp,
    trunc,
    floor,
    PI
} from '../../../r-func.mjs';
const printer = debug('besselI');
function BesselI(x, nu, exponScaled = false) {
    if (isNaN(x) || isNaN(nu))
        return x + nu;
    if (x < 0) {
        ML_ERROR2(ME.ME_RANGE, 'bessel_i', printer);
        return NaN;
    }
    const ize = exponScaled ? 2 : 1;
    const na = floor(nu);
    if (nu < 0) {
        return BesselI(x, -nu, exponScaled) + (nu === na ? 0 : BesselK(x, -nu, exponScaled) * (ize === 1 ? 2 : 2 * exp(-2 * x)) / PI * sinpi(-nu));
    }
    const nb = 1 + trunc(na);
    nu -= nb - 1;
    const rc = I_bessel(x, nu, nb, ize);
    if (rc.ncalc !== rc.nb) {
        if (rc.ncalc < 0)
            printer('bessel_i(%d): ncalc (=%d) != nb (=%d); nu=%d. Arg. out of range?', x, rc.ncalc, rc.nb, nu);
        else
            printer('bessel_i(%d,nu=%d): precision lost in result\n', rc.x, nu + rc.nb - 1);
    }
    x = rc.x;
    return x;
}
export default BesselI;