import { debug } from '@mangos/debug';
import {
    ME,
    ML_ERROR2
} from '../../../../packages/common/logger.mjs';
import { cospi } from '../../../trigonometry/cospi.mjs';
import { sinpi } from '../../../trigonometry/sinpi.mjs';
import BesselY from '../besselY/index.mjs';
import { J_bessel } from './Jbessel.mjs';
import { floor } from '../../../r-func.mjs';
const printer = debug('BesselJ');
function BesselJ(x, nu) {
    if (isNaN(x) || isNaN(nu))
        return x + nu;
    if (x < 0) {
        ML_ERROR2(ME.ME_RANGE, 'BesselJ', printer);
        return NaN;
    }
    const na = floor(nu);
    if (nu < 0) {
        let rc;
        if (nu - na === 0.5) {
            rc = 0;
        } else {
            rc = BesselJ(x, -nu) * cospi(nu);
        }
        if (nu !== na) {
            rc += BesselY(x, -nu) * sinpi(nu);
        }
        return rc;
    } else if (nu > 10000000) {
        printer('besselJ(x, nu): nu=%d too large for bessel_j() algorithm', nu);
        return NaN;
    }
    const nb = 1 + na;
    nu -= na;
    const rc = J_bessel(x, nu, nb);
    printer('debug (nu=%d, na=%d, nb=%d, rc=%j', nu, na, nb, rc);
    if (rc.ncalc !== nb) {
        if (rc.ncalc < 0) {
            printer('bessel_j(%d): ncalc (=%d) != nb (=%d); nu=%d. Arg. out of range?', x, rc.ncalc, rc.nb, nu);
        } else {
            printer('bessel_j(%d,nu=%d): precision lost in result', x, nu + nb - 1);
        }
    }
    return rc.x;
}
export default BesselJ;