import { debug } from '@mangos/debug';
import {
    ME,
    ML_ERROR2
} from '../../../../packages/common/logger.mjs';
import {
    isNaN,
    floor
} from '../../../r-func.mjs';
import { K_bessel } from './Kbessel.mjs';
const printer = debug('BesselK');
function BesselK(x, nu, exponScaled = false) {
    if (isNaN(x) || isNaN(nu))
        return x + nu;
    if (x < 0) {
        ML_ERROR2(ME.ME_RANGE, 'bessel_k', printer);
        return NaN;
    }
    const ize = exponScaled ? 2 : 1;
    if (nu < 0)
        nu = -nu;
    const nb = 1 + floor(nu);
    nu -= nb - 1;
    const rc = K_bessel(x, nu, nb, ize);
    if (rc.ncalc !== rc.nb) {
        if (rc.ncalc < 0)
            printer('bessel_k(%d): ncalc (=%d) != nb (=%d); nu=%d. Arg. out of range?\n', rc.x, rc.ncalc, rc.nb, nu);
        else
            printer('bessel_k(%d,nu=%d): precision lost in result\n', rc.x, nu + rc.nb - 1);
    }
    x = rc.x;
    return x;
}
export default BesselK;