import { debug } from '@mangos/debug';
import {
    ME,
    ML_ERROR2
} from '../../../../packages/common/logger.mjs';
import { cospi } from '../../../trigonometry/cospi.mjs';
import { sinpi } from '../../../trigonometry/sinpi.mjs';
import BesselJ from '../besselJ/index.mjs';
import { Y_bessel } from './Ybessel.mjs';
import { floor } from '../../../r-func.mjs';
const printer = debug('BesselY');
function BesselY(x, nu) {
    if (isNaN(x) || isNaN(nu))
        return x + nu;
    if (x < 0) {
        ML_ERROR2(ME.ME_RANGE, 'BesselY', printer);
        return NaN;
    }
    const na = floor(nu);
    if (nu < 0) {
        return (nu - na === 0.5 ? 0 : BesselY(x, -nu) * cospi(nu)) - (nu === na ? 0 : BesselJ(x, -nu) * sinpi(nu));
    } else if (nu > 10000000) {
        printer('besselY(x, nu): nu=%d too large for bessel_y() algorithm', nu);
        return NaN;
    }
    const nb = 1 + na;
    nu -= nb - 1;
    const rc = Y_bessel(x, nu, nb);
    if (rc.ncalc !== nb) {
        if (rc.ncalc === -1) {
            return Infinity;
        } else if (rc.ncalc < -1) {
            printer('bessel_y(%d): ncalc (=%d) != nb (=%d); nu=%d. Arg. out of range?\n', rc.x, rc.ncalc, nb, nu);
        } else {
            printer('bessel_y(%d,nu=%d): precision lost in result\n', rc.x, nu + nb - 1);
        }
    }
    return rc.x;
}
export default BesselY;