import { debug } from '@mangos/debug';
import {
    ME,
    ML_ERR_return_NAN2,
    lineInfo4,
    ML_ERROR2
} from '../../../packages/common/logger.mjs';
import { gamma } from '../gamma/index.mjs';
import lbeta from './lbeta.mjs';
import { exp } from '../../r-func.mjs';
const xmax = 171.61447887182297;
const lnsml = -708.3964185322641;
const printer_beta = debug('beta');
function beta(a, b) {
    if (isNaN(a) || isNaN(b))
        return a + b;
    if (a < 0 || b < 0)
        return ML_ERR_return_NAN2(printer_beta, lineInfo4);
    else if (a === 0 || b === 0)
        return Infinity;
    else if (!isFinite(a) || !isFinite(b))
        return 0;
    if (a + b < xmax) {
        return 1 / gamma(a + b) * gamma(a) * gamma(b);
    } else {
        const val = lbeta(a, b);
        if (val < lnsml) {
            ML_ERROR2(ME.ME_UNDERFLOW, 'beta', printer_beta);
        }
        return exp(val);
    }
}
export default beta;