import { debug } from '@mangos/debug';
import {
    ML_ERR_return_NAN2,
    lineInfo4
} from '../../../packages/common/logger.mjs';
import { M_LN_SQRT_2PI } from '../../r-func.mjs';
import { gamma } from '../gamma/index.mjs';
import { lgammafn_sign } from '../gamma/lgammafn_sign.mjs';
import { lgammacor } from '../gamma/lgammacor.mjs';
const printer = debug('lbeta');
function lbeta(a, b) {
    let corr;
    let p;
    let q;
    if (isNaN(a) || isNaN(b))
        return a + b;
    p = q = a;
    if (b < p)
        p = b;
    if (b > q)
        q = b;
    if (p < 0)
        return ML_ERR_return_NAN2(printer, lineInfo4);
    else if (p === 0) {
        return Infinity;
    } else if (!isFinite(q)) {
        return -Infinity;
    }
    if (p >= 10) {
        corr = lgammacor(p) + lgammacor(q) - lgammacor(p + q);
        return Math.log(q) * -0.5 + M_LN_SQRT_2PI + corr + (p - 0.5) * Math.log(p / (p + q)) + q * Math.log1p(-p / (p + q));
    } else if (q >= 10) {
        corr = lgammacor(q) - lgammacor(p + q);
        return lgammafn_sign(p) + corr + p - p * Math.log(p + q) + (q - 0.5) * Math.log1p(-p / (p + q));
    } else {
        if (p < 1e-306)
            return lgammafn_sign(p) + (lgammafn_sign(q) - lgammafn_sign(p + q));
        else
            return Math.log(gamma(p) * (gamma(q) / gamma(p + q)));
    }
}
export default lbeta;