import { debug } from '@mangos/debug';
import {
    isOdd,
    abs,
    log,
    exp,
    round,
    isInteger
} from '../../r-func.mjs';
import lbeta from '../beta/lbeta.mjs';
import { lgammafn_sign } from '../gamma/lgammafn_sign.mjs';
function lfastchoose(n, k) {
    return -log(n + 1) - lbeta(n - k + 1, k + 1);
}
function lfastchoose2(n, k, sChoose) {
    const r = lgammafn_sign(n - k + 1, sChoose);
    return lgammafn_sign(n + 1) - lgammafn_sign(k + 1) - r;
}
const printer_lchoose = debug('lchoose');
function lchoose(n, k) {
    const k0 = k;
    k = Math.round(k);
    if (isNaN(n) || isNaN(k))
        return n + k;
    if (abs(k - k0) > 1e-7)
        printer_lchoose('"k" (%d) must be integer, rounded to %d', k0, k);
    if (k < 2) {
        if (k < 0)
            return -Infinity;
        if (k === 0)
            return 0;
        return log(abs(n));
    }
    if (n < 0) {
        return lchoose(-n + k - 1, k);
    } else if (isInteger(n)) {
        n = round(n);
        if (n < k)
            return -Infinity;
        if (n - k < 2)
            return lchoose(n, n - k);
        return lfastchoose(n, k);
    }
    if (n < k - 1) {
        return lfastchoose2(n, k);
    }
    return lfastchoose(n, k);
}
const k_small_max = 30;
const printer_choose = debug('choose');
function choose(n, k) {
    let r;
    const k0 = k;
    k = round(k);
    if (isNaN(n) || isNaN(k)) {
        return n + k;
    }
    if (abs(k - k0) > 1e-7) {
        printer_choose('k (%d) must be integer, rounded to %d', k0, k);
    }
    if (k < k_small_max) {
        let j;
        if (n - k < k && n >= 0 && isInteger(n)) {
            k = n - k;
        }
        if (k < 0) {
            return 0;
        }
        if (k === 0) {
            return 1;
        }
        r = n;
        for (j = 2; j <= k; j++) {
            r *= (n - j + 1) / j;
        }
        return isInteger(n) ? round(r) : r;
    }
    if (n < 0) {
        r = choose(-n + k - 1, k);
        if (isOdd(k)) {
            r = -r;
        }
        return r;
    } else if (isInteger(n)) {
        n = round(n);
        if (n < k)
            return 0;
        if (n - k < k_small_max)
            return choose(n, n - k);
        return round(exp(lfastchoose(n, k)));
    }
    if (n < k - 1) {
        const schoose = new Int32Array(1);
        r = lfastchoose2(n, k, schoose);
        return schoose[0] * exp(r);
    }
    return exp(lfastchoose(n, k));
}
export {
    lfastchoose,
    lchoose,
    choose
};