import { debug } from '@mangos/debug';
import {
    fmod,
    M_LN_SQRT_2PI,
    M_LN_SQRT_PId2,
    trunc,
    floor,
    abs,
    log
} from '../../r-func.mjs';
import {
    ME,
    ML_ERROR2
} from '../../../packages/common/logger.mjs';
import { sinpi } from '../../trigonometry/sinpi.mjs';
import { lgammacor } from './lgammacor.mjs';
import { gammafn } from './gamma_fn.mjs';
const printer_sign = debug('lgammafn_sign');
const xmax = 2.5327372760800758e+305;
const dxrel = 1.4901161193847656e-8;
const ML_POSINF = Infinity;
export function lgammafn_sign(x, sgn) {
    if (isNaN(x))
        return x;
    if (sgn !== undefined && sgn !== null) {
        sgn[0] = 1;
        if (x < 0 && fmod(floor(-x), 2) === 0) {
            sgn[0] = -1;
        }
    }
    if (x <= 0 && x === trunc(x)) {
        ML_ERROR2(ME.ME_RANGE, 'lgamma', printer_sign);
        return ML_POSINF;
    }
    const y = abs(x);
    if (y < 1e-306)
        return -log(y);
    if (y <= 10)
        return log(abs(gammafn(x)));
    if (y > xmax) {
        ML_ERROR2(ME.ME_RANGE, 'lgamma', printer_sign);
        return ML_POSINF;
    }
    if (x > 0) {
        if (x > 100000000000000000)
            return x * (log(x) - 1);
        else if (x > 4934720)
            return M_LN_SQRT_2PI + (x - 0.5) * log(x) - x;
        else
            return M_LN_SQRT_2PI + (x - 0.5) * log(x) - x + lgammacor(x);
    }
    const sinpiy = abs(sinpi(y));
    const ans = M_LN_SQRT_PId2 + (x - 0.5) * log(y) - x - log(sinpiy) - lgammacor(y);
    if (abs((x - trunc(x - 0.5)) * ans / x) < dxrel) {
        ML_ERROR2(ME.ME_PRECISION, 'lgamma', printer_sign);
    }
    return ans;
}