import { debug } from '@mangos/debug';
import {
    DBL_MANT_DIG,
    DBL_MAX_EXP,
    DBL_MIN_EXP,
    imin2,
    M_LOG10_2,
    R_pow_di,
    DBL_EPSILON,
    pow,
    abs,
    max,
    min,
    exp,
    log,
    sin,
    cos,
    PI,
    round
} from '../../r-func.mjs';
const printer = debug('dpsifn');
const n_max = 100;
const trm = new Float64Array(23);
const trmr = new Float64Array(n_max + 1);
const r1m5 = M_LOG10_2;
const r1m4 = DBL_EPSILON * 0.5;
const wdtol = max(r1m4, 5e-19);
const lrg = 1 / (2 * DBL_EPSILON);
const bvalues = new Float64Array([
    1,
    -0.5,
    0.16666666666666666,
    -0.03333333333333333,
    0.023809523809523808,
    -0.03333333333333333,
    0.07575757575757576,
    -0.2531135531135531,
    1.1666666666666667,
    -7.092156862745098,
    54.971177944862156,
    -529.1242424242424,
    6192.123188405797,
    -86580.25311355312,
    1425517.1666666667,
    -27298231.067816094,
    601580873.9006424,
    -15116315767.092157,
    429614643061.1667,
    -13711655205088.332,
    488332318973593.2,
    -19296579341940068
]);
function dpsifn(x, n, kode, m, ans, nz, ierr) {
    let nx;
    let xinc = 0;
    let xdmln = 0;
    let i;
    let j;
    let k;
    let mm;
    let mx;
    let nn;
    let np;
    let fn;
    let arg;
    let den;
    let eps;
    let fln;
    let fx;
    let rln;
    let s;
    let slope;
    let t;
    let tk;
    let tol;
    let tols;
    let tss;
    let tt;
    let t1;
    let t2;
    let xdmy = 0;
    let xln = 0;
    let xm;
    let xmin;
    let xq;
    let yint;
    trm.fill(0);
    trmr.fill(0);
    ierr[0] = 0;
    if (n < 0 || kode < 1 || kode > 2 || m < 1) {
        ierr[0] = 1;
        return;
    }
    if (x <= 0) {
        if (x === round(x)) {
            for (j = 0; j < m; j++)
                ans[j] = (j + n) % 2 ? Infinity : NaN;
            return;
        }
        dpsifn(1 - x, n, 1, m, ans, nz, ierr);
        if (m > 1 || n > 3) {
            ierr[0] = 4;
            return;
        }
        x *= PI;
        if (n === 0)
            tt = cos(x) / sin(x);
        else if (n === 1)
            tt = -1 / R_pow_di(sin(x), 2);
        else if (n === 2)
            tt = 2 * cos(x) / R_pow_di(sin(x), 3);
        else if (n === 3)
            tt = -2 * (2 * R_pow_di(cos(x), 2) + 1) / R_pow_di(sin(x), 4);
        else
            tt = NaN;
        s = n % 2 ? -1 : 1;
        t1 = t2 = s = 1;
        for (k = 0, j = k - n; j < m; k++, j++, s = -s) {
            t1 *= PI;
            if (k >= 2)
                t2 *= k;
            if (j >= 0)
                ans[j] = s * (ans[j] + t1 / t2 * tt);
        }
        if (n === 0 && kode === 2)
            ans[0] += xln;
        return;
    }
    nz[0] = 0;
    xln = log(x);
    if (kode === 1 && m === 1) {
        if (n === 0 && x * xln > lrg) {
            ans[0] = -xln;
            return;
        } else if (n >= 1 && x > n * lrg) {
            ans[0] = exp(-n * xln) / n;
            return;
        }
    }
    mm = m;
    nx = imin2(-DBL_MIN_EXP, DBL_MAX_EXP);
    const elim = 2.302 * (nx * r1m5 - 3);
    let L10 = false;
    let L20 = false;
    let L30 = false;
    for (;;) {
        nn = n + mm - 1;
        fn = nn;
        t = (fn + 1) * xln;
        if (abs(t) > elim) {
            if (t <= 0) {
                nz[0] = 0;
                ierr[0] = 2;
                return;
            }
        } else {
            if (x < wdtol) {
                ans[0] = R_pow_di(x, -n - 1);
                if (mm !== 1) {
                    for (k = 1; k < mm; k++)
                        ans[k] = ans[k - 1] / x;
                }
                if (n === 0 && kode === 2)
                    ans[0] += xln;
                return;
            }
            rln = r1m5 * DBL_MANT_DIG;
            rln = min(rln, 18.06);
            fln = max(rln, 3) - 3;
            yint = 3.5 + 0.4 * fln;
            slope = 0.21 + fln * (0.0006038 * fln + 0.008677);
            xm = yint + slope * fn;
            mx = (xm >> 0) + 1;
            xmin = mx;
            if (n !== 0) {
                xm = -2.302 * rln - min(0, xln);
                arg = xm / n;
                arg = min(0, arg);
                eps = exp(arg);
                xm = 1 - eps;
                if (abs(arg) < 0.001)
                    xm = -arg;
                fln = x * xm / eps;
                xm = xmin - x;
                if (xm > 7 && fln < 15)
                    break;
            }
            xdmy = x;
            xdmln = xln;
            xinc = 0;
            if (x < xmin) {
                nx = x >> 0;
                xinc = xmin - nx;
                xdmy = x + xinc;
                xdmln = log(xdmy);
            }
            t = fn * xdmln;
            t1 = xdmln + xdmln;
            t2 = t + xdmln;
            tk = max(abs(t), max(abs(t1), abs(t2)));
            if (tk <= elim) {
                L10 = true;
                break;
            }
        }
        nz[0]++;
        mm--;
        ans[mm] = 0;
        if (mm === 0) {
            return;
        }
    }
    if (!L10) {
        nn = (fln >> 0) + 1;
        np = n + 1;
        t1 = (n + 1) * xln;
        t = exp(-t1);
        s = t;
        den = x;
        for (i = 1; i <= nn; i++) {
            den += 1;
            trm[i] = pow(den, -np);
            s += trm[i];
        }
        ans[0] = s;
        if (n === 0 && kode === 2)
            ans[0] = s + xln;
        if (mm !== 1) {
            tol = wdtol / 5;
            for (j = 1; j < mm; j++) {
                t /= x;
                s = t;
                tols = t * tol;
                den = x;
                for (i = 1; i <= nn; i++) {
                    den += 1;
                    trm[i] /= den;
                    s += trm[i];
                    if (trm[i] < tols)
                        break;
                }
                ans[j] = s;
            }
        }
        return;
    }
    tss = exp(-t);
    tt = 0.5 / xdmy;
    t1 = tt;
    const tst = wdtol * tt;
    if (nn !== 0)
        t1 = tt + 1 / fn;
    const rxsq = 1 / (xdmy * xdmy);
    const ta = 0.5 * rxsq;
    t = (fn + 1) * ta;
    s = t * bvalues[2];
    if (abs(s) >= tst) {
        tk = 2;
        for (k = 4; k <= 22; k++) {
            t = t * ((tk + fn + 1) / (tk + 1)) * ((tk + fn) / (tk + 2)) * rxsq;
            trm[k] = t * bvalues[k - 1];
            if (abs(trm[k]) < tst)
                break;
            s += trm[k];
            tk += 2;
        }
    }
    s = (s + t1) * tss;
    for (;;) {
        if (xinc !== 0) {
            nx = xinc >> 0;
            np = nn + 1;
            if (nx > n_max) {
                nz[0] = 0;
                ierr[0] = 3;
                return;
            } else {
                if (nn === 0) {
                    L20 = true;
                    break;
                }
                xm = xinc - 1;
                fx = x + xm;
                for (i = 1; i <= nx; i++) {
                    trmr[i] = pow(fx, -np);
                    s += trmr[i];
                    xm -= 1;
                    fx = x + xm;
                }
            }
        }
        ans[mm - 1] = s;
        if (fn === 0) {
            L30 = true;
            break;
        }
        for (j = 2; j <= mm; j++) {
            fn--;
            tss *= xdmy;
            t1 = tt;
            if (fn !== 0)
                t1 = tt + 1 / fn;
            t = (fn + 1) * ta;
            s = t * bvalues[2];
            if (abs(s) >= tst) {
                tk = 4 + fn;
                for (k = 4; k <= 22; k++) {
                    trm[k] = trm[k] * (fn + 1) / tk;
                    if (abs(trm[k]) < tst)
                        break;
                    s += trm[k];
                    tk += 2;
                }
            }
            s = (s + t1) * tss;
            if (xinc !== 0) {
                if (fn === 0) {
                    L20 = true;
                    break;
                }
                xm = xinc - 1;
                fx = x + xm;
                for (i = 1; i <= nx; i++) {
                    trmr[i] = trmr[i] * fx;
                    s += trmr[i];
                    xm -= 1;
                    fx = x + xm;
                }
            }
            ans[mm - j] = s;
            if (fn === 0) {
                L30 = true;
                break;
            }
        }
        return;
    }
    printer(L20 ? 'goto L20 was set!' : 'goto L20 was not set');
    if (!L30) {
        for (i = 1; i <= nx; i++) {
            s += 1 / (x + (nx - i));
        }
    }
    if (kode !== 2)
        ans[0] = s - xdmln;
    else if (xdmy !== x) {
        xq = xdmy / x;
        ans[0] = s - log(xq);
    }
    return;
}
const print_psigamma = debug('psigamma');
function _render(x, calculate, final) {
    const ans = new Float64Array(1);
    const nz = new Uint8Array();
    const ierr = new Uint8Array(1);
    ans[0] = 0;
    nz[0] = 0;
    ierr[0] = 0;
    if (isNaN(x)) {
        return NaN;
    }
    calculate(x, ans, nz, ierr);
    if (ierr[0] !== 0) {
        return NaN;
    }
    return final(ans[0]);
}
export function psigamma(x, deriv) {
    deriv = round(deriv);
    const n = deriv >> 0;
    if (n > n_max) {
        print_psigamma('"deriv = %d > %d (= n_max)', n, n_max);
    }
    return _render(x, (x0, ans, nz, ierr) => {
        if (n > n_max) {
            ans[0] = NaN;
            return;
        }
        dpsifn(x0, n, 1, 1, ans, nz, ierr);
    }, v => {
        v = -v;
        for (let k = 1; k <= n; k++)
            v *= -k;
        return v;
    });
}
export function pentagamma(x) {
    return _render(x, (x0, ans, nz, ierr) => dpsifn(x0, 3, 1, 1, ans, nz, ierr), v => v * 6);
}
export function tetragamma(x) {
    return _render(x, (x0, ans, nz, ierr) => dpsifn(x0, 2, 1, 1, ans, nz, ierr), v => v * -2);
}
export function trigamma(x) {
    return _render(x, (x0, ans, nz, ierr) => dpsifn(x0, 1, 1, 1, ans, nz, ierr), v => v);
}
export function digamma(x) {
    return _render(x, (x0, ans, nz, ierr) => dpsifn(x0, 0, 1, 1, ans, nz, ierr), v => v * -1);
}