import { debug } from '@mangos/debug';
import { getLineInfo } from '@mangos/debug';
const debug_R_Q_P01_boundaries = debug('R_Q_P01_boundaries');
const debug_R_Q_P01_check = debug('R_Q_P01_check');
export function createLineInfo(n) {
    return function () {
        const info = getLineInfo(n);
        return `${ info.fnName }, line:${ info.line }, col:${ info.column }`;
    };
}
const lineInfo4 = createLineInfo(4);
export {
    lineInfo4
};
export var ME;
(function (ME) {
    ME[ME['ME_NONE'] = 0] = 'ME_NONE';
    ME[ME['ME_DOMAIN'] = 1] = 'ME_DOMAIN';
    ME[ME['ME_RANGE'] = 2] = 'ME_RANGE';
    ME[ME['ME_NOCONV'] = 4] = 'ME_NOCONV';
    ME[ME['ME_PRECISION'] = 8] = 'ME_PRECISION';
    ME[ME['ME_UNDERFLOW'] = 16] = 'ME_UNDERFLOW';
}(ME || (ME = {})));
export const mapErr = new Map([
    [
        ME.ME_NONE,
        'No error'
    ],
    [
        ME.ME_DOMAIN,
        'argument out of domain in \'%s\''
    ],
    [
        ME.ME_RANGE,
        'argument out of range in \'%s\''
    ],
    [
        ME.ME_NOCONV,
        'convergence failed in \'%s\''
    ],
    [
        ME.ME_PRECISION,
        'full precision may not have been achieved in \'%s\''
    ],
    [
        ME.ME_UNDERFLOW,
        'underflow occurred in \'%s\''
    ]
]);
export function ML_ERROR2(x, s, printer) {
    if (!printer.enabled) {
        return;
    }
    const str = mapErr.get(x);
    const val = typeof s === 'function' ? s() : s;
    if (str) {
        printer(str, val);
    }
}
export function ML_ERR_return_NAN2(printer, getExtraInfo) {
    if (printer.enabled) {
        ML_ERROR2(ME.ME_DOMAIN, getExtraInfo(), printer);
    }
    return NaN;
}
export function R_Q_P01_boundaries(lower_tail, log_p, p, _LEFT_, _RIGHT_) {
    if (log_p) {
        if (p > 0) {
            return ML_ERR_return_NAN2(debug_R_Q_P01_boundaries, lineInfo4);
        }
        if (p === 0)
            return lower_tail ? _RIGHT_ : _LEFT_;
        if (p === -Infinity)
            return lower_tail ? _LEFT_ : _RIGHT_;
    } else {
        if (p < 0 || p > 1) {
            return ML_ERR_return_NAN2(debug_R_Q_P01_boundaries, lineInfo4);
        }
        if (p === 0)
            return lower_tail ? _LEFT_ : _RIGHT_;
        if (p === 1)
            return lower_tail ? _RIGHT_ : _LEFT_;
    }
    return undefined;
}
export function R_Q_P01_check(logP, p) {
    if (logP && p > 0 || !logP && (p < 0 || p > 1)) {
        return ML_ERR_return_NAN2(debug_R_Q_P01_check, lineInfo4);
    }
    return undefined;
}